Package: pls
Title: Partial Least Squares and Principal Component Regression
Version: 2.9-0
Date: 2026-02-20
Authors@R: c(person("Kristian Hovde", "Liland", role = c("aut", "cre"),
                    email = "kristian.liland@nmbu.no"),
	     person("Bjørn-Helge", "Mevik", role = "aut" ,
                    email = "b-h@mevik.net"),
             person("Ron", "Wehrens", role = "aut"),
             person("Paul",  "Hiemstra", role = "ctb"))
Author: Kristian Hovde Liland [aut, cre],
  Bjørn-Helge Mevik [aut],
  Ron Wehrens [aut],
  Paul Hiemstra [ctb]
Maintainer: Kristian Hovde Liland <kristian.liland@nmbu.no>
Encoding: UTF-8
LazyData: yes
Description: Multivariate regression methods
	Partial Least Squares Regression (PLSR), Principal Component
	Regression (PCR) and Canonical Powered Partial Least Squares (CPPLS).
Depends: R (>= 3.5.0)
Imports: grDevices, graphics, methods, stats
Suggests: MASS, parallel, Rmpi, testthat, RUnit, knitr
License: GPL-2
URL: https://github.com/khliland/pls, https://khliland.github.io/pls/
BugReports: https://github.com/khliland/pls/issues
Config/testthat/edition: 2
RoxygenNote: 7.3.3
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2026-02-20 20:26:00 UTC; kristian
Repository: CRAN
Date/Publication: 2026-02-21 06:10:07 UTC
Built: R 4.6.0; ; 2026-04-25 16:27:32 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(MSEP,mvr)
S3method(R2,mvr)
S3method(RMSEP,mvr)
S3method(as.data.frame,mvrVal)
S3method(biplot,mvr)
S3method(coef,mvr)
S3method(fitted,mvr)
S3method(loadingplot,default)
S3method(loadings,default)
S3method(makepredictcall,msc)
S3method(makepredictcall,stdized)
S3method(model.frame,mvr)
S3method(model.matrix,mvr)
S3method(plot,loadings)
S3method(plot,mvr)
S3method(plot,mvrVal)
S3method(plot,scores)
S3method(predict,msc)
S3method(predict,mvr)
S3method(predict,stdized)
S3method(predplot,default)
S3method(predplot,mvr)
S3method(print,jacktest)
S3method(print,mvr)
S3method(print,mvrVal)
S3method(residuals,mvr)
S3method(scoreplot,default)
S3method(scores,default)
S3method(summary,mvr)
S3method(vcov,mvr)
export(MSEP)
export(R2)
export(RMSEP)
export(Yloadings)
export(Yscores)
export(coefplot)
export(compnames)
export(corrplot)
export(cppls)
export(cppls.fit)
export(crossval)
export(cvsegments)
export(explvar)
export(fac2seg)
export(jack.test)
export(kernelpls.fit)
export(loading.weights)
export(loadingplot)
export(loadings)
export(msc)
export(mvr)
export(mvrValstats)
export(nipals)
export(nipals.fit)
export(nipalspc.fit)
export(nipalspcr)
export(oscorespls.fit)
export(pcr)
export(pls.options)
export(plsr)
export(prednames)
export(predplot)
export(predplotXy)
export(respnames)
export(scoreplot)
export(scores)
export(selectNcomp)
export(simpls.fit)
export(stdize)
export(svdpc.fit)
export(validationplot)
export(var.jack)
export(widekernelpls.fit)
importFrom(grDevices,dev.interactive)
importFrom(grDevices,devAskNewPage)
importFrom(graphics,abline)
importFrom(graphics,arrows)
importFrom(graphics,axTicks)
importFrom(graphics,axis)
importFrom(graphics,legend)
importFrom(graphics,matplot)
importFrom(graphics,mtext)
importFrom(graphics,pairs)
importFrom(graphics,par)
importFrom(graphics,plot)
importFrom(graphics,points)
importFrom(graphics,segments)
importFrom(graphics,text)
importFrom(graphics,title)
importFrom(methods,hasArg)
importFrom(stats,.checkMFClasses)
importFrom(stats,coef)
importFrom(stats,cor)
importFrom(stats,delete.response)
importFrom(stats,drop.terms)
importFrom(stats,fitted)
importFrom(stats,formula)
importFrom(stats,makepredictcall)
importFrom(stats,median)
importFrom(stats,model.frame)
importFrom(stats,model.matrix)
importFrom(stats,model.response)
importFrom(stats,na.omit)
importFrom(stats,na.pass)
importFrom(stats,napredict)
importFrom(stats,optimize)
importFrom(stats,predict)
importFrom(stats,printCoefmat)
importFrom(stats,pt)
importFrom(stats,residuals)
importFrom(stats,runif)
importFrom(stats,sd)
importFrom(stats,terms)
importFrom(stats,update)
importFrom(stats,var)

<h1 align="center">
  <img height="128px" src="https://raw.githubusercontent.com/dhruvkb/pls/main/readme_assets/pls.svg"/>
</h1>

<p align="center">
  <a href="https://pypi.org/project/pls/">
    <img src="https://img.shields.io/pypi/v/pls" alt="pls on PyPI"/>
  </a>
  <a href="https://www.python.org">
    <img src="https://img.shields.io/pypi/pyversions/pls" alt="Python versions"/>
  </a>
  <a href="https://github.com/dhruvkb/pls/blob/main/LICENSE">
    <img src="https://img.shields.io/github/license/dhruvkb/pls" alt="GPL-3.0"/>
  </a>
  <a href="https://github.com/dhruvkb/pls/actions/workflows/ci.yml">
    <img src="https://github.com/dhruvkb/pls/actions/workflows/ci.yml/badge.svg" alt="CI status"/>
  </a>
</p>

<p align="center">
  <img src="https://raw.githubusercontent.com/dhruvkb/pls/main/readme_assets/demo.png" alt="Demo of `pls`"/>
</p>

`pls` is a better `ls` for developers. The "p" stands for "pro" (as in "professional"/"programmer") or "prettier", whichever helps you remember the command name.

It works in a manner similar to `ls`, in  that it lists directories and files in a given directory, but it adds many more [developer-friendly features](#features).

Note that `pls` is not a replacement for `ls`. `ls` is a tried, tested and trusted command with lots of features. `pls`, on the other hand, is a simple tool for people who just want to see the contents of their directories.

## Features

`pls` provides many features over  `ls` command. `pls` can:

- show Nerd Font icons or emoji next to files and directories making it easier to read the output
- colour output to elevate important files or dim unimportant ones
- use a more nuanced approach to hidden files than plainly hiding files with a leading dot `.`
- group directories and shows them all before files
- ignore leading dots `.` and normalise case when sorting files
- align files names by first character
- generate visibility rules by parsing `.gitignore`
- show technical two-letter Git status for files and directories
- cascade formatting rule specs by based on specificity levels
- read [`.pls.yml`](.pls.yml) files from the directory to augment its configuration
- show more details like permissions, owner and size in columns
- run quite fast, thanks to multiprocessing

The icon, color and most behaviour in the application can be [configured using plain-text YAML files](src/pls/data/README.md) for the pros who prefer to tweak their tools.

## Upcoming features

In the future `pls` will be able to

- add MIME type as another method for matching files to specs
- use complete path based matching for files
- link files and hide derived files behind the main ones
- generate tree-like output for subdirectories

If you want to help implement any of these features, feel free to submit a PR. `pls` is free and open-source software.

## Comparison with similar tools

There are a lot of `ls` replacements. Here are some of the most popular ones.

- [`exa`](https://github.com/ogham/exa)
- [`lsd`](https://github.com/Peltoche/lsd)
- [`colorls`](https://github.com/athityakumar/colorls)
- [`ls-go`](https://github.com/acarl005/ls-go)

`pls` aims to stand out because of some very specific choices.

- Does not intend to replace `ls`. `pls`, as a command, is just as easy to type.
- Targets a more tech-savvy audience in its [features](#features).
- Intelligently [maps file type](src/pls/data/README.md). Just comparing the file extension would be too generic.
- Meticulously chosen iconography for the appreciating eyes.
- Highly customisable at a project level using a simple [`.pls.yml`](.pls.yml) file.
- Built in a friendly language, Python. This makes it easy to fork and change it yourself.

## Installation

To get the best of `pls`, [install a Nerd Font](https://github.com/ryanoasis/nerd-fonts/blob/master/readme.md#font-installation) on your computer. [Nerd Fonts](https://www.nerdfonts.com) come patched with many icons from different popular icon sets. If you're a "pro" (the target audience for `pls`) these fonts are basically a must.

`pls` is a pure-Python codebase and is deployed to PyPI. So installing it on any system with a supported Python version is quite straightforward.

```shell
$ python3 -m pip install --user pls
```

There are no native packages _yet_.

## Usage

`pls` has a very simple API with easy to memorise flags. There are no mandatory arguments. Just run `pls` anywhere on your disk.

```shell
$ pls
```

There are a few optional arguments and flags you can use to tweak the behaviour. You can see the complete list of arguments and their description by passing the `--help` or `-h` flags.

```shell
$ pls --help
```

### Directory

The only positional argument is a directory. Pass this to see the contents of a different folder rather than the current working directory.

```shell
$ pls path/to/somewhere/else
```

### Icons

`pls` supports many icons for popular languages out of the box and will show icons by default. If you don't have a Nerd Font (why?), you can switch to emoji icons using `--icons emoji` or `-iemoji`. Be warned they are quite bad. If you are a sad person, you turn icons off using `--icon none` or `-inone`.

**Note:** The built-in icon configuration is intentionally lean. The whole idea is for `pls` to be [customisable by you](src/pls/data/README.md).

### Filtering

You can choose to hide files or folders from the output using `--no-files` and `--no-dirs` respectively. Passing both will lead to a blank output. On the other hand if you want to see files and directories that `pls` would not show otherwise, pass `--all`.

### Sorting

By default `pls` will place all directories first, followed by files with both sorted alphabetically from A to Z. You can prevent folders from being first by passing the `--no-dirs-first` flag. You can change the sort to go from Z to A using `--sort desc` or `-sdesc`. Leading dots are ignored during sorting.

### Alignment

A lot of code related files start with a leading dot `.` for no valid reason. `pls` by default

- moves their name left by one character to line up the actual alphabets
- dims their leading dot

If you don't like this, you can set `--no-align` to turn off all this behaviour in one swoop.

### Details

When you need more infomation about your files, pass the `--details` flag. This expands the list into a table, with

- permissions
- owner name
- size

added to the output. The permissions are presented as `rwx` triplets. The size is presented in binary compound-units (the ones with the "i" like "*iB"). You can switch to decimal units by passing `--units decimal` or `-udecimal`. This flag has no effect unless the `--detail` flag is passed too.

# Node specs

The node spec is arguably the most important component of the codebase.

## Cascade

When a node is being matched against specs, the specs are compared in order of their definition in the `node_specs.yml` file. All matching specs are stored in the node. Then attributes are then read from the matches in first to last order. Any missing attributes will be read from the next match. This will enable the more specific specs to override certain attributes of the more generic ones.

For an example, refer to this project's own `.pls.yml` file.

## Importance

Specs have two functions.

- mark some files with a leading dot as meaningful enough to be shown  
  This can be accomplished with a simple spec containing just a `name`.
- mark some files without a leading dot as meaningless  
  This can be accomplished with a spec with -2 importance.
- decorate files with icons and colors  
  These improve the output both aesthetically and functionally.

## Specification

A node spec instance consists of
- one of
  - [name](#name): `str`
  - [pattern](#pattern): `str`
  - [extension](#extension): `str`
- [icon](#icon): `str?`
- [color](#color): `str?`
- [importance](#importance): `int?`

When matching nodes to specs, care must be taken to ensure that name matches supersede pattern matches which supersede extension matches, by placing them in this specific order.

### name

This is a string which describes the entire file name, including the extension.

#### Examples

- `Pipfile`
  - matches `Pipfile`
  - does not match `Pipfile.lock`

### pattern

This is a string containing a regular expression compared against the entire file name, including the extension. The file is considered to be a match if the regex matches the file name from the start. This is the most versatile of all identification approaches.

Note that these are regex patterns and not globs. Also since the regex is matched from the start of the string, the leading caret `^` is not needed. However, to match the end of the string, a trailing dollar sign `$` is.

#### Examples

- `Dockerfile\b.*`:
  - matches `docker-compose.yml`
  - matches `docker-compose.prod.yml`
  - does not match `docker-compose.yml.prod`

### extension

This is a string which describes the file extension without a leading dot `.` character. To match files to extensions, the entire extension is compared.

#### Examples

- `js`:
  - matches `index.js`
  - does not match `component.jsx`

### icon

The name of the icon to show beside nodes matching this spec. This name must correspond to entries in `emoji_icons.yml` and `nerd_icons.yml` to actually render the icon.

There are some caveats with emojis and nerd icons. These are a result of these icons visually taking up more than one terminal row but only occupying one. Most of these can be resolved by using a more advanced terminal application such as [iTerm2 for macOS](https://iterm2.com).

#### References

- [Nerd Fonts](https://www.nerdfonts.com/cheat-sheet)
- [Emojipedia](https://emojipedia.org)

### color

The name of the color in which to write nodes matching this spec. This must be one of the following.

- name of a standard color
- hex code `#ffffff`
- RGB triplet `rgb(255,255,255)`

#### References

- [Rich docs](https://rich.readthedocs.io/en/stable/appendix/colors.html)

### importance

The visual importance to associate with nodes matching this spec. This must be a number between -2 and 1.

- -2: the least importance, _hidden_  
  This corresponds to regular files you would not want to see at all
- -1: slightly lower importance, _dimmed_  
  This corresponds to autogenerated or downloaded files
- 0: normal importance  
  This is the default importance of all files unless specified
- 1: slightly higher importance, _bold_  
  This corresponds to important files like source code
- 2: the highest importance, _underline_  
  This corresponds to extremely important files that must be immediately seen

## Shorthand spec

In the interest of brevity, `name`, `pattern` and `extension` can be replaced with `names`, `patterns` and `extensions`. These plural forms take an array with the rules applying to each item of the array. These shorthand specs are expanded with each item in the plural key collated with the other attributes.

### utils.R: Utilities for the test suites
### By Bjørn-Helge Mevik
### Started 2007-10-18

## scalecomps:  Utility function to scale and/or remove sign
## differences of all scores/loadings in a fit
scalecomps <- function(fit, scale = FALSE, sign = FALSE) {
    ## Matrices that are to be scaled and/or signed:
    for (nm in c("loadings", "scores", "loading.weights", "Yscores",
                 "Yloadings", "projection")) {
        if(!is.null(fit[[nm]])) {
            if (scale) {
                fit[[nm]] <- scale(fit[[nm]], center = FALSE)
                attr(fit[[nm]], "scaled:scale") <- NULL
            }
            if (sign) fit[[nm]] <- abs(fit[[nm]])
        }
    }
    fit
}

### runit.algorithms.R: test functions for checking algorithm accurracy
### By Bjørn-Helge Mevik
### Started 2007-08-22

###
### Check that all algorithms give the same results
###

## test.plsrRes: check that the plsr algorithms give the same results
test.plsrRes <- function() {
    ## Load single response reference model (created with oscorespls):
    load("RUnit/common/ref_singresp.RData")
    oscmod <- mvr(form, nc = nc, data = mydata, method = "oscorespls")
    oscmod$method <- oscmod$call <- oscmod$fit.time <- oscmod$nobj <- oscmod$nobjOrig <- NULL
    checkEquals(refmod, oscmod)
    kernmod <- mvr(form, nc = nc, data = mydata, method = "kernelpls")
    kernmod$method <- kernmod$call <- kernmod$fit.time <- kernmod$nobj <- kernmod$nobjOrig <- NULL
    ## MSWin: ok with 49 comps, Linux/64: 47:
    ## oscores Yscores (5) have different scaling
    checkEquals(refmod[-5], kernmod[-5])
    widekernmod <- mvr(form, nc = nc, data = mydata, method = "widekernelpls")
    widekernmod$method <- widekernmod$call <- widekernmod$fit.time <- widekernmod$nobj <- widekernmod$nobjOrig <- NULL
    widekernmod$nits <- NULL            # widekern also returns nits
    ## MSWin: ok with 44 comps, Linux/64: 43:
    ## oscores Yscores have different scaling
    checkEquals(refmod[-5], widekernmod[-5])
    ## MSWin: ok with 44 comps, Linux/64: 43:
    checkEquals(kernmod, widekernmod)
    simplsmod <- mvr(form, nc = nc, data = mydata, method = "simpls")
    simplsmod$method <- simplsmod$call <- simplsmod$fit.time <- simplsmod$nobj <- simplsmod$nobjOrig <- NULL
    scaled.simplsmod <- scalecomps(simplsmod, scale = TRUE)
    ## MSWin: ok with 36 comps, Linux/64: 27:
    refmod$loading.weights <- NULL      # simpls doesn't give loading weights
    checkEquals(scalecomps(refmod, scale = TRUE), scaled.simplsmod)
    ## MSWin: ok with 36 comps, Linux/64: 27:
    kernmod$loading.weights <- NULL      # simpls doesn't give loading weights
    checkEquals(scalecomps(kernmod, scale = TRUE), scaled.simplsmod)
    ## MSWin: ok with 36 comps, Linux/64: 27:
    widekernmod$loading.weights <- NULL # simpls doesn't give loading weights
    widekernmod$nits <- NULL            # widekern also returns nits
    checkEquals(scalecomps(widekernmod, scale = TRUE), scaled.simplsmod)

    ## Multi-response model:
    ## Load multi response reference model (created with oscorespls):
    load("RUnit/common/ref_multiresp.RData")
    ## Note: the algs give sign differences, in general, for scores
    ## and loadings
    oscmod <- mvr(form, nc = nc, data = mydata, method = "oscorespls")
    oscmod$method <- oscmod$call <- oscmod$fit.time <- oscmod$nobj <- oscmod$nobjOrig <- NULL
    for (i in 2:7) oscmod[[i]] <- abs(oscmod[[i]])
    checkEquals(refmod, oscmod)
    kernmod <- mvr(form, nc = nc, data = mydata, method = "kernelpls")
    kernmod$method <- kernmod$call <- kernmod$fit.time <- kernmod$nobj <- kernmod$nobjOrig <- NULL
    for (i in 2:7) kernmod[[i]] <- abs(kernmod[[i]])
    ## MSWin: ok with 5 comps, Linux/64: 5:
    checkEquals(refmod, kernmod)
    widekernmod <- mvr(form, nc = nc, data = mydata, method = "widekernelpls")
    widekernmod$method <- widekernmod$call <- widekernmod$fit.time <- widekernmod$nobj <- widekernmod$nobjOrig <- NULL
    widekernmod$nits <- NULL            # widekern also returns nits
    for (i in 2:7) widekernmod[[i]] <- abs(widekernmod[[i]])
    ## MSWin: ok with 4 comps, Linux/64: 4:
    checkEquals(refmod, widekernmod)
    ## MSWin: ok with 4 comps, Linux/64: 4:
    checkEquals(kernmod, widekernmod)

    ## SimPLS is supposed to give (slightly) different results for
    ## multi-response models, so we do not compare it
}

### runit.CV.R: test functions for checking cross-validation functions
### By Bjørn-Helge Mevik
### Started 2007-10-18


## Basic cross-validation:
test.CV <- function() {
    ##  1 resp, 5-fold, specified # comps
    cvmod1 <- mvr(density ~ NIR, ncomp = 10, data = yarn, validation = "CV",
                   segments = 5, segment.type = "interleaved")
    omod <- mvr(density ~ NIR, ncomp = 10, data = yarn)
    cvmod2 <- crossval(omod, segments = 5, segment.type = "interleaved")
    ## Check validation component:
    checkEquals(cvmod1$validation, cvmod2$validation, "1 resp, validation")
    ## Remove components that will not match or have been tested:
    cvmod1$call <- cvmod2$call <- NULL
    cvmod1$fit.time <- cvmod2$fit.time <- NULL
    cvmod1$validation <- cvmod2$validation <- NULL
    checkEquals(cvmod1, cvmod2, "1 resp, the rest")

    ## 5 resps, LOO, unspecified # comps, without standardisation
    cvmod1 <- mvr(sensory ~ chemical, data = oliveoil,
                   validation = "LOO")
    omod <- mvr(sensory ~ chemical, data = oliveoil)
    cvmod2 <- crossval(omod, length.seg = 1)
    ## Check validation component:
    cvmod1$validation$segments <- cvmod2$validation$segments <- NULL # Segment
                                        # order and attributes will not match
    checkEquals(cvmod1$validation, cvmod2$validation, "5 resps, validation")
    ## Remove components that will not match or have been tested:
    cvmod1$call <- cvmod2$call <- NULL
    cvmod1$fit.time <- cvmod2$fit.time <- NULL
    cvmod1$validation <- cvmod2$validation <- NULL
    checkEquals(cvmod1, cvmod2, "5 resps, the rest")

    ## 5 resps, LOO, unspecified # comps, with standardisation
    cvmod1 <- mvr(sensory ~ chemical, data = oliveoil,
                   validation = "LOO", scale = TRUE)
    omod <- mvr(sensory ~ stdize(chemical), data = oliveoil)
    cvmod2 <- crossval(omod, length.seg = 1)
    ## Check the scaling used:
    checkEquals(cvmod1$scale, attr(model.frame(cvmod2)[[2]], "stdized:scale"),
               "5 resps, std, scale")
    ## Check the validation component:
    cvmod1$validation$segments <- cvmod2$validation$segments <- NULL # Segment
                                        # order and attributes will not match
    checkEquals(cvmod1$validation, cvmod2$validation,
                "5 resps, std, validation")
    ## Remove components that will not match or have been tested:
    cvmod1$call <- cvmod2$call <- NULL      # The calls will not match
    cvmod1$Xmeans <- cvmod2$Xmeans <- NULL  # cvmod2$Xmeans == 0
    cvmod1$terms <- cvmod2$terms <- NULL
    cvmod1$model <- cvmod2$model <- NULL
    cvmod1$scale <- NULL                    # crossval() doesn't add scale
    cvmod1$fit.time <- cvmod2$fit.time <- NULL
    cvmod1$validation <- cvmod2$validation <- NULL
    cvmod1 <- scalecomps(cvmod1, sign = TRUE) # Remove sign differences
    cvmod2 <- scalecomps(cvmod2, sign = TRUE) # Remove sign differences
    checkTrue(all.equal(cvmod1, cvmod2, check.attributes = FALSE),
              "5 resps, std, the rest")

    ## 5 resps, LOO, unspecified # comps, without centering
    cvmod1 <- mvr(sensory ~ chemical, data = oliveoil,
                   validation = "LOO", center = FALSE)
    omod <- mvr(sensory ~ chemical, data = oliveoil, center = FALSE)
    cvmod2 <- crossval(omod, length.seg = 1)
    ## Check the validation component:
    cvmod1$validation$segments <- cvmod2$validation$segments <- NULL # Segment
                                        # order and attributes will not match
    checkEquals(cvmod1$validation, cvmod2$validation,
                "5 resps, uncent, validation")
    ## Remove components that will not match or have been tested:
    cvmod1$call <- cvmod2$call <- NULL      # The calls will not match
    cvmod1$fit.time <- cvmod2$fit.time <- NULL
    cvmod1$validation <- cvmod2$validation <- NULL
    checkTrue(all.equal(cvmod1, cvmod2, check.attributes = FALSE),
              "5 resps, uncent, the rest")
}

## Test cvsegments()
test.cvsegments <- function() {
    ## Data set without replicates (nrep = 1)
    N <- 10
    ok_segs <- list(random = list(), consecutive = list(), interleaved = list())
    ok_segs[[c("consecutive", 1)]] <- list(1:10)
    ok_segs[[c("consecutive", 2)]] <- list(1:5, 6:10)
    ok_segs[[c("consecutive", 3)]] <- list(1:4, 5:7, 8:10)
    ok_segs[[c("interleaved", 1)]] <- list(1:10)
    ok_segs[[c("interleaved", 2)]] <- list(c(1,3,5,7,9), c(2,4,6,8,10))
    ok_segs[[c("interleaved", 3)]] <- list(c(1,4,7,10), c(2,5,8), c(3,6,9))
    for (type in c("random", "consecutive", "interleaved")) {
        for (k in c(1,2,3)) {
            segs <- cvsegments(N = N, k = k, type = type)
            test_id <- paste("N = ", N, ", k = ", k, ", type = ", type)
            if (type == "random") {
                switch(as.character(k),
                       "1" = {
                           checkTrue(all.equal(1:N, sort(segs[[1]]),
                                               check.attributes = FALSE),
                                     test_id)
                       },
                       "2" = {
                           checkTrue(
                               length(segs) == k &&
                               all(sapply(segs, length) == N/k) &&
                               length(intersect(segs[[1]], segs[[2]])) == 0,
                               test_id)
                       },
                       "3" = {
                           checkTrue(
                               length(segs) == k &&
                               all(sapply(segs, length) == c(4, 3, 3)) &&
                               length(intersect(segs[[1]], segs[[2]])) == 0 &&
                               length(intersect(segs[[1]], segs[[3]])) == 0 &&
                               length(intersect(segs[[2]], segs[[3]])) == 0,
                               test_id)
                       }
                       )
            } else {
                checkTrue(all.equal(ok_segs[[c(type, k)]], segs,
                                    check.attributes = FALSE),
                          test_id)
            }
        }
    }

    ## Data set with replicates (nrep > 1)
    N <- 20
    nrep <- 2
    ok_segs <- list(random = list(), consecutive = list(), interleaved = list())
    ok_segs[[c("consecutive", 1)]] <- list(1:20)
    ok_segs[[c("consecutive", 2)]] <- list(1:10, 11:20)
    ok_segs[[c("consecutive", 3)]] <- list(1:8, 9:14, 15:20)
    ok_segs[[c("interleaved", 1)]] <- list(1:20)
    ok_segs[[c("interleaved", 2)]] <- list(c(1,2,5,6,9,10,13,14,17,18),
                                           c(3,4,7,8,11,12,15,16,19,20))
    ok_segs[[c("interleaved", 3)]] <- list(c(1,2,7,8,13,14,19,20),
                                           c(3,4,9,10,15,16),
                                           c(5,6,11,12,17,18))
    for (type in c("random", "consecutive", "interleaved")) {
        for (k in c(1,2,3)) {
            if (k == 3) options(warn = -1) # Ignore warning
            segs <- cvsegments(N = N, k = k, nrep = nrep, type = type)
            if (k == 3) options(warn = 0)
            test_id <- paste("N = ", N, ", k = ", k, ", nrep = ", nrep,
                             ", type = ", type)
            if (type == "random") {
                switch(as.character(k),
                       "1" = {
                           checkTrue(all.equal(1:N, sort(segs[[1]]),
                                               check.attributes = FALSE),
                                     test_id)
                       },
                       "2" = {
                           checkTrue(
                               length(segs) == k &&
                               all(sapply(segs, length) == N/k) &&
                               length(intersect(segs[[1]], segs[[2]])) == 0,
                               test_id)
                       },
                       "3" = {
                           checkTrue(
                               length(segs) == k &&
                               all(sapply(segs, length) == c(8, 6, 6)) &&
                               length(intersect(segs[[1]], segs[[2]])) == 0 &&
                               length(intersect(segs[[1]], segs[[3]])) == 0 &&
                               length(intersect(segs[[2]], segs[[3]])) == 0,
                               test_id)
                       }
                       )
            } else {
                checkTrue(all.equal(ok_segs[[c(type, k)]], segs,
                                    check.attributes = FALSE),
                          test_id)
            }
        }
    }
}

randomiz.test <- function(residualsNew, residualsReference, nperm) {
    d <- residualsNew^2 - residualsReference^2
    md <- mean(d)
    N <- length(d)

    signs <- round(matrix(runif(N * nperm), N, nperm)) * 2 - 1
    dsigns <- d * signs
    mdsigns <- colMeans(dsigns)

    count <- sum(mdsigns >= md) ## equality will never occur, right?
    (count + .5) / (nperm + 1)
}

## Starting from the minimum in the CV results do formal randomization
## tests for the models with fewer components or apply the one-sigma
## criterion. This is done on the validation data. At this moment only
## for univariate Y, and for the "CV" estimate.


#' @title Suggestions for the optimal number of components in PCR and PLSR models
#'
#' @description Choosing the best number of components in PCR and PLSR models is difficult
#' and usually done on the basis of visual inspection of the validation plots.
#' In cases where large numbers of models are built this choice needs to be
#' automated. This function implements two proposals, one based on
#' randomization (permutation) testing, and an approach based on the standard
#' error of the cross-validation residuals.
#'
#' @details In both approaches the results of cross-validation are used, so the model
#' should have been calculated with some form of cross-validation. First, the
#' absolute minimum in the CV curve is determined (considering only the first
#' ncomp components), leading to the reference model. The randomization test
#' approach (Van der Voet, 1994) checks whether the squared prediction errors
#' of models with fewer components are significantly larger than in the
#' reference model. This leads for each model considered to a \eqn{p} value;
#' the smallest model not significantly worse than the reference model is
#' returned as the selected one.
#'
#' The approach \code{"onesigma"} simply returns the first model where the
#' optimal CV is within one standard error of the absolute optimum (Hastie,
#' Tibshirani and Friedman, 2009). Note that here we simply use the standard
#' deviation of the cross-validation residuals, in line with the procedure used
#' to calculate the error measure itself. Some other packages implementing
#' similar procedures (such as \code{glmnet}) calculate an error measure for
#' each validation segment separately and use the average as the final
#' estimate. In such cases the standard error across segments is the relevant
#' measure of spread. For LOO, the two procedures are identical. In other forms
#' of validation, small differences will occur.
#'
#' @param object an \code{mvr} object.  The fitted model. It should contain a
#' \code{validation} element.
#' @param method character string, indicating the heuristic to use.
#' @param nperm number of permutations in the \code{"randomization"} approach -
#' not used in the \code{"onesigma"} approach.
#' @param alpha cutoff for p values in the \code{"randomization"} approach -
#' not used in the \code{"onesigma"} approach.
#' @param ncomp maximum number of components to consider when determining the
#' global minimum in the cross-validation curve.
#' @param plot whether or not to show a cross-validation plot. The plot for the
#' \code{"randomization"} approach shows models that do not differ
#' significantly from the global RMSEP minimum with open circles; the
#' \code{"onesigma"} approach shows the one-sigma bands around the RMSEP
#' values. In both cases, the selection is indicated with a blue dashed line.
#' @param \dots Further plotting arguments, e.g., to add a title to the plot,
#' or to limit the plotting range.
#' @return A number indicating the suggested number of components in the model.
#' @author Ron Wehrens, Hilko van der Voet and Gerie van der Heijden
#' @seealso \code{\link{mvr}}, \code{\link{summary.mvr}}
#' @references Van der Voet, H. (1994) Comparing the predictive accuracy of
#' models using a simple randomization test. Chemom. Intell. Lab. Syst. 25 (2),
#' 313-323
#'
#' Hastie, T., Friedman, J. and Tibshirani, R. The Elements of Statistical
#' Learning: data mining, inference, and prediction, Springer (2013), 10th
#' printing with corrections, paragraph 7.10.
#' @keywords regression multivariate
#' @examples
#'
#' data(yarn)
#' yarn.pls <- plsr(density ~ NIR, data = yarn, scale = TRUE,
#'                  ncomp = 20, validation = "LOO")
#' selectNcomp(yarn.pls, "onesigma", plot = TRUE, ylim = c(0, 3))
#' selectNcomp(yarn.pls, "randomization", plot = TRUE)
#' selectNcomp(yarn.pls, "randomization", plot = TRUE,
#'             ncomp = 10, ylim = c(0, 3))
#'
#' @export
selectNcomp <- function(object,
                        method = c("randomization", "onesigma"),
                        nperm = 999, alpha = 0.01, ncomp = object$ncomp,
                        plot = FALSE, ...) {
    if (!isTRUE(object$validation$method == "CV"))
        stop("No cross-validation data available in model")
    ## check that Y is univariate
    if (dim(residuals(object))[2] > 1)
        stop("Only univariate response supported")

    rmseps <- c(RMSEP(object, "CV")$val) ## includes zero
    maxIdx <- ncomp + 1
    absBest <- which.min(rmseps[seq_len(maxIdx)])

    if (absBest > 0) {
        method <- match.arg(method)
        if (is.null(origResponse <- object$y))
            origResponse <-
                c(predict(object, ncomp = 1) + residuals(object)[,1,1])

        ## include LOO prediction with zero components (i.e., the
        ## mean). For the mean we should also use the LOO estimate...
        allresids <- cbind(origResponse -
                               (sum(origResponse) - origResponse) /
                                   (length(origResponse) - 1),
                           object$validation$pred[,1,] - origResponse)

        if (method == "randomization") {
            pvals <- sapply(seq_len(absBest - 1),
                            function(ii) randomiz.test(allresids[,ii],
                                                       allresids[,absBest],
                                                       nperm = nperm))
            idx <- which(pvals > alpha)
            selection <- min(c(idx, absBest)) - 1
        } else {
            residsds <- apply(allresids, 2, sd) / sqrt(nrow(allresids))
            uls <- rmseps - residsds
            selection <- min(which(uls < rmseps[absBest])) - 1
        }

        if (isTRUE(plot)) {
            xvals <- seq_along(rmseps) - 1
            plot(xvals, rmseps, ylab = "RMSEP",
                 xlab = "Number of components", type = "b", ...)
            if (method == "onesigma") {
                arrows(xvals, uls, xvals, rmseps + residsds,
                       code = 3, col = "gray", angle = 90, length = .1)
            } else {
                points(xvals[idx], rmseps[idx], cex = 2, col = 4)
            }
            abline(h = rmseps[absBest], col = "gray", lty = 3)
            abline(v = absBest - 1, col = "gray", lty = 3)
            abline(v = selection, col = "blue", lty = 2)
            legend("topright", legend = c("Abs. minimum", "Selection"),
                   lty = 3:2, col = c("gray", "blue"), bg = "white")
        }

        selection
    } else {
        warning("Lowest CV error found at 0 components, no testing performed")
        0
    }
}

### svdpc.fit.R: SVD PC fit algorithm



#' @title Principal Component Regression
#'
#' @description Fits a PCR model using the singular value decomposition.
#'
#' @details This function should not be called directly, but through the generic
#' functions \code{pcr} or \code{mvr} with the argument \code{method="svdpc"}.
#' The singular value decomposition is used to calculate the principal
#' components.
#'
#' @param X a matrix of observations.  \code{NA}s and \code{Inf}s are not
#' allowed.
#' @param Y a vector or matrix of responses.  \code{NA}s and \code{Inf}s are
#' not allowed.
#' @param ncomp the number of components to be used in the modelling.
#' @param center logical, determines if the \eqn{X} and \eqn{Y} matrices are
#' mean centered or not. Default is to perform mean centering.
#' @param stripped logical.  If \code{TRUE} the calculations are stripped as
#' much as possible for speed; this is meant for use with cross-validation or
#' simulations when only the coefficients are needed.  Defaults to
#' \code{FALSE}.
#' @param \dots other arguments.  Currently ignored.
#' @return A list containing the following components is returned:
#' \item{coefficients}{an array of regression coefficients for 1, \ldots{},
#' \code{ncomp} components.  The dimensions of \code{coefficients} are
#' \code{c(nvar, npred, ncomp)} with \code{nvar} the number of \code{X}
#' variables and \code{npred} the number of variables to be predicted in
#' \code{Y}.} \item{scores}{a matrix of scores.} \item{loadings}{a matrix of
#' loadings.} \item{Yloadings}{a matrix of Y-loadings.} \item{projection}{the
#' projection matrix used to convert X to scores.} \item{Xmeans}{a vector of
#' means of the X variables.} \item{Ymeans}{a vector of means of the Y
#' variables.} \item{fitted.values}{an array of fitted values.  The dimensions
#' of \code{fitted.values} are \code{c(nobj, npred, ncomp)} with \code{nobj}
#' the number samples and \code{npred} the number of Y variables.}
#' \item{residuals}{an array of regression residuals.  It has the same
#' dimensions as \code{fitted.values}.} \item{Xvar}{a vector with the amount of
#' X-variance explained by each component.} \item{Xtotvar}{Total variance in
#' \code{X}.}
#'
#' If \code{stripped} is \code{TRUE}, only the components \code{coefficients},
#' \code{Xmeans} and \code{Ymeans} are returned.
#' @author Ron Wehrens and Bjørn-Helge Mevik
#' @seealso \code{\link{mvr}} \code{\link{plsr}} \code{\link{pcr}}
#' \code{\link{cppls}}
#' @references Martens, H., Næs, T. (1989) \emph{Multivariate calibration.}
#' Chichester: Wiley.
#' @keywords regression multivariate
#' @export
svdpc.fit <- function(X, Y, ncomp, center = TRUE, stripped = FALSE, ...) {
    Y <- as.matrix(Y)
    if (!stripped) {
        ## Save dimnames:
        dnX <- dimnames(X)
        dnY <- dimnames(Y)
    }
    ## Remove dimnames during calculation  (doesn't seem to matter; in fact,
    ## as far as it has any effect, it hurts a tiny bit in most situations).
    ## dimnames(X) <- dimnames(Y) <- NULL

    nobj  <- dim(X)[1]
    npred <- dim(X)[2]
    nresp <- dim(Y)[2]

    B <- array(0, dim = c(npred, nresp, ncomp))
    if (!stripped) fitted <- array(0, dim = c(nobj, nresp, ncomp))

    ## Center variables:
    if (center) {
        Xmeans <- colMeans(X)
        X <- X - rep(Xmeans, each = nobj)
        Ymeans <- colMeans(Y)
        Y <- Y - rep(Ymeans, each = nobj)
    } else {
        ## Set means to zero. Will ensure that predictions do not take the
        ## mean into account.
        Xmeans <- rep_len(0, npred)
        Ymeans <- rep_len(0, nresp)
    }

    huhn <- La.svd(X)
    D  <- huhn$d[1:ncomp]
    TT <- huhn$u[,1:ncomp, drop=FALSE] %*% diag(D, nrow = ncomp)
    P  <- t(huhn$vt[1:ncomp,, drop=FALSE])
    tQ <- crossprod(TT, Y) / D^2

    for (a in 1:ncomp) {
        B[,,a] <- P[,1:a, drop=FALSE] %*% tQ[1:a,]
        if (!stripped) fitted[,,a] <- TT[,1:a, drop=FALSE] %*% tQ[1:a,]
    }

    if (stripped) {
        ## Return as quickly as possible
        list(coefficients = B, Xmeans = Xmeans, Ymeans = Ymeans)
    } else {
        residuals <- c(Y) - fitted
        fitted <- fitted + rep(Ymeans, each = nobj) # Add mean

        ## Add dimnames and classes:
        objnames <- dnX[[1]]
        if (is.null(objnames)) objnames <- dnY[[1]]
        prednames <- dnX[[2]]
        respnames <- dnY[[2]]
        compnames <- paste("Comp", 1:ncomp)
        nCompnames <- paste(1:ncomp, "comps")
        dimnames(TT) <- list(objnames, compnames)
        dimnames(P) <- list(prednames, compnames)
        dimnames(tQ) <- list(compnames, respnames)
        dimnames(B) <- list(prednames, respnames, nCompnames)
        dimnames(fitted) <- dimnames(residuals) <-
            list(objnames, respnames, nCompnames)
        names(D) <- compnames
        class(TT) <- "scores"
        R <- P                          # To avoid class "loadings" on projection
        class(P) <- class(tQ) <- "loadings"

        list(coefficients = B,
             scores = TT, loadings = P,
             Yloadings = t(tQ),
             projection = R,
             Xmeans = Xmeans, Ymeans = Ymeans,
             fitted.values = fitted, residuals = residuals,
             Xvar = D^2, Xtotvar = sum(X * X))
    }
}


#' @title NIPALS PCR with missing values
#'
#' @description A NIPALS-based PCR that tolerates missing entries in both
#' predictors and responses by only using observed cells when updating scores
#' and loadings.  It follows the same API as \code{svdpc.fit} so it can be used
#' whenever low-level PCR needs to handle incomplete data.
#'
#' @param X numeric matrix (or coercible) of predictors. Missing values are
#' allowed and handled internally.
#' @param Y numeric matrix (or coercible) of responses. Missing values are also
#' handled internally during the final regression step.
#' @param ncomp number of PCR components to extract.
#' @param center logical. If \code{TRUE} both \code{X} and \code{Y} are
#' centered column-wise (ignoring missing entries).
#' @param stripped logical. When \code{TRUE} only the coefficients and mean
#' vectors are returned for faster use in resampling.
#' @param maxiter maximum number of inner iterations per component.
#' @param tol convergence tolerance used when the direction vector stabilizes.
#' @param ... currently ignored.
#'
#' @return A list mirroring the return value of \code{svdpc.fit} but computed
#' via the NA-robust NIPALS PCR updates.
#' @export
nipalspc.fit <- function(X, Y, ncomp, center = TRUE, stripped = FALSE,
                         maxiter = 500, tol = 1e-06, ...)
{
    X <- as.matrix(X)
    Y <- as.matrix(Y)
    if (!stripped) {
        dnX <- dimnames(X)
        dnY <- dimnames(Y)
    }
    dimnames(X) <- dimnames(Y) <- NULL

    nobj <- nrow(X)
    npred <- ncol(X)
    nresp <- ncol(Y)

    mean_ignore_na <- function(mat) {
        colMeans(mat, na.rm = TRUE)
    }

    if (center) {
        Xmeans <- mean_ignore_na(X)
        Ymeans <- mean_ignore_na(Y)
        X <- sweep(X, 2, Xmeans, "-")
        Y <- sweep(Y, 2, Ymeans, "-")
    } else {
        Xmeans <- rep_len(0, npred)
        Ymeans <- rep_len(0, nresp)
    }

    Xcentre <- X
    Ycentre <- Y
    Xresid <- X
    maskX <- !is.na(Xcentre)

    safe_crossprod <- function(mat, vec) {
        apply(mat, 2, function(col) sum(col * vec, na.rm = TRUE))
    }

    safe_matvec <- function(mat, vec) {
        res <- numeric(nrow(mat))
        for (j in seq_len(ncol(mat))) {
            col <- mat[, j]
            idx <- !is.na(col)
            if (any(idx)) {
                res[idx] <- res[idx] + col[idx] * vec[j]
            }
        }
        res
    }

    safe_times <- function(mat, coeff) {
        res <- matrix(0, nrow = nrow(mat), ncol = ncol(coeff))
        for (j in seq_len(ncol(coeff))) {
            res[, j] <- safe_matvec(mat, coeff[, j])
        }
        res
    }

    normalize <- function(v) {
        n <- sum(v * v)
        if (n <= .Machine$double.eps) return(v)
        v / sqrt(n)
    }

    TT <- matrix(0, nrow = nobj, ncol = ncomp)
    P <- matrix(0, nrow = npred, ncol = ncomp)
    tsqs <- numeric(ncomp)
    B <- array(0, c(npred, nresp, ncomp))
    tQ <- matrix(0, nrow = ncomp, ncol = nresp)
    nused <- 0
    if (!stripped) {
        fitted <- array(0, c(nobj, nresp, ncomp))
    }

    for (a in seq_len(ncomp)) {
        start <- Xresid[, 1]
        start[is.na(start)] <- 0
        if (sum(start * start) == 0) {
            start <- rep_len(1, nobj)
        }
        w <- normalize(safe_crossprod(Xresid, start))
        if (sum(w * w) == 0) {
            warning("component ", a, " has zero loading weights", call. = FALSE)
            break
        }

        for (iter in seq_len(maxiter)) {
            t1 <- safe_matvec(Xresid, w)
            if (sum(t1 * t1) == 0) break
            p <- safe_crossprod(Xresid, t1) / sum(t1 * t1)
            w1 <- normalize(p)
            if (sum((w1 - w)^2) < tol^2) {
                w <- w1
                break
            }
            w <- w1
        }

        t1 <- safe_matvec(Xresid, w)
        tt <- sum(t1 * t1)
        if (tt == 0) {
            warning("component ", a, " collapsed to zero scores", call. = FALSE)
            break
        }

        p <- safe_crossprod(Xresid, t1) / tt
        TT[, a] <- t1
        P[, a] <- p
        tsqs[a] <- tt
        nused <- a

        reconX <- tcrossprod(t1, p)
        Xresid[maskX] <- Xresid[maskX] - reconX[maskX]
    }

    for (a in seq_len(nused)) {
        if (tsqs[a] <= 0) break
        tQ[a, ] <- safe_crossprod(Ycentre, TT[, a]) / tsqs[a]
    }

    for (a in seq_len(ncomp)) {
        if (a > nused) break
        B[,,a] <- P[, 1:a, drop = FALSE] %*% tQ[1:a, , drop = FALSE]
    }

    if (!stripped) {
        for (a in seq_len(ncomp)) {
            fitted[,,a] <- safe_times(Xcentre, matrix(B[,,a]))
        }
        residuals <- array(NA, c(nobj, nresp, ncomp))
        for (a in seq_len(ncomp)) {
            residuals[,,a] <- Ycentre - fitted[,,a]
        }
        if (center) {
            fitted <- fitted + rep(Ymeans, each = nobj)
        }
    }

    Xvar <- tsqs
    Xtotvar <- sum(Xcentre[maskX]^2)

    if (stripped) {
        return(list(coefficients = B, Xmeans = Xmeans, Ymeans = Ymeans))
    }

    objnames <- dnX[[1]]
    if (is.null(objnames)) objnames <- dnY[[1]]
    prednames <- dnX[[2]]
    respnames <- dnY[[2]]
    compnames <- paste("Comp", seq_len(ncomp))
    nCompnames <- paste(seq_len(ncomp), "comps")

    dimnames(TT) <- list(objnames, compnames)
    dimnames(P) <- list(prednames, compnames)
    dimnames(tQ) <- list(compnames, respnames)
    dimnames(B) <- list(prednames, respnames, nCompnames)
    dimnames(fitted) <- dimnames(residuals) <-
        list(objnames, respnames, nCompnames)
    names(Xvar) <- compnames
    class(TT) <- "scores"
    R <- P
    class(P) <- class(tQ) <- "loadings"

    list(coefficients = B,
         scores = TT, loadings = P,
         Yloadings = t(tQ),
         projection = R,
         Xmeans = Xmeans, Ymeans = Ymeans,
         fitted.values = fitted, residuals = residuals,
         Xvar = Xvar, Xtotvar = Xtotvar)
}

### jackknife.R: Jackknife variance estimation of regression coefficients.

## var.jack: Calculate jackknife variance (or covariance) estimates


#' @title Jackknife Variance Estimates of Regression Coefficients
#'
#' @description Calculates jackknife variance or covariance estimates of regression
#' coefficients.
#'
#' The original (Tukey) jackknife variance estimator is defined as \eqn{(g-1)/g
#' \sum_{i=1}^g(\tilde\beta_{-i} - \bar\beta)^2}, where \eqn{g} is the number
#' of segments, \eqn{\tilde\beta_{-i}} is the estimated coefficient when
#' segment \eqn{i} is left out (called the jackknife replicates), and
#' \eqn{\bar\beta} is the mean of the \eqn{\tilde\beta_{-i}}.  The most common
#' case is delete-one jackknife, with \eqn{g = n}, the number of observations.
#'
#' This is the definition \code{var.jack} uses by default.
#'
#' However, Martens and Martens (2000) defined the estimator as \eqn{(g-1)/g
#' \sum_{i=1}^g(\tilde\beta_{-i} - \hat\beta)^2}, where \eqn{\hat\beta} is the
#' coefficient estimate using the entire data set.  I.e., they use the original
#' fitted coefficients instead of the mean of the jackknife replicates.  Most
#' (all?) other jackknife implementations for PLSR use this estimator.
#' \code{var.jack} can be made to use this definition with \code{use.mean =
#' FALSE}.  In practice, the difference should be small if the number of
#' observations is sufficiently large.  Note, however, that all theoretical
#' results about the jackknife refer to the `proper' definition.  (Also note
#' that this option might disappear in a future version.)
#'
#' @param object an \code{mvr} object.  A cross-validated model fitted with
#' \code{jackknife = TRUE}.
#' @param ncomp the number of components to use for estimating the
#' (co)variances
#' @param covariance logical.  If \code{TRUE}, covariances are calculated;
#' otherwise only variances.  The default is \code{FALSE}.
#' @param use.mean logical.  If \code{TRUE} (default), the mean coefficients
#' are used when estimating the (co)variances; otherwise the coefficients from
#' a model fitted to the entire data set.  See Details.
#' @return If \code{covariance} is \code{FALSE}, an \eqn{p\times q \times c}
#' array of variance estimates, where \eqn{p} is the number of predictors,
#' \eqn{q} is the number of responses, and \eqn{c} is the number of components.
#'
#' If \code{covariance} id \code{TRUE}, an \eqn{pq\times pq \times c} array of
#' variance-covariance estimates.
#' @section Warning: Note that the Tukey jackknife variance estimator is not
#' unbiased for the variance of regression coefficients (Hinkley 1977).  The
#' bias depends on the \eqn{X} matrix.  For ordinary least squares regression
#' (OLSR), the bias can be calculated, and depends on the number of
#' observations \eqn{n} and the number of parameters \eqn{k} in the mode.  For
#' the common case of an orthogonal design matrix with \eqn{\pm 1}{?1} levels,
#' the delete-one jackknife estimate equals \eqn{(n-1)/(n-k)} times the
#' classical variance estimate for the regression coefficients in OLSR.
#' Similar expressions hold for delete-d estimates.  Modifications have been
#' proposed to reduce or eliminate the bias for the OLSR case, however, they
#' depend on the number of parameters used in the model.  See e.g. Hinkley
#' (1977) or Wu (1986).
#'
#' Thus, the results of \code{var.jack} should be used with caution.
#' @author Bjørn-Helge Mevik
#' @seealso \code{\link{mvrCv}}, \code{\link{jack.test}}
#' @references Tukey J.W. (1958) Bias and Confidence in Not-quite Large
#' Samples. (Abstract of Preliminary Report).  \emph{Annals of Mathematical
#' Statistics}, \bold{29}(2), 614.
#'
#' Martens H. and Martens M. (2000) Modified Jack-knife Estimation of Parameter
#' Uncertainty in Bilinear Modelling by Partial Least Squares Regression
#' (PLSR).  \emph{Food Quality and Preference}, \bold{11}, 5--16.
#'
#' Hinkley D.V. (1977), Jackknifing in Unbalanced Situations.
#' \emph{Technometrics}, \bold{19}(3), 285--292.
#'
#' Wu C.F.J. (1986) Jackknife, Bootstrap and Other Resampling Methods in
#' Regression Analysis.  \emph{Te Annals of Statistics}, \bold{14}(4),
#' 1261--1295.
#' @keywords univar
#' @examples
#'
#' data(oliveoil)
#' mod <- pcr(sensory ~ chemical, data = oliveoil, validation = "LOO",
#'            jackknife = TRUE)
#' var.jack(mod, ncomp = 2)
#'
#' @export
var.jack <- function(object, ncomp = object$ncomp, covariance = FALSE,
                     use.mean = TRUE)
{
    if (!inherits(object, "mvr"))
        stop("Not an 'mvr' object")
    if (is.null(object$validation) || is.null(object$validation$coefficients))
        stop("'object' was not fit with jackknifing enabled")

    seglengths <- sapply(object$validation$segments, length)
    if (any(diff(seglengths) != 0))
        warning("Unequal segment lengths.  Estimator currently ignores that")
    nseg <- length(seglengths)
    if (isTRUE(use.mean)) {
        ## The `proper' version of the jackknife
        cent <-
            rowMeans(object$validation$coefficients[,,ncomp,, drop=FALSE],
                     dims = 3)
    } else {
        ## The `sloppy' version, used by e.g. Westad FIXME: ref
        cent <- object$coefficients[,,ncomp, drop=FALSE]
    }
    dnB <- dimnames(object$validation$coefficients[,,ncomp,, drop=FALSE])
    Bdiff <- object$validation$coefficients[,,ncomp,, drop=FALSE] - c(cent)
    if (isTRUE(covariance)) {
        BdiffSq <- apply(Bdiff, 3:4, function(x) tcrossprod(c(x)))
        dims <- dim(Bdiff)
        dims[1:2] <- dims[1] * dims[2]
        dim(BdiffSq) <- dims
        est <- (nseg - 1) * rowMeans(BdiffSq, dims = 3)
        if (length(dnB[[2]]) == 1) {
            nxy <- dnB[[1]]
        } else if (length(dnB[[1]]) == 1) {
            nxy <- dnB[[2]]
        } else {
            nxy <- c(t(outer(dnB[[2]], dnB[[1]], paste, sep = ":")))
        }
        dimnames(est) <- list(nxy, nxy, dnB[[3]])
    } else {
        BdiffSq <- apply(Bdiff, 3:4, function(x) c(x)^2)
        est <- (nseg - 1) * rowMeans(BdiffSq, dims = 2)
        dim(est) <- dim(cent)
        dimnames(est) <- dnB[1:3]
    }
    return(est)
}

## jack.test: Use jackknife variance estimates to test B = 0


#' @name jack.test
#' @title Jackknife approximate t tests of regression coefficients
#'
#' @description Performes approximate t tests of regression coefficients based on jackknife
#' variance estimates.
#'
#' @details \code{jack.test} uses the variance estimates from \code{var.jack} to perform
#' \eqn{t} tests of the regression coefficients.  The resulting object has a
#' print method, \code{print.jacktest}, which uses \code{\link{printCoefmat}}
#' for the actual printing.
#'
#' @aliases jack.test print.jacktest
#' @param object an \code{mvr} object.  A cross-validated model fitted with
#' \code{jackknife = TRUE}.
#' @param ncomp the number of components to use for estimating the variances
#' @param use.mean logical.  If \code{TRUE} (default), the mean coefficients
#' are used when estimating the (co)variances; otherwise the coefficients from
#' a model fitted to the entire data set.  See \code{\link{var.jack}} for
#' details.
#' @param x an \code{jacktest} object, the result of \code{jack.test}.
#' @param P.values logical.  Whether to print \eqn{p} values (default).
#' @param \dots Further arguments sent to the underlying print function
#' \code{\link{printCoefmat}}.
#' @return \code{jack.test} returns an object of class \code{"jacktest"}, with
#' components \item{coefficients }{The estimated regression coefficients}
#' \item{sd}{The square root of the jackknife variance estimates}
#' \item{tvalues}{The \eqn{t} statistics} \item{df}{The `degrees of freedom'
#' used for calculating \eqn{p} values} \item{pvalues}{The calculated \eqn{p}
#' values}
#'
#' \code{print.jacktest} returns the \code{"jacktest"} object (invisibly).
#' @section Warning: The jackknife variance estimates are known to be biased
#' (see \code{\link{var.jack}}).  Also, the distribution of the regression
#' coefficient estimates and the jackknife variance estimates are unknown (at
#' least in PLSR/PCR).  Consequently, the distribution (and in particular, the
#' degrees of freedom) of the resulting \eqn{t} statistics is unknown.  The
#' present code simply assumes a \eqn{t} distribution with \eqn{m - 1} degrees
#' of freedom, where \eqn{m} is the number of cross-validation segments.
#'
#' Therefore, the resulting \eqn{p} values should not be used uncritically, and
#' should perhaps be regarded as mere indicator of (non-)significance.
#'
#' Finally, also keep in mind that as the number of predictor variables
#' increase, the problem of multiple tests increases correspondingly.
#' @author Bjørn-Helge Mevik
#' @seealso \code{\link{var.jack}}, \code{\link{mvrCv}}
#' @references Martens H. and Martens M. (2000) Modified Jack-knife Estimation
#' of Parameter Uncertainty in Bilinear Modelling by Partial Least Squares
#' Regression (PLSR).  \emph{Food Quality and Preference}, \bold{11}, 5--16.
#' @keywords htest
#' @examples
#'
#' data(oliveoil)
#' mod <- pcr(sensory ~ chemical, data = oliveoil, validation = "LOO", jackknife = TRUE)
#' jack.test(mod, ncomp = 2)
#'
#' @export
jack.test <- function(object, ncomp = object$ncomp, use.mean = TRUE) {
    nresp <- dim(object$coefficients)[2]
    sdjack <- sqrt(var.jack(object, ncomp = ncomp, covariance = FALSE,
                           use.mean = use.mean))
    B <- coef(object, ncomp = ncomp)
    ## FIXME: This is an approximation at best:
    df <- length(object$validation$segments) - 1
    tvals <- B / sdjack
    pvals <- 2 * pt(abs(tvals), df = df, lower.tail = FALSE)
    structure(list(coefficients = B, sd = sdjack,
                   tvalues = tvals, df = df, pvalues = pvals),
              class = "jacktest")
}

## print.jacktest: Print method for jacktest objects
#' @rdname jack.test
#' @export
print.jacktest <- function(x, P.values = TRUE, ...) {
    nresp <- dim(x$coefficients)[2]
    respnames <- dimnames(x$coefficients)[[2]]
    nmod <- dim(x$coefficients)[3]
    modnames <- dimnames(x$coefficients)[[3]]
    for (resp in 1:nresp) {
        for (mod in 1:nmod) {
            if (resp > 1 || mod > 1) cat("\n")
            cat("Response ", respnames[resp], " (", modnames[mod], "):\n",
                sep = "")
            coefmat <- cbind(Estimate = x$coefficients[,resp,mod],
                             "Std. Error" = x$sd[,resp,mod],
                             Df = x$df,
                             "t value" = x$tvalues[,resp,mod],
                             "Pr(>|t|)" = x$pvalues[,resp,mod])
            printCoefmat(coefmat, P.values = isTRUE(P.values),
                         cs.ind = 1:2, tst.ind = 4, ...)
        }
    }
    invisible(x)
}

### runit.jackknife.R: test functions for jackknifing
### By Bjørn-Helge Mevik
### Started 2007-08-02

## test.var.jack: Check dimensions of var.jack return value
test.var.jack <- function() {

    ## Single-response model, CV:
    npred <- ncol(gasoline$NIR)
    nresp <- 1
    mod <- mvr(octane ~ NIR, ncomp = 5, data = gasoline, validation = "CV",
                jackknife = TRUE)
    ## Dimensions
    checkEquals(dim(var.jack(mod)), c(npred, nresp, 1))
    checkEquals(dim(var.jack(mod, ncomp = 1:4)), c(npred, nresp, 4))
    ## Dimensions, with covariance
    checkEquals(dim(var.jack(mod, covariance = TRUE)),
                c(npred*nresp, npred*nresp, 1))
    checkEquals(dim(var.jack(mod, ncomp = 1:2, covariance = TRUE)),
                c(npred*nresp, npred*nresp, 2))

    ## Multi-response model, LOO:
    npred <- ncol(oliveoil$chemical)
    nresp <- ncol(oliveoil$sensory)
    mod <- mvr(sensory ~ chemical, data = oliveoil, validation = "LOO",
                jackknife = TRUE, ncomp = 4)
    ## Dimensions
    checkEquals(dim(var.jack(mod)), c(npred, nresp, 1))
    checkEquals(dim(var.jack(mod, ncomp = 1:4)), c(npred, nresp, 4))
    ## Dimensions, with covariance
    checkEquals(dim(var.jack(mod, covariance = TRUE)),
                c(npred*nresp, npred*nresp, 1))
    checkEquals(dim(var.jack(mod, ncomp = 1:2, covariance = TRUE)),
                c(npred*nresp, npred*nresp, 2))
}

## test.jack.test: Test output from jack.test
test.jack.test <- function() {

    ## Single-response model, CV:
    npred <- ncol(gasoline$NIR)
    nresp <- 1
    mod <- mvr(octane ~ NIR, ncomp = 6, data = gasoline, validation = "CV",
                jackknife = TRUE)
    ## Things to change:
    ## ncomp (== object$ncomp, length > 1)
    ## use.mean (FALSE/TRUE)
    for (nc in list(6, 4:5)) {
        for (use.mean in c(FALSE, TRUE)) {
            res <- if (length(nc) == 1)
                jack.test(mod, use.mean = use.mean)
            else
                jack.test(mod, ncomp = nc, use.mean = use.mean)
            ## Check for all finite values:
            checkTrue(all(sapply(res, function(x) all(is.finite(x)))),
                      paste("uniresp", length(nc), use.mean, "finite"))
        }
    }

    ## Multi-response model, LOO:
    npred <- ncol(oliveoil$chemical)
    nresp <- ncol(oliveoil$sensory)
    mod <- mvr(sensory ~ chemical, data = oliveoil, validation = "LOO",
                jackknife = TRUE, ncomp = 5)
    ## Things to change:
    ## ncomp (== object$ncomp, length > 1)
    ## use.mean (FALSE/TRUE)
    for (nc in list(6, 3:4)) {
        for (use.mean in c(FALSE, TRUE)) {
            res <- if (length(nc) == 1)
                jack.test(mod, use.mean = use.mean)
            else
                jack.test(mod, ncomp = nc, use.mean = use.mean)
            ## Check for all finite values:
            checkTrue(all(sapply(res, function(x) all(is.finite(x)))),
                      paste("uniresp", length(nc), use.mean, "finite"))
        }
    }
}

### oscorespls.fit.R: The multiresponse orthogonal scores algorithm
###
### Implements an adapted version of the `orthogonal scores' algorithm as
###   described in Martens and Naes, pp. 121--122 and 157--158.



#' @title Orthogonal scores PLSR
#'
#' @description Fits a PLSR model with the orthogonal scores algorithm (aka the NIPALS
#' algorithm).
#'
#' @details This function should not be called directly, but through the generic
#' functions \code{plsr} or \code{mvr} with the argument
#' \code{method="oscorespls"}.  It implements the orthogonal scores algorithm,
#' as described in \cite{Martens and Næs (1989)}.  This is one of the two
#' \dQuote{classical} PLSR algorithms, the other being the orthogonal loadings
#' algorithm.
#'
#' @param X a matrix of observations.  \code{NA}s and \code{Inf}s are not
#' allowed.
#' @param Y a vector or matrix of responses.  \code{NA}s and \code{Inf}s are
#' not allowed.
#' @param ncomp the number of components to be used in the modelling.
#' @param center logical, determines if the \eqn{X} and \eqn{Y} matrices are
#' mean centered or not. Default is to perform mean centering.
#' @param stripped logical.  If \code{TRUE} the calculations are stripped as
#' much as possible for speed; this is meant for use with cross-validation or
#' simulations when only the coefficients are needed.  Defaults to
#' \code{FALSE}.
#' @param tol numeric.  The tolerance used for determining convergence in
#' multi-response models.
#' @param maxit positive integer.  The maximal number of iterations used in the
#' internal Eigenvector calculation.
#' @param \dots other arguments.  Currently ignored.
#' @return A list containing the following components is returned:
#' \item{coefficients}{an array of regression coefficients for 1, \ldots{},
#' \code{ncomp} components.  The dimensions of \code{coefficients} are
#' \code{c(nvar, npred, ncomp)} with \code{nvar} the number of \code{X}
#' variables and \code{npred} the number of variables to be predicted in
#' \code{Y}.} \item{scores}{a matrix of scores.} \item{loadings}{a matrix of
#' loadings.} \item{loading.weights}{a matrix of loading weights.}
#' \item{Yscores}{a matrix of Y-scores.} \item{Yloadings}{a matrix of
#' Y-loadings.} \item{projection}{the projection matrix used to convert X to
#' scores.} \item{Xmeans}{a vector of means of the X variables.}
#' \item{Ymeans}{a vector of means of the Y variables.} \item{fitted.values}{an
#' array of fitted values.  The dimensions of \code{fitted.values} are
#' \code{c(nobj, npred, ncomp)} with \code{nobj} the number samples and
#' \code{npred} the number of Y variables.} \item{residuals}{an array of
#' regression residuals.  It has the same dimensions as \code{fitted.values}.}
#' \item{Xvar}{a vector with the amount of X-variance explained by each
#' component.} \item{Xtotvar}{Total variance in \code{X}.}
#'
#' If \code{stripped} is \code{TRUE}, only the components \code{coefficients},
#' \code{Xmeans} and \code{Ymeans} are returned.
#' @author Ron Wehrens and Bjørn-Helge Mevik
#' @seealso \code{\link{mvr}} \code{\link{plsr}} \code{\link{pcr}}
#' \code{\link{kernelpls.fit}} \code{\link{widekernelpls.fit}}
#' \code{\link{simpls.fit}}
#' @references Martens, H., Næs, T. (1989) \emph{Multivariate calibration.}
#' Chichester: Wiley.
#' @keywords regression multivariate
#' @export
oscorespls.fit <- function(X, Y, ncomp, center = TRUE, stripped = FALSE,
                           tol = .Machine$double.eps^0.5, maxit = 100, ...)
{
    ## Initialise
    Y <- as.matrix(Y)
    if (!stripped) {
        ## Save dimnames
        dnX <- dimnames(X)
        dnY <- dimnames(Y)
    }
    ## Remove dimnames for performance (doesn't seem to matter; in fact,
    ## as far as it has any effect, it hurts a tiny bit in most situations.
    dimnames(X) <- dimnames(Y) <- NULL

    nobj  <- dim(X)[1]
    npred <- dim(X)[2]
    nresp <- dim(Y)[2]

    W <- P <- matrix(0, nrow = npred, ncol = ncomp)
    tQ <- matrix(0, nrow = ncomp, ncol = nresp) # Y loadings; transposed
    B <- array(0, dim = c(npred, nresp, ncomp))
    if (!stripped) {
        TT <- U <- matrix(0, nrow = nobj, ncol = ncomp)
        tsqs <- numeric(ncomp)          # t't
        fitted <- residuals <- array(0, dim = c(nobj, nresp, ncomp))
    }

    ## C1
    if (center) {
        Xmeans <- colMeans(X)
        X <- X - rep(Xmeans, each = nobj)
        Ymeans <- colMeans(Y)
        Y <- Y - rep(Ymeans, each = nobj)
    } else {
        ## Set means to zero. Will ensure that predictions do not take the
        ## mean into account.
        Xmeans <- rep_len(0, npred)
        Ymeans <- rep_len(0, nresp)
    }

    ## Must be done here due to the deflation of X
    if (!stripped) Xtotvar <- sum(X * X)

    for (a in 1:ncomp) {
        ## Initial values:
        if (nresp == 1) {               # pls1
            u.a <- Y                    # FIXME: scale?
        } else {                        # pls2
            ## The coloumn of Y with largest sum of squares:
            u.a <- Y[,which.max(colSums(Y * Y))]
            t.a.old <- 0
        }
        nit <- 0
        repeat {
            nit <- nit + 1              # count the iterations
            ## C2.1
            w.a <- crossprod(X, u.a)
            w.a <- w.a / sqrt(c(crossprod(w.a)))

            ## C2.2
            t.a <- X %*% w.a

            ## C2.3
            tsq <- c(crossprod(t.a))
            t.tt <- t.a / tsq

            ## C2.4
            q.a <- crossprod(Y, t.tt)

            if (nresp == 1)
                break                   # pls1: no iteration

            ## C2.4b-c
            ## Convergence check for pls2:
            if (sum(abs((t.a - t.a.old) / t.a), na.rm = TRUE) < tol)
                break
            else {
                u.a <- Y %*% q.a / c(crossprod(q.a))
                t.a.old <- t.a          # Save for comparison
            }
            if (nit >= maxit) {
              warning("No convergence in ", maxit, " iterations\n")
              break
            }
        }

        ## C2.3 contd.
        p.a <- crossprod(X, t.tt)

        ## C2.5
        X <- X - t.a %*% t(p.a)
        Y <- Y - t.a %*% t(q.a)

        ## Save scores etc:
        W[,a]  <- w.a
        P[,a]  <- p.a
        tQ[a,] <- q.a
        if (!stripped) {
            TT[,a] <- t.a
            U[,a]  <- u.a
            tsqs[a] <- tsq
            ## (For very tall, slim X and Y, X0 %*% B[,,a] is slightly faster,
            ## due to less overhead.)
            fitted[,,a] <- TT[,1:a] %*% tQ[1:a,, drop=FALSE]
            residuals[,,a] <- Y
        }
    }

    ## Calculate rotation matrix:
    if (ncomp == 1) {
        ## For 1 component, R == W:
        R <- W
    } else {
        PW <- crossprod(P, W)
        ## It is known that P^tW is right bi-diagonal (one response) or upper
        ## triangular (multiple responses), with all diagonal elements equal to 1.
        if (nresp == 1) {
            ## For single-response models, direct calculation of (P^tW)^-1 is
            ## simple, and faster than using backsolve.
            PWinv <- diag(ncomp)
            bidiag <- - PW[row(PW) == col(PW)-1]
            for (a in 1:(ncomp - 1))
                PWinv[a,(a+1):ncomp] <- cumprod(bidiag[a:(ncomp-1)])
        } else {
            PWinv <- backsolve(PW, diag(ncomp))
        }
    R <- W %*% PWinv
    }

    ## Calculate regression coefficients:
    for (a in 1:ncomp) {
        B[,,a] <- R[,1:a, drop=FALSE] %*% tQ[1:a,, drop=FALSE]
    }

    if (stripped) {
        ## Return as quickly as possible
        list(coefficients = B, Xmeans = Xmeans, Ymeans = Ymeans)
    } else {
        fitted <- fitted + rep(Ymeans, each = nobj) # Add mean

        ## Add dimnames and classes:
        objnames <- dnX[[1]]
        if (is.null(objnames)) objnames <- dnY[[1]]
        prednames <- dnX[[2]]
        respnames <- dnY[[2]]
        compnames <- paste("Comp", 1:ncomp)
        nCompnames <- paste(1:ncomp, "comps")
        dimnames(TT) <- dimnames(U) <- list(objnames, compnames)
        dimnames(R) <- dimnames(W) <- dimnames(P) <-
            list(prednames, compnames)
        dimnames(tQ) <- list(compnames, respnames)
        dimnames(B) <- list(prednames, respnames, nCompnames)
        dimnames(fitted) <- dimnames(residuals) <-
            list(objnames, respnames, nCompnames)
        class(TT) <- class(U) <- "scores"
        class(P) <- class(W) <- class(tQ) <- "loadings"

        list(coefficients = B,
             scores = TT, loadings = P,
             loading.weights = W,
             Yscores = U, Yloadings = t(tQ),
             projection = R,
             Xmeans = Xmeans, Ymeans = Ymeans,
             fitted.values = fitted, residuals = residuals,
             Xvar = colSums(P * P) * tsqs,
             Xtotvar = Xtotvar)
    }
}

#' @title NIPALS PLS with missing values
#'
#' @description A NIPALS implementation that tolerates \code{NA}s in both
#' \code{X} and \code{Y} by ignoring them when updating scores and loadings.
#' This is useful when the design matrix is incomplete but the number of
#' components is relatively low.
#'
#' @param X numeric matrix (or coercible) of predictors. Missing values are
#' allowed and handled internally.
#' @param Y numeric matrix (or coercible) of responses. Missing values are also
#' handled internally.
#' @param ncomp number of PLS components to extract.
#' @param center logical whether to center \code{X} and \code{Y} before
#' fitting. Means ignore missing entries.
#' @param stripped logical. If \code{TRUE} only the coefficients and the mean
#' vectors are returned.
#' @param maxiter maximum number of inner iterations to force convergence on
#' each component.
#' @param tol tolerance used to stop the inner loop when the direction vector
#' changes very little.
#' @param ... currently ignored.
#'
#' @return A list with the same components as \code{nipals.fit}, but the
#' computations never fail in the presence of missing entries.
#' @export
nipals.fit <- function(X, Y, ncomp, center = TRUE, stripped = FALSE,
                       maxiter = 500, tol = 1e-06, ...)
{
  X <- as.matrix(X)
  Y <- as.matrix(Y)
  if (!stripped) {
    dnX <- dimnames(X)
    dnY <- dimnames(Y)
  }
  dimnames(X) <- dimnames(Y) <- NULL

  nobj <- nrow(X)
  npred <- ncol(X)
  nresp <- ncol(Y)

  mean_ignore_na <- function(mat) {
    colMeans(mat, na.rm = TRUE)
  }

  if (center) {
    Xmeans <- mean_ignore_na(X)
    Ymeans <- mean_ignore_na(Y)
    X <- sweep(X, 2, Xmeans, "-")
    Y <- sweep(Y, 2, Ymeans, "-")
  } else {
    Xmeans <- rep_len(0, npred)
    Ymeans <- rep_len(0, nresp)
  }

  Xcentre <- X
  Ycentre <- Y
  Xresid <- X
  Yresid <- Y
  maskX <- !is.na(Xcentre)
  maskY <- !is.na(Ycentre)

  safe_crossprod <- function(mat, vec) {
    apply(mat, 2, function(col) sum(col * vec, na.rm = TRUE))
  }

  safe_matvec <- function(mat, vec) {
    res <- numeric(nrow(mat))
    for (j in seq_len(ncol(mat))) {
      col <- mat[, j]
      idx <- !is.na(col)
      if (any(idx)) {
        res[idx] <- res[idx] + col[idx] * vec[j]
      }
    }
    res
  }

  safe_times <- function(mat, coeff) {
    res <- matrix(0, nrow = nrow(mat), ncol = ncol(coeff))
    for (j in seq_len(ncol(coeff))) {
      res[, j] <- safe_matvec(mat, coeff[, j])
    }
    res
  }

  normalize <- function(v) {
    n <- sum(v * v)
    if (n <= .Machine$double.eps) return(v)
    v / sqrt(n)
  }

  R <- P <- matrix(0, nrow = npred, ncol = ncomp)
  Q <- matrix(0, nrow = ncomp, ncol = nresp)
  B <- array(0, c(npred, nresp, ncomp))
  tsqs <- numeric(ncomp)

  W <- matrix(0, npred, ncomp)
  Tmat <- matrix(0, nrow = nobj, ncol = ncomp)
  U <- matrix(0, nrow = nobj, ncol = ncomp)
  if (!stripped) {
    fitted <- array(0, c(nobj, nresp, ncomp))
  }

  for (a in seq_len(ncomp)) {
    start <- Yresid[, 1]
    start[is.na(start)] <- 0
    if (sum(start * start) == 0) {
      start <- rep_len(1, nobj)
    }
    v <- normalize(start)

    w <- normalize(safe_crossprod(Xresid, v))
    if (sum(w * w) == 0) {
      warning("component ", a, " has zero loading weights", call. = FALSE)
      break
    }

    for (iter in seq_len(maxiter)) {
      t1 <- safe_matvec(Xresid, w)
      if (sum(t1 * t1) == 0) break
      z <- normalize(safe_crossprod(Yresid, t1))
      if (sum(z * z) == 0) break
      v <- normalize(safe_matvec(Yresid, z))
      if (sum(v * v) == 0) break
      w1 <- normalize(safe_crossprod(Xresid, v))
      if (sum(w1 * w1) == 0) break
      if (sum((w1 - w)^2) < tol^2) {
        w <- w1
        break
      }
      w <- w1
    }

    t1 <- safe_matvec(Xresid, w)
    tt <- sum(t1 * t1)
    if (tt == 0) {
      warning("component ", a, " collapsed to zero scores", call. = FALSE)
      break
    }

    q <- safe_crossprod(Yresid, t1) / tt
    p <- safe_crossprod(Xresid, t1) / tt
    W[, a] <- w
    Tmat[, a] <- t1
    U[, a] <- safe_matvec(Yresid, q)
    Q[a, ] <- q
    P[, a] <- p
    tsqs[a] <- tt

    reconX <- tcrossprod(t1, p)
    reconY <- tcrossprod(t1, q)
    Xresid[maskX] <- Xresid[maskX] - reconX[maskX]
    Yresid[maskY] <- Yresid[maskY] - reconY[maskY]
  }

  R <- W %*% solve(crossprod(P, W))
  for (a in seq_len(ncomp)) {
    Qsub <- Q[seq_len(a), , drop = FALSE]
    Rsub <- R[, seq_len(a), drop = FALSE]
    B[,,a] <- Rsub %*% Qsub
  }

  if (!stripped) {
    for (a in seq_len(ncomp)) {
      fitted[,,a] <- safe_times(Xcentre, matrix(B[,,a]))
    }
    residuals <- array(NA, c(nobj, nresp, ncomp))
    for (a in seq_len(ncomp)) {
      residuals[,,a] <- Ycentre - fitted[,,a]
    }
    if (center) {
      fitted <- fitted + rep(Ymeans, each = nobj)
    }
  }

  Xvar <- colSums(P * P) * tsqs
  Xtotvar <- sum(Xcentre[!is.na(Xcentre)]^2)

  if (stripped) {
    return(list(coefficients = B, Xmeans = Xmeans, Ymeans = Ymeans))
  }

  objnames <- dnX[[1]]
  if (is.null(objnames)) objnames <- dnY[[1]]
  prednames <- dnX[[2]]
  respnames <- dnY[[2]]
  compnames <- paste("Comp", seq_len(ncomp))
  nCompnames <- paste(seq_len(ncomp), "comps")

  dimnames(Tmat) <- dimnames(U) <- list(objnames, compnames)
  dimnames(R) <- dimnames(W) <- dimnames(P) <- list(prednames, compnames)
  dimnames(Q) <- list(compnames, respnames)
  dimnames(B) <- list(prednames, respnames, nCompnames)
  dimnames(fitted) <- dimnames(residuals) <-
    list(objnames, respnames, nCompnames)
  class(Tmat) <- class(U) <- "scores"
  class(P) <- class(W) <- class(Q) <- "loadings"

  list(coefficients = B,
       scores = Tmat, loadings = P,
       loading.weights = W,
       Yscores = U, Yloadings = t(Q),
       projection = R,
       Xmeans = Xmeans, Ymeans = Ymeans,
       fitted.values = fitted, residuals = residuals,
       Xvar = Xvar, Xtotvar = Xtotvar)
}

#'
#'
#' #' @title Kernel PLS (Dayal and MacGregor)
#' #'
#' #' @description Fits a PLSR model with the kernel algorithm.
#' #'
#' #' @details This function should not be called directly, but through the generic
#' #' functions \code{plsr} or \code{mvr} with the argument
#' #' \code{method="kernelpls"} (default).  Kernel PLS is particularly efficient
#' #' when the number of objects is (much) larger than the number of variables.
#' #' The results are equal to the NIPALS algorithm.  Several different forms of
#' #' kernel PLS have been described in literature, e.g.  by De Jong and Ter
#' #' Braak, and two algorithms by Dayal and MacGregor.  This function implements
#' #' the fastest of the latter, not calculating the crossproduct matrix of X.  In
#' #' the Dyal & MacGregor paper, this is \dQuote{algorithm 1}.
#' #'
#' #' @param X a matrix of observations.  \code{NA}s and \code{Inf}s are not
#' #' allowed.
#' #' @param Y a vector or matrix of responses.  \code{NA}s and \code{Inf}s are
#' #' not allowed.
#' #' @param ncomp the number of components to be used in the modelling.
#' #' @param center logical, determines if the \eqn{X} and \eqn{Y} matrices are
#' #' mean centered or not. Default is to perform mean centering.
#' #' @param stripped logical.  If \code{TRUE} the calculations are stripped as
#' #' much as possible for speed; this is meant for use with cross-validation or
#' #' simulations when only the coefficients are needed.  Defaults to
#' #' \code{FALSE}.
#' #' @param \dots other arguments.  Currently ignored.
#' #' @return A list containing the following components is returned:
#' #' \item{coefficients}{an array of regression coefficients for 1, \ldots{},
#' #' \code{ncomp} components.  The dimensions of \code{coefficients} are
#' #' \code{c(nvar, npred, ncomp)} with \code{nvar} the number of \code{X}
#' #' variables and \code{npred} the number of variables to be predicted in
#' #' \code{Y}.} \item{scores}{a matrix of scores.} \item{loadings}{a matrix of
#' #' loadings.} \item{loading.weights}{a matrix of loading weights.}
#' #' \item{Yscores}{a matrix of Y-scores.} \item{Yloadings}{a matrix of
#' #' Y-loadings.} \item{projection}{the projection matrix used to convert X to
#' #' scores.} \item{Xmeans}{a vector of means of the X variables.}
#' #' \item{Ymeans}{a vector of means of the Y variables.} \item{fitted.values}{an
#' #' array of fitted values.  The dimensions of \code{fitted.values} are
#' #' \code{c(nobj, npred, ncomp)} with \code{nobj} the number samples and
#' #' \code{npred} the number of Y variables.} \item{residuals}{an array of
#' #' regression residuals.  It has the same dimensions as \code{fitted.values}.}
#' #' \item{Xvar}{a vector with the amount of X-variance explained by each
#' #' component.} \item{Xtotvar}{Total variance in \code{X}.}
#' #'
#' #' If \code{stripped} is \code{TRUE}, only the components \code{coefficients},
#' #' \code{Xmeans} and \code{Ymeans} are returned.
#' #' @author Ron Wehrens and Bjørn-Helge Mevik
#' #' @seealso \code{\link{mvr}} \code{\link{plsr}} \code{\link{cppls}}
#' #' \code{\link{pcr}} \code{\link{widekernelpls.fit}} \code{\link{simpls.fit}}
#' #' \code{\link{oscorespls.fit}}
#' #' @references de Jong, S. and ter Braak, C. J. F. (1994) Comments on the PLS
#' #' kernel algorithm.  \emph{Journal of Chemometrics}, \bold{8}, 169--174.
#' #'
#' #' Dayal, B. S. and MacGregor, J. F. (1997) Improved PLS algorithms.
#' #' \emph{Journal of Chemometrics}, \bold{11}, 73--85.
#' #' @keywords regression multivariate
#' #' @export
#' nipals.fit <- function(X, Y, ncomp, center = TRUE,
#'                           stripped = FALSE, ...)
#' {
#'     Y <- as.matrix(Y)
#'     if (!stripped) {
#'         ## Save dimnames:
#'         dnX <- dimnames(X)
#'         dnY <- dimnames(Y)
#'     }
#'     ## Remove dimnames during calculation.  (Doesn't seem to make a
#'     ## difference here (2.3.0).)
#'     dimnames(X) <- dimnames(Y) <- NULL
#'
#'     nobj  <- dim(X)[1]
#'     npred <- dim(X)[2]
#'     nresp <- dim(Y)[2]
#'
#'     ## Center variables:
#'     if (center) {
#'         Xmeans <- colMeans(X)
#'         X <- X - rep(Xmeans, each = nobj)
#'         Ymeans <- colMeans(Y)
#'         Y <- Y - rep(Ymeans, each = nobj)
#'     } else {
#'         ## Set means to zero. Will ensure that predictions do not take the
#'         ## mean into account.
#'         Xmeans <- rep_len(0, npred)
#'         Ymeans <- rep_len(0, nresp)
#'     }
#'     X.orig <- X
#'
#'     ## Projection, loadings
#'     R <- P <- matrix(0, ncol = ncomp, nrow = npred)
#'     Q <- matrix(0, ncol = nresp, nrow = ncomp)# Y loadings; transposed
#'     B <- array(0, c(npred, nresp, ncomp))
#'
#'     if (!stripped) {
#'         W <- P                        # Loading weights
#'         U <- T <- matrix(0, ncol = ncomp, nrow = nobj)# scores
#'         fitted <- array(0, c(nobj, nresp, ncomp))
#'     }
#'
#'     for(a in 1:ncomp){
#'       v <- Y[,1]
#'       w <- crossprod(X,v);  w  <- w/sqrt(sum(w*w))
#'       u <- X %*% w;         u  <- u/sqrt(sum(u*u))
#'       z <- crossprod(Y,u);  z  <- z/sqrt(sum(z*z))
#'       v <- Y %*% z;         v  <- v/sqrt(sum(v*v))
#'       w1 <- crossprod(X,v); w1 <- w1/sqrt(sum(w1*w1))
#'       while(abs(crossprod(w1,w)-crossprod(w1)) > 0.0001){
#'         w <- w1
#'         u <- X %*% w;         u  <- u/sqrt(sum(u*u))
#'         z <- crossprod(Y,u);  z  <- z/sqrt(sum(z*z))
#'         v <- Y %*% z;         v  <- v/sqrt(sum(v*v))
#'         w1 <- crossprod(X,v); w1 <- w1/sqrt(sum(w1*w1))
#'       }
#'       W[,a] <- w1
#'       t1 <- X %*% w1
#'       T[,a] <- t1
#'       tt <- sum(t1*t1)
#'       Q[,a] <- crossprod(Y, t1)/tt
#'       P[,a] <- crossprod(X, t1)/tt
#'       X <- X - tcrossprod(t1, P[,a])
#'       Y <- Y - tcrossprod(t1, Q[,a])
#'
#'     }
#'     R <- W %*% solve(crossprod(P,W))
#'     for(a in 1:ncomp){
#'       B[,,a] <- tcrossprod(R[,1:a], Q[,1:a])
#'       fitted[,,a] <- X.orig %*% B[,,a]
#'     }
#'
#'     if (stripped) {
#'         ## Return as quickly as possible
#'         list(coefficients = B, Xmeans = Xmeans, Ymeans = Ymeans)
#'     } else {
#'         residuals <- - fitted + c(Y)
#'         if (center) {
#'             fitted <- fitted + rep(Ymeans, each = nobj) # Add mean
#'         }
#'
#'         ## Add dimnames:
#'         objnames <- dnX[[1]]
#'         if (is.null(objnames)) objnames <- dnY[[1]]
#'         prednames <- dnX[[2]]
#'         respnames <- dnY[[2]]
#'         compnames <- paste("Comp", 1:ncomp)
#'         nCompnames <- paste(1:ncomp, "comps")
#'         dimnames(TT) <- dimnames(U) <- list(objnames, compnames)
#'         dimnames(R) <- dimnames(W) <- dimnames(P) <-
#'             list(prednames, compnames)
#'         dimnames(tQ) <- list(compnames, respnames)
#'         dimnames(B) <- list(prednames, respnames, nCompnames)
#'         dimnames(fitted) <- dimnames(residuals) <-
#'             list(objnames, respnames, nCompnames)
#'         class(TT) <- class(U) <- "scores"
#'         class(P) <- class(W) <- class(tQ) <- "loadings"
#'
#'         list(coefficients = B,
#'              scores = TT, loadings = P,
#'              loading.weights = W,
#'              Yscores = U, Yloadings = Q,
#'              projection = R,
#'              Xmeans = Xmeans, Ymeans = Ymeans,
#'              fitted.values = fitted, residuals = residuals,
#'              Xvar = colSums(P * P) * tsqs,
#'              Xtotvar = sum(X * X))
#'     }
#' }

### Plots for mvr objects.  Some of them also work for other
### objects, but that is not a priority.

###
### Plot method for mvr objects
###



#' @title Plot Method for MVR objects
#'
#' @description \code{plot.mvr} plots predictions, coefficients, scores, loadings, biplots,
#' correlation loadings or validation plots (RMSEP curves, etc.).
#'
#' @details The function is simply a wrapper for the underlying plot functions used to
#' make the selected plots.  See \code{\link{predplot.mvr}},
#' \code{\link{validationplot}}, \code{\link{coefplot}},
#' \code{\link{scoreplot}}, \code{\link{loadingplot}}, \code{\link{biplot.mvr}}
#' or \code{\link{corrplot}} for details.  Note that all arguments except
#' \code{x} and \code{plottype} must be named.
#'
#' @param x an object of class \code{mvr}.  The fitted model to plot.
#' @param plottype character.  What kind of plot to plot.
#' @param \dots further arguments, sent to the underlying plot functions.
#' @return \code{plot.mvr} returns whatever the underlying plot function
#' returns.
#' @author Ron Wehrens and Bjørn-Helge Mevik
#' @seealso \code{\link{mvr}}, \code{\link{predplot.mvr}},
#' \code{\link{validationplot}}, \code{\link{coefplot}},
#' \code{\link{scoreplot}}, \code{\link{loadingplot}},
#' \code{\link{biplot.mvr}}, \code{\link{corrplot}}
#' @keywords regression multivariate hplot
#' @examples
#'
#' data(yarn)
#' nir.pcr <- pcr(density ~ NIR, ncomp = 9, data = yarn, validation = "CV")
#' \dontrun{
#' plot(nir.pcr, ncomp = 5) # Plot of cross-validated predictions
#' plot(nir.pcr, "scores") # Score plot
#' plot(nir.pcr, "loadings", comps = 1:3) # The three first loadings
#' plot(nir.pcr, "coef", ncomp = 5) # Coefficients
#' plot(nir.pcr, "val") # RMSEP curves
#' plot(nir.pcr, "val", val.type = "MSEP", estimate = "CV") # CV MSEP
#' }
#'
#' @export
plot.mvr <- function(x, plottype = c("prediction", "validation",
                                     "coefficients", "scores", "loadings", "biplot",
                                     "correlation"),
                     ...)
{
  plottype <- match.arg(plottype)
  plotFunc <- switch(plottype,
                     prediction = predplot.mvr,
                     validation = validationplot,
                     coefficients = coefplot,
                     scores = scoreplot,
                     loadings = loadingplot,
                     biplot = biplot.mvr,
                     correlation = corrplot)
  plotFunc(x, ...)
}


###
### Scoreplot
###



#' @name scoreplot
#' @title Plots of Scores, Loadings and Correlation Loadings
#'
#' @description  Functions to make scatter plots of scores or correlation loadings, and
#' scatter or line plots of loadings.
#'
#' @details \code{plot.scores} is simply a wrapper calling \code{scoreplot}, passing all
#' arguments.  Similarly for \code{plot.loadings}.
#'
#' \code{scoreplot} is generic, currently with a default method that works for
#' matrices and any object for which \code{\link{scores}} returns a matrix.
#' The default \code{scoreplot} method makes one or more scatter plots of the
#' scores, depending on how many components are selected.  If one or two
#' components are selected, and \code{identify} is \code{TRUE}, the function
#' \code{\link{identify}} is used to interactively identify points.
#'
#' Also \code{loadingplot} is generic, with a default method that works for
#' matrices and any object where \code{\link{loadings}} returns a matrix.  If
#' \code{scatter} is \code{TRUE}, the default method works exactly like the
#' default \code{scoreplot} method.  Otherwise, it makes a lineplot of the
#' selected loading vectors, and if \code{identify} is \code{TRUE}, uses
#' \code{\link{identify}} to interactively identify points.  Also, if
#' \code{legendpos} is given, a legend is drawn at the position indicated.
#'
#' \code{corrplot} works exactly like the default \code{scoreplot} method,
#' except that at least two components must be selected.  The
#' \dQuote{correlation loadings}, i.e. the correlations between each variable
#' and the selected components (see References), are plotted as pairwise
#' scatter plots, with concentric circles of radii given by \code{radii}.  Each
#' point corresponds to a variable.  The squared distance between the point and
#' origin equals the fraction of the variance of the variable explained by the
#' components in the panel.  The default \code{radii} corresponds to 50\% and
#' 100\% explained variance.  By default, only the correlation loadings of the
#' \eqn{X} variables are plotted, but if \code{ploty} is \code{TRUE}, also the
#' \eqn{Y} correlation loadings are plotted.
#'
#' \code{scoreplot}, \code{loadingplot} and \code{corrplot} can also be called
#' through the plot method for \code{mvr} objects, by specifying
#' \code{plottype} as \code{"scores"}, \code{"loadings"} or
#' \code{"correlation"}, respectively.  See \code{\link{plot.mvr}}.
#'
#' The argument \code{labels} can be a vector of labels or one of
#' \code{"names"} and \code{"numbers"}.
#'
#' If a scatter plot is produced (i.e., \code{scoreplot}, \code{corrplot}, or
#' \code{loadingplot} with \code{scatter = TRUE}), the labels are used instead
#' of plot symbols for the points plotted.  If \code{labels} is \code{"names"}
#' or \code{"numbers"}, the row names or row numbers of the matrix (scores,
#' loadings or correlation loadings) are used.
#'
#' If a line plot is produced (i.e., \code{loadingplot}), the labels are used
#' as \eqn{x} axis labels.  If \code{labels} is \code{"names"} or
#' \code{"numbers"}, the variable names are used as labels, the difference
#' being that with \code{"numbers"}, the variable names are converted to
#' numbers, if possible.  Variable names of the forms \samp{"number"} or
#' \samp{"number text"} (where the space is optional), are handled.
#'
#' The argument \code{pretty.xlabels} is only used when \code{labels} is
#' specified for a line plot.  If \code{TRUE} (default), the code tries to use
#' a \sQuote{pretty} selection of labels.  If \code{labels} is
#' \code{"numbers"}, it also uses the numerical values of the labels for
#' horisontal spacing.  If one has excluded parts of the spectral region, one
#' might therefore want to use \code{pretty.xlabels = FALSE}.
#'
#' @aliases scoreplot scoreplot.default plot.scores loadingplot
#' loadingplot.default plot.loadings corrplot
#' @param object an object.  The fitted model.
#' @param estimate optional character vector passed to \code{scores()} so
#'   \code{scoreplot()} can request the training, CV or test estimate when
#'   plotting.
#' @param newdata optional data frame supplied to \code{scores()} when
#'   \code{estimate = "test"}.
#' @param comps integer vector.  The components to plot.
#' @param scatter logical.  Whether the loadings should be plotted as a scatter
#' instead of as lines.
#' @param labels optional.  Alternative plot labels or \eqn{x} axis labels.
#' See Details.
#' @param plotx locical.  Whether to plot the \eqn{X} correlation loadings.
#' Defaults to \code{TRUE}.
#' @param ploty locical.  Whether to plot the \eqn{Y} correlation loadings.
#' Defaults to \code{FALSE}.
#' @param radii numeric vector, giving the radii of the circles drawn in
#' \code{corrplot}.  The default radii represent 50\% and 100\% explained
#' variance of the \eqn{X} variables by the chosen components.
#' @param identify logical.  Whether to use \code{identify} to interactively
#' identify points.  See below.
#' @param type character.  What type of plot to make.  Defaults to \code{"p"}
#' (points) for scatter plots and \code{"l"} (lines) for line plots.  See
#' \code{\link{plot}} for a complete list of types (not all types are
#' possible/meaningful for all plots).
#' @param lty vector of line types (recycled as neccessary).  Line types can be
#' specified as integers or character strings (see \code{\link{par}} for the
#' details).
#' @param lwd vector of positive numbers (recycled as neccessary), giving the
#' width of the lines.
#' @param pch plot character.  A character string or a vector of single
#' characters or integers (recycled as neccessary).  See \code{\link{points}}
#' for all alternatives.
#' @param cex numeric vector of character expansion sizes (recycled as
#' neccessary) for the plotted symbols.
#' @param col character or integer vector of colors for plotted lines and
#' symbols (recycled as neccessary).  See \code{\link{par}} for the details.
#' @param legendpos Legend position.  Optional.  Ignored if \code{scatter} is
#' \code{TRUE}.  If present, a legend is drawn at the given position.  The
#' position can be specified symbolically (e.g., \code{legendpos =
#' "topright"}).  This requires >= 2.1.0.  Alternatively, the position can be
#' specified explicitly (\code{legendpos = t(c(x,y))}) or interactively
#' (\code{legendpos = \link{locator}()}).
#' @param xlab,ylab titles for \eqn{x} and \eqn{y} axes.  Typically character
#' strings, but can be expressions or lists.  See \code{\link{title}} for
#' details.
#' @param pretty.xlabels logical.  If \code{TRUE}, \code{loadingplot} tries to
#' plot the \eqn{x} labels more nicely.  See Details.
#' @param xlim optional vector of length two, with the \eqn{x} limits of the
#' plot.
#' @param x a \code{scores} or \code{loadings} object.  The scores or loadings
#' to plot.
#' @param \dots further arguments sent to the underlying plot function(s).
#' @return The functions return whatever the underlying plot function (or
#' \code{identify}) returns.
#' @note \code{\link{legend}} has many options.  If you want greater control
#' over the appearance of the legend, omit the \code{legendpos} argument and
#' call \code{legend} manually.
#'
#' Graphical parametres (such as \code{pch} and \code{cex}) can also be used
#' with \code{scoreplot} and \code{corrplot}.  They are not listed in the
#' argument list simply because they are not handled specifically in the
#' function (unlike in \code{loadingplot}), but passed directly to the
#' underlying plot functions by \code{\dots{}}.
#'
#' Tip: If the labels specified with \code{labels} are too long, they get
#' clipped at the border of the plot region.  This can be avoided by supplying
#' the graphical parameter \code{xpd = TRUE} in the plot call.
#'
#' The handling of \code{labels} and \code{pretty.xlabels} in \code{coefplot}
#' is experimental.
#' @author Ron Wehrens and Bjørn-Helge Mevik
#' @seealso \code{\link{mvr}}, \code{\link{plot.mvr}}, \code{\link{scores}},
#' \code{\link{loadings}}, \code{\link{identify}}, \code{\link{legend}}
#' @references Martens, H., Martens, M. (2000) Modified Jack-knife Estimation
#' of Parameter Uncertainty in Bilinear Modelling by Partial Least Squares
#' Regression (PLSR).  \emph{Food Quality and Preference}, \bold{11}(1--2),
#' 5--16.
#' @keywords regression multivariate hplot
#' @examples
#'
#' data(yarn)
#' mod <- plsr(density ~ NIR, ncomp = 10, data = yarn)
#' ## These three are equivalent:
#' \dontrun{
#' scoreplot(mod, comps = 1:5)
#' plot(scores(mod), comps = 1:5)
#' plot(mod, plottype = "scores", comps = 1:5)
#'
#' loadingplot(mod, comps = 1:5)
#' loadingplot(mod, comps = 1:5, legendpos = "topright") # With legend
#' loadingplot(mod, comps = 1:5, scatter = TRUE) # Plot as scatterplots
#'
#' corrplot(mod, comps = 1:2)
#' corrplot(mod, comps = 1:3)
#' }
#'
#' # Use of labels in plots and x scales
#' data(gasoline)
#' colnames(gasoline$NIR) <- paste(seq(900, 1700, 2), "nm")
#' gas <- plsr(octane ~ NIR, ncomp = 10, data = gasoline)
#' loadingplot(gas, labels="numbers")
#' loadingplot(gas, labels="names")
#' loadingplot(gas, labels="names", scatter=TRUE)
#'
#' @export
scoreplot <- function(object, ...) UseMethod("scoreplot")

#' @rdname scoreplot
#' @export
scoreplot.default <- function(object, comps = 1:2, labels, identify = FALSE,
                              type = "p", xlab, ylab, estimate, newdata, ...)
{
  ## Check arguments
  nComps <- length(comps)
  if (nComps == 0) stop("At least one component must be selected.")
  if((inherits(object, "scores") && ncol(object) == 1 ) ||
     (inherits(object, "mvr") && length(object$Xvar) == 1)){ # Check for single component in model
    comps <- comps[1]
    nComps <- length(comps)
  }
  ## Get the scores
  if (is.matrix(object)) {
    ## Assume this is already a score matrix
    S <- object[,comps, drop = FALSE]
  } else {
    ## Try to get the scores using the supplied estimate/newdata
    scoreArgs <- list()
    if (!missing(estimate)) scoreArgs$estimate <- estimate
    if (!missing(newdata)) scoreArgs$newdata <- newdata
    args <- if (length(scoreArgs))
      c(list(object), scoreArgs)
    else
      list(object)
    scoreObj <- do.call(scores, args)
    S <- scoreObj[,comps, drop = FALSE]
    expl_attr <- attr(scoreObj, "explvar")
    if (!is.null(expl_attr))
      attr(S, "explvar") <- expl_attr[seq_len(ncol(S))]
    class(S) <- class(scoreObj)
    if (is.null(S))
      stop("`", deparse(substitute(object)), "' has no scores.")
  }
  if (!missing(labels)) {
    ## Set up point labels
    if (length(labels) == 1) {
      labels <- switch(match.arg(labels, c("names", "numbers")),
                       names = rownames(S),
                       numbers = 1:nrow(S)
      )
    }
    labels <- as.character(labels)
    type <- "n"
  }
  varlab <- compnames(S, comps, explvar = TRUE)
  if (nComps <= 2) {
    if (nComps == 1) {
      ## One component versus index
      if (missing(xlab)) xlab <- "observation"
      if (missing(ylab)) ylab <- varlab
    } else {
      ## Second component versus first
      if (missing(xlab)) xlab <- varlab[1]
      if (missing(ylab)) ylab <- varlab[2]
    }
    plot(S, xlab = xlab, ylab = ylab, type = type, ...)
    if (!missing(labels)) text(S, labels, ...)
    if (isTRUE(identify)) {
      if (!is.null(rownames(S))) {
        identify(S, labels = rownames(S))
      } else {
        identify(S)
      }
    }
  } else {
    ## Pairwise scatterplots of several components
    panel <- if (missing(labels))
      function(x, y, ...) points(x, y, type = type, ...) else
        function(x, y, ...) text(x, y, labels = labels, ...)
    pairs(S, labels = varlab, panel = panel, ...)
  }
}

## A plot method for scores:
#' @rdname scoreplot
#' @export
plot.scores <- function(x, ...) scoreplot(x, ...)


###
### Loadingplot
###

#' @rdname scoreplot
#' @export
loadingplot <- function(object, ...) UseMethod("loadingplot")

#' @rdname scoreplot
#' @export
loadingplot.default <- function(object, comps = 1:2, scatter = FALSE, labels,
                                identify = FALSE, type, lty, lwd = NULL, pch,
                                cex = NULL, col, legendpos, xlab, ylab,
                                pretty.xlabels = TRUE, xlim, ...)
{
  ## Check arguments
  nComps <- length(comps)
  if (nComps == 0) stop("At least one component must be selected.")
  if((inherits(object, "loadings") && ncol(object) == 1 ) ||
     (inherits(object, "mvr") && length(object$Xvar) == 1)){ # Check for single component in model
    comps <- comps[1]
    nComps <- length(comps)
  }
  if (!missing(type) &&
      (length(type) != 1 || is.na(nchar(type, "c")) || nchar(type, "c") != 1))
    stop("Invalid plot type.")
  ## Get the loadings
  if (is.matrix(object)) {
    ## Assume this is already a loading matrix
    L <- object[,comps, drop = FALSE]
  } else {
    ## Try to get the loadings:
    L <- loadings(object)[,comps, drop = FALSE]
    if (is.null(L))
      stop("`", deparse(substitute(object)), "' has no loadings.")
  }
  varlab <- compnames(object, comps, explvar = TRUE)
  if (isTRUE(scatter)) {
    ## Scatter plots
    if (missing(type)) type <- "p"
    if (!missing(labels)) {
      ## Set up point/tick mark labels
      if (length(labels) == 1) {
        labels <- switch(match.arg(labels, c("names", "numbers")),
                         names = {
                           if (is.null(rnames <- rownames(L))) {
                             stop("The loadings have no row names.")
                           } else {
                             rnames
                           }},
                         numbers = 1:nrow(L)
        )
      }
      labels <- as.character(labels)
      type <- "n"
    }
    if (missing(lty)) lty <- NULL
    if (missing(pch)) pch <- NULL
    if (missing(col)) col <- par("col") # `NULL' means `no colour'
    if (nComps <= 2) {
      if (nComps == 1) {
        ## One component versus index
        if (missing(xlab)) xlab <- "variable"
        if (missing(ylab)) ylab <- varlab
      } else {
        ## Second component versus first
        if (missing(xlab)) xlab <- varlab[1]
        if (missing(ylab)) ylab <- varlab[2]
      }
      if(missing(xlim)) xlim <- NULL
      plot(L, xlab = xlab, ylab = ylab, type = type, lty = lty,
           lwd = lwd, pch = pch, cex = cex, col = col, xlim = xlim, ...)
      if (!missing(labels)) text(L, labels, cex = cex, col = col, ...)
      if (isTRUE(identify))
        identify(L, labels = paste(1:nrow(L), rownames(L), sep = ": "))
    } else {
      ## Pairwise scatterplots of several components
      panel <- if (missing(labels)) {
        function(x, y, ...)
          points(x, y, type = type, lty = lty, lwd = lwd,
                 pch = pch, col = col, ...)
      } else {
        function(x, y, ...)
          text(x, y, labels = labels, col = col, ...)
      }
      pairs(L, labels = varlab, panel = panel, cex = cex, ...)
    }
  } else {                            # if (!isTRUE(scatter))
    ## Line plots
    if (missing(type)) type <- "l"
    if (missing(lty))  lty  <- 1:nComps
    if (missing(pch))  pch  <- 1:nComps
    if (missing(col))  col  <- 1:nComps
    if (missing(xlab)) xlab <- "variable"
    if (missing(ylab)) ylab <- "loading value"
    xnum <- 1:nrow(L)
    if (missing(labels)) {
      xaxt <- par("xaxt")
    } else {
      xaxt <- "n"
      if (length(labels) == 1) {
        xnam <- rownames(L)
        switch(match.arg(labels, c("names", "numbers")),
               names = {        # Simply use the names as is
                 labels <- xnam
               },
               numbers = {      # Try to use them as numbers
                 if (length(grep("^[-0-9.]+[^0-9]*$", xnam)) ==
                     length(xnam)) {
                   ## Labels are on "num+text" format
                   labels <- sub("[^0-9]*$", "", xnam)
                   if (isTRUE(pretty.xlabels)) {
                     xnum <- as.numeric(labels)
                     xaxt <- par("xaxt")
                   }
                 } else {
                   stop("Could not convert variable names to numbers.")
                 }
               }
        )
      } else {
        labels <- as.character(labels)
      }
    }
    if (missing(xlim)) xlim <- xnum[c(1, length(xnum))] # Needed for reverted scales
    matplot(xnum, L, xlab = xlab, ylab = ylab, type = type,
            lty = lty, lwd = lwd, pch = pch, cex = cex, col = col,
            xaxt = xaxt, xlim = xlim, ...)
    if (!missing(labels) && xaxt == "n") {
      if (isTRUE(pretty.xlabels)) {
        ticks <- axTicks(1)
        ticks <- ticks[ticks >= 1 & ticks <= length(labels)]
      } else {
        ticks <- 1:length(labels)
      }
      axis(1, ticks, labels[ticks], ...)
    }
    if (!missing(legendpos)) {
      ## Are we plotting lines?
      dolines <- type %in% c("l", "b", "c", "o", "s", "S", "h")
      ## Are we plotting points?
      dopoints <- type %in% c("p", "b", "o")
      if (length(lty) > nComps) lty <- lty[1:nComps]
      do.call("legend", c(list(legendpos, varlab, col = col),
                          if (dolines) list(lty = lty, lwd = lwd),
                          if (dopoints) list(pch = pch, pt.cex = cex,
                                             pt.lwd = lwd)))
    }
    if (isTRUE(identify))
      identify(c(row(L)), c(L),
               labels = paste(c(col(L)), rownames(L), sep = ": "))
  }                                   # if (isTRUE(scatter))
}

## A plot method for loadings (loadings, loading.weights or Yloadings):
#' @rdname scoreplot
#' @export
plot.loadings <- function(x, ...) loadingplot(x, ...)


###
### Correlation loadings plot
###

#' @rdname scoreplot
#' @export
corrplot <- function(object, comps = 1:2, labels, plotx = TRUE, ploty = FALSE,
                     radii = c(sqrt(1/2), 1), identify = FALSE,
                     type = "p", xlab, ylab, col, ...)
{
  nComps <- length(comps)
  if (nComps < 2) stop("At least two components must be selected.")
  if (is.matrix(object)) {
    ## Assume this is already a correlation matrix
    cl <- object[,comps, drop = FALSE]
    numX <- nrow(cl)
    numY <- 0
    varlab <- colnames(cl)
  } else {
    S <- scores(object)[,comps, drop = FALSE]
    if (is.null(S))
      stop("`", deparse(substitute(object)), "' has no scores.")

    if (isTRUE(plotx)) {
      clX <- cor(model.matrix(object), S)
      numX <- nrow(clX)
    } else {
      clX <- NULL
      numX <- 0
    }
    if (isTRUE(ploty)) {
      clY <- cor(model.response(model.frame(object)), S)
      numY <- nrow(clY)
      if (numY == 1) {
        ## Add response name for single response models
        rownames(clY) <- all.vars(formula(object))[1]
      }
    } else {
      clY <- NULL
      numY <- 0
    }
    cl <- rbind(clX, clY)
    varlab <- compnames(object, comps, explvar = TRUE)
  }
  if (missing(col)) {
    ## Set up default colours:
    col <- c(rep(1, numX), rep(2, numY))
  }
  if (!missing(labels)) {
    ## Set up point labels
    if (length(labels) == 1) {
      labels <- switch(match.arg(labels, c("names", "numbers")),
                       names = rownames(cl),
                       numbers = 1:nrow(cl)
      )
    }
    labels <- as.character(labels)
    type <- "n"
  }
  ## Build the expression to add circles:
  if (length(radii)) {
    addcircles <- substitute(symbols(cent, cent, circles = radii,
                                     inches = FALSE, add = TRUE),
                             list(cent = rep(0, length(radii))))
  } else {
    addcircles <- expression()
  }
  if (nComps == 2) {
    ## Second component versus first
    if (missing(xlab)) xlab <- varlab[1]
    if (missing(ylab)) ylab <- varlab[2]
    plot(cl, xlim = c(-1,1), ylim = c(-1,1), asp = 1,
         xlab = xlab, ylab = ylab, type = type, col = col, ...)
    eval(addcircles)
    segments(x0 = c(-1, 0), y0 = c(0, -1), x1 = c(1, 0), y1 = c(0, 1))
    if (!missing(labels)) text(cl, labels, col = col, ...)
    if (isTRUE(identify)) {
      if (!is.null(rownames(cl))) {
        identify(cl, labels = rownames(cl))
      } else {
        identify(cl)
      }
    }
  } else {
    ## Pairwise scatterplots of several components
    pointsOrText <- if (missing(labels)) {
      function(x, y, ...) points(x, y, type = type, ...)
    } else {
      function(x, y, ...) text(x, y, labels = labels, ...)
    }
    panel <- function(x, y, ...) {
      ## Ignore the leading `ghost points':
      pointsOrText(x[-(1:2)], y[-(1:2)], ...)
      eval(addcircles)
      segments(x0 = c(-1, 0), y0 = c(0, -1), x1 = c(1, 0),
               y1 = c(0, 1))
    }
    ## Call `pairs' with two leading `ghost points', to get
    ## correct xlim and ylim:
    pairs(rbind(-1, 1, cl), labels = varlab, panel = panel, asp = 1,
          col = col, ...)
  }
}


###
### prediction plot
###

## Generic:


#' @name predplot
#' @title Prediction Plots
#'
#' @description Functions to plot predicted values against measured values for a fitted
#' model.
#'
#' @details \code{predplot} is a generic function for plotting predicted versus measured
#' response values, with default and \code{mvr} methods currently implemented.
#' The default method is very simple, and doesn't handle multiple responses or
#' new data.
#'
#' The \code{mvr} method, handles multiple responses, model sizes and types of
#' predictions by making one plot for each combination.  It can also be called
#' through the plot method for \code{mvr}, by specifying \code{plottype =
#' "prediction"} (the default).
#'
#' The argument \code{main} can be used to specify the main title of the plot.
#' It is handled in a non-standard way.  If there is only on (sub) plot,
#' \code{main} will be used as the main title of the plot.  If there is
#' \emph{more} than one (sub) plot, however, the presence of \code{main} will
#' produce a corresponding \sQuote{global} title on the page.  Any graphical
#' parametres, e.g., \code{cex.main}, supplied to \code{coefplot} will only
#' affect the \sQuote{ordinary} plot titles, not the \sQuote{global} one.  Its
#' appearance can be changed by setting the parameters with \code{\link{par}},
#' which will affect \emph{both} titles (with the exception of \code{font.main}
#' and \code{cex.main}, which will only affect the \sQuote{global} title when
#' there is more than one plot).  (To have different settings for the two
#' titles, one can override the \code{par} settings with arguments to
#' \code{predplot}.)
#'
#' \code{predplotXy} is an internal function and is not meant for interactive
#' use.  It is called by the \code{predplot} methods, and its arguments, e.g,
#' \code{line}, can be given in the \code{predplot} call.
#'
#' @aliases predplot predplot.default predplot.mvr predplotXy
#' @param object a fitted model.
#' @param ncomp integer vector.  The model sizes (numbers of components) to use
#' for prediction.
#' @param which character vector.  Which types of predictions to plot.  Should
#' be a subset of \code{c("train", "validation", "test")}.  If not specified,
#' \code{plot.mvr} selects test set predictions if \code{newdata} is supplied,
#' otherwise cross-validated predictions if the model has been cross-validated,
#' otherwise fitted values from the calibration data.
#' @param newdata data frame.  New data to predict.
#' @param nCols,nRows integer.  The number of coloumns and rows the plots will
#' be laid out in.  If not specified, \code{plot.mvr} tries to be intelligent.
#' @param xlab,ylab titles for \eqn{x} and \eqn{y} axes.  Typically character
#' strings, but can be expressions or lists.  See \code{\link{title}} for
#' details.
#' @param labels optional.  Alternative plot labels to use.  Either a vector of
#' labels, or \code{"names"} or \code{"numbers"} to use the row names or row
#' numbers of the data as labels.
#' @param type character.  What type of plot to make.  Defaults to \code{"p"}
#' (points).  See \code{\link{plot}} for a complete list of types.  The
#' argument is ignored if \code{labels} is specified.
#' @param main optional main title for the plot.  See Details.
#' @param ask logical.  Whether to ask the user before each page of a plot.
#' @param font.main font to use for main titles.  See \code{\link{par}} for
#' details.  Also see Details below.
#' @param cex.main numeric.  The magnification to be used for main titles
#' relative to the current size.  Also see Details below.
#' @param x numeric vector.  The observed response values.
#' @param y numeric vector.  The predicted response values.
#' @param line logical.  Whether a target line should be drawn.
#' @param line.col,line.lty,line.lwd character or numeric.  The \code{col},
#' \code{lty} and \code{lwd} parametres for the target line.  See
#' \code{\link{par}} for details.
#' @param \dots further arguments sent to underlying plot functions.
#' @return The functions invisibly return a matrix with the (last) plotted
#' data.
#' @note The \code{font.main} and \code{cex.main} must be (completely) named.
#' This is to avoid that any argument \code{cex} or \code{font} matches them.
#'
#' Tip: If the labels specified with \code{labels} are too long, they get
#' clipped at the border of the plot region.  This can be avoided by supplying
#' the graphical parameter \code{xpd = TRUE} in the plot call.
#' @author Ron Wehrens and Bjørn-Helge Mevik
#' @seealso \code{\link{mvr}}, \code{\link{plot.mvr}}
#' @keywords regression multivariate hplot
#' @examples
#'
#' data(yarn)
#' mod <- plsr(density ~ NIR, ncomp = 10, data = yarn[yarn$train,], validation = "CV")
#' \dontrun{
#' predplot(mod, ncomp = 1:6)
#' plot(mod, ncomp = 1:6) # Equivalent to the previous
#' ## Both cross-validated and test set predictions:
#' predplot(mod, ncomp = 4:6, which = c("validation", "test"),
#'          newdata = yarn[!yarn$train,])
#' }
#'
#' data(oliveoil)
#' mod.sens <- plsr(sensory ~ chemical, ncomp = 4, data = oliveoil)
#' \dontrun{plot(mod.sens, ncomp = 2:4) # Several responses gives several plots}
#'
#' @export
predplot <- function(object, ...)
  UseMethod("predplot")

## Default method:
#' @rdname predplot
#' @export
predplot.default <- function(object, ...) {
  measured <- model.response(model.frame(object))
  predicted <- predict(object)
  predplotXy(measured, predicted, ...)
}

## Method for mvr objects:
#' @rdname predplot
#' @export
predplot.mvr <- function(object, ncomp = object$ncomp, which, newdata,
                         nCols, nRows, xlab = "measured", ylab = "predicted",
                         main,
                         ask = nRows * nCols < nPlots && dev.interactive(),
                         ..., font.main, cex.main)
{
  ## Select type(s) of prediction
  if (missing(which)) {
    ## Pick the `best' alternative.
    if (!missing(newdata)) {
      which <- "test"
    } else {
      if (!is.null(object$validation)) {
        which <- "validation"
      } else {
        which <- "train"
      }
    }
  } else {
    ## Check the supplied `which'
    allTypes <- c("train", "validation", "test")
    which <- allTypes[pmatch(which, allTypes)]
    if (length(which) == 0 || any(is.na(which)))
      stop("`which' should be a subset of ",
           paste(allTypes, collapse = ", "))
  }

  ## Help variables
  nEst  <- length(which)
  nSize <- length(ncomp)
  nResp <- dim(object$fitted.values)[2]

  ## Set plot parametres as needed:
  dims <- c(nEst, nSize, nResp)
  dims <- dims[dims > 1]
  nPlots <- prod(dims)
  if (nPlots > 1) {
    ## Set up default font.main and cex.main for individual titles:
    if (missing(font.main)) font.main <- 1
    if (missing(cex.main)) cex.main <- 1.1
    ## Show the *labs in the margin:
    mXlab <- xlab
    mYlab <- ylab
    xlab <- ylab <- ""
    if(missing(nCols)) nCols <- min(c(3, dims[1]))
    if(missing(nRows))
      nRows <- min(c(3, ceiling(prod(dims[1:2], na.rm = TRUE) / nCols)))
    opar <- par(no.readonly = TRUE)
    on.exit(par(opar))
    par(mfrow = c(nRows, nCols),
        oma = c(1, 1, if(missing(main)) 0 else 2, 0) + 0.1,
        mar = c(3,3,3,1) + 0.1)
    if (isTRUE(ask)) {
      oask <- devAskNewPage(TRUE)
      on.exit(devAskNewPage(oask))
    }
  } else {
    ## Set up default font.main and cex.main for the main title:
    if (missing(font.main)) font.main <- par("font.main")
    if (missing(cex.main)) cex.main <- par("cex.main")
    nCols <- nRows <- 1
  }

  ## Set up measured and predicted for all model sizes, responses and
  ## estimates:
  if ("train" %in% which) {
    train.measured <- as.matrix(model.response(model.frame(object)))
    train.predicted <- object$fitted.values[,,ncomp, drop = FALSE]
  }
  if ("validation" %in% which) {
    if (is.null(object$validation)) stop("`object' has no `validation' component.")
    if(!exists("train.measured"))
      train.measured <- as.matrix(model.response(model.frame(object)))
    validation.predicted <- object$validation$pred[,,ncomp, drop = FALSE]
  }
  if ("test" %in% which) {
    if (missing(newdata)) stop("Missing `newdata'.")
    test.measured <- as.matrix(model.response(model.frame(formula(object),
                                                          data = newdata)))
    test.predicted <- predict(object, ncomp = ncomp, newdata = newdata)
  }

  ## Do the plots
  plotNo <- 0
  for (resp in 1:nResp) {
    for (size in 1:nSize) {
      for (est in 1:nEst) {
        plotNo <- plotNo + 1
        if (nPlots == 1 && !missing(main)) {
          lmain <- main
        } else {
          lmain <- sprintf("%s, %d comps, %s",
                           respnames(object)[resp],
                           ncomp[size], which[est])
        }
        sub <- which[est]
        switch(which[est],
               train = {
                 measured <- train.measured[,resp]
                 predicted <- train.predicted[,resp,size]
               },
               validation = {
                 measured <- train.measured[,resp]
                 predicted <- validation.predicted[,resp,size]
               },
               test = {
                 measured <- test.measured[,resp]
                 predicted <- test.predicted[,resp,size]
               }
        )
        xy <- predplotXy(measured, predicted, main = lmain,
                         font.main = font.main, cex.main = cex.main,
                         xlab = xlab, ylab = ylab, ...)
        if (nPlots > 1 &&
            (plotNo %% (nCols * nRows) == 0 || plotNo == nPlots)) {
          ## Last plot on a page; add outer margin text and title:
          mtext(mXlab, side = 1, outer = TRUE)
          mtext(mYlab, side = 2, outer = TRUE)
          if (!missing(main)) title(main = main, outer = TRUE)
        }
      }
    }
  }
  invisible(xy)
}

## The workhorse function:
#' @rdname predplot
#' @export
predplotXy <- function(x, y, line = FALSE, labels, type = "p",
                       main = "Prediction plot", xlab = "measured response",
                       ylab = "predicted response", line.col = par("col"),
                       line.lty = NULL, line.lwd = NULL, ...)
{
  if (!missing(labels)) {
    ## Set up point labels
    if (length(labels) == 1) {
      labels <- switch(match.arg(labels, c("names", "numbers")),
                       names = names(y),
                       numbers = as.character(1:length(y))
      )
    }
    ## Override plot type:
    type <- "n"
  }
  plot(y ~ x, type = type, main = main, xlab = xlab, ylab = ylab, ...)
  if (!missing(labels)) text(x, y, labels, ...)
  if (isTRUE(line)) abline(0, 1, col = line.col, lty = line.lty, lwd = line.lwd)
  invisible(cbind(measured = x, predicted = as.vector(y)))
}


###
### Coefficient plot
###



#' @title Plot Regression Coefficients of PLSR and PCR models
#'
#' @description  Function to plot the regression coefficients of an \code{mvr} object.
#'
#' @details \code{coefplot} handles multiple responses by making one plot for each
#' response.  If \code{separate} is \code{TRUE}, separate plots are made for
#' each combination of model size and response.  The plots are laid out in a
#' rectangular fashion.
#'
#' If \code{legendpos} is given, a legend is drawn at the given position
#' (unless \code{separate} is \code{TRUE}).
#'
#' The argument \code{labels} can be a vector of labels or one of
#' \code{"names"} and \code{"numbers"}.  The labels are used as \eqn{x} axis
#' labels.  If \code{labels} is \code{"names"} or \code{"numbers"}, the
#' variable names are used as labels, the difference being that with
#' \code{"numbers"}, the variable names are converted to numbers, if possible.
#' Variable names of the forms \samp{"number"} or \samp{"number text"} (where
#' the space is optional), are handled.
#'
#' The argument \code{main} can be used to specify the main title of the plot.
#' It is handled in a non-standard way.  If there is only on (sub) plot,
#' \code{main} will be used as the main title of the plot.  If there is
#' \emph{more} than one (sub) plot, however, the presence of \code{main} will
#' produce a corresponding \sQuote{global} title on the page.  Any graphical
#' parametres, e.g., \code{cex.main}, supplied to \code{coefplot} will only
#' affect the \sQuote{ordinary} plot titles, not the \sQuote{global} one.  Its
#' appearance can be changed by setting the parameters with \code{\link{par}},
#' which will affect \emph{both} titles.  (To have different settings for the
#' two titles, one can override the \code{par} settings with arguments to
#' \code{coefplot}.)
#'
#' The argument \code{pretty.xlabels} is only used when \code{labels} is
#' specified.  If \code{TRUE} (default), the code tries to use a
#' \sQuote{pretty} selection of labels.  If \code{labels} is \code{"numbers"},
#' it also uses the numerical values of the labels for horisontal spacing.  If
#' one has excluded parts of the spectral region, one might therefore want to
#' use \code{pretty.xlabels = FALSE}.
#'
#' When \code{separate} is \code{TRUE}, the arguments \code{lty}, \code{col},
#' and \code{pch} default to their \code{par()} setting.  Otherwise, the
#' default for all of them is \code{1:nLines}, where \code{nLines} is the
#' number of model sizes specified, i.e., the length of \code{ncomp} or
#' \code{comps}.
#'
#' The function can also be called through the \code{mvr} plot method by
#' specifying \code{plottype = "coefficients"}.
#'
#' @param object an \code{mvr} object.  The fitted model.
#' @param ncomp,comps vector of positive integers.  The components to plot.
#' See \code{\link{coef.mvr}} for details.
#' @param separate logical.  If \code{TRUE}, coefficients for different model
#' sizes are blotted in separate plots.
#' @param se.whiskers logical.  If \code{TRUE}, whiskers at plus/minus 1
#' estimated standard error are added to the plot.  This is only available if
#' the model was cross-validated with \code{jackknife = TRUE}.  Also, in the
#' current implementation, \code{intercept} must be \code{FALSE}, and
#' \code{separate} must be \code{TRUE} if \code{length(ncomp) > 1}.
#' @param intercept logical.  Whether coefficients for the intercept should be
#' plotted.  Ignored if \code{comps} is specified.  Defaults to \code{FALSE}.
#' See \code{\link{coef.mvr}} for details.
#' @param nCols,nRows integer.  The number of coloumns and rows the plots will
#' be laid out in.  If not specified, \code{coefplot} tries to be intelligent.
#' @param labels optional.  Alternative \eqn{x} axis labels.  See Details.
#' @param type character.  What type of plot to make.  Defaults to \code{"l"}
#' (lines).  Alternative types include \code{"p"} (points) and \code{"b"}
#' (both).  See \code{\link{plot}} for a complete list of types.
#' @param lty vector of line types (recycled as neccessary).  Line types can be
#' specified as integers or character strings (see \code{\link{par}} for the
#' details).
#' @param lwd vector of positive numbers (recycled as neccessary), giving the
#' width of the lines.
#' @param pch plot character.  A character string or a vector of single
#' characters or integers (recycled as neccessary).  See \code{\link{points}}
#' for all alternatives.
#' @param cex numeric vector of character expansion sizes (recycled as
#' neccessary) for the plotted symbols.
#' @param col character or integer vector of colors for plotted lines and
#' symbols (recycled as neccessary).  See \code{\link{par}} for the details.
#' @param legendpos Legend position.  Optional.  Ignored if \code{separate} is
#' \code{TRUE}.  If present, a legend is drawn at the given position.  The
#' position can be specified symbolically (e.g., \code{legendpos =
#' "topright"}).  This requires >= 2.1.0.  Alternatively, the position can be
#' specified explicitly (\code{legendpos = t(c(x,y))}) or interactively
#' (\code{legendpos = \link{locator}()}).  This only works well for plots of
#' single-response models.
#' @param xlab,ylab titles for \eqn{x} and \eqn{y} axes.  Typically character
#' strings, but can be expressions (e.g., \code{expression(R^2)} or lists.  See
#' \code{\link{title}} for details.
#' @param main optional main title for the plot.  See Details.
#' @param pretty.xlabels logical.  If \code{TRUE}, \code{coefplot} tries to
#' plot the \eqn{x} labels more nicely.  See Details.
#' @param xlim,ylim optional vector of length two, with the \eqn{x} or \eqn{y}
#' limits of the plot.
#' @param ask logical.  Whether to ask the user before each page of a plot.
#' @param \dots Further arguments sent to the underlying plot functions.
#' @note \code{\link{legend}} has many options.  If you want greater control
#' over the appearance of the legend, omit the \code{legendpos} argument and
#' call \code{legend} manually.
#'
#' The handling of \code{labels} and \code{pretty.xlabels} is experimental.
#' @author Ron Wehrens and Bjørn-Helge Mevik
#' @seealso \code{\link{mvr}}, \code{\link{plot.mvr}}, \code{\link{coef.mvr}},
#' \code{\link{plot}}, \code{\link{legend}}
#' @keywords regression multivariate hplot
#' @examples
#'
#' data(yarn)
#' mod.nir <- plsr(density ~ NIR, ncomp = 8, data = yarn)
#' \dontrun{
#' coefplot(mod.nir, ncomp = 1:6)
#' plot(mod.nir, plottype = "coefficients", ncomp = 1:6) # Equivalent to the previous
#' ## Plot with legend:
#' coefplot(mod.nir, ncom = 1:6, legendpos = "bottomright")
#' }
#'
#' data(oliveoil)
#' mod.sens <- plsr(sensory ~ chemical, ncomp = 4, data = oliveoil)
#' \dontrun{coefplot(mod.sens, ncomp = 2:4, separate = TRUE)}
#'
#' @export
coefplot <- function(object, ncomp = object$ncomp, comps, intercept = FALSE,
                     separate = FALSE, se.whiskers = FALSE,
                     nCols, nRows, labels,
                     type = "l", lty, lwd = NULL,
                     pch, cex = NULL, col, legendpos,
                     xlab = "variable", ylab = "regression coefficient",
                     main, pretty.xlabels = TRUE, xlim, ylim,
                     ask = nRows * nCols < nPlots && dev.interactive(), ...)
{
  ## This simplifies code below:
  if (missing(comps)) comps <- NULL
  separate <- isTRUE(separate)
  se.whiskers <- isTRUE(se.whiskers)

  ## Help variables
  nLines <- if (is.null(comps)) length(ncomp) else length(comps)
  nSize <- if (separate) nLines else 1
  nResp <- dim(object$fitted.values)[2]

  ## Set plot parametres as needed:
  dims <- c(nSize, nResp)
  dims <- dims[dims > 1]
  nPlots <- prod(dims)
  if (nPlots > 1) {
    ## Show the *labs in the margin:
    mXlab <- xlab
    mYlab <- ylab
    xlab <- ylab <- ""
    if (missing(nCols)) nCols <- min(c(3, dims[1]))
    if (missing(nRows))
      nRows <- min(c(3, ceiling(prod(dims[1:2], na.rm = TRUE) / nCols)))
    opar <- par(no.readonly = TRUE)
    on.exit(par(opar))
    par(mfrow = c(nRows, nCols),
        oma = c(1, 1, if(missing(main)) 0 else 2, 0) + 0.1,
        mar = c(3,3,3,1) + 0.1)
    if (isTRUE(ask)) {
      oask <- devAskNewPage(TRUE)
      on.exit(devAskNewPage(oask))
    }
  } else {
    nCols <- nRows <- 1
  }
  if (missing(col)) col <- if (separate) par("col") else 1:nLines
  if (missing(pch)) pch <- if (separate) par("pch") else 1:nLines
  if (missing(lty)) lty <- if (separate) par("lty") else 1:nLines
  if (length(lty) > nLines) lty <- lty[1:nLines] # otherwise legend chokes
  if (length(type) != 1 || is.na(nchar(type, "c")) || nchar(type, "c") != 1)
    stop("Invalid plot type.")
  ## Are we plotting lines?
  dolines <- type %in% c("l", "b", "c", "o", "s", "S", "h")
  ## Are we plotting points?
  dopoints <- type %in% c("p", "b", "o")

  ## Get the coefficients:
  coefs <- coef(object, ncomp = ncomp, comps = comps, intercept = intercept)
  complabs <- dimnames(coefs)[[3]]

  ## Optionally, get the standard errors:
  if (se.whiskers) {
    if (isTRUE(intercept))
      stop(sQuote("se.whiskers"), " not supported when ",
           sQuote("intercept"), " is TRUE")
    if (!is.null(comps))
      stop(sQuote("se.whiskers"), " not supported when ",
           sQuote("comps"), " is specified")
    if (dim(coefs)[3] > 1 && !separate)
      stop(sQuote("se.whiskers"), " not supported when ",
           sQuote("separate"), " is FALSE and length(ncomp) > 1")
    SEs <- sqrt(var.jack(object, ncomp = ncomp))
    npred <- dim(SEs)[1]
    if (!hasArg("ylim")) {
      ## Calculate new ylims:
      miny <- apply(coefs - SEs, 2:3, min)
      maxy <- apply(coefs + SEs, 2:3, max)
    }
  }

  ## Set up the x labels:
  xnum <- 1:dim(coefs)[1]
  if (missing(labels)) {
    xaxt <- par("xaxt")
  } else {
    xaxt <- "n"
    if (length(labels) == 1) {
      xnam <- prednames(object, intercept = intercept)
      switch(match.arg(labels, c("names", "numbers")),
             names = {            # Simply use the names as is
               labels <- xnam
             },
             numbers = {          # Try to use them as numbers
               if (length(grep("^[-0-9.]+[^0-9]*$", xnam)) ==
                   length(xnam)) {
                 ## Labels are on "num+text" format
                 labels <- sub("[^0-9]*$", "", xnam)
                 if (isTRUE(pretty.xlabels)) {
                   xnum <- as.numeric(labels)
                   xaxt <- par("xaxt")
                 }
               } else {
                 stop("Could not convert variable names to numbers.")
               }
             }
      )
    } else {
      labels <- as.character(labels)
    }
  }
  if (missing(xlim)) xlim <- xnum[c(1, length(xnum))] # Needed for reverted scales
  ## Do the plots
  plotNo <- 0
  for (resp in 1:nResp) {
    respname <- respnames(object)[resp]
    for (size in 1:nSize) {
      plotNo <- plotNo + 1

      if (nPlots == 1 && !missing(main)) {
        lmain <- main
      } else if (separate) {
        lmain <- paste(respname, complabs[size], sep = ", ")
      } else {
        lmain <- respname
      }
      if (separate) {
        if (missing(ylim)) {
          if (se.whiskers) {
            ylims <- c(miny[resp,size], maxy[resp,size])
          } else {
            ylims <- range(coefs[,resp,size])
          }
        } else {
          ylims <- ylim
        }
        plot(xnum, coefs[,resp,size],
             main = lmain, xlab = xlab, ylab = ylab, type = type,
             lty = lty, lwd = lwd, pch = pch, cex = cex,
             col = col, xaxt = xaxt, xlim = xlim, ylim = ylims, ...)
        if (se.whiskers) {
          arrows(1:npred, (coefs - SEs)[,resp,size],
                 1:npred, (coefs + SEs)[,resp,size], length = 0.05,
                 angle = 90, code = 3, col = 2)
        }
      } else {
        if (missing(ylim)) {
          if (se.whiskers) {
            ylims <- c(miny[resp,], maxy[resp,])
          } else {
            ylims <- range(coefs[,resp,])
          }
        } else {
          ylims <- ylim
        }
        matplot(xnum, coefs[,resp,], main = lmain, xlab = xlab,
                ylab = ylab, type = type, lty = lty, lwd = lwd,
                pch = pch, cex = cex, col = col, xaxt = xaxt,
                xlim = xlim, ylim = ylims, ...)
        if (se.whiskers) {
          arrows(1:npred, (coefs - SEs)[,resp,],
                 1:npred, (coefs + SEs)[,resp,], length = 0.05,
                 angle = 90, code = 3, col = 2)
        }
        if (!missing(legendpos)) {
          do.call("legend", c(list(legendpos, complabs, col = col),
                              if(dolines) list(lty = lty, lwd = lwd),
                              if(dopoints) list(pch = pch,
                                                pt.cex = cex,
                                                pt.lwd = lwd)))
        }
      }
      if (!missing(labels) && xaxt == "n") {
        if (isTRUE(pretty.xlabels)) {
          ticks <- axTicks(1)
          ticks <- ticks[ticks >= 1 & ticks <= length(labels)]
        } else {
          ticks <- 1:length(labels)
        }
        axis(1, ticks, labels[ticks], ...)
      }
      abline(h = 0, col = "gray")

      if (nPlots > 1 &&
          (plotNo %% (nCols * nRows) == 0 || plotNo == nPlots)) {
        ## Last plot on a page; add outer margin text and title:
        mtext(mXlab, side = 1, outer = TRUE)
        mtext(mYlab, side = 2, outer = TRUE)
        if (!missing(main)) title(main, outer = TRUE)
      }
    }
  }
}


###
### Validation plot (MSEP/RMSEP/R2)
###



#' @name validationplot
#' @title Validation Plots
#'
#' @description Functions to plot validation statistics, such as RMSEP or \eqn{R^2}, as a
#' function of the number of components.
#'
#' @details \code{validationplot} calls the proper validation function (currently
#' \code{\link{MSEP}}, \code{\link{RMSEP}} or \code{\link{R2}}) and plots the
#' results with \code{plot.mvrVal}.  \code{validationplot} can be called
#' through the \code{mvr} plot method, by specifying \code{plottype =
#' "validation"}.
#'
#' \code{plot.mvrVal} creates one plot for each response variable in the model,
#' laid out in a rectangle.  It uses \code{\link{matplot}} for performing the
#' actual plotting.  If \code{legendpos} is given, a legend is drawn at the
#' given position.
#'
#' The argument \code{main} can be used to specify the main title of the plot.
#' It is handled in a non-standard way.  If there is only on (sub) plot,
#' \code{main} will be used as the main title of the plot.  If there is
#' \emph{more} than one (sub) plot, however, the presence of \code{main} will
#' produce a corresponding \sQuote{global} title on the page.  Any graphical
#' parametres, e.g., \code{cex.main}, supplied to \code{coefplot} will only
#' affect the \sQuote{ordinary} plot titles, not the \sQuote{global} one.  Its
#' appearance can be changed by setting the parameters with \code{\link{par}},
#' which will affect \emph{both} titles.  (To have different settings for the
#' two titles, one can override the \code{par} settings with arguments to the
#' plot function.)
#'
#' @aliases validationplot plot.mvrVal
#' @param object an \code{mvr} object.
#' @param val.type character.  What type of validation statistic to plot.
#' @param estimate character.  Which estimates of the statistic to calculate.
#' See \code{\link{RMSEP}}.
#' @param newdata data frame.  Optional new data used to calculate statistic.
#' @param ncomp,comps integer vector.  The model sizes to compute the statistic
#' for.  See \code{\link{RMSEP}}.
#' @param intercept logical.  Whether estimates for a model with zero
#' components should be calculated as well.
#' @param x an \code{mvrVal} object.  Usually the result of a
#' \code{\link{RMSEP}}, \code{\link{MSEP}} or \code{\link{R2}} call.
#' @param nCols,nRows integers.  The number of coloumns and rows the plots will
#' be laid out in.  If not specified, \code{plot.mvrVal} tries to be
#' intelligent.
#' @param type character.  What type of plots to create.  Defaults to
#' \code{"l"} (lines).  Alternative types include \code{"p"} (points) and
#' \code{"b"} (both).  See \code{\link{plot}} for a complete list of types.
#' @param lty vector of line types (recycled as neccessary).  Line types can be
#' specified as integers or character strings (see \code{\link{par}} for the
#' details).
#' @param lwd vector of positive numbers (recycled as neccessary), giving the
#' width of the lines.
#' @param pch plot character.  A character string or a vector of single
#' characters or integers (recycled as neccessary).  See \code{\link{points}}
#' for all alternatives.
#' @param cex numeric vector of character expansion sizes (recycled as
#' neccessary) for the plotted symbols.
#' @param col character or integer vector of colors for plotted lines and
#' symbols (recycled as neccessary).  See \code{\link{par}} for the details.
#' @param legendpos Legend position.  Optional.  If present, a legend is drawn
#' at the given position.  The position can be specified symbolically (e.g.,
#' \code{legendpos = "topright"}).  This requires >= 2.1.0.  Alternatively, the
#' position can be specified explicitly (\code{legendpos = t(c(x,y))}) or
#' interactively (\code{legendpos = \link{locator}()}).  This only works well
#' for plots of single-response models.
#' @param xlab,ylab titles for \eqn{x} and \eqn{y} axes.  Typically character
#' strings, but can be expressions (e.g., \code{expression(R^2)} or lists.  See
#' \code{\link{title}} for details.
#' @param main optional main title for the plot.  See Details.
#' @param ask logical.  Whether to ask the user before each page of a plot.
#' @param \dots Further arguments sent to underlying plot functions.
#' @note \code{\link{legend}} has many options.  If you want greater control
#' over the appearance of the legend, omit the \code{legendpos} argument and
#' call \code{legend} manually.
#' @author Ron Wehrens and Bjørn-Helge Mevik
#' @seealso \code{\link{mvr}}, \code{\link{plot.mvr}}, \code{\link{RMSEP}},
#' \code{\link{MSEP}}, \code{\link{R2}}, \code{\link{matplot}},
#' \code{\link{legend}}
#' @keywords regression multivariate hplot
#' @examples
#'
#' data(oliveoil)
#' mod <- plsr(sensory ~ chemical, data = oliveoil, validation = "LOO")
#' \dontrun{
#' ## These three are equivalent:
#' validationplot(mod, estimate = "all")
#' plot(mod, "validation", estimate = "all")
#' plot(RMSEP(mod, estimate = "all"))
#' ## Plot R2:
#' plot(mod, "validation", val.type = "R2")
#' ## Plot R2, with a legend:
#' plot(mod, "validation", val.type = "MSEP", legendpos = "top") # R >= 2.1.0
#' }
#'
#' @export
validationplot <- function(object, val.type = c("RMSEP", "MSEP", "R2"),
                           estimate, newdata, ncomp, comps, intercept, ...)
{
  cl <- match.call(expand.dots = FALSE)
  cl[[1]] <- as.name(match.arg(val.type))
  cl$val.type <- NULL
  x <- eval(cl, parent.frame())
  plot(x, ...)
}

## A plot method for mvrVal objects:
#' @rdname validationplot
#' @export
plot.mvrVal <- function(x, nCols, nRows, type = "l", lty = 1:nEst,
                        lwd = par("lwd"), pch = 1:nEst, cex = 1, col = 1:nEst,
                        legendpos, xlab = "number of components",
                        ylab = x$type, main,
                        ask = nRows * nCols < nResp && dev.interactive(), ...)
{
  if (!is.null(x$call$cumulative) && eval(x$call$cumulative) == FALSE)
    stop("`cumulative = FALSE' not supported.")
  ## Set plot parametres as needed:
  nResp <- dim(x$val)[2]              # Number of response variables
  if (nResp > 1) {
    ## Show the *labs in the margin:
    mXlab <- xlab
    mYlab <- ylab
    xlab <- ylab <- ""
    if(missing(nCols)) nCols <- min(c(3, nResp))
    if(missing(nRows)) nRows <- min(c(3, ceiling(nResp / nCols)))
    opar <- par(no.readonly = TRUE)
    on.exit(par(opar))
    par(mfrow = c(nRows, nCols),
        oma = c(1, 1, if(missing(main)) 0 else 2, 0) + 0.1,
        mar = c(3,3,3,1) + 0.1)
    if (isTRUE(ask)) {
      oask <- devAskNewPage(TRUE)
      on.exit(devAskNewPage(oask))
    }
  } else {
    nCols <- nRows <- 1
  }
  ynames <- dimnames(x$val)[[2]]      # Names of response variables
  estnames <- dimnames(x$val)[[1]]    # Names of estimators
  nEst <- length(estnames)
  if (length(lty) > nEst) lty <- lty[1:nEst] # otherwise legend chokes
  if (length(type) != 1 || is.na(nchar(type, "c")) || nchar(type, "c") != 1)
    stop("Invalid plot type.")
  ## Are we plotting lines?
  dolines <- type %in% c("l", "b", "c", "o", "s", "S", "h")
  ## Are we plotting points?
  dopoints <- type %in% c("p", "b", "o")

  for (resp in 1:nResp) {
    if (nResp == 1 && !missing(main)) {
      lmain <- main
    } else {
      lmain <- ynames[resp]
    }
    y <- x$val[,resp,]
    if (is.matrix(y)) y <- t(y)
    if (isTRUE(all.equal(x$comps, min(x$comps):max(x$comps)))) {
      matplot(x$comps, y, xlab = xlab, ylab = ylab, main = lmain,
              type = type, lty = lty, lwd = lwd, pch = pch, cex = cex,
              col = col, ...)
    } else {
      ## Handle irregular x$comps:
      matplot(y, xlab = xlab, ylab = ylab, main = lmain,
              xaxt = "n", type = type, lty = lty, lwd = lwd,
              pch = pch, cex = cex, col = col, ...)
      axis(1, seq(along = x$comps), x$comps)
    }
    if (!missing(legendpos)) {
      do.call("legend", c(list(legendpos, estnames, col = col),
                          if (dolines) list(lty = lty, lwd = lwd),
                          if (dopoints) list(pch = pch, pt.cex = cex,
                                             pt.lwd = lwd)))
    }
    if (nResp > 1 && (resp %% (nCols * nRows) == 0 || resp == nResp)) {
      ## Last plot on a page; add outer margin text and title:
      mtext(mXlab, side = 1, outer = TRUE)
      mtext(mYlab, side = 2, outer = TRUE)
      if (!missing(main)) title(main, outer = TRUE)
    }
  }
}


###
### biplot
###



#' @name biplot.mvr
#' @title Biplots of PLSR and PCR Models.
#'
#' @description Biplot method for \code{mvr} objects.
#'
#' @details \code{biplot.mvr} can also be called through the \code{mvr} plot method by
#' specifying \code{plottype = "biplot"}.
#'
#' @param x an \code{mvr} object.
#' @param comps integer vector of length two.  The components to plot.
#' @param which character.  Which matrices to plot.  One of \code{"x"} (X
#' scores and loadings), \code{"y"} (Y scores and loadings), \code{"scores"} (X
#' and Y scores) and \code{"loadings"} (X and Y loadings).
#' @param var.axes logical.  If \code{TRUE}, the second set of points have
#' arrows representing them.
#' @param xlabs either a character vector of labels for the first set of
#' points, or \code{FALSE} for no labels.  If missing, the row names of the
#' first matrix is used as labels.
#' @param ylabs either a character vector of labels for the second set of
#' points, or \code{FALSE} for no labels.  If missing, the row names of the
#' second matrix is used as labels.
#' @param main character.  Title of plot.  If missing, a title is constructed
#' by \code{biplot.mvr}.
#' @param \dots Further arguments passed on to \code{biplot.default}.
#' @author Ron Wehrens and Bjørn-Helge Mevik
#' @seealso \code{\link{mvr}}, \code{\link{plot.mvr}},
#' \code{\link{biplot.default}}
#' @keywords regression multivariate hplot
#' @examples
#'
#' data(oliveoil)
#' mod <- plsr(sensory ~ chemical, data = oliveoil)
#' \dontrun{
#' ## These are equivalent
#' biplot(mod)
#' plot(mod, plottype = "biplot")
#'
#' ## The four combinations of x and y points:
#' par(mfrow = c(2,2))
#' biplot(mod, which = "x") # Default
#' biplot(mod, which = "y")
#' biplot(mod, which = "scores")
#' biplot(mod, which = "loadings")
#' }
#'
#' @export
biplot.mvr <- function(x, comps = 1:2,
                       which = c("x", "y", "scores", "loadings"),
                       var.axes = FALSE, xlabs, ylabs, main, ...)
{
  if (length(comps) != 2) stop("Exactly 2 components must be selected.")
  which <- match.arg(which)
  switch(which,
         x = {
           objects <- x$scores
           vars <- x$loadings
           title <- "X scores and X loadings"
         },
         y = {
           objects <- x$Yscores
           vars <- x$Yloadings
           title <- "Y scores and Y loadings"
         },
         scores = {
           objects <- x$scores
           vars <- x$Yscores
           title <- "X scores and Y scores"
         },
         loadings = {
           objects <- x$loadings
           vars <- x$Yloadings
           title <- "X loadings and Y loadings"
         }
  )
  if (is.null(objects) || is.null(vars))
    stop("'x' lacks the required scores/loadings.")
  ## Build a call to `biplot'
  mc <- match.call()
  mc$comps <- mc$which <- NULL
  mc$x <- objects[,comps, drop = FALSE]
  mc$y <- vars[,comps, drop = FALSE]
  if (missing(main)) mc$main <- title
  if (missing(var.axes)) mc$var.axes = FALSE
  if (!missing(xlabs) && isFALSE(xlabs))
    mc$xlabs <- rep("o", nrow(objects))
  if (!missing(ylabs) && isFALSE(ylabs))
    mc$ylabs <- rep("o", nrow(vars))
  mc[[1]] <- quote(biplot)
  ## Evaluate the call:
  eval(mc, parent.frame())
}

### kernelpls.fit.R: Kernel PLS fit algorithm for tall data.
###
### Implements an adapted version of the `algorithm 1' described in
###   Dayal, B. S. and MacGregor, J. F. (1997) Improved PLS algorithms.
###   \emph{Journal of Chemometrics}, \bold{11}, 73--85.
### (This is a modification of the algorithm described in
###   Lindgren F, Geladi P, Wold S (1993) The kernel algorithm for PLS.
###   J. Chemometrics 7, 45-59,
### incorporating the changes in
###   de Jong, S. and ter Braak,  C. J. F. (1994) Comments on the PLS kernel
###   algorithm.  \emph{Journal of Chemometrics}, \bold{8}, 169--174.



#' @title Kernel PLS (Dayal and MacGregor)
#'
#' @description Fits a PLSR model with the kernel algorithm.
#'
#' @details This function should not be called directly, but through the generic
#' functions \code{plsr} or \code{mvr} with the argument
#' \code{method="kernelpls"} (default).  Kernel PLS is particularly efficient
#' when the number of objects is (much) larger than the number of variables.
#' The results are equal to the NIPALS algorithm.  Several different forms of
#' kernel PLS have been described in literature, e.g.  by De Jong and Ter
#' Braak, and two algorithms by Dayal and MacGregor.  This function implements
#' the fastest of the latter, not calculating the crossproduct matrix of X.  In
#' the Dyal & MacGregor paper, this is \dQuote{algorithm 1}.
#'
#' @param X a matrix of observations.  \code{NA}s and \code{Inf}s are not
#' allowed.
#' @param Y a vector or matrix of responses.  \code{NA}s and \code{Inf}s are
#' not allowed.
#' @param ncomp the number of components to be used in the modelling.
#' @param center logical, determines if the \eqn{X} and \eqn{Y} matrices are
#' mean centered or not. Default is to perform mean centering.
#' @param stripped logical.  If \code{TRUE} the calculations are stripped as
#' much as possible for speed; this is meant for use with cross-validation or
#' simulations when only the coefficients are needed.  Defaults to
#' \code{FALSE}.
#' @param \dots other arguments.  Currently ignored.
#' @return A list containing the following components is returned:
#' \item{coefficients}{an array of regression coefficients for 1, \ldots{},
#' \code{ncomp} components.  The dimensions of \code{coefficients} are
#' \code{c(nvar, npred, ncomp)} with \code{nvar} the number of \code{X}
#' variables and \code{npred} the number of variables to be predicted in
#' \code{Y}.} \item{scores}{a matrix of scores.} \item{loadings}{a matrix of
#' loadings.} \item{loading.weights}{a matrix of loading weights.}
#' \item{Yscores}{a matrix of Y-scores.} \item{Yloadings}{a matrix of
#' Y-loadings.} \item{projection}{the projection matrix used to convert X to
#' scores.} \item{Xmeans}{a vector of means of the X variables.}
#' \item{Ymeans}{a vector of means of the Y variables.} \item{fitted.values}{an
#' array of fitted values.  The dimensions of \code{fitted.values} are
#' \code{c(nobj, npred, ncomp)} with \code{nobj} the number samples and
#' \code{npred} the number of Y variables.} \item{residuals}{an array of
#' regression residuals.  It has the same dimensions as \code{fitted.values}.}
#' \item{Xvar}{a vector with the amount of X-variance explained by each
#' component.} \item{Xtotvar}{Total variance in \code{X}.}
#'
#' If \code{stripped} is \code{TRUE}, only the components \code{coefficients},
#' \code{Xmeans} and \code{Ymeans} are returned.
#' @author Ron Wehrens and Bjørn-Helge Mevik
#' @seealso \code{\link{mvr}} \code{\link{plsr}} \code{\link{cppls}}
#' \code{\link{pcr}} \code{\link{widekernelpls.fit}} \code{\link{simpls.fit}}
#' \code{\link{oscorespls.fit}}
#' @references de Jong, S. and ter Braak, C. J. F. (1994) Comments on the PLS
#' kernel algorithm.  \emph{Journal of Chemometrics}, \bold{8}, 169--174.
#'
#' Dayal, B. S. and MacGregor, J. F. (1997) Improved PLS algorithms.
#' \emph{Journal of Chemometrics}, \bold{11}, 73--85.
#' @keywords regression multivariate
#' @export
kernelpls.fit <- function(X, Y, ncomp, center = TRUE,
                          stripped = FALSE, ...)
{
    Y <- as.matrix(Y)
    if (!stripped) {
        ## Save dimnames:
        dnX <- dimnames(X)
        dnY <- dimnames(Y)
    }
    ## Remove dimnames during calculation.  (Doesn't seem to make a
    ## difference here (2.3.0).)
    dimnames(X) <- dimnames(Y) <- NULL

    nobj  <- dim(X)[1]
    npred <- dim(X)[2]
    nresp <- dim(Y)[2]

    ## Center variables:
    if (center) {
        Xmeans <- colMeans(X)
        X <- X - rep(Xmeans, each = nobj)
        Ymeans <- colMeans(Y)
        Y <- Y - rep(Ymeans, each = nobj)
    } else {
        ## Set means to zero. Will ensure that predictions do not take the
        ## mean into account.
        Xmeans <- rep_len(0, npred)
        Ymeans <- rep_len(0, nresp)
    }

    ## Projection, loadings
    R <- P <- matrix(0, ncol = ncomp, nrow = npred)
    tQ <- matrix(0, ncol = nresp, nrow = ncomp)# Y loadings; transposed
    B <- array(0, c(npred, nresp, ncomp))

    if (!stripped) {
        W <- P                        # Loading weights
        U <- TT <- matrix(0, ncol = ncomp, nrow = nobj)# scores
        tsqs <- rep.int(1, ncomp)       # t't
        fitted <- array(0, c(nobj, nresp, ncomp))
    }

    ## 1.
    XtY <- crossprod(X, Y)

    for (a in 1:ncomp) {
        ## 2.
        if (nresp == 1) {
            w.a <- XtY / sqrt(c(crossprod(XtY)))
        } else {
            if (nresp < npred) {
                ## FIXME: is q proportional to q.a?
                q <- eigen(crossprod(XtY), symmetric = TRUE)$vectors[,1]
                w.a <- XtY %*% q
                w.a <- w.a / sqrt(c(crossprod(w.a)))
            } else {
                w.a <- eigen(XtY %*% t(XtY), symmetric = TRUE)$vectors[,1]
            }
        }

        ## 3.
        r.a <- w.a
        if (a > 5) {
            ## This is faster when a > 5:
            r.a <- r.a - colSums(crossprod(w.a, P[,1:(a-1), drop=FALSE]) %*%
                               t(R[,1:(a-1), drop=FALSE]))
        } else if (a > 1) {
            for (j in 1:(a - 1))
                r.a <- r.a - c(P[,j] %*% w.a) * R[,j]
        }

        ## 4.
        t.a <- X %*% r.a
        tsq <- c(crossprod(t.a))
        p.a <- crossprod(X, t.a) / tsq
        q.a <- crossprod(XtY, r.a) / tsq

        ## 5.
        XtY <- XtY - (tsq * p.a) %*% t(q.a)

        ## 6.-8.
        R[,a]  <- r.a
        P[,a]  <- p.a
        tQ[a,] <- q.a
        B[,,a] <- R[,1:a, drop=FALSE] %*% tQ[1:a,, drop=FALSE]
        if (!stripped) {
            tsqs[a] <- tsq
            ## Extra step to calculate Y scores:
            u.a <- Y %*% q.a / c(crossprod(q.a)) # Ok for nresp == 1 ??
            ## make u orth to previous X scores:
            if (a > 1) u.a <- u.a - TT %*% (crossprod(TT, u.a) / tsqs)
            U[,a]  <- u.a
            TT[,a] <- t.a
            W[,a]  <- w.a
            ## (For very tall, slim X and Y, X %*% B[,,a] is slightly faster
            ## due to less overhead.)
            fitted[,,a] <- TT[,1:a] %*% tQ[1:a,, drop=FALSE]
        }
    }

    if (stripped) {
        ## Return as quickly as possible
        list(coefficients = B, Xmeans = Xmeans, Ymeans = Ymeans)
    } else {
        residuals <- - fitted + c(Y)
        if (center) {
            fitted <- fitted + rep(Ymeans, each = nobj) # Add mean
        }

        ## Add dimnames:
        objnames <- dnX[[1]]
        if (is.null(objnames)) objnames <- dnY[[1]]
        prednames <- dnX[[2]]
        respnames <- dnY[[2]]
        compnames <- paste("Comp", 1:ncomp)
        nCompnames <- paste(1:ncomp, "comps")
        dimnames(TT) <- dimnames(U) <- list(objnames, compnames)
        dimnames(R) <- dimnames(W) <- dimnames(P) <-
            list(prednames, compnames)
        dimnames(tQ) <- list(compnames, respnames)
        dimnames(B) <- list(prednames, respnames, nCompnames)
        dimnames(fitted) <- dimnames(residuals) <-
            list(objnames, respnames, nCompnames)
        class(TT) <- class(U) <- "scores"
        class(P) <- class(W) <- class(tQ) <- "loadings"

        list(coefficients = B,
             scores = TT, loadings = P,
             loading.weights = W,
             Yscores = U, Yloadings = t(tQ),
             projection = R,
             Xmeans = Xmeans, Ymeans = Ymeans,
             fitted.values = fitted, residuals = residuals,
             Xvar = colSums(P * P) * tsqs,
             Xtotvar = sum(X * X))
    }
}

### mvr.R: plsr/pcr modelling functions
###
### The top level user function.  Implements a formula interface and calls the
### correct fit function to do the work.
### The function borrows heavily from lm().



#' @name mvr
#' @title Partial Least Squares and Principal Component Regression
#'
#' @description Functions to perform partial least squares regression (PLSR), canonical
#' powered partial least squares (CPPLS) or principal component regression
#' (PCR), with a formula interface.  Cross-validation can be used.  Prediction,
#' model extraction, plot, print and summary methods exist.
#'
#' @details The functions fit PLSR, CPPLS or PCR models with 1, \eqn{\ldots},
#' \code{ncomp} number of components.  Multi-response models are fully
#' supported.
#'
#' The type of model to fit is specified with the \code{method} argument. Four
#' PLSR algorithms are available: the kernel algorithm (\code{"kernelpls"}),
#' the wide kernel algorithm (\code{"widekernelpls"}), SIMPLS (\code{"simpls"})
#' and the classical orthogonal scores algorithm (\code{"oscorespls"}). One
#' CPPLS algorithm is available (\code{"cppls"}) providing several extensions
#' to PLS. One PCR algorithm is available: using the singular value
#' decomposition (\code{"svdpc"}).  If \code{method} is \code{"model.frame"},
#' the model frame is returned.  The functions \code{pcr}, \code{plsr} and
#' \code{cppls} are wrappers for \code{mvr}, with different values for
#' \code{method}.
#'
#' The \code{formula} argument should be a symbolic formula of the form
#' \code{response ~ terms}, where \code{response} is the name of the response
#' vector or matrix (for multi-response models) and \code{terms} is the name of
#' one or more predictor matrices, usually separated by \code{+}, e.g.,
#' \code{water ~ FTIR} or \code{y ~ X + Z}.  See \code{\link{lm}} for a
#' detailed description.  The named variables should exist in the supplied
#' \code{data} data frame or in the global environment.  Note: Do not use
#' \code{mvr(mydata$y ~ mydata$X, \ldots{})}, instead use \code{mvr(y ~ X, data
#' = mydata, \ldots{})}.  Otherwise, \code{\link{predict.mvr}} will not work
#' properly.  The chapter \samp{Statistical models in R} of the manual \samp{An
#' Introduction to R} distributed with is a good reference on formulas in .
#'
#' The number of components to fit is specified with the argument \code{ncomp}.
#' It this is not supplied, the maximal number of components is used (taking
#' account of any cross-validation).
#'
#' All implemented algorithms mean-center both predictor and response matrices.
#' This can be turned off by specifying \code{center = FALSE}.  See Seasholtz
#' and Kowalski for a discussion about centering in PLS regression.
#'
#' If \code{validation = "CV"}, cross-validation is performed.  The number and
#' type of cross-validation segments are specified with the arguments
#' \code{segments} and \code{segment.type}.  See \code{\link{mvrCv}} for
#' details.  If \code{validation = "LOO"}, leave-one-out cross-validation is
#' performed.  It is an error to specify the segments when \code{validation =
#' "LOO"} is specified.
#'
#' By default, the cross-validation will be performed serially.  However, it
#' can be done in parallel using functionality in the \code{\link{parallel}}
#' package by setting the option \code{parallel} in \code{\link{pls.options}}.
#' See \code{\link{pls.options}} for the differnt ways to specify the
#' parallelism.  See also Examples below.
#'
#' Note that the cross-validation is optimised for speed, and some generality
#' has been sacrificed.  Especially, the model matrix is calculated only once
#' for the complete cross-validation, so models like \code{y ~ msc(X)} will not
#' be properly cross-validated.  However, scaling requested by \code{scale =
#' TRUE} is properly cross-validated.  For proper cross-validation of models
#' where the model matrix must be updated/regenerated for each segment, use the
#' separate function \code{\link{crossval}}.
#'
#' @aliases mvr pcr plsr cppls
#' @param formula a model formula.  Most of the \code{\link{lm}} formula
#' constructs are supported.  See below.
#' @param ncomp the number of components to include in the model (see below).
#' @param Y.add a vector or matrix of additional responses containing relevant
#' information about the observations.  Only used for \code{cppls}.
#' @param data an optional data frame with the data to fit the model from.
#' @param subset an optional vector specifying a subset of observations to be
#' used in the fitting process.
#' @param na.action a function which indicates what should happen when the data
#' contain missing values.  The default is set by the \code{na.action} setting
#' of \code{\link{options}}, and is \code{\link{na.fail}} if that is unset.
#' The \sQuote{factory-fresh} default is \code{\link{na.omit}}.  Another
#' possible value is \code{NULL}, no action.  Value \code{\link{na.exclude}}
#' can be useful.  See \code{\link{na.omit}} for other alternatives.
#' @param method the multivariate regression method to be used.  If
#' \code{"model.frame"}, the model frame is returned.
#' @param scale numeric vector, or logical.  If numeric vector, \eqn{X} is
#' scaled by dividing each variable with the corresponding element of
#' \code{scale}.  If \code{scale} is \code{TRUE}, \eqn{X} is scaled by dividing
#' each variable by its sample standard deviation.  If cross-validation is
#' selected, scaling by the standard deviation is done for every segment.
#' @param center logical, determines if the \eqn{X} and \eqn{Y} matrices are
#' mean centered or not. Default is to perform mean centering.
#' @param validation character.  What kind of (internal) validation to use.
#' See below.
#' @param model a logical.  If \code{TRUE}, the model frame is returned.
#' @param x a logical.  If \code{TRUE}, the model matrix is returned.
#' @param y a logical.  If \code{TRUE}, the response is returned.
#' @param weights a vector of individual weights for the observations.  Only
#' used for \code{cppls}.  (Optional)
#' @param \dots additional optional arguments, passed to the underlying fit
#' functions, and \code{\link{mvrCv}}.
#'
#' Currently, the fit functions \code{\link{oscorespls.fit}} and
#' \code{\link{widekernelpls.fit}} implement these extra arguments: \describe{
#' \item{tol:}{numeric.  Tolerance used for determining convergence.}
#' \item{maxit:}{positive integer.  The maximal number of iterations used.} }
#' and \code{\link{cppls.fit}} implements: \describe{ \item{lower:}{a vector of
#' lower limits for power optimisation.} \item{upper:}{a vector of upper limits
#' for power optimisation.} \item{trunc.pow:}{logical. Whether to use an
#' experimental alternative power algorithm.} } \code{\link{mvrCv}} implements
#' several arguments; the following are probably the most useful of them:
#' \describe{ \item{segments:}{the number of segments to use, or a list with
#' segments.} \item{segment.type:}{the type of segments to use.}
#' \item{length.seg:}{Positive integer.  The length of the segments to use.}
#' \item{jackknife:}{logical.  Whether to perform jackknifing of regression
#' coefficients.} }
#'
#' See the functions' documentation for details.
#' @return If \code{method = "model.frame"}, the model frame is returned.
#' Otherwise, an object of class \code{mvr} is returned.  The object contains
#' all components returned by the underlying fit function.  In addition, it
#' contains the following components: \item{validation}{if validation was
#' requested, the results of the cross-validation.  See \code{\link{mvrCv}} for
#' details.} \item{fit.time}{the elapsed time for the fit.  This is used by
#' \code{\link{crossval}} to decide whether to turn on tracing.}
#' \item{na.action}{if observations with missing values were removed,
#' \code{na.action} contains a vector with their indices.  The class of this
#' vector is used by functions like \code{fitted} to decide how to treat the
#' observations.} \item{ncomp}{the number of components of the model.}
#' \item{method}{the method used to fit the model.  See the argument
#' \code{method} for possible values.} \item{center}{use of centering in the model}
#' \item{scale}{if scaling was requested
#' (with \code{scale}), the scaling used.} \item{call}{the function call.}
#' \item{terms}{the model terms.} \item{model}{if \code{model = TRUE}, the
#' model frame.} \item{x}{if \code{x = TRUE}, the model matrix.} \item{y}{if
#' \code{y = TRUE}, the model response.}
#' @author Ron Wehrens and Bjørn-Helge Mevik
#' @seealso \code{\link{kernelpls.fit}}, \code{\link{widekernelpls.fit}},
#' \code{\link{simpls.fit}}, \code{\link{oscorespls.fit}}, \code{\link{nipals.fit}},
#' \code{\link{cppls.fit}}, \code{\link{svdpc.fit}}, \code{\link{mvrCv}},
#' \code{\link{crossval}}, \code{\link[stats]{loadings}}, \code{\link{scores}},
#' \code{\link{loading.weights}}, \code{\link{coef.mvr}},
#' \code{\link{predict.mvr}}, \code{\link{R2}}, \code{\link{MSEP}},
#' \code{\link{RMSEP}}, \code{\link{plot.mvr}}
#' @references Martens, H., Næs, T. (1989) \emph{Multivariate calibration.}
#' Chichester: Wiley.
#'
#' Seasholtz, M. B. and Kowalski, B. R. (1992) The effect of mean centering on
#' prediction in multivariate calibration.  \emph{Journal of Chemometrics},
#' \bold{6}(2), 103--111.
#' @keywords regression multivariate
#' @examples
#'
#' data(yarn)
#' ## Default methods:
#' yarn.pcr <- pcr(density ~ NIR, 6, data = yarn, validation = "CV")
#' yarn.pls <- plsr(density ~ NIR, 6, data = yarn, validation = "CV")
#' yarn.cppls <- cppls(density ~ NIR, 6, data = yarn, validation = "CV")
#'
#' ## Alternative methods:
#' yarn.oscorespls <- mvr(density ~ NIR, 6, data = yarn, validation = "CV",
#'                       method = "oscorespls")
#' yarn.simpls <- mvr(density ~ NIR, 6, data = yarn, validation = "CV",
#'                   method = "simpls")
#' # See ?simpls.fit for example of numerical instability in SIMPLS
#'
#' \dontrun{
#' ## Parallelised cross-validation, using transient cluster:
#' pls.options(parallel = 4) # use mclapply
#' pls.options(parallel = quote(makeCluster(4, type = "PSOCK"))) # use parLapply
#' ## A new cluster is created and stopped for each cross-validation:
#' yarn.pls <- plsr(density ~ NIR, 6, data = yarn, validation = "CV")
#' yarn.pcr <- pcr(density ~ NIR, 6, data = yarn, validation = "CV")
#'
#' ## Parallelised cross-validation, using persistent cluster:
#' library(parallel)
#' ## This creates the cluster:
#' pls.options(parallel = makeCluster(4, type = "PSOCK"))
#' ## The cluster can be used several times:
#' yarn.pls <- plsr(density ~ NIR, 6, data = yarn, validation = "CV")
#' yarn.pcr <- pcr(density ~ NIR, 6, data = yarn, validation = "CV")
#' ## The cluster should be stopped manually afterwards:
#' stopCluster(pls.options()$parallel)
#'
#' ## Parallelised cross-validation, using persistent MPI cluster:
#' ## This requires the packages snow and Rmpi to be installed
#' library(parallel)
#' ## This creates the cluster:
#' pls.options(parallel = makeCluster(4, type = "MPI"))
#' ## The cluster can be used several times:
#' yarn.pls <- plsr(density ~ NIR, 6, data = yarn, validation = "CV")
#' yarn.pcr <- pcr(density ~ NIR, 6, data = yarn, validation = "CV")
#' ## The cluster should be stopped manually afterwards:
#' stopCluster(pls.options()$parallel)
#' ## It is good practice to call mpi.exit() or mpi.quit() afterwards:
#' mpi.exit()
#' }
#'
#' ## Multi-response models:
#' data(oliveoil)
#' sens.pcr <- pcr(sensory ~ chemical, ncomp = 4, scale = TRUE, data = oliveoil)
#' sens.pls <- plsr(sensory ~ chemical, ncomp = 4, scale = TRUE, data = oliveoil)
#'
#' ## Classification
#' # A classification example utilizing additional response information
#' # (Y.add) is found in the cppls.fit manual ('See also' above).
#'
#' @export
mvr <- function(formula, ncomp, Y.add, data, subset, na.action,
                method = pls.options()$mvralg, scale = FALSE,
                center = TRUE, validation = c("none", "CV", "LOO"),
                model = TRUE, x = FALSE, y = FALSE, ...)
{
  ret.x  <- isTRUE(x)             # More useful names
  ret.y  <- isTRUE(y)
  center <- isTRUE(center)        # Make sure it is a single logical

  ## Get the model frame
  mf <- match.call(expand.dots = FALSE)
  if (!missing(Y.add)) {
    ## Temporarily add Y.add to the formula
    Y.addname <- as.character(substitute(Y.add))
    mf$formula <- update(formula, paste("~ . +", Y.addname))
  }
  # Find the number of observations before any na.action is applied
  if (missing(data)) {
    nobjOrig <- eval(call("nrow", model.frame(formula)), parent.frame())
  } else {
    nobjOrig <- eval(call("nrow", data), parent.frame())
  }
  m <- match(c("formula", "data", "subset", "na.action"), names(mf), 0)
  mf <- mf[c(1, m)]                # Retain only the named arguments
  mf[[1]] <- quote(model.frame)
  # Ensure that na.action is set to NULL for nipalspls
  if(missing(na.action) && method %in% c("nipalspls", "nipalspc")){
    mf$na.action <- quote("na.pass")
    na.action <- "na.pass"
  }
  mf <- eval(mf, parent.frame())
  method <- match.arg(method, c("kernelpls", "widekernelpls", "simpls",
                                "oscorespls", "nipalspls", "cppls", "svdpc", "nipalspc", "model.frame"))
  if (method == "model.frame") return(mf)
  ## Get the terms
  mt <- attr(mf, "terms")        # This is to include the `predvars'
  # attribute of the terms
  ## Get the data matrices
  Y <- model.response(mf, "numeric")
  if (is.matrix(Y)) {
    if (is.null(colnames(Y)))
      colnames(Y) <- paste("Y", 1:dim(Y)[2], sep = "")
  } else {
    Y <- as.matrix(Y)
    colnames(Y) <- deparse(formula[[2]])
  }
  if (missing(Y.add)) {
    Y.add <- NULL
  } else {
    Y.add <- mf[,Y.addname]
    ## Remove Y.add from the formula again
    mt <- drop.terms(mt, which(attr(mt, "term.labels") == Y.addname),
                     keep.response = TRUE)
  }
  X <- delete.intercept(model.matrix(mt, mf))

  nobj  <- dim(X)[1]
  npred <- dim(X)[2]

  ## model.matrix prepends the term name to the colnames of matrices.
  ## If there is only one predictor term, and the corresponding matrix
  ## has colnames, remove the prepended term name:
  if (length(attr(mt, "term.labels")) == 1 &&
      !is.null(colnames(mf[[attr(mt, "term.labels")]])))
    colnames(X) <- sub(attr(mt, "term.labels"), "", colnames(X))

  ## Set or check the number of components:
  if (missing(ncomp)) {
    ncomp <- min(nobj - 1, npred)
    ncompWarn <- FALSE              # Don't warn about changed `ncomp'
  } else {
    if (ncomp < 1 || ncomp > min(nobj - 1, npred))
      stop("Invalid number of components, ncomp")
    ncompWarn <- TRUE
  }

  ## Handle any fixed scaling before the the validation
  sdscale <- isTRUE(scale)            # Signals scaling by sd
  if (is.numeric(scale))
    if (length(scale) == npred)
      X <- X / rep(scale, each = nobj)
  else stop("length of 'scale' must equal the number of x variables")

  ## Optionally, perform validation:
  switch(match.arg(validation),
         CV = {
           val <- mvrCv(X, Y, ncomp, Y.add = Y.add, method = method,
                        scale = sdscale, center = center, ...)
         },
         LOO = {
           segments <- as.list(1:nobj)
           attr(segments, "type") <- "leave-one-out"
           val <- mvrCv(X, Y, ncomp, Y.add = Y.add, method = method,
                        scale = sdscale, center = center,
                        segments = segments, ...)
         },
         none = {
           val <- NULL
         }
  )
  ## Check and possibly adjust ncomp:
  if (identical(TRUE, ncomp > val$ncomp)) {
    ncomp <- val$ncomp
    if (ncompWarn) warning("`ncomp' reduced to ", ncomp,
                           " due to cross-validation")
  }

  ## Select fit function:
  fitFunc <- switch(method,
                    kernelpls = kernelpls.fit,
                    widekernelpls = widekernelpls.fit,
                    simpls = simpls.fit,
                    oscorespls = oscorespls.fit,
                    nipalspls = nipals.fit,
                    cppls = cppls.fit,
                    svdpc = svdpc.fit,
                    nipalspc = nipalspc.fit)

  ## Perform any scaling by sd:
  if (sdscale) {
    if(missing(na.action) || !(na.action %in% c(NULL, "na.pass"))){
      ## This is faster than sd(X), but cannot handle missing values:
      scale <- sqrt(colSums((X - rep(colMeans(X), each = nobj))^2) /
                      (nobj - 1))
    } else {
      scale <- apply(X, 2, sd, na.rm=TRUE)
    }
    if (any(abs(scale) < .Machine$double.eps^0.5))
      warning("Scaling with (near) zero standard deviation")
    X <- X / rep(scale, each = nobj)
  }

  ## Fit the model:
  start.time <- proc.time()[3]
  z <- fitFunc(X, Y, ncomp, Y.add = Y.add, center = center, ...)
  z$fit.time <- proc.time()[3] - start.time

  ## Build and return the object:
  class(z) <- "mvr"
  z$na.action <- attr(mf, "na.action")
  z$ncomp <- ncomp
  z$method <- method
  z$center <- center
  if (is.numeric(scale)) z$scale <- scale
  z$validation <- val
  z$call <- match.call()
  z$terms <- mt
  if (isTRUE(model)) z$model <- mf
  if (ret.x) z$x <- X
  if (ret.y) z$y <- Y
  z$nobjOrig <- nobjOrig
  z$nobj <- nobj
  z
}

### runit.predict.R: test functions for predict.mvr()
### By Bjørn-Helge Mevik
### Started 2007-06-11

###
### Handling of missing values
###

## test.NAtrain: test handling of missing values in training data
test.NAtrain <- function() {
    ## NAs in train X:
    olx <- oliveoil
    olx$chemical[2,3] <- NA
    n <- nrow(olx)
    oltest <- oliveoil[1:5,]
    ntest <- nrow(oltest)
    ## parametres:
    ## - train: na.action (omit, exclude)
    ## - ncomp / comps
    ## - type: response / scores
    ## - predict: fitted (default) / newdata

    ## na.action = omit (default)
    mod <- mvr(sensory ~ chemical, data = olx)
    checkEquals(nrow(predict(mod)), n - 1)
    checkEquals(nrow(predict(mod, type = "scores")), n - 1)
    checkEquals(nrow(predict(mod, comps = 1:mod$ncomp)), n - 1)
    checkEquals(nrow(predict(mod, comps = 1:mod$ncomp, type = "scores")), n - 1)
    checkEquals(nrow(predict(mod, newdata = oltest)), ntest)
    checkEquals(nrow(predict(mod, newdata = oltest, type = "scores")), ntest)
    checkEquals(nrow(predict(mod, newdata = oltest, comps = 1:mod$ncomp)), ntest)
    checkEquals(nrow(predict(mod, newdata = oltest, comps = 1:mod$ncomp, type = "scores")), ntest)
    ## na.action = exclude
    mod <- mvr(sensory ~ chemical, data = olx, na.action = na.exclude)
    checkEquals(nrow(predict(mod)), n)
    checkEquals(nrow(predict(mod, type = "scores")), n)
    checkEquals(nrow(predict(mod, comps = 1:mod$ncomp)), n)
    checkEquals(nrow(predict(mod, comps = 1:mod$ncomp, type = "scores")), n)
    checkEquals(nrow(predict(mod, newdata = oltest)), ntest)
    checkEquals(nrow(predict(mod, newdata = oltest, type = "scores")), ntest)
    checkEquals(nrow(predict(mod, newdata = oltest, comps = 1:mod$ncomp)), ntest)
    checkEquals(nrow(predict(mod, newdata = oltest, comps = 1:mod$ncomp, type = "scores")), ntest)

    ## NAs in train Y:
    oly <- oliveoil
    oly$sensory[3,2] <- NA
    n <- nrow(oly)
    ## parametres:
    ## - train: na.action (omit, exclude)
    ## - ncomp / comps
    ## - type: response / scores

    ## na.action = omit (default)
    mod <- mvr(sensory ~ chemical, data = oly)
    checkEquals(nrow(predict(mod)), n - 1)
    checkEquals(nrow(predict(mod, type = "scores")), n - 1)
    checkEquals(nrow(predict(mod, comps = 1:mod$ncomp)), n - 1)
    checkEquals(nrow(predict(mod, comps = 1:mod$ncomp, type = "scores")), n - 1)
    checkEquals(nrow(predict(mod, newdata = oltest)), ntest)
    checkEquals(nrow(predict(mod, newdata = oltest, type = "scores")), ntest)
    checkEquals(nrow(predict(mod, newdata = oltest, comps = 1:mod$ncomp)), ntest)
    checkEquals(nrow(predict(mod, newdata = oltest, comps = 1:mod$ncomp, type = "scores")), ntest)
    ## na.action = na.exclude
    mod <- mvr(sensory ~ chemical, data = oly, na.action = na.exclude)
    checkEquals(nrow(predict(mod)), n)
    checkEquals(nrow(predict(mod, type = "scores")), n)
    checkEquals(nrow(predict(mod, comps = 1:mod$ncomp)), n)
    checkEquals(nrow(predict(mod, comps = 1:mod$ncomp, type = "scores")), n)
    checkEquals(nrow(predict(mod, newdata = oltest)), ntest)
    checkEquals(nrow(predict(mod, newdata = oltest, type = "scores")), ntest)
    checkEquals(nrow(predict(mod, newdata = oltest, comps = 1:mod$ncomp)), ntest)
    checkEquals(nrow(predict(mod, newdata = oltest, comps = 1:mod$ncomp, type = "scores")), ntest)
}

## test.NAtest: test handling of missing values in test data
test.NAtest <- function() {
    ## NAs in test X:
    olx <- oliveoil[1:5,]
    olx$chemical[2,3] <- NA
    n <- nrow(olx)
    mod <- mvr(sensory ~ chemical, data = oliveoil)
    ## parametres:
    ## - test: na.action (pass (default), omit, exclude, fail)
    ## - ncomp (default) / comps
    ## - type: response (default) / scores

    ## na.action = pass (default)
    checkEquals(nrow(predict(mod, newdata = olx)), n, "pass, 1")
    checkEquals(nrow(predict(mod, newdata = olx, type = "scores")), n,
                "pass, 2")
    checkEquals(nrow(predict(mod, newdata = olx, comps = 1:mod$ncomp)), n,
                "pass, 3")
    checkEquals(nrow(predict(mod, newdata = olx, comps = 1:mod$ncomp, type =
                             "scores")), n, "pass, 4")

    ## na.action = omit
    checkEquals(nrow(predict(mod, newdata = olx, na.action = na.omit)),
                n - 1, "omit, 1")
    checkEquals(nrow(predict(mod, newdata = olx, type = "scores", na.action
                             = na.omit)), n - 1, "omit, 2")
    checkEquals(nrow(predict(mod, newdata = olx, comps = 1:mod$ncomp,
                             na.action = na.omit)), n - 1, "omit, 3")
    checkEquals(nrow(predict(mod, newdata = olx, comps = 1:mod$ncomp, type =
                             "scores", na.action = na.omit)), n - 1, "omit, 4")

    ## na.action = exclude
    checkEquals(nrow(predict(mod, newdata = olx, na.action = na.exclude)),
                n - 1, "exclude, 1")
    checkEquals(nrow(predict(mod, newdata = olx, type = "scores", na.action
                             = na.exclude)), n - 1, "exclude, 2")
    checkEquals(nrow(predict(mod, newdata = olx, comps = 1:mod$ncomp,
                             na.action = na.exclude)), n - 1, "exclude, 3")
    checkEquals(nrow(predict(mod, newdata = olx, comps = 1:mod$ncomp, type =
                             "scores", na.action = na.exclude)), n - 1,
                "exclude, 4")

    ## na.action = fail
    checkException(predict(mod, newdata = olx, na.action = na.fail), "fail",
                   silent = TRUE)

    ## NAs in test Y:
    oly <- oliveoil[1:5,]
    oly$sensory[3,2] <- NA
    n <- nrow(oly)

    checkEquals(nrow(predict(mod, newdata = oly)), n, "missing Y")
}

### stdize.R: Standardization by centering and scaling

## This is a somewhat modified version of scale.default


#' @name stdize
#' @title Standardization of Data Matrices
#'
#' @description Performs standardization (centering and scaling) of a data matrix.
#'
#' @details \code{makepredictcall.stdized} is an internal utility function; it is not
#' meant for interactive use.  See \code{\link{makepredictcall}} for details.
#'
#' If \code{center} is \code{TRUE}, \code{x} is centered by subtracting the
#' coloumn mean from each coloumn.  If \code{center} is a numeric vector, it is
#' used in place of the coloumn means.
#'
#' If \code{scale} is \code{TRUE}, \code{x} is scaled by dividing each coloumn
#' by its sample standard deviation.  If \code{scale} is a numeric vector, it
#' is used in place of the standard deviations.
#'
#' @aliases stdize predict.stdized makepredictcall.stdized
#' @param x,newdata numeric matrices.  The data to standardize.
#' @param center logical value or numeric vector of length equal to the number
#' of coloumns of \code{x}.
#' @param scale logical value or numeric vector of length equal to the number
#' of coloumns of \code{x}.
#' @param object an object inheriting from class \code{"stdized"}, normally the
#' result of a call to \code{stdize}.
#' @param var A variable.
#' @param call The term in the formula, as a call.
#' @param \dots other arguments.  Currently ignored.
#' @return Both \code{stdize} and \code{predict.stdized} return a scaled and/or
#' centered matrix, with attributes \code{"stdized:center"} and/or
#' \code{"stdized:scale"} the vector used for centering and/or scaling.  The
#' matrix is given class \code{c("stdized", "matrix")}.
#' @note \code{stdize} is very similar to \code{\link[base]{scale}}.  The
#' difference is that when \code{scale = TRUE}, \code{stdize} divides the
#' coloumns by their standard deviation, while \code{scale} uses the
#' root-mean-square of the coloumns.  If \code{center} is \code{TRUE}, this is
#' equivalent, but in general it is not.
#' @author Bjørn-Helge Mevik and Ron Wehrens
#' @seealso \code{\link{mvr}}, \code{\link{pcr}}, \code{\link{plsr}},
#' \code{\link{msc}}, \code{\link[base]{scale}}
#' @keywords regression multivariate
#' @examples
#'
#' data(yarn)
#' ## Direct standardization:
#' Ztrain <- stdize(yarn$NIR[yarn$train,])
#' Ztest <- predict(Ztrain, yarn$NIR[!yarn$train,])
#'
#' ## Used in formula:
#' mod <- plsr(density ~ stdize(NIR), ncomp = 6, data = yarn[yarn$train,])
#' pred <- predict(mod, newdata = yarn[!yarn$train,]) # Automatically standardized
#'
#' @export
stdize <- function(x, center = TRUE, scale = TRUE) {
    nc <- ncol(x)
    if (is.logical(center)) {
        if (isTRUE(center)) {
            center <- colMeans(x, na.rm = TRUE)
            x <- sweep(x, 2, center)
        }
    } else if (is.numeric(center) && length(center) == nc)
        x <- sweep(x, 2, center)
    else stop("invalid 'center'")
    if (is.logical(scale)) {
        if (isTRUE(scale)) {
            ## This is faster than sd(x), but cannot handle missing values:
            scale <- sqrt(colSums(sweep(x, 2, colMeans(x))^2) / (nrow(x) - 1))
            x <- sweep(x, 2, scale, "/")
        }
    } else if (is.numeric(scale) && length(scale) == nc)
        x <- sweep(x, 2, scale, "/")
    else stop("invalid 'scale'")
    if (is.numeric(center)) attr(x, "stdized:center") <- center
    if (is.numeric(scale))  attr(x, "stdized:scale")  <- scale
    class(x) <- c("stdized", "matrix")
    return(x)
}

## This is not really needed for `stdize' to work with formulas, but might
## be nice to have for manually manipulating data:
#' @rdname stdize
#' @export
predict.stdized <- function(object, newdata, ...) {
    if (missing(newdata)) return(object)
    if (is.null(center <- attr(object, "stdized:center")))
        center <- FALSE
    if (is.null(scale <- attr(object, "stdized:scale")))
        scale <- FALSE
    stdize(newdata, center = center, scale = scale)
}

## This method makes things like
## `predict(plsr(y ~ stdize(X), data = foo), newdata = bar)' work.
## This is a slightly modified version of makepredictcall.default.
#' @rdname stdize
#' @export
makepredictcall.stdized <- function(var, call) {
    if (as.character(call)[1] != "stdize")
        return(call)
    if (!is.null(z <- attr(var, "stdized:center")))
        call$center <- z
    if (!is.null(z <- attr(var, "stdized:scale")))
        call$scale <- z
    call
}

### simpls.fit.R: SimPLS fit algorithm.
###
### Implements an adapted version of the SIMPLS algorithm described in
###   de Jong, S. (1993) SIMPLS: an alternative approach to partial least
###   squares regression.  \emph{Chemometrics and Intelligent Laboratory
###   Systems}, \bold{18}, 251--263.



#' @title Sijmen de Jong's SIMPLS
#'
#' @description Fits a PLSR model with the SIMPLS algorithm.
#'
#' @details This function should not be called directly, but through the generic
#' functions \code{plsr} or \code{mvr} with the argument
#' \code{method="simpls"}.  SIMPLS is much faster than the NIPALS algorithm,
#' especially when the number of X variables increases, but gives slightly
#' different results in the case of multivariate Y.  SIMPLS truly maximises the
#' covariance criterion.  According to de Jong, the standard PLS2 algorithms
#' lie closer to ordinary least-squares regression where a precise fit is
#' sought; SIMPLS lies closer to PCR with stable predictions.
#'
#' @param X a matrix of observations.  \code{NA}s and \code{Inf}s are not
#' allowed.
#' @param Y a vector or matrix of responses.  \code{NA}s and \code{Inf}s are
#' not allowed.
#' @param ncomp the number of components to be used in the modelling.
#' @param center logical, determines if the \eqn{X} and \eqn{Y} matrices are
#' mean centered or not. Default is to perform mean centering.
#' @param orthScores logical.  If \code{TRUE} the scores are orthogonalised for
#' increased numerical precision (default = FALSE, for speed).
#' @param stripped logical.  If \code{TRUE} the calculations are stripped as
#' much as possible for speed; this is meant for use with cross-validation or
#' simulations when only the coefficients are needed.  Defaults to
#' \code{FALSE}.
#' @param \dots other arguments.  Currently ignored.
#' @return A list containing the following components is returned:
#' \item{coefficients}{an array of regression coefficients for 1, \ldots{},
#' \code{ncomp} components.  The dimensions of \code{coefficients} are
#' \code{c(nvar, npred, ncomp)} with \code{nvar} the number of \code{X}
#' variables and \code{npred} the number of variables to be predicted in
#' \code{Y}.} \item{scores}{a matrix of scores.} \item{loadings}{a matrix of
#' loadings.} \item{Yscores}{a matrix of Y-scores.} \item{Yloadings}{a matrix
#' of Y-loadings.} \item{projection}{the projection matrix used to convert X to
#' scores.} \item{Xmeans}{a vector of means of the X variables.}
#' \item{Ymeans}{a vector of means of the Y variables.} \item{fitted.values}{an
#' array of fitted values.  The dimensions of \code{fitted.values} are
#' \code{c(nobj, npred, ncomp)} with \code{nobj} the number samples and
#' \code{npred} the number of Y variables.} \item{residuals}{an array of
#' regression residuals.  It has the same dimensions as \code{fitted.values}.}
#' \item{Xvar}{a vector with the amount of X-variance explained by each
#' component.} \item{Xtotvar}{Total variance in \code{X}.}
#'
#' If \code{stripped} is \code{TRUE}, only the components \code{coefficients},
#' \code{Xmeans} and \code{Ymeans} are returned.
#' @author Ron Wehrens and Bjørn-Helge Mevik
#' @seealso \code{\link{mvr}} \code{\link{plsr}} \code{\link{pcr}}
#' \code{\link{kernelpls.fit}} \code{\link{widekernelpls.fit}}
#' \code{\link{oscorespls.fit}}
#' @references de Jong, S. (1993) SIMPLS: an alternative approach to partial
#' least squares regression.  \emph{Chemometrics and Intelligent Laboratory
#' Systems}, \bold{18}, 251--263.
#' @keywords regression multivariate
#' @examples
#' ## Simulation of SIMPLS stability
#' # The graphics produced, demonstrate the numeric instability of the original
#' # SIMPLS without score orthogonalization.
#' set.seed(42)
#' N <- 100
#' p <- 2000
#' ncomp <- 40
#' simData <- data.frame(X = I(matrix(rnorm(N*p),N)), y = rnorm(N))
#' pls    <- plsr(y ~ X, data=simData, ncomp=ncomp)
#' simps  <- plsr(y ~ X, data=simData, ncomp=ncomp, method="simpls")
#' simpsO <- plsr(y ~ X, data=simData, ncomp=ncomp, method="simpls", orthScores=TRUE)
#' normScores <- pls$scores
#' for(i in 1:ncomp)
#'   normScores[,i] <- normScores[,i]/sqrt(sum(normScores[,i]^2))
#'
#' # Check number of equal digits
#' eqDig <- function(x,y){
#'   xy <- abs(x - y)
#'   xy[xy == 0] <- 10^-16
#'   -colMeans(log10(xy))
#' }
#' eqDig_PLS_oSIMPLS    <- eqDig(normScores, simpsO$scores)
#' eqDig_SIMPLS_PLS     <- eqDig(simps$scores, normScores)
#' eqDig_SIMPLS_oSIMPLS <- eqDig(simps$scores, simpsO$scores)
#' # Correlation between models
#' cor_PLS_oSIMPLS    <- diag(cor(pls$scores,simps$scores))
#' cor_SIMPLS_oSIMPLS <- diag(cor(pls$scores,simpsO$scores))
#' cor_SIMPLS_PLS     <- diag(cor(simps$scores,simpsO$scores))
#'
#' par.old <- par(mfrow=c(2,1), mar=c(4,4,1,1), las=1)
#' matplot(2:ncomp,cbind(eqDig_PLS_oSIMPLS,eqDig_SIMPLS_PLS, eqDig_SIMPLS_oSIMPLS)[-1,],
#'         type="l", xlab="component", ylab="equal digits", ylim=c(0,17),
#'         panel.first=grid())
#' legend("bottomleft", legend=c("PLS, SIMPLS", "PLS, OrthSIMPLS", "SIMPLS, OrthSIMPLS"),
#'        col=1:3, lty=1:3)
#' matplot(1:ncomp,cbind(cor_PLS_oSIMPLS,cor_SIMPLS_oSIMPLS,cor_SIMPLS_PLS), type="l",
#'         ylab="correlation", xlab="component", panel.first=grid())
#' legend("bottomleft", legend=c("PLS, SIMPLS", "PLS, OrthSIMPLS", "SIMPLS, OrthSIMPLS"),
#'        col=1:3, lty=1:3)
#' par(par.old)
#' @export
simpls.fit <- function(X, Y, ncomp, center = TRUE, orthScores = FALSE, stripped = FALSE, ...) {
    Y <- as.matrix(Y)
    if (!stripped) {
        ## Save dimnames:
        dnX <- dimnames(X)
        dnY <- dimnames(Y)
    }
    ## Remove dimnames during calculation (doesn't seem to matter; in fact,
    ## as far as it has any effect, it hurts a tiny bit in most situations).
    dimnames(X) <- dimnames(Y) <- NULL

    nobj  <- dim(X)[1] # n in paper
    npred <- dim(X)[2] # p in paper
    nresp <- dim(Y)[2]

    V <- R <- matrix(0, nrow = npred, ncol = ncomp)
    tQ <- matrix(0, nrow = ncomp, ncol = nresp) # Y loadings; transposed
    B <- array(0, dim = c(npred, nresp, ncomp))
    if (!stripped) {
        P <- R
        U <- TT <- matrix(0, nrow = nobj, ncol = ncomp)
        fitted <- array(0, dim = c(nobj, nresp, ncomp))
    }

    ## Center variables:
    if (center) {
        Xmeans <- colMeans(X)
        X <- X - rep(Xmeans, each = nobj) # This is not strictly neccessary
                                        # (but might be good for accuracy?)!
        Ymeans <- colMeans(Y)
        Y <- Y - rep(Ymeans, each = nobj)
    } else {
        ## Set means to zero. Will ensure that predictions do not take the
        ## mean into account.
        Xmeans <- rep_len(0, npred)
        Ymeans <- rep_len(0, nresp)
    }

    S <- crossprod(X, Y)

    for (a in 1:ncomp) {
        ## A more efficient way of calculating the Y block factor weights
        ## q.a <- svd(S)$v[,1]:
        if (nresp == 1) {
            q.a <- 1
        } else {
            if (nresp < npred) {
                q.a <- eigen(crossprod(S), symmetric = TRUE)$vectors[,1]
            } else {
                q.a <- c(crossprod(S, eigen(S %*% t(S),
                                            symmetric = TRUE)$vectors[,1]))
                q.a <- q.a / sqrt(c(crossprod(q.a)))
            }
        }
        r.a <- S %*% q.a                 # X block factor weights
        t.a <- X %*% r.a
        if (center) {
            t.a <- t.a - mean(t.a)       # center scores
        }
        if(a > 1 && orthScores && !stripped) {
            t.a <- t.a - TT %*% crossprod(TT, t.a) # t.a orth to previous t.a values
        }
        tnorm <- sqrt(c(crossprod(t.a)))
        t.a <- t.a / tnorm               # normalize scores
        r.a <- r.a / tnorm               # adapt weights accordingly
        p.a <- crossprod(X, t.a)         # X block factor loadings
        q.a <- crossprod(Y, t.a)         # Y block factor loadings
        v.a <- p.a			 # init orthogonal loadings
        if (a > 1) {
            v.a <- v.a - V %*% crossprod(V, p.a) # v.a orth to previous loadings
        }
        v.a <- v.a / sqrt(c(crossprod(v.a))) # normalize orthogonal loadings
        S <- S - v.a %*% crossprod(v.a, S) # deflate S

        R[,a]  <- r.a
        tQ[a,] <- q.a
        V[,a]  <- v.a

        B[,,a] <- R[,1:a, drop=FALSE] %*% tQ[1:a,, drop=FALSE]

        if (!stripped) {
            u.a <- Y %*% q.a # Y block factor scores
            if (a > 1)
                u.a <- u.a - TT %*% crossprod(TT, u.a) # u.a orth to previous t.a values
            P[,a]  <- p.a
            TT[,a] <- t.a
            U[,a]  <- u.a
            ## (For very tall, slim X and Y, X %*% B[,,a] is slightly faster,
            ## due to less overhead.)
            fitted[,,a] <- TT[,1:a] %*% tQ[1:a,, drop=FALSE]
        }
    }

    if (stripped) {
        ## Return as quickly as possible
        list(coefficients = B, Xmeans = Xmeans, Ymeans = Ymeans)
    } else {
        residuals <- - fitted + c(Y)
        fitted <- fitted + rep(Ymeans, each = nobj) # Add mean

        ## Add dimnames and classes:
        objnames <- dnX[[1]]
        if (is.null(objnames)) objnames <- dnY[[1]]
        prednames <- dnX[[2]]
        respnames <- dnY[[2]]
        compnames <- paste("Comp", 1:ncomp)
        nCompnames <- paste(1:ncomp, "comps")
        dimnames(TT) <- dimnames(U) <- list(objnames, compnames)
        dimnames(R) <- dimnames(P) <- list(prednames, compnames)
        dimnames(tQ) <- list(compnames, respnames)
        dimnames(B) <- list(prednames, respnames, nCompnames)
        dimnames(fitted) <- dimnames(residuals) <-
            list(objnames, respnames, nCompnames)
        class(TT) <- class(U) <- "scores"
        class(P) <- class(tQ) <- "loadings"

        list(coefficients = B,
             scores = TT, loadings = P,
             Yscores = U, Yloadings = t(tQ),
             projection = R,
             Xmeans = Xmeans, Ymeans = Ymeans,
             fitted.values = fitted, residuals = residuals,
             Xvar = colSums(P * P), Xtotvar = sum(X * X))
    }
}

### widekernelpls.fit.R: Kernel PLS fit algorithm for wide data.
###
### Implements an adapted version of the algorithm described in
###  Rannar, S., Lindgren, F., Geladi, P. and Wold, S. (1994) A PLS
###  Kernel Algorithm for Data Sets with Many Variables and Fewer
###  Objects.  Part 1: Theory and Algorithm.
###  \emph{Journal of Chemometrics}, \bold{8}, 111--125.



#' @title Wide Kernel PLS (Rännar et al.)
#'
#' @description Fits a PLSR model with the wide kernel algorithm.
#'
#' @details This function should not be called directly, but through the generic
#' functions \code{plsr} or \code{mvr} with the argument
#' \code{method="widekernelpls"}.  The wide kernel PLS algorithm is efficient
#' when the number of variables is (much) larger than the number of
#' observations.  For very wide \code{X}, for instance 12x18000, it can be
#' twice as fast as \code{\link{kernelpls.fit}} and \code{\link{simpls.fit}}.
#' For other matrices, however, it can be much slower.  The results are equal
#' to the results of the NIPALS algorithm.
#'
#' @param X a matrix of observations.  \code{NA}s and \code{Inf}s are not
#' allowed.
#' @param Y a vector or matrix of responses.  \code{NA}s and \code{Inf}s are
#' not allowed.
#' @param ncomp the number of components to be used in the modelling.
#' @param center logical, determines if the \eqn{X} and \eqn{Y} matrices are
#' mean centered or not. Default is to perform mean centering.
#' @param stripped logical.  If \code{TRUE} the calculations are stripped as
#' much as possible for speed; this is meant for use with cross-validation or
#' simulations when only the coefficients are needed.  Defaults to
#' \code{FALSE}.
#' @param tol numeric.  The tolerance used for determining convergence in the
#' algorithm.
#' @param maxit positive integer.  The maximal number of iterations used in the
#' internal Eigenvector calculation.
#' @param \dots other arguments.  Currently ignored.
#' @return A list containing the following components is returned:
#' \item{coefficients}{an array of regression coefficients for 1, \ldots{},
#' \code{ncomp} components.  The dimensions of \code{coefficients} are
#' \code{c(nvar, npred, ncomp)} with \code{nvar} the number of \code{X}
#' variables and \code{npred} the number of variables to be predicted in
#' \code{Y}.} \item{scores}{a matrix of scores.} \item{loadings}{a matrix of
#' loadings.} \item{loading.weights}{a matrix of loading weights.}
#' \item{Yscores}{a matrix of Y-scores.} \item{Yloadings}{a matrix of
#' Y-loadings.} \item{projection}{the projection matrix used to convert X to
#' scores.} \item{Xmeans}{a vector of means of the X variables.}
#' \item{Ymeans}{a vector of means of the Y variables.} \item{fitted.values}{an
#' array of fitted values.  The dimensions of \code{fitted.values} are
#' \code{c(nobj, npred, ncomp)} with \code{nobj} the number samples and
#' \code{npred} the number of Y variables.} \item{residuals}{an array of
#' regression residuals.  It has the same dimensions as \code{fitted.values}.}
#' \item{Xvar}{a vector with the amount of X-variance explained by each
#' component.} \item{Xtotvar}{Total variance in \code{X}.}
#'
#' If \code{stripped} is \code{TRUE}, only the components \code{coefficients},
#' \code{Xmeans} and \code{Ymeans} are returned.
#' @note The current implementation has not undergone extensive testing yet,
#' and should perhaps be regarded as experimental.  Specifically, the internal
#' Eigenvector calculation does not always converge in extreme cases where the
#' Eigenvalue is close to zero.  However, when it does converge, it always
#' converges to the same results as \code{\link{kernelpls.fit}}, up to
#' numerical inacurracies.
#'
#' The algorithm also has a bit of overhead, so when the number of observations
#' is moderately high, \code{\link{kernelpls.fit}} can be faster even if the
#' number of predictors is much higher.  The relative speed of the algorithms
#' can also depend greatly on which BLAS and/or LAPACK library is linked
#' against.
#' @author Bjørn-Helge Mevik
#' @seealso \code{\link{mvr}} \code{\link{plsr}} \code{\link{cppls}}
#' \code{\link{pcr}} \code{\link{kernelpls.fit}} \code{\link{simpls.fit}}
#' \code{\link{oscorespls.fit}}
#' @references Rännar, S., Lindgren, F., Geladi, P. and Wold, S. (1994) A PLS
#' Kernel Algorithm for Data Sets with Many Variables and Fewer Objects.  Part
#' 1: Theory and Algorithm.  \emph{Journal of Chemometrics}, \bold{8},
#' 111--125.
#' @keywords regression multivariate
#' @export
widekernelpls.fit <- function(X, Y, ncomp, center = TRUE, stripped = FALSE,
                              tol = .Machine$double.eps^0.5,
                              maxit = 100, ...)
{
    ## Initialise
    Y <- as.matrix(Y)
    if(!stripped) {
        ## Save dimnames:
        dnX <- dimnames(X)
        dnY <- dimnames(Y)
    }
    ## Remove dimnames during calculation.
    dimnames(X) <- dimnames(Y) <- NULL

    nobj  <- dim(X)[1]
    npred <- dim(X)[2]
    nresp <- dim(Y)[2]

    TT <- U <- matrix(0, ncol = ncomp, nrow = nobj)# scores
    B <- array(0, c(npred, nresp, ncomp))
    In <- diag(nobj)
    nits <- numeric(ncomp)              # for debugging
    if (!stripped) {
        fitted <- array(0, dim = c(nobj, nresp, ncomp))
        Xresvar <- numeric(ncomp)
    }

    ## Center variables:
    if (center) {
        Xmeans <- colMeans(X)
        X <- X - rep(Xmeans, each = nobj)
        Ymeans <- colMeans(Y)
        Y <- Y - rep(Ymeans, each = nobj)
    } else {
        ## Set means to zero. Will ensure that predictions do not take the
        ## mean into account.
        Xmeans <- rep_len(0, npred)
        Ymeans <- rep_len(0, nresp)
    }

    XXt <- tcrossprod(X)
    YYt <- tcrossprod(Y)

    if (!stripped) Xtotvar <- sum(diag(XXt))

    for (a in 1:ncomp) {
        XXtYYt <- XXt %*% YYt
        ## This avoids problems with negative eigenvalues due to roundoff
        ## errors in zero rank cases, and can potentionally give slightly
        ## faster and/or more accurate results:
        XXtYYt <- XXtYYt %*% XXtYYt

        ## Initial values:
        t.a.old <- Y[,1]
        nit <- 0                        # for debugging
        repeat {
            nit <- nit + 1              # for debugging
            t.a <- XXtYYt %*% t.a.old
            t.a <- t.a / sqrt(c(crossprod(t.a)))
            if (sum(abs((t.a - t.a.old) / t.a), na.rm = TRUE) < tol)
                break
            else
                t.a.old <- t.a
            if (nit >= maxit) {         # for debugging
              warning("No convergence in ", maxit, " iterations\n")
              break
            }
        }
        nits[a] <- nit                  # for debugging

        u.a <- YYt %*% t.a
        utmp <- u.a / c(crossprod(t.a, u.a))
        wpw <- sqrt(c(crossprod(utmp, XXt) %*% utmp))
        TT[,a] <- t.a * wpw
        U[,a]  <- utmp * wpw

        G <- In - tcrossprod(t.a)
        XXt <- G %*% XXt %*% G
        YYt <- G %*% YYt %*% G

        if (!stripped) Xresvar[a] <- sum(diag(XXt))
    }

    W <- crossprod(X, U)
    W <- W / rep(sqrt(colSums(W * W)), each = npred)

    TTtTinv <- TT %*% diag(1 / colSums(TT * TT), ncol = ncol(TT))
    P <- crossprod(X, TTtTinv)
    Q <- crossprod(Y, TTtTinv)

    ## Calculate rotation matrix:
    if (ncomp == 1) {
        ## For 1 component, R == W:
        R <- W
    } else {
        PW <- crossprod(P, W)
        ## It is known that P^tW is right bi-diagonal (one response) or upper
        ## triangular (multiple responses), with all diagonal elements equal to 1.
        if (nresp == 1) {
            ## For single-response models, direct calculation of (P^tW)^-1 is
            ## simple, and faster than using backsolve.
            PWinv <- diag(ncomp)
            bidiag <- - PW[row(PW) == col(PW)-1]
            for (a in 1:(ncomp - 1))
                PWinv[a,(a+1):ncomp] <- cumprod(bidiag[a:(ncomp-1)])
        } else {
            PWinv <- backsolve(PW, diag(ncomp))
        }
        R <- W %*% PWinv
    }

    ## Calculate regression coefficients:
    for (a in 1:ncomp) {
        B[,,a] <- tcrossprod(R[,1:a, drop=FALSE], Q[,1:a, drop=FALSE])
    }

    if (stripped) {
        ## Return as quickly as possible
        list(coefficients = B, Xmeans = Xmeans, Ymeans = Ymeans)
    } else {
        ## Fitted values, residuals etc:
        for (a in 1:ncomp)
            fitted[,,a] <- tcrossprod(TT[,1:a, drop=FALSE], Q[,1:a, drop=FALSE])
        residuals <- - fitted + c(Y)
        fitted <- fitted + rep(Ymeans, each = nobj) # Add mean
        Xvar <- diff(-c(Xtotvar, Xresvar))

        ## Add dimnames:
        objnames <- dnX[[1]]
        if (is.null(objnames)) objnames <- dnY[[1]]
        prednames <- dnX[[2]]
        respnames <- dnY[[2]]
        compnames <- paste("Comp", 1:ncomp)
        nCompnames <- paste(1:ncomp, "comps")
        dimnames(TT) <- dimnames(U) <- list(objnames, compnames)
        dimnames(R) <- dimnames(W) <- dimnames(P) <-
            list(prednames, compnames)
        dimnames(Q) <- list(respnames, compnames)
        dimnames(B) <- list(prednames, respnames, nCompnames)
        dimnames(fitted) <- dimnames(residuals) <-
            list(objnames, respnames, nCompnames)
        names(Xvar) <- compnames
        class(TT) <- class(U) <- "scores"
        class(P) <- class(W) <- class(Q) <- "loadings"

        list(coefficients = B,
             scores = TT, loadings = P,
             loading.weights = W,
             Yscores = U, Yloadings = Q,
             projection = R,
             Xmeans = Xmeans, Ymeans = Ymeans,
             fitted.values = fitted, residuals = residuals,
             Xvar = Xvar, Xtotvar = Xtotvar,
             nits = nits)               # for debugging
    }
}

### pls.options.R:  Package specific options mechanism.
###
### Implements a slightly modified version of the sm.options() as found in
### sm 2.1-0.  The difference is that the option list is stored in an
### environment '.pls.data'.

## The list of initial options:
.pls.data <- new.env(parent = emptyenv())
.pls.data$options <-
    list(mvralg = "kernelpls", plsralg = "kernelpls", cpplsalg = "cppls",
         pcralg = "svdpc", parallel = NULL,
         w.tol = .Machine$double.eps, X.tol = 10^-12)




#' @title Set or return options for the pls package
#'
#' @description A function to set options for the \pkg{pls} package, or to return the
#' current options.
#'
#' @details If called with no arguments, or with an empty list as the single argument,
#' \code{pls.options} returns the current options.
#'
#' If called with a character vector as the single argument, a list with the
#' arguments named in the vector are returned.
#'
#' If called with a non-empty list as the single argument, the list elements
#' should be named, and are treated as named arguments to the function.
#'
#' Otherwise, \code{pls.options} should be called with one or more named
#' arguments \var{name = value}.  For each argument, the option named
#' \var{name} will be given the value \var{value}.
#'
#' The recognised options are: \describe{ \item{mvralg}{The fit method to use
#' in \code{\link{mvr}} and \code{\link{mvrCv}}.  The value should be one of
#' the allowed methods.  Defaults to \code{"kernelpls"}.  Can be overridden
#' with the argument \code{method} in \code{mvr} and \code{mvrCv}.}
#' \item{pcralg}{The fit method to use in \code{\link{pcr}}.  The value should
#' be one of the allowed methods.  Defaults to \code{"svdpc"}.  Can be
#' overridden with the argument \code{method} in \code{pcr}.}
#' \item{plsralg}{The fit method to use in \code{\link{plsr}}.  The value
#' should be one of the allowed methods.  Defaults to \code{"kernelpls"}.  Can
#' be overridden with the argument \code{method} in \code{plsr}.}
#' \item{cpplsalg}{The fit method to use in \code{\link{cppls}}.  The value
#' should be one of the allowed methods.  Defaults to \code{"cppls"}.  Can be
#' overridden with the argument \code{method} in \code{cppls}.}
#' \item{parallel}{Specification of how the cross-validation (CV) in
#' \code{\link{mvr}} should be performed.  If the specification is \code{NULL}
#' (default) or \code{1}, the CV is done serially, otherwise it is done in
#' parallel using functionality from the \code{\link{parallel}} package.
#'
#' If it is an integer greater than 1, the CV is done in parallel with the
#' specified number of processes, using \code{\link{mclapply}}.
#'
#' If it is a cluster object created by \code{\link{makeCluster}}, the CV is
#' done in parallel on that cluster, using \code{\link{parLapply}}.  The user
#' should stop the cluster herself when it is no longer needed, using
#' \code{\link{stopCluster}}.
#'
#' Finally, if the specification is an unevaluated call to
#' \code{\link{makeCluster}}, the call is evaluated, and the CV is done in
#' parallel on the resulting cluster, using \code{\link{parLapply}}.  In this
#' case, the cluster will be stopped (with \code{\link{stopCluster}}) after the
#' CV.  Thus, in the final case, the cluster is created and destroyed for each
#' CV, just like when using \code{\link{mclapply}}.} \item{w.tol}{The tolerance
#' used for removing values close to 0 in the vectors of loading weights in
#' \code{\link{cppls}}.  Defaults to .Machine$double.eps.} \item{X.tol}{The
#' tolerance used for removing predictor variables with L1 norms close to 0 in
#' \code{\link{cppls}}.  Defaults to 10^-12.} }
#'
#' @param \dots a single list, a single character vector, or any number of
#' named arguments (\var{name = value}).
#' @return A list with the (possibly changed) options.  If any named argument
#' (or list element) was provided, the list is returned invisibly.
#' @note The function is a slight modification of the function
#' \code{\link[sm]{sm.options}} from the package \pkg{sm}.
#' @author Bjørn-Helge Mevik and Ron Wehrens
#' @keywords regression multivariate
#' @examples
#'
#' ## Return current options:
#' pls.options()
#' pls.options("plsralg")
#' pls.options(c("plsralg", "pcralg"))
#'
#' ## Set options:
#' pls.options(plsralg = "simpls", mvralg = "simpls")
#' pls.options(list(plsralg = "simpls", mvralg = "simpls")) # Equivalent
#' pls.options()
#'
#' ## Restore `factory settings':
#' pls.options(list(mvralg = "kernelpls", plsralg = "kernelpls", cpplsalg = "cppls",
#'                  pcralg = "svdpc", parallel = NULL,
#'                  w.tol = .Machine$double.eps, X.tol = 10^-12))
#' pls.options()
#'
#' @export
pls.options <- function(...) {
    if (nargs() == 0) return(.pls.data$options)
    current <- .pls.data$options
    temp <- list(...)
    if (length(temp) == 1 && is.null(names(temp))) {
        arg <- temp[[1]]
        switch(mode(arg),
               list = temp <- arg,
               character = return(.pls.data$options[arg]),
               stop("invalid argument: ", sQuote(arg)))
    }
    if (length(temp) == 0) return(current)
    n <- names(temp)
    if (is.null(n)) stop("options must be given by name")
    changed <- current[n]
    current[n] <- temp
    .pls.data$options <- current
    invisible(current)
}

### runit.options.R: test the pls.options() function
### By Bjørn-Helge Mevik
### Started 2013-07-30

## test.options: test calling pls::pls.options directly
test.options <- function() {
    ## Save original options for later restore:
    old_opts <- pls::pls.options()
    checkTrue(is.list(old_opts),
              "pls::pls.options() doesn't return a list")
    
    pls::pls.options(mvralg = "val1")
    checkEquals(pls::pls.options()$mvralg, "val1",
                "pls::pls.options(mvralg = 'val1') didn't set 'mvralg'")

    ## Restore the original options:
    pls::pls.options(old_opts)
}

### runit.options.R: tests for pls.options()
### By Bjørn-Helge Mevik
### Started 2013-07-30

test.options <- function() {
    ## Check results when called with allowed arguments:

    ## none
    ## Save original options for later restore:
    old_opts <- pls.options()
    checkTrue(is.list(old_opts),
              "pls.options() doesn't return a list")

    ## a single list
    checkTrue(is.list(pls.options(list())),
              "pls.options(list()) doesn't return a list")
    pls.options(list(mvralg = "val1"))
    checkEquals(pls.options()$mvralg, "val1",
                "pls.options(list(mvralg = 'val1')) didn't set 'mvralg'")
    pls.options(list(mvralg = "val2", plsralg = "val3"))
    checkEquals(pls.options()$mvralg, "val2",
        "pls.options(list(mvralg = 'val2', plsralg = 'val3')) didn't set 'mvralg'")
    checkEquals(pls.options()$plsralg, "val3",
        "pls.options(list(mvralg = 'val2', plsralg = 'val3')) didn't set 'plsralg'")

    ## a single character vector
    res <- pls.options("mvralg")
    checkTrue(is.list(res), "pls.options('mvralg') doesn't return a list")
    checkEquals(names(res), "mvralg",
                "pls.options('mvralg') doesn't return the correct option")
    res <- pls.options(c("mvralg", "plsralg"))
    checkTrue(is.list(res),
              "pls.options(c('mvralg', 'plsralg') doesn't return a list")
    checkEquals(names(res), c("mvralg", "plsralg"),
        "pls.options(c('mvralg', 'plsralg') doesn't return the correct options")

    ## any number of named arguments
    pls.options(mvralg = "val4")
    checkEquals(pls.options()$mvralg, "val4",
                "pls.options(mvralg = 'val4') didn't set 'mvralg'")
    pls.options(mvralg = "val5", plsralg = "val6")
    checkEquals(pls.options()$mvralg, "val5",
        "pls.options(mvralg = 'val5', plsralg = 'val6') didn't set 'mvralg'")
    checkEquals(pls.options()$plsralg, "val6",
        "pls.options(mvralg = 'val5', plsralg = 'val6')) didn't set 'plsralg'")

    ## Check handling invalid arguments
    checkException(pls.options(list("mvralg")),
        "pls.options(list('mvralg')) didn't throw an error")
    checkException(pls.options(list("mvralg", "plsralg")),
        "pls.options(list('mvralg', 'plsralg')) didn't throw an error")

    ## Restore the original options:
    pls.options(old_opts)
}

### extract.R:  Extraction functions

## coef.mvr: Extract the base variable regression coefficients from
## an mvr object.


#' @name coef.mvr
#' @title Extract Information From a Fitted PLSR or PCR Model
#'
#' @description Functions to extract information from \code{mvr} objects: Regression
#' coefficients, fitted values, residuals, the model frame, the model matrix,
#' names of the variables and components, and the \eqn{X} variance explained by
#' the components.
#'
#' @details These functions are mostly used inside other functions.  (Functions
#' \code{coef.mvr}, \code{fitted.mvr} and \code{residuals.mvr} are usually
#' called through their generic functions \code{\link{coef}},
#' \code{\link{fitted}} and \code{\link{residuals}}, respectively.)
#'
#' \code{coef.mvr} is used to extract the regression coefficients of a model,
#' i.e. the \eqn{B} in \eqn{y = XB} (for the \eqn{Q} in \eqn{y = TQ} where
#' \eqn{T} is the scores, see \code{\link{Yloadings}}).  An array of dimension
#' \code{c(nxvar, nyvar, length(ncomp))} or \code{c(nxvar, nyvar,
#' length(comps))} is returned.
#'
#' If \code{comps} is missing (or is \code{NULL}), \code{coef()[,,ncomp[i]]}
#' are the coefficients for models with \code{ncomp[i]} components, for \eqn{i
#' = 1, \ldots, length(ncomp)}.  Also, if \code{intercept = TRUE}, the first
#' dimension is \eqn{nxvar + 1}, with the intercept coefficients as the first
#' row.
#'
#' If \code{comps} is given, however, \code{coef()[,,comps[i]]} are the
#' coefficients for a model with only the component \code{comps[i]}, i.e. the
#' contribution of the component \code{comps[i]} on the regression
#' coefficients.
#'
#' \code{fitted.mvr} and \code{residuals.mvr} return the fitted values and
#' residuals, respectively.  If the model was fitted with \code{na.action =
#' na.exclude} (or after setting the default \code{na.action} to
#' \code{"na.exclude"} with \code{\link{options}}), the fitted values (or
#' residuals) corresponding to excluded observations are returned as \code{NA};
#' otherwise, they are omitted.
#'
#' \code{model.frame.mvr} returns the model frame; i.e. a data frame with all
#' variables neccessary to generate the model matrix.  See
#' \code{\link[stats]{model.frame}} for details.
#'
#' \code{model.matrix.mvr} returns the (possibly coded) matrix used as \eqn{X}
#' in the fitting.  See \code{\link[stats]{model.matrix}} for details.
#'
#' \code{prednames}, \code{respnames} and \code{compnames} extract the names of
#' the \eqn{X} variables, responses and components, respectively.  With
#' \code{intercept = TRUE} in \code{prednames}, the name of the intercept
#' variable (i.e. \code{"(Intercept)"}) is returned as well.  \code{compnames}
#' can also extract component names from score and loading matrices.  If
#' \code{explvar = TRUE} in \code{compnames}, the explained variance for each
#' component (if available) is appended to the component names.  For optimal
#' formatting of the explained variances when not all components are to be
#' used, one should specify the desired components with the argument
#' \code{comps}.
#'
#' \code{explvar} extracts the amount of \eqn{X} variance (in per cent)
#' explained by each component in the model.  It can also handle score and
#' loading matrices returned by \code{\link{scores}} and
#' \code{\link{loadings}}.
#'
#' @aliases coef.mvr fitted.mvr residuals.mvr model.frame.mvr model.matrix.mvr
#' prednames respnames compnames explvar
#' @param object,formula an \code{mvr} object.  The fitted model.
#' @param ncomp,comps vector of positive integers.  The components to include
#' in the coefficients or to extract the names of.  See below.
#' @param intercept logical.  Whether coefficients for the intercept should be
#' included.  Ignored if \code{comps} is specified.  Defaults to \code{FALSE}.
#' @param explvar logical.  Whether the explained \eqn{X} variance should be
#' appended to the component names.
#' @param \dots other arguments sent to underlying functions.  Currently only
#' used for \code{model.frame.mvr} and \code{model.matrix.mvr}.
#' @return \code{coef.mvr} returns an array of regression coefficients.
#'
#' \code{fitted.mvr} returns an array with fitted values.
#'
#' \code{residuals.mvr} returns an array with residuals.
#'
#' \code{model.frame.mvr} returns a data frame.
#'
#' \code{model.matrix.mvr} returns the \eqn{X} matrix.
#'
#' \code{prednames}, \code{respnames} and \code{compnames} return a character
#' vector with the corresponding names.
#'
#' \code{explvar} returns a numeric vector with the explained variances, or
#' \code{NULL} if not available.
#' @author Ron Wehrens and Bjørn-Helge Mevik
#' @seealso \code{\link{mvr}}, \code{\link{coef}}, \code{\link{fitted}},
#' \code{\link{residuals}}, \code{\link{model.frame}},
#' \code{\link{model.matrix}}, \code{\link{na.omit}}
#' @keywords regression multivariate
#' @examples
#'
#' data(yarn)
#' mod <- pcr(density ~ NIR, data = yarn[yarn$train,], ncomp = 5)
#' B <- coef(mod, ncomp = 3, intercept = TRUE)
#' ## A manual predict method:
#' stopifnot(drop(B[1,,] + yarn$NIR[!yarn$train,] %*% B[-1,,]) ==
#'           drop(predict(mod, ncomp = 3, newdata = yarn[!yarn$train,])))
#'
#' ## Note the difference in formatting:
#' mod2 <- pcr(density ~ NIR, data = yarn[yarn$train,])
#' compnames(mod2, explvar = TRUE)[1:3]
#' compnames(mod2, comps = 1:3, explvar = TRUE)
#'
#' @export
coef.mvr <- function(object, ncomp = object$ncomp, comps, intercept = FALSE,
                     ...)
{
  if (missing(comps) || is.null(comps)) {
    ## Cumulative coefficients:
    B <- object$coefficients[,,ncomp, drop=FALSE]
    if (isTRUE(intercept)) {      # Intercept only has meaning for
      # cumulative coefficients
      dB <- dim(B)
      dB[1] <- dB[1] + 1
      dnB <- dimnames(B)
      dnB[[1]] <- c("(Intercept)", dnB[[1]])
      BInt <- array(dim = dB, dimnames = dnB)
      BInt[-1,,] <- B
      for (i in seq(along = ncomp))
        BInt[1,,i] <- object$Ymeans - object$Xmeans %*% B[,,i]
      B <- BInt
    }
  } else {
    ## Individual coefficients:
    B <- object$coefficients[,,comps, drop=FALSE]
    g1 <- which(comps > 1)
    ## Indiv. coef. must be calculated since object$coefficients is
    ## cumulative coefs.
    B[,,g1] <- B[,,g1, drop=FALSE] -
      object$coefficients[,,comps[g1] - 1, drop=FALSE]
    dimnames(B)[[3]] <- paste("Comp", comps)
  }
  return(B)
}

## fitted.mvr: Extract the fitted values.  It is needed because the case
## na.action == "na.exclude" must be treated differently from what is done
## in fitted.default.
#' @rdname coef.mvr
#' @export
fitted.mvr <- function(object, ...) {
  if (inherits(object$na.action, "exclude")) {
    naExcludeMvr(object$na.action, object$fitted.values)
  } else {
    object$fitted.values
  }
}

## residuals.mvr: Extract the residuals.  It is needed because the case
## na.action == "na.exclude" must be treated differently from what is done
## in residuals.default.
#' @rdname coef.mvr
#' @export
residuals.mvr <- function(object, ...) {
  if (inherits(object$na.action, "exclude")) {
    naExcludeMvr(object$na.action, object$residuals)
  } else {
    object$residuals
  }
}

## naExcludeMvr: Perform the equivalent of naresid.exclude and
## napredict.exclude on three-dimensional arrays where the first dimension
## corresponds to the observations.
## Almost everything here is lifted verbatim from naresid.exclude (R 2.2.0)


#' @title Adjust for Missing Values
#'
#' @description Use missing value information to adjust residuals and predictions.  This is
#' the \sQuote{mvr equivalent} of the \code{naresid.exclude} and
#' \code{napredict.exclude} functions.
#'
#' @details This is a utility function used to allow \code{predict.mvr} and
#' \code{residuals.mvr} to compensate for the removal of \code{NA}s in the
#' fitting process.
#'
#' It is called only when the \code{na.action} is \code{na.exclude}, and pads
#' \code{x} with \code{NA}s in the correct positions to have the same number of
#' rows as the original data frame.
#'
#' @param omit an object produced by an \code{na.action} function, typically
#' the \code{"na.action"} attribute of the result of \code{na.omit} or
#' \code{na.exclude}.
#' @param x a three-dimensional array to be adjusted based upon the missing
#' value information in \code{omit}.
#' @param \dots further arguments.  Currently not used.
#' @return \code{x}, padded with \code{NA}s along the first dimension
#' (\sQuote{rows}).
#' @author Bjørn-Helge Mevik and Ron Wehrens
#' @seealso \code{\link{predict.mvr}}, \code{\link{residuals.mvr}},
#' \code{\link{napredict}}, \code{\link{naresid}}
#' @keywords regression multivariate internal
naExcludeMvr <- function(omit, x, ...) {
  if (length(omit) == 0 || !is.numeric(omit))
    stop("invalid argument 'omit'")
  if (length(x) == 0)
    return(x)
  n <- nrow(x)
  keep <- rep.int(NA, n + length(omit))
  keep[-omit] <- 1:n
  x <- x[keep,,, drop = FALSE]        # This is where the real difference is!
  temp <- rownames(x)
  if (length(temp)) {
    temp[omit] <- names(omit)
    rownames(x) <- temp
  }
  return(x)
}

## loadings is in stats, but doesn't work for prcomp objects, and is not
## generic, so we build our own:
#' @name scores
#' @title Extract Scores and Loadings from PLSR and PCR Models
#'
#' @description These functions extract score and loading matrices from fitted \code{mvr}
#' models.
#'
#' @details All functions extract the indicated matrix from the fitted model, and will
#' work with any object having a suitably named component.
#'
#' The default \code{scores} and \code{loadings} methods also handle
#' \code{prcomp} objects (their scores and loadings components are called
#' \code{x} and \code{rotation}, resp.), and add an attribute \code{"explvar"}
#' with the variance explained by each component, if this is available.  (See
#' \code{\link{explvar}} for details.)
#'
#' @aliases scores scores.default loadings loadings.default loading.weights
#' Yscores Yloadings
#' @param object a fitted model to extract from.
#' @param estimate optional character vector ("train", "CV", "test") used
#'   by \code{scores} to select the desired estimate.
#' @param newdata optional data frame passed to \code{scores}
#'   when \code{estimate = "test"}.
#' @param \dots extra arguments, currently not used.
#' @return A matrix with scores or loadings.
#' @note There is a \code{loadings} function in package \pkg{stats}.  It simply
#' returns any element named \code{"loadings"}.  See
#' \code{\link[stats]{loadings}} for details.  The function can be accessed as
#' \code{stats::loadings(...)}.
#' @author Ron Wehrens and Bjørn-Helge Mevik
#' @seealso \code{\link{mvr}}, \code{\link{coef.mvr}}
#' @keywords regression multivariate
#' @examples
#'
#' data(yarn)
#' plsmod <- plsr(density ~ NIR, 6, data = yarn)
#' scores(plsmod)
#' loadings(plsmod)[,1:4]
#'
#' @export
loadings <- function(object, ...) UseMethod("loadings")
#' @rdname scores
#' @export
loadings.default <- function(object, ...) {
  L <- if (inherits(object, "prcomp")) object$rotation else object$loadings
  if (!(inherits(L, "loadings") || inherits(L, "list")))
    class(L) <- "loadings"
  attr(L, "explvar") <- explvar(object)
  L
}

## scores: Return the scores (also works for prcomp/princomp objects):
#' @rdname scores
#' @export
scores <- function(object, ...) UseMethod("scores")
# #' @rdname scores
# #' @export
# scores.default <- function(object, ...) {
#   S <- if (inherits(object, "prcomp")) object$x else object$scores
#   if (!(inherits(S, "scores") || inherits(S, "list")))
#     class(S) <- "scores"
#   attr(S, "explvar") <- explvar(object)
#   S
# }

#' @rdname scores
#' @export
scores.default <- function(object, estimate, newdata, ...){
  allEstimates <- c("all", "train", "CV", "test")
  scoreList <- list()
  if (missing(estimate)) {
    ## Select the `best' available estimate
    if (!missing(newdata)) {
      estimate = "test"
    } else {
      estimate = "train"
    }
  } else {
    estimate <- allEstimates[pmatch(estimate, allEstimates)]
    if (any(is.na(estimate)))
      stop("`estimate' should be a subset of ",
           paste(allEstimates, collapse = ", "))
    if (any(estimate == "all")) {
      estimate <- allEstimates[-1] # Try all estimates (except "all")
      if (missing(newdata))
        estimate <- setdiff(estimate, "test")
      if (is.null(object$validation))
        estimate <- setdiff(estimate, "CV")
    }
  }
  if("train" %in% estimate){
    S <- if (inherits(object, "prcomp")) object$x else object$scores
    if (!(inherits(S, "scores") || inherits(S, "list")))
      class(S) <- "scores"
    attr(S, "explvar") <- explvar(object)
    scoreList$train <- S
  }
  if("test" %in% estimate){
    if (missing(newdata))
      stop("Missing `newdata'.")
    mat <- buildDesignMatrix(newdata, object)
    centerX <- centerDesignMatrix(mat, object)
    totalXvar <- sum(centerX^2, na.rm = TRUE)
    S <- predict(object, newdata=newdata, type="scores")
    if (!(inherits(S, "scores") || inherits(S, "list")))
      class(S) <- "scores"
    contrib <- colSums(S^2) * colSums(object$loadings^2)
    S <- attachExplvar(S, contrib, totalXvar, object)
    scoreList$test <- S
  }
  if("CV" %in% estimate){
    if (is.null(object$validation))
      stop("`object' has no `validation' component")
    N <- nrow(object$scores)
    S <- matrix(0.0, N, ncol(object$scores))
    dimnames(S) <- dimnames(object$scores)
    contrib_seg <- rep(0, ncol(object$scores))
    for(seg in seq_along(object$validation$segments)){
      rows <- object$validation$segments[[seg]]
      newd <- object$model[rows,,drop=FALSE]
      cvfit <- update(object, subset=!((1:N)%in%rows))
      preds <- predict(cvfit, newdata = newd, type = "scores")
      S[rows,] <- preds
      newres <- centerDesignMatrix(buildDesignMatrix(newd, object), object)
      prev_ss <- sum(newres^2)
      for(a in seq_len(ncol(S))){
        recon <- tcrossprod(preds[,a], cvfit$loadings[,a])
        newres <- newres - recon
        contrib_seg[a] <- contrib_seg[a] + (prev_ss - sum(newres^2))
        prev_ss <- sum(newres^2)
      }
    }
    if (!(inherits(S, "scores") || inherits(S, "list")))
      class(S) <- "scores"
    totalXvar <- if (!is.null(object$Xtotvar)) object$Xtotvar
                 else sum(centerDesignMatrix(model.matrix(object), object)^2, na.rm = TRUE)
    S <- attachExplvar(S, contrib_seg, totalXvar, object)
    scoreList$CV <- S
  }
  if(length(scoreList) == 1){
    return(scoreList[[1]])
  } else {
    class(scoreList) <- "scoreList"
    return(scoreList)
  }
}

## Yscores: Return the Yscores
#' @rdname scores
#' @export
Yscores <- function(object) object$Yscores

## loading.weights: Return the loading weights:
#' @rdname scores
#' @export
loading.weights <- function(object) object$loading.weights

## Yloadings: Return the Yloadings
#' @rdname scores
#' @export
Yloadings <- function(object) object$Yloadings

## model.frame.mvr: Extract or generate the model frame from a `mvr' object.
## It is simply a slightly modified `model.frame.lm'.
#' @rdname coef.mvr
#' @export
model.frame.mvr <- function(formula, ...) {
  dots <- list(...)
  nargs <- dots[match(c("data", "na.action", "subset"), names(dots), 0)]
  if (length(nargs) || is.null(formula$model)) {
    fcall <- formula$call
    fcall$method <- "model.frame"
    fcall[[1]] <- quote(pls::mvr)
    fcall[names(nargs)] <- nargs
    env <- environment(formula$terms)
    if (is.null(env)) env <- parent.frame()
    eval(fcall, env, parent.frame())
  }
  else formula$model
}

## model.matrix.mvr: Extract the model matrix from an `mvr' object.
## It is a modified version of model.matrix.lm.
#' @rdname coef.mvr
#' @export
model.matrix.mvr <- function(object, ...) {
  if (n_match <- match("x", names(object), 0))
    object[[n_match]]
  else {
    data <- model.frame(object, ...)
    mm <- NextMethod("model.matrix", data = data)
    mm <- delete.intercept(mm) # Deletes any intercept coloumn
    ## model.matrix.default prepends the term name to the colnames of
    ## matrices.  If there is only one predictor term, and the
    ## corresponding matrix has colnames, remove the prepended term name:
    mt <- terms(object)
    if (length(attr(mt, "term.labels")) == 1 &&
        !is.null(colnames(data[[attr(mt, "term.labels")]])))
      colnames(mm) <- sub(attr(mt, "term.labels"), "", colnames(mm))
    return(mm)
  }
}

## delete.intercept: utility function that deletes the response coloumn from
## a model matrix, and adjusts the "assign" attribute:


#' @title Delete intercept from model matrix
#'
#' @description A utility function to delete any intercept column from a model matrix, and
#' adjust the \code{"assign"} attribute correspondingly.  It is used by formula
#' handling functions like \code{mvr} and \code{model.matrix.mvr}.
#'
#'
#' @param mm Model matrix.
#' @return A model matrix without intercept column.
#' @author Bjørn-Helge Mevik and Ron Wehrens
#' @seealso \code{\link{mvr}}, \code{\link{model.matrix.mvr}}
#' @keywords internal
delete.intercept <- function(mm) {
  ## Save the attributes prior to removing the intercept coloumn:
  saveattr <- attributes(mm)
  ## Find the intercept coloumn:
  intercept <- which(saveattr$assign == 0)
  ## Return if there was no intercept coloumn:
  if (!length(intercept)) return(mm)
  ## Remove the intercept coloumn:
  mm <- mm[,-intercept, drop=FALSE]
  ## Update the attributes with the new dimensions:
  saveattr$dim <- dim(mm)
  saveattr$dimnames <- dimnames(mm)
  ## Remove the assignment of the intercept from the attributes:
  saveattr$assign <- saveattr$assign[-intercept]
  ## Restore the (modified) attributes:
  attributes(mm) <- saveattr
  ## Return the model matrix:
  mm
}

## The following "extraction" functions are mostly used in plot and summary
## functions.

## The names of the response variables:
#' @rdname coef.mvr
#' @export
respnames <- function(object)
  dimnames(fitted(object))[[2]]

## The names of the prediction variables:
#' @rdname coef.mvr
#' @export
prednames <- function(object, intercept = FALSE) {
  if (isTRUE(intercept))
    c("(Intercept)", rownames(object$loadings))
  else
    rownames(object$loadings)
}

## The names of the components:
## Note: The components must be selected prior to the format statement
#' @rdname coef.mvr
#' @export
compnames <- function(object, comps, explvar = FALSE, ...) {
  M <- if (is.matrix(object)) object else scores(object)
  labs <- colnames(M)
  if (missing(comps))
    comps <- seq(along = labs)
  else
    labs <- labs[comps]
  if (isTRUE(explvar) && !is.null(evar <- explvar(M)[comps]))
    labs <- paste(labs, " (", format(evar, digits = 2, trim = TRUE),
                  " %)", sep = "")
  return(labs)
}


## The explained X variance:
#' @rdname coef.mvr
#' @export
explvar <- function(object)
  switch(class(object)[1],
         mvr = 100 * object$Xvar / object$Xtotvar,
         princomp =,
         prcomp = 100 * object$sdev^2 / sum(object$sdev^2),
         scores =,
         loadings = attr(object, "explvar")
  )


## Internal functions
buildDesignMatrix <- function(data, object) {
  if (is.matrix(data)) {
    if (!is.null(object$Xmeans) && ncol(data) != length(object$Xmeans))
      stop("'newdata' does not have the correct number of columns")
    mat <- data
  } else {
    Terms <- delete.response(terms(object))
    mf <- model.frame(Terms, data, na.action = na.pass)
    if (!is.null(cl <- attr(Terms, "dataClasses")))
      .checkMFClasses(cl, mf)
    mat <- delete.intercept(model.matrix(Terms, mf))
  }
  if (!is.null(object$scale))
    mat <- mat / rep(object$scale, each = nrow(mat))
  mat
}

centerDesignMatrix <- function(mat, object) {
  if (is.null(object$Xmeans))
    return(mat)
  mat - rep(object$Xmeans, each = nrow(mat))
}

attachExplvar <- function(S, contribution, totalXvar, object) {
  if (length(contribution) != ncol(S))
    contribution <- contribution[seq_len(ncol(S))]
  expl <- if (totalXvar > 0) contribution / totalXvar * 100
          else rep(NA_real_, ncol(S))
  if (!is.null(colnames(S)))
    names(expl) <- colnames(S)
  else if (inherits(object, "mvr")) {
    cn <- if (!is.null(object$loading.weights)) colnames(object$loading.weights)
          else paste("Comp", seq_len(ncol(S)))
    names(expl) <- cn[seq_len(length(expl))]
  }
  attr(S, "explvar") <- expl
  S
}

### msc.R: Multiplicative scatter/signal correction



#' @name msc
#' @title Multiplicative Scatter Correction
#'
#' @description Performs multiplicative scatter/signal correction on a data matrix.
#'
#' @details \code{makepredictcall.msc} is an internal utility function; it is not meant
#' for interactive use.  See \code{\link{makepredictcall}} for details.
#'
#' @aliases msc predict.msc makepredictcall.msc
#' @param X,newdata numeric matrices.  The data to scatter correct.
#' @param reference numeric vector.  Spectre to use as reference.  If
#' \code{NULL}, the column means of \code{X} are used.
#' @param object an object inheriting from class \code{"msc"}, normally the
#' result of a call to \code{msc} with a single matrix argument.
#' @param var A variable.
#' @param call The term in the formula, as a call.
#' @param \dots other arguments.  Currently ignored.
#' @return Both \code{msc} and \code{predict.msc} return a multiplicative
#' scatter corrected matrix, with attribute \code{"reference"} the vector used
#' as reference spectre. The matrix is given class \code{c("msc", "matrix")}.
#' For \code{predict.msc}, the \code{"reference"} attribute of \code{object} is
#' used as reference spectre.
#' @author Bjørn-Helge Mevik and Ron Wehrens
#' @seealso \code{\link{mvr}}, \code{\link{pcr}}, \code{\link{plsr}},
#' \code{\link{stdize}}
#' @references Martens, H., Næs, T. (1989) \emph{Multivariate calibration.}
#' Chichester: Wiley.
#' @keywords regression multivariate
#' @examples
#'
#' data(yarn)
#' ## Direct correction:
#' Ztrain <- msc(yarn$NIR[yarn$train,])
#' Ztest <- predict(Ztrain, yarn$NIR[!yarn$train,])
#'
#' ## Used in formula:
#' mod <- plsr(density ~ msc(NIR), ncomp = 6, data = yarn[yarn$train,])
#' pred <- predict(mod, newdata = yarn[!yarn$train,]) # Automatically scatter corrected
#'
#' @export
msc <- function(X, reference = NULL) {
    if (is.null(reference)) reference <- colMeans(X)
    Z <- cbind(1, reference)
    ## The estimated regression coefficients (a_i, b_i), one pair per row:
    B <- t(solve(crossprod(Z), t(X %*% Z)))
    res <- (X - B[,1]) / B[,2]
    attr(res, "reference") <- reference
    class(res) <- c("msc", "matrix")
    return(res)
}

#' @rdname msc
#' @export
predict.msc <- function(object, newdata, ...) {
    if (missing(newdata)) return(object)
    msc(newdata, reference = attr(object, "reference"))
}

## This method makes things like
## `predict(plsr(y ~ msc(X), data = foo), newdata = bar)' work.
#' @rdname msc
#' @export
makepredictcall.msc <- function(var, call) {
    if (as.character(call)[1] != "msc")
        return(call)
    call$reference <- attr(var, "reference")
    call
}

#' Octane numbers and NIR spectra of gasoline
#'
#' A data set with NIR spectra and octane numbers of 60 gasoline samples.  The
#' NIR spectra were measured using diffuse reflectance as log(1/R) from 900 nm
#' to 1700 nm in 2 nm intervals, giving 401 wavelengths.  Many thanks to John
#' H. Kalivas.
#'
#'
#' @name gasoline
#' @docType data
#' @format A data frame with 60 observations on the following 2 variables.
#' \describe{ \item{octane}{a numeric vector.  The octane number.}
#' \item{NIR}{a matrix with 401 columns.  The NIR spectrum.} }
#' @source Kalivas, John H. (1997) Two Data Sets of Near Infrared Spectra
#' \emph{Chemometrics and Intelligent Laboratory Systems}, \bold{37}, 255--259.
#' @keywords datasets
NULL





#' NIR measurements and oil types of mayonnaise
#'
#' Raw NIR measurements (351 wavelengths, 1100-2500 nm in steps of 4 nm) taken
#' on 54 samples of mayonnaise based on six different oil types (soybean,
#' sunflower, canola, olive, corn, and grapeseed). The resulting 54 samples
#' were measured in triplicates, resulting in 54 x 3 = 162 different spectra
#' (120/42 training/test).
#'
#'
#' @name mayonnaise
#' @docType data
#' @format A data frame with 162 observations on the following 4 variables.
#' \describe{ \item{NIR}{a matrix with 351 columns}
#' \item{oil.type}{a numeric vector} \item{design}{a matrix
#' with 5 columns} \item{train}{a logical vector} }
#' @source Indahl U, Sahni NS, Kirkhus B, Næs T.  Multivariate strategies for
#' classification based on NIR-spectra-with application to mayonnaise.
#' Chemometr. Intell. Lab. Sys. 1999; 49: 19-31.
#' @keywords datasets
NULL





#' Sensory and physico-chemical data of olive oils
#'
#' A data set with scores on 6 attributes from a sensory panel and measurements
#' of 5 physico-chemical quality parameters on 16 olive oil samples.  The first
#' five oils are Greek, the next five are Italian and the last six are Spanish.
#'
#'
#' @name oliveoil
#' @docType data
#' @format A data frame with 16 observations on the following 2 variables.
#' \describe{ \item{sensory}{a matrix with 6 columns.  Scores for
#' attributes \sQuote{yellow}, \sQuote{green}, \sQuote{brown}, \sQuote{glossy},
#' \sQuote{transp}, and \sQuote{syrup}.} \item{chemical}{a matrix with
#' 5 columns.  Measurements of acidity, peroxide, K232, K270, and DK.} }
#' @source Massart, D. L., Vandeginste, B. G. M., Buydens, L. M. C., de Jong,
#' S., Lewi, P. J., Smeyers-Verbeke, J. (1998) \emph{Handbook of Chemometrics
#' and Qualimetrics: Part B}.  Elsevier. Tables 35.1 and 35.4.
#' @keywords datasets
NULL





#' NIR spectra and density measurements of PET yarns
#'
#' A training set consisting of 21 NIR spectra of PET yarns, measured at 268
#' wavelengths, and 21 corresponding densities.  A test set of 7 samples is
#' also provided.  Many thanks to Erik Swierenga.
#'
#'
#' @name yarn
#' @docType data
#' @format A data frame with components \describe{ \item{NIR}{Numeric matrix of
#' NIR measurements} \item{density}{Numeric vector of densities}
#' \item{train}{Logical vector with \code{TRUE} for the training samples and
#' \code{FALSE} for the test samples} }
#' @source Swierenga H., de Weijer A. P., van Wijk R. J., Buydens L. M. C.
#' (1999) Strategy for constructing robust multivariate calibration models
#' \emph{Chemometrics and Intelligent Laboratoryy Systems}, \bold{49}(1),
#' 1--17.
#' @keywords datasets
NULL

### test.R: Runs all RUnit tests that work when the functions are accessed by direct
### import (i.e., pls::mvr(...))


## Get warnings at once
options(warn = 1)

## Make sure the global environment is clean:
rm(list = ls(all = TRUE))

## Get the data
data(yarn, package = "pls")
data(oliveoil, package = "pls")
data(gasoline, package = "pls")

## Get RUnit and utility functions
library(RUnit)
source("RUnit/common/utils.R")

## Make checkException silent
opts <- getOption("RUnit")
opts$silent <- TRUE
options(RUnit = opts)

## Define and check test suite
testdirs <- c("RUnit/import")
plsTestSuite <- defineTestSuite("pls", testdirs)
isValidTestSuite(plsTestSuite)

## Run test suite
printTextProtocol(res <- runTestSuite(plsTestSuite))
if (res$pls$nFail > 0 || res$pls$nErr > 0) stop("One or more tests failed")

### summaries.R: print and summary methods.

## Print method for mvr objects:
#' @rdname summary.mvr
#' @export
print.mvr <- function(x, ...) {
  switch(x$method,
         kernelpls = {
           ana = "Partial least squares regression"
           alg = "kernel"
         },
         widekernelpls = {
           ana = "Partial least squares regression"
           alg = "wide kernel"
         },
         simpls = {
           ana = "Partial least squares regression"
           alg = "simpls"
         },
         oscorespls = {
           ana = "Partial least squares regression"
           alg = "orthogonal scores"
         },
         nipalspls = {
           ana = "Partial least squares regression"
           alg = "NIPALS"
         },
         cppls = {
           ana = "Canonical powered partial least squares"
           alg = "cppls"
         },
         svdpc = {
           ana = "Principal component regression"
           alg = "singular value decomposition"
         },
         nipalspc = {
           ana = "Principal component regression"
           alg = "nipals algorithm"
         },
         stop("Unknown fit method.")
  )
  cat(ana, ", fitted with the ", alg, " algorithm.", sep = "")
  if (!is.null(x$validation))
    cat("\nCross-validated using", length(x$validation$segments),
        attr(x$validation$segments, "type"), "segments.")
  cat("\nCall:\n", deparse(x$call), "\n", sep = "")
  invisible(x)
}

## Summary method for mvr objects


#' @name summary.mvr
#' @title Summary and Print Methods for PLSR and PCR objects
#'
#' @description Summary and print methods for \code{mvr} and \code{mvrVal} objects.
#'
#' @details If \code{what} is \code{"training"}, the explained variances are given; if
#' it is \code{"validation"}, the cross-validated RMSEPs (if available) are
#' given; if it is \code{"all"}, both are given.
#'
#' @aliases summary.mvr print.mvr print.mvrVal
#' @param x,object an \code{mvr} object
#' @param what one of \code{"all"}, \code{"validation"} or \code{"training"}
#' @param digits integer.  Minimum number of significant digits in the output.
#' Default is 4.
#' @param print.gap Integer.  Gap between coloumns of the printed tables.
#' @param shortAlgs Logical.  Shorten algorithm names (default = TRUE).
#' @param row.names NULL or a character vector giving the row names for the data frame. Missing values are not allowed.
#' @param optional Not used, only included to match signature of \code{as.data.frame}.
#' @param \dots Other arguments sent to underlying methods.
#' @return \code{print.mvr} and \code{print.mvrVal} return the object
#' invisibly.
#' @author Ron Wehrens and Bjørn-Helge Mevik
#' @seealso \code{\link{mvr}}, \code{\link{pcr}}, \code{\link{plsr}},
#' \code{\link{RMSEP}}, \code{\link{MSEP}}
#' @keywords regression multivariate
#' @examples
#'
#' data(yarn)
#' nir.mvr <- mvr(density ~ NIR, ncomp = 8, validation = "LOO", data = yarn)
#' nir.mvr
#' summary(nir.mvr)
#' RMSEP(nir.mvr)
#' # Extract MVR validation statistics as data.frame:
#' as.data.frame(RMSEP(nir.mvr, estimate = "CV"))
#' as.data.frame(R2(nir.mvr))
#'
#' @export
summary.mvr <- function(object, what = c("all", "validation", "training"),
                        digits = 4, print.gap = 2, ...)
{
  what <- match.arg(what)
  if (what == "all") what <- c("validation", "training")
  if (is.null(object$validation)) what <- "training"

  nobj <- nrow(object$scores)
  nresp <- length(object$Ymeans)
  yvarnames <- respnames(object)
  cat("Data: \tX dimension:", nobj, length(object$Xmeans),
      "\n\tY dimension:", nobj, nresp)
  cat("\nFit method:", object$method)
  cat("\nNumber of components considered:", object$ncomp)

  for (wh in what) {
    if (wh == "training") {
      cat("\nTRAINING: % variance explained\n")
      xve <- explvar(object)
      yve <- 100 * drop(R2(object, estimate = "train",
                           intercept = FALSE)$val)
      if(length(xve) == 1)
        yve <- as.matrix(yve)
      tbl <- rbind(cumsum(xve), yve)
      dimnames(tbl) <- list(c("X", yvarnames),
                            paste(1:object$ncomp, "comps"))
      print(tbl, digits = digits, print.gap = print.gap, ...)
    } else {
      cat("\n\nVALIDATION: RMSEP")
      cat("\nCross-validated using", length(object$validation$segments),
          attr(object$validation$segments, "type"), "segments.\n")
      print(RMSEP(object), digits = digits, print.gap = print.gap, ...)
    }
  }
  # Warn if missing data have been removed
  if(object$nobj < object$nobjOrig){
    cat("(", object$nobjOrig - object$nobj,
        " observations deleted due to missingness)\n", sep = "")
  }
}

## Print method for mvrVal objects:
#' @rdname summary.mvr
#' @export
print.mvrVal <- function(x, digits = 4, print.gap = 2, ...) {
  nresp <- dim(x$val)[2]
  yvarnames <- dimnames(x$val)[[2]]
  names(dimnames(x$val)) <- NULL
  for (i in 1:nresp) {
    if (nresp > 1) cat("\nResponse:", yvarnames[i], "\n")
    print(x$val[,i,], digits = digits, print.gap = print.gap, ...)
  }
  invisible(x)
}

#' @rdname summary.mvr
#' @export
as.data.frame.mvrVal <- function(x, row.names = NULL, optional = FALSE,
                                 shortAlgs = TRUE, ...){
  # mvrVal values
  mvrVals <- x$val

  # Reshape and add columns
  dnames <- dimnames(mvrVals)
  dims <- dim(mvrVals)
  obj <- data.frame(estimate = rep(dnames$estimate, dims[2]*dims[3]),
                    response = rep(dnames$response, dims[1]*dims[3]),
                    comps    = rep(x$comps, each = dims[1]*dims[2]))

  # Fitting method and validation type
  object <- eval(x$call$object, envir = parent.frame())
  fit <- as.character(object$call[[1]])
  val <- object$call$validation
  switch(object$method,
         kernelpls = {
           alg = "kernel"
         },
         widekernelpls = {
           if(shortAlgs)
             alg = "wide"
           else
             alg = "wide kernel"
         },
         simpls = {
           alg = "simpls"
         },
         nipalspls = {
           alg = "nipalspls"
         },
         oscorespls = {
           if(shortAlgs)
             alg = "orthScores"
           else
             alg = "orthogonal scores"
         },
         cppls = {
           alg = "cppls"
         },
         svdpc = {
           if(shortAlgs)
             alg = "svd"
           else
             alg = "singular value decomposition"
         },
         nipalspc = {
           alg = "nipals pca"
         },
         stop("Unknown fit method.")
  )
  obj$validation <- rep(val, prod(dims))
  obj$method <- rep(fit, prod(dims))
  obj$algorithm <- rep(alg, prod(dims))
  obj$value <- c(mvrVals)
  if(!is.null(row.names))
    rownames(obj) <- row.names
  obj
}

#' Factor to Segments
#'
#' @param fac A factor where each level represents a segment
#'
#' @return A list of vectors, each vector contains the indices of the elements
#' of the corresponding segment
#' @export
#'
#' @seealso CV segments by various patterns: \code{\link{cvsegments}}.
#' @examples
#' fac <- factor(c("a", "b", "a", "b", "c", "c"))
#' fac2seg(fac)
fac2seg <- function(fac){
  if(!is.factor(fac))
    stop("Input must be a factor")
  lapply(1:length(levels(fac)),
         function(i) which(fac==levels(fac)[i]))
}

## Creates reference models to compare later implementations with

## The models were created with R 3.3.2 an pls 2.5-0

require(pls)
sessionInfo()

## Single response model:
rm(list = ls(all = TRUE))
mydata <- gasoline
form <- octane ~ NIR
nc <- 27                            # Max on MSWin: 36, Linux/64: 27
refmod <-  mvr(form, nc = nc, data = mydata, method = "oscorespls")
refmod$method <- refmod$call <- refmod$fit.time <- NULL
save.image(file = "ref_singresp.RData")

## Multi-response model:
rm(list = ls(all = TRUE))
mydata <- oliveoil
form <- sensory ~ chemical
nc <- 4                             # 5 == max
## Note: the algs give sign differences, in general, for scores
## and loadings
refmod <- mvr(form, nc = nc, data = mydata, method = "oscorespls")
refmod$method <- refmod$call <- refmod$fit.time <-NULL
for (i in 2:7) refmod[[i]] <- abs(refmod[[i]])
save.image(file = "ref_multiresp.RData")

#' @title Calculate Variance-Covariance Matrix for a Fitted Model Object
#'
#' @description Returns the variance-covariance matrix of the coefficients
#' of a Principal Component Regression.
#'
#' @param object a fitted PCR object of class \code{mvr}.
#' @param ncomp number of principal components to estimate \code{vcov} for.
#' @param ... additional arguments (not used).
#'
#' @return A matrix of estimated covariances between regression coefficients.
#' @export
#'
#' @examples
#' data(yarn)
#' yarn.pcr <- pcr(density ~ NIR, 6, data = yarn)
#' vc <- vcov(yarn.pcr, 3)
#'
#' # Standard error of coefficients
#' se <- sqrt(diag(vc))
#' beta <- coef(yarn.pcr, ncomp = 3)
#'
#' # Plot regression coefficients with two standard errors shading.
#' plot(beta, type = 'l',
#'      panel.first = polygon(x = c(1:268, 268:1),
#'                            y = c(beta+2*se, rev(beta-2*se)),
#'                            col = 'lightblue',
#'                            border = NA))
vcov.mvr <- function(object, ncomp, ...){
  if(object$method != "svdpc")
    stop("For 'mvr' objects, only method 'svdpc' has a 'vcov' definition.")
  if(missing(ncomp)){
    ncomp <- object$ncomp
    warning(paste0("Missing parameter 'ncomp', defaulting to maximum: ", ncomp))
  }
  intercept <- ifelse(object$center, 1, 0)
  sigma2 <- sum(object$residuals[,,ncomp]^2)/(nrow(object$scores)-ncomp-intercept)
  vc <- sigma2 * object$loadings[,1:ncomp,drop=FALSE] %*% solve(crossprod(object$scores[,1:ncomp,drop=FALSE])) %*% t(object$loadings[,1:ncomp,drop=FALSE])
  return(vc)
}

#' @title Partial Least Squares and Principal Component Regression
#'
#' @description
#' Multivariate regression methods Partial Least Squares Regression (PLSR), Principal Component
#' Regression (PCR) and Canonical Powered Partial Least Squares (CPPLS).
#'
#' @importFrom grDevices dev.interactive devAskNewPage
#' @importFrom graphics abline arrows axTicks axis legend matplot mtext pairs par plot points segments text title
#' @importFrom methods hasArg
#' @importFrom stats .checkMFClasses coef cor delete.response drop.terms fitted formula makepredictcall median model.frame model.matrix model.response na.omit na.pass napredict optimize predict printCoefmat pt residuals runif sd terms update var
#' @docType package
#' @name pls
#' @keywords internal
"_PACKAGE"

### crossval.R: Cross-validation functions.

## The basic cross-validation function


#' @title Cross-validation
#'
#' @description Performs the cross-validation calculations for \code{mvr}.
#'
#' This function is not meant to be called directly, but through the generic
#' functions \code{pcr}, \code{plsr}, \code{cppls} or \code{mvr} with the
#' argument \code{validation} set to \code{"CV"} or \code{"LOO"}.  All
#' arguments to \code{mvrCv} can be specified in the generic function call.
#'
#' If \code{segments} is a list, the arguments \code{segment.type} and
#' \code{length.seg} are ignored.  The elements of the list should be integer
#' vectors specifying the indices of the segments.  See
#' \code{\link{cvsegments}} for details.
#'
#' Otherwise, segments of type \code{segment.type} are generated.  How many
#' segments to generate is selected by specifying the number of segments in
#' \code{segments}, or giving the segment length in \code{length.seg}.  If both
#' are specified, \code{segments} is ignored.
#'
#' If \code{jackknife} is \code{TRUE}, jackknifed regression coefficients are
#' returned, which can be used for for variance estimation
#' (\code{\link{var.jack}}) or hypothesis testing (\code{\link{jack.test}}).
#'
#' \code{X} and \code{Y} do not need to be centered.
#'
#' Note that this function cannot be used in situations where \eqn{X} needs to
#' be recalculated for each segment (except for scaling by the standard
#' deviation), for instance with \code{msc} or other preprocessing.  For such
#' models, use the more general (but slower) function \code{\link{crossval}}.
#'
#' Also note that if needed, the function will silently(!) reduce \code{ncomp}
#' to the maximal number of components that can be cross-validated, which is
#' \eqn{n - l - 1}, where \eqn{n} is the number of observations and \eqn{l} is
#' the length of the longest segment.  The (possibly reduced) number of
#' components is returned as the component \code{ncomp}.
#'
#' By default, the cross-validation will be performed serially.  However, it
#' can be done in parallel using functionality in the \code{\link{parallel}}
#' package by setting the option \code{parallel} in \code{\link{pls.options}}.
#' See \code{\link{pls.options}} for the different ways to specify the
#' parallelism.
#'
#' @param X a matrix of observations.  \code{NA}s and \code{Inf}s are not
#' allowed.
#' @param Y a vector or matrix of responses.  \code{NA}s and \code{Inf}s are
#' not allowed.
#' @param ncomp the number of components to be used in the modelling.
#' @param Y.add a vector or matrix of additional responses containing relevant
#' information about the observations.  Only used for \code{cppls}.
#' @param weights a vector of individual weights for the observations.  Only
#' used for \code{cppls}.  (Optional)
#' @param method the multivariate regression method to be used.
#' @param scale logical.  If \code{TRUE}, the learning \eqn{X} data for each
#' segment is scaled by dividing each variable by its sample standard
#' deviation.  The prediction data is scaled by the same amount.
#' @param segments the number of segments to use, or a list with segments (see
#' below).
#' @param segment.type the type of segments to use.  Ignored if \code{segments}
#' is a list.
#' @param length.seg Positive integer.  The length of the segments to use.  If
#' specified, it overrides \code{segments} unless \code{segments} is a list.
#' @param jackknife logical.  Whether jackknifing of regression coefficients
#' should be performed.
#' @param trace logical; if \code{TRUE}, the segment number is printed for each
#' segment.
#' @param \dots additional arguments, sent to the underlying fit function.
#' @return A list with the following components: \item{method}{equals
#' \code{"CV"} for cross-validation.} \item{pred}{an array with the
#' cross-validated predictions.} \item{coefficients}{(only if \code{jackknife}
#' is \code{TRUE}) an array with the jackknifed regression coefficients.  The
#' dimensions correspond to the predictors, responses, number of components,
#' and segments, respectively.} \item{PRESS0}{a vector of PRESS values (one for
#' each response variable) for a model with zero components, i.e., only the
#' intercept.} \item{PRESS}{a matrix of PRESS values for models with 1,
#' \ldots{}, \code{ncomp} components.  Each row corresponds to one response
#' variable.} \item{adj}{a matrix of adjustment values for calculating bias
#' corrected MSEP.  \code{MSEP} uses this.} \item{segments}{the list of
#' segments used in the cross-validation.} \item{ncomp}{the actual number of
#' components used.} \item{gamma}{if method \code{cppls} is used, gamma values
#' for the powers of each CV segment are returned.}
#' @note The \code{PRESS0} is always cross-validated using leave-one-out
#' cross-validation.  This usually makes little difference in practice, but
#' should be fixed for correctness.
#'
#' The current implementation of the jackknife stores all jackknife-replicates
#' of the regression coefficients, which can be very costly for large matrices.
#' This might change in a future version.
#' @author Ron Wehrens and Bjørn-Helge Mevik
#' @seealso \code{\link{mvr}} \code{\link{crossval}} \code{\link{cvsegments}}
#' \code{\link{MSEP}} \code{\link{var.jack}} \code{\link{jack.test}}
#' @references Mevik, B.-H., Cederkvist, H. R. (2004) Mean Squared Error of
#' Prediction (MSEP) Estimates for Principal Component Regression (PCR) and
#' Partial Least Squares Regression (PLSR).  \emph{Journal of Chemometrics},
#' \bold{18}(9), 422--429.
#' @keywords regression multivariate
#' @examples
#'
#' data(yarn)
#' yarn.pcr <- pcr(density ~ NIR, 6, data = yarn, validation = "CV", segments = 10)
#' \dontrun{plot(MSEP(yarn.pcr))}
#'
mvrCv <- function(X, Y, ncomp, Y.add = NULL, weights = NULL,
                  method = pls.options()$mvralg,
                  scale = FALSE, segments = 10,
                  segment.type = c("random", "consecutive", "interleaved"),
                  length.seg, jackknife = FALSE, trace = FALSE, ...)
{
    ## Initialise:
    Y <- as.matrix(Y)
    if (!(missing(Y.add) || is.null(Y.add)))
        Y.add <- as.matrix(Y.add)

    ## Save dimnames:
    dnX <- dimnames(X)
    dnY <- dimnames(Y)

    ## Remove dimnames for performance (doesn't seem to matter; in fact,
    ## as far as it has any effect, it hurts a tiny bit in most situations).
    ## dimnames(X) <- dimnames(Y) <- NULL

    ## Save dimensions:
    nobj  <- dim(X)[1]
    npred <- dim(X)[2]
    nresp <- dim(Y)[2]

    ## Check the `scale' parameter:
    if (!is.logical(scale) || length(scale) != 1)
        stop("'scale' must be 'TRUE' or 'FALSE'")

    ## Set up segments:
    if (is.list(segments)) {
        if (is.null(attr(segments, "type")))
            attr(segments, "type") <- "user supplied"
    } else {
        if (missing(length.seg)) {
            segments <- cvsegments(nobj, k = segments, type = segment.type)
        } else {
            segments <- cvsegments(nobj, length.seg = length.seg,
                                   type = segment.type)
        }
    }

    ## Reduce ncomp, if neccessary:
    ncomp <- min(ncomp, nobj - max(sapply(segments, length)) - 1)

    ## Select fit function:
    method <- match.arg(method,c("kernelpls", "widekernelpls", "simpls",
                                 "oscorespls", "nipalspls", "cppls", "svdpc", "nipalspc"))
    fitFunc <- switch(method,
                      kernelpls = kernelpls.fit,
                      widekernelpls = widekernelpls.fit,
                      simpls = simpls.fit,
                      oscorespls = oscorespls.fit,
                      nipalspls = nipals.fit,
                      cppls = cppls.fit,
                      svdpc = svdpc.fit,
                      nipalspc = nipalspc.fit)

    ## Helper function to perform the cross-validatoin for one segment.
    ## Defined inside mvrCv to be able to access local variables:
    mvrCvSeg <- function(n.seg) {
        if (trace) cat(n.seg, "")

        ## Set up train data:
        seg <- segments[[n.seg]]
        Xtrain <- X[-seg,, drop=FALSE]
        if (scale) {
            ntrain <- nrow(Xtrain)
            if(anyNA(Xtrain)){
              sdtrain <- apply(Xtrain, 2, sd, na.rm=TRUE)
            } else {
              ## This is faster than sd(X), but cannot handle missing values:
              sdtrain <-
                sqrt(colSums((Xtrain - rep(colMeans(Xtrain), each = ntrain))^2) /
                       (ntrain - 1))
            }
            if (any(abs(sdtrain) < .Machine$double.eps^0.5))
                warning("Scaling with (near) zero standard deviation")
            Xtrain <- Xtrain / rep(sdtrain, each = ntrain)
        }

        ## Fit the model:
        fit <- fitFunc(Xtrain, Y[-seg,, drop=FALSE], ncomp,
                       Y.add = Y.add[-seg,, drop=FALSE], stripped = TRUE,
                       weights = weights[-seg], ...)

        ## Set up test data:
        Xtest <- X
        if (scale) Xtest <- Xtest / rep(sdtrain, each = nobj)
        Xtest <- Xtest - rep(fit$Xmeans, each = nobj)

        ## Predict test data:
        pred <- array(0, dim = c(nobj, nresp, ncomp))
        Ymeansrep <- rep(fit$Ymeans, each = nobj)
        for (a in 1:ncomp)
            pred[,,a] <- Xtest %*% fit$coefficients[,,a] + Ymeansrep
        if(method %in% c("nipalspls", "nipalspc"))
          adj <- length(seg) * colSums((pred - c(Y))^2, na.rm=TRUE)
        else
          adj <- length(seg) * colSums((pred - c(Y))^2)
        return(list(adj = adj,
                    cvPred = pred[seg,,, drop=FALSE],
                    gammas = if (method == "cppls") fit$gammas else NULL,
                    cvCoef = if (jackknife) fit$coefficients else NULL
                    ))
    }

    ## Perform the cross-validation, optionally in parallel:
    if (trace) cat("Segment: ")
    results <- lapplyFunc(pls.options()$parallel, seq_along(segments), mvrCvSeg)
    if (trace) cat("\n")

    ## Variables to save CV results in:
    adj <- matrix(0, nrow = nresp, ncol = ncomp)
    cvPred <- array(0, dim = c(nobj, nresp, ncomp))
    if (jackknife)
        cvCoef <- array(dim = c(npred, nresp, ncomp, length(segments)))
    if (method == "cppls") gammas <- list()

    ## Collect the results:
    for (n.seg in seq_along(segments)) {
        res <- results[[n.seg]]
        adj <- adj + res$adj
        cvPred[segments[[n.seg]],,] <- res$cvPred
        if (jackknife) cvCoef[,,,n.seg] <- res$cvCoef
        if (method == "cppls") gammas[[n.seg]] <- res$gammas
    }

    ## Calculate validation statistics:
    PRESS0 <- apply(Y, 2, var) * nobj^2 / (nobj - 1) # FIXME: Only correct for loocv!
    if(method %in% c("nipalspls", "nipalspc"))
      PRESS <- colSums((cvPred - c(Y))^2, na.rm=TRUE)
    else
      PRESS <- colSums((cvPred - c(Y))^2)

    ## Add dimnames:
    objnames <- dnX[[1]]
    if (is.null(objnames)) objnames <- dnY[[1]]
    respnames <- dnY[[2]]
    nCompnames <- paste(1:ncomp, "comps")
    names(PRESS0) <- respnames
    dimnames(adj) <- dimnames(PRESS) <-
        list(respnames, nCompnames)
    dimnames(cvPred) <- list(objnames, respnames, nCompnames)
    if (jackknife)
        dimnames(cvCoef) <- list(dnX[[2]], respnames, nCompnames,
                                 paste("Seg", seq_along(segments)))

    list(method = "CV", pred = cvPred, coefficients = if (jackknife) cvCoef,
         gammas = if (method == "cppls") gammas,
         PRESS0 = PRESS0, PRESS = PRESS, adj = adj / nobj^2,
         segments = segments, ncomp = ncomp)
}


## Genereral cross-validation function.


#' @title Cross-validation of PLSR and PCR models
#'
#' @description A \dQuote{stand alone} cross-validation function for \code{mvr} objects.
#'
#' @details This function performs cross-validation on a model fit by \code{mvr}.  It
#' can handle models such as \code{plsr(y ~ msc(X), \dots{})} or other models
#' where the predictor variables need to be recalculated for each segment.
#' When recalculation is not needed, the result of
#' \code{crossval(mvr(\dots{}))} is identical to \code{mvr(\dots{}, validation
#' = "CV")}, but slower.
#'
#' Note that to use \code{crossval}, the data \emph{must} be specified with a
#' \code{data} argument when fitting \code{object}.
#'
#' If \code{segments} is a list, the arguments \code{segment.type} and
#' \code{length.seg} are ignored.  The elements of the list should be integer
#' vectors specifying the indices of the segments.  See
#' \code{\link{cvsegments}} for details.
#'
#' Otherwise, segments of type \code{segment.type} are generated.  How many
#' segments to generate is selected by specifying the number of segments in
#' \code{segments}, or giving the segment length in \code{length.seg}.  If both
#' are specified, \code{segments} is ignored.
#'
#' If \code{jackknife} is \code{TRUE}, jackknifed regression coefficients are
#' returned, which can be used for for variance estimation
#' (\code{\link{var.jack}}) or hypothesis testing (\code{\link{jack.test}}).
#'
#' When tracing is turned on, the segment number is printed for each segment.
#'
#' By default, the cross-validation will be performed serially.  However, it
#' can be done in parallel using functionality in the \code{\link{parallel}}
#' package by setting the option \code{parallel} in \code{\link{pls.options}}.
#' See \code{\link{pls.options}} for the different ways to specify the
#' parallelism.  See also Examples below.
#'
#' @param object an \code{mvr} object; the regression to cross-validate.
#' @param segments the number of segments to use, or a list with segments (see
#' below).
#' @param segment.type the type of segments to use.  Ignored if \code{segments}
#' is a list.
#' @param length.seg Positive integer.  The length of the segments to use.  If
#' specified, it overrides \code{segments} unless \code{segments} is a list.
#' @param jackknife logical.  Whether jackknifing of regression coefficients
#' should be performed.
#' @param trace if \code{TRUE}, tracing is turned on.  If numeric, it denotes a
#' time limit (in seconds).  If the estimated total time of the
#' cross-validation exceeds this limit, tracing is turned on.
#' @param \dots additional arguments, sent to the underlying fit function.
#' @return The supplied \code{object} is returned, with an additional component
#' \code{validation}, which is a list with components \item{method}{euqals
#' \code{"CV"} for cross-validation.} \item{pred}{an array with the
#' cross-validated predictions.} \item{coefficients}{(only if \code{jackknife}
#' is \code{TRUE}) an array with the jackknifed regression coefficients.  The
#' dimensions correspond to the predictors, responses, number of components,
#' and segments, respectively.} \item{PRESS0}{a vector of PRESS values (one for
#' each response variable) for a model with zero components, i.e., only the
#' intercept.} \item{PRESS}{a matrix of PRESS values for models with 1,
#' \ldots{}, \code{ncomp} components.  Each row corresponds to one response
#' variable.} \item{adj}{a matrix of adjustment values for calculating bias
#' corrected MSEP.  \code{MSEP} uses this.} \item{segments}{the list of
#' segments used in the cross-validation.} \item{ncomp}{the number of
#' components.} \item{gammas}{if method \code{cppls} is used, gamma values for
#' the powers of each CV segment are returned.}
#' @note The \code{PRESS0} is always cross-validated using leave-one-out
#' cross-validation.  This usually makes little difference in practice, but
#' should be fixed for correctness.
#'
#' The current implementation of the jackknife stores all jackknife-replicates
#' of the regression coefficients, which can be very costly for large matrices.
#' This might change in a future version.
#' @author Ron Wehrens and Bjørn-Helge Mevik
#' @seealso \code{\link{mvr}} \code{\link{mvrCv}} \code{\link{cvsegments}}
#' \code{\link{MSEP}} \code{\link{var.jack}} \code{\link{jack.test}}
#' @references Mevik, B.-H., Cederkvist, H. R. (2004) Mean Squared Error of
#' Prediction (MSEP) Estimates for Principal Component Regression (PCR) and
#' Partial Least Squares Regression (PLSR).  \emph{Journal of Chemometrics},
#' \bold{18}(9), 422--429.
#' @keywords regression multivariate
#' @examples
#'
#' data(yarn)
#' yarn.pcr <- pcr(density ~ msc(NIR), 6, data = yarn)
#' yarn.cv <- crossval(yarn.pcr, segments = 10)
#' \dontrun{plot(MSEP(yarn.cv))}
#'
#' \dontrun{
#' ## Parallelised cross-validation, using transient cluster:
#' pls.options(parallel = 4) # use mclapply (not available on Windows)
#' pls.options(parallel = quote(parallel::makeCluster(4, type = "PSOCK"))) # parLapply
#' ## A new cluster is created and stopped for each cross-validation:
#' yarn.cv <- crossval(yarn.pcr)
#' yarn.loocv <- crossval(yarn.pcr, length.seg = 1)
#'
#' ## Parallelised cross-validation, using persistent cluster:
#' library(parallel)
#' ## This creates the cluster:
#' pls.options(parallel = makeCluster(4, type = "FORK")) # not available on Windows
#' pls.options(parallel = makeCluster(4, type = "PSOCK"))
#' ## The cluster can be used several times:
#' yarn.cv <- crossval(yarn.pcr)
#' yarn.loocv <- crossval(yarn.pcr, length.seg = 1)
#' ## The cluster should be stopped manually afterwards:
#' stopCluster(pls.options()$parallel)
#'
#' ## Parallelised cross-validation, using persistent MPI cluster:
#' ## This requires the packages snow and Rmpi to be installed
#' library(parallel)
#' ## This creates the cluster:
#' pls.options(parallel = makeCluster(4, type = "MPI"))
#' ## The cluster can be used several times:
#' yarn.cv <- crossval(yarn.pcr)
#' yarn.loocv <- crossval(yarn.pcr, length.seg = 1)
#' ## The cluster should be stopped manually afterwards:
#' stopCluster(pls.options()$parallel)
#' ## It is good practice to call mpi.exit() or mpi.quit() afterwards:
#' mpi.exit()
#' }
#'
#' @export
crossval <- function(object, segments = 10,
                     segment.type = c("random", "consecutive", "interleaved"),
                     length.seg, jackknife = FALSE, trace = 15, ...)
{
    if (!inherits(object, "mvr")) stop("`object' not an mvr object.")
    ## Get data frame
    fitCall <- object$call
    data <- eval(fitCall$data, parent.frame())
    if (is.null(data)) stop("`object' must be fit with a `data' argument.")
    ## Optionally get weights
    if (cppls <- (object$method == "cppls")) {
        weights <- eval(fitCall$weights, parent.frame())
    }
    else weights <- NULL

    if (!is.null(fitCall$subset)) {
        ## Handle "subset" argument
        data <- data[eval(fitCall$subset, parent.frame()),]
        object$call$subset <- NULL
    }

    ## Handle NAs (according to na.action)
    if (is.na(match("na.action", names(fitCall)))) {
        ## Cannot use is.null(fitCall$na.action) here, since the meaning of
        ## `na.action = NULL' is not the same as that of a missing na.action
        ## argument.
        mf <- model.frame(formula(object), data = data)
    } else {
        mf <- model.frame(formula(object), data = data,
                          na.action = fitCall$na.action)
    }
    if (!is.null(NAs <- attr(mf, "na.action"))) {
        ## Some observations were dropped due to NAs.  Skip the same in data:
        data <- data[-NAs,]
    }

    ## Get response:
    Y <- as.matrix(model.response(mf))
    nresp <- dim(Y)[2]
    npred <- length(object$Xmeans)
    ## Calculate effective number of observations
    nobj <- nrow(data)

    ## Set up segments
    if (is.list(segments)) {
        if (is.null(attr(segments, "type")))
            attr(segments, "type") <- "user supplied"
    } else {
        if (missing(length.seg)) {
            segments <- cvsegments(nobj, k = segments, type = segment.type)
        } else {
            segments <- cvsegments(nobj, length.seg = length.seg,
                                   type = segment.type)
        }
    }

    jackknife <- isTRUE(jackknife)
    ncomp <- object$ncomp
    if (ncomp > nobj - max(sapply(segments, length)) - 1)
        stop("`ncomp' too large for cross-validation.",
             "\nPlease refit with `ncomp' less than ",
             nobj - max(sapply(segments, length)))

    ## Optionally turn on tracing:
    if (is.numeric(trace)) {
        trace <- object$fit.time * length(segments) > trace
    }

    ## Helper function to perform the cross-validatoin for one segment.
    ## Defined inside crossval to be able to access local variables:
    crossvalSeg <- function(n.seg) {
        if (trace) cat(n.seg, "")

        ## Run cv, using update and predict
        seg <- segments[[n.seg]]
        fit <- update(object, data = data[-seg,], weights = weights[-seg])
        pred <- predict(fit, newdata = data)

        return(list(adj = length(seg) * colSums((pred - c(Y))^2),
                    cvPred = pred[seg,,, drop=FALSE],
                    gammas = if (cppls) fit$gammas else NULL,
                    cvCoef = if (jackknife) fit$coefficients else NULL
                    ))
    }

    ## Perform the cross-validation, optionally in parallel:
    if (trace) cat("Segment: ")
    results <- lapplyFunc(pls.options()$parallel,
                          seq_along(segments), crossvalSeg,
                          quote(parallel::clusterCall(parSpec, library, "pls",
                                                      character.only = TRUE,
                                                      warn.conflicts = FALSE)))
    if (trace) cat("\n")

    ## Variables to save CV results in:
    cvPred <- array(dim = c(nobj, nresp, ncomp))
    adj <- matrix(0, nrow = nresp, ncol = ncomp)
    if (jackknife)
        cvCoef <- array(dim = c(npred, nresp, ncomp, length(segments)))
    if (cppls) gammas <- list()

    ## Collect the results:
    for (n.seg in seq_along(segments)) {
        res <- results[[n.seg]]
        adj <- adj + res$adj
        cvPred[segments[[n.seg]],,] <- res$cvPred
        if (jackknife) cvCoef[,,,n.seg] <- res$cvCoef
        if (cppls) gammas[[n.seg]] <- res$gammas
    }

    ## Calculate validation statistics:
    PRESS0 <- apply(Y, 2, var) * nobj^2 / (nobj - 1) # FIXME: Only correct for loocv!
    PRESS <- colSums((cvPred - c(Y))^2)

    ## Add dimnames:
    objnames <- rownames(data)
    if (is.null(objnames)) objnames <- rownames(Y)
    dimnames(cvPred) <- c(list(objnames), dimnames(fitted(object))[-1])
    if (is.null(names(PRESS0))) names(PRESS0) <- dimnames(object$Yloadings)[[1]]
    dimnames(PRESS) <- dimnames(adj)
    if (jackknife)
        dimnames(cvCoef) <- c(dimnames(coef(object)),
                              list(paste("Seg", seq_along(segments))))

    ## Return the original object, with a component `validation' added
    object$validation <- list(method = "CV", pred = cvPred,
                              coefficients = if (jackknife) cvCoef,
                              gammas = if (cppls) gammas,
                              PRESS0 = PRESS0, PRESS = PRESS,
                              adj = adj / nobj^2,
                              segments = segments, ncomp = ncomp)
    return(object)
}

## Internal function to apply FUN over X, optionally in parallel:
lapplyFunc <- function(parSpec, X, FUN, nonForkInit) {
    if (is.null(parSpec) || (is.numeric(parSpec) && parSpec == 1)) {
        ## Serially
        results <- lapply(X, FUN)
    } else {
        ## Parallel
        stop_cluster <- FALSE           # Whether to kill the workers afterwards

        if (is.numeric(parSpec) && parSpec > 1) {
            ## Number => number of workers with mclapply
            results <- parallel::mclapply(X, FUN, mc.cores = parSpec)
        } else {
            if (is.call(parSpec)) {
                ## Unevaluated call => evaluate it to create the cluster:
                parSpec <- eval(parSpec)
                stop_cluster <- TRUE
            }

            if (inherits(parSpec, "cluster")) {
                ## Run library(pls) on cluster if type != FORK
                if (!inherits(parSpec[[1]], "forknode")
                    && !missing(nonForkInit)) {
                    eval(nonForkInit)
                }
                results <- parallel::parLapply(parSpec, X, FUN)

                if (stop_cluster) {
                    parallel::stopCluster(parSpec)
                }
            } else {
                stop("Unknown parallelity specification: '", parSpec, "'")
            }
        }
    }

    return(results)
}

### runit.CV.R: test functions for checking cross-validation functions
### By Bjørn-Helge Mevik
### Started 2007-10-18

## Parallel computations, crossval:
test.crossval.parallel <- function() {
    ## Make sure we start with a `clean slate'
    pls.options(parallel = NULL)

    ## Create a baseline version to compare the parallel results with:
    plsmod <- plsr(octane ~ NIR, ncomp = 5, data = gasoline)
    serial <- crossval(plsmod, length.seg = 1, segment.type = "cons")

    ## mclapply:
    if (.Platform$OS.type != "windows") {
        pls.options(parallel = 2)
        parallel <- crossval(plsmod, length.seg = 1, segment.type = "cons")
        checkEquals(serial, parallel, "mclapply")
    }

    ## Auto-created FORK cluster:
    if (.Platform$OS.type != "windows") {
        pls.options(parallel = quote(parallel::makeCluster(2, type = "FORK")))
        parallel <- crossval(plsmod, length.seg = 1, segment.type = "cons")
        checkEquals(serial, parallel, "parLapply, auto-created FORK cluster")
    }

    ## Auto-created PSOCK cluster:
    pls.options(parallel = quote(parallel::makeCluster(2, type = "PSOCK")))
    parallel <- crossval(plsmod, length.seg = 1, segment.type = "cons")
    checkEquals(serial, parallel, "parLapply, auto-created PSOCK cluster")

    ## The rest of the tests use parallel functions directly:
    require(parallel)

    ## Permanent FORK cluster:
    if (.Platform$OS.type != "windows") {
        pls.options(parallel = makeCluster(2, type = "FORK"))
        parallel <- crossval(plsmod, length.seg = 1, segment.type = "cons")
        stopCluster(pls.options()$parallel)
        checkEquals(serial, parallel, "parLapply, permanent FORK cluster")
    }

    ## Permanent PSOCK cluster:
    pls.options(parallel = makeCluster(2, type = "PSOCK"))
    parallel <- crossval(plsmod, length.seg = 1, segment.type = "cons")
    stopCluster(pls.options()$parallel)
    checkEquals(serial, parallel, "parLapply, permanent PSOCK cluster")

    ## Clean up.  Note: this will not run if any check above fails:
    pls.options(parallel = NULL)
}

## Parallel computations, calling pls:::mvrCv directly:
test.mvrCv.parallel <- function() {
    ## Make sure we start with a `clean slate'
    pls.options(parallel = NULL)

    ## Create a baseline version to compare the parallel results with:
    serial <- pls:::mvrCv(gasoline$NIR, gasoline$octane, 5,
                          length.seg = 1, segment.type = "cons")

    ## mclapply:
    if (.Platform$OS.type != "windows") {
        pls.options(parallel = 2)
        parallel <- pls:::mvrCv(gasoline$NIR, gasoline$octane, 5,
                                length.seg = 1, segment.type = "cons")
        checkEquals(serial, parallel, "mclapply")
    }

    ## Auto-created FORK cluster:
    if (.Platform$OS.type != "windows") {
        pls.options(parallel = quote(parallel::makeCluster(2, type = "FORK")))
        parallel <- pls:::mvrCv(gasoline$NIR, gasoline$octane, 5,
                                length.seg = 1, segment.type = "cons")
        checkEquals(serial, parallel, "parLapply, auto-created FORK cluster")
    }

    ## Auto-created PSOCK cluster:
    pls.options(parallel = quote(parallel::makeCluster(2, type = "PSOCK")))
    parallel <- pls:::mvrCv(gasoline$NIR, gasoline$octane, 5,
                            length.seg = 1, segment.type = "cons")
    checkEquals(serial, parallel, "parLapply, auto-created PSOCK cluster")

    ## The rest of the tests use parallel functions directly:
    require(parallel)

    ## Permanent FORK cluster:
    if (.Platform$OS.type != "windows") {
        pls.options(parallel = makeCluster(2, type = "FORK"))
        parallel <- pls:::mvrCv(gasoline$NIR, gasoline$octane, 5,
                                length.seg = 1, segment.type = "cons")
        stopCluster(pls.options()$parallel)
        checkEquals(serial, parallel, "parLapply, permanent FORK cluster")
    }

    ## Permanent PSOCK cluster:
    pls.options(parallel = makeCluster(2, type = "PSOCK"))
    parallel <- pls:::mvrCv(gasoline$NIR, gasoline$octane, 5,
                            length.seg = 1, segment.type = "cons")
    stopCluster(pls.options()$parallel)
    checkEquals(serial, parallel, "parLapply, permanent PSOCK cluster")

    ## Clean up.  Note: this will not run if any check above fails:
    pls.options(parallel = NULL)
}


## A fake test function to make sure parallelism is turned off after
## the test.*.parallel tests:
test.parallel.cleanup <- function() {
    pls.options(parallel = NULL)
}

### predict.mvr.R: A predict method



#' @title Predict Method for PLSR and PCR
#'
#' @description Prediction for mvr (PCR, PLSR) models.  New responses or scores are
#' predicted using a fitted model and a new matrix of observations.
#'
#' @details When \code{type} is \code{"response"} (default), predicted response values
#' are returned.  If \code{comps} is missing (or is \code{NULL}), predictions
#' for \code{length(ncomp)} models with \code{ncomp[1]} components,
#' \code{ncomp[2]} components, etc., are returned.  Otherwise, predictions for
#' a single model with the exact components in \code{comps} are returned.
#' (Note that in both cases, the intercept is always included in the
#' predictions.  It can be removed by subtracting the \code{Ymeans} component
#' of the fitted model.)
#'
#' When \code{type} is \code{"scores"}, predicted score values are returned for
#' the components given in \code{comps}.  If \code{comps} is missing or
#' \code{NULL}, \code{ncomps} is used instead.
#'
#' It is also possible to supply a matrix instead of a data frame as
#' \code{newdata}, which is then assumed to be the \eqn{X} data matrix.  Note
#' that the usual checks for the type of the data are then omitted.  Also note
#' that this is \emph{only} possible with \code{predict}; it will not work in
#' functions like \code{\link{predplot}}, \code{\link{RMSEP}} or
#' \code{\link{R2}}, because they also need the response variable of the new
#' data.
#'
#' @param object an \code{mvr} object.  The fitted model
#' @param newdata a data frame.  The new data.  If missing, the training data
#' is used.
#' @param ncomp,comps vector of positive integers.  The components to use in
#' the prediction.  See below.
#' @param type character.  Whether to predict scores or response values
#' @param na.action function determining what should be done with missing
#' values in \code{newdata}.  The default is to predict \code{NA}.  See
#' \code{\link{na.omit}} for alternatives.
#' @param \dots further arguments.  Currently not used
#' @return When \code{type} is \code{"response"}, a three dimensional array of
#' predicted response values is returned.  The dimensions correspond to the
#' observations, the response variables and the model sizes, respectively.
#'
#' When \code{type} is \code{"scores"}, a score matrix is returned.
#' @note A warning message like \samp{'newdata' had 10 rows but variable(s)
#' found have 106 rows} means that not all variables were found in the
#' \code{newdata} data frame.  This (usually) happens if the formula contains
#' terms like \code{yarn$NIR}.  Do not use such terms; use the \code{data}
#' argument instead.  See \code{\link{mvr}} for details.
#' @author Ron Wehrens and Bjørn-Helge Mevik
#' @seealso \code{\link{mvr}}, \code{\link{summary.mvr}},
#' \code{\link{coef.mvr}}, \code{\link{plot.mvr}}
#' @keywords regression multivariate
#' @examples
#'
#' data(yarn)
#' nir.mvr <- mvr(density ~ NIR, ncomp = 5, data = yarn[yarn$train,])
#'
#' ## Predicted responses for models with 1, 2, 3 and 4 components
#' pred.resp <- predict(nir.mvr, ncomp = 1:4, newdata = yarn[!yarn$train,])
#'
#' ## Predicted responses for a single model with components 1, 2, 3, 4
#' predict(nir.mvr, comps = 1:4, newdata = yarn[!yarn$train,])
#'
#' ## Predicted scores
#' predict(nir.mvr, comps = 1:3, type = "scores", newdata = yarn[!yarn$train,])
#'
#' @export
predict.mvr <- function(object, newdata, ncomp = 1:object$ncomp, comps,
                        type = c("response", "scores"),
                        na.action = na.pass, ...)
{
    if (missing(newdata) || is.null(newdata))
        newX <- model.matrix(object)
    else if (is.matrix(newdata)) {
        ## For matrices, simply check dimension:
        if (ncol(newdata) != length(object$Xmeans))
            stop("'newdata' does not have the correct number of columns")
        newX <- newdata
    } else {
        Terms <- delete.response(terms(object))
        m <- model.frame(Terms, newdata, na.action = na.action)
        if (!is.null(cl <- attr(Terms, "dataClasses")))
            .checkMFClasses(cl, m)
        newX <- delete.intercept(model.matrix(Terms, m))
    }

    nobs <- dim(newX)[1]

    ## Perform any scaling:
    if (!is.null(object$scale)) newX <- newX / rep(object$scale, each = nobs)
    type <- match.arg(type)
    if (type == "response") {
        if (missing(comps) || is.null(comps)) {
            ## Predict with models containing ncomp[1] components,
            ## ncomp[2] components, etc.
            if (missing(newdata)) return(fitted(object)[,,ncomp, drop=FALSE])
            B <- coef(object, ncomp = ncomp, intercept = TRUE)
            dPred <- dim(B)
            dPred[1] <- dim(newX)[1]
            dnPred <- dimnames(B)
            dnPred[1] <-
                if(is.null(dimnames(newX))) list(NULL) else dimnames(newX)[1]
            pred <- array(dim = dPred, dimnames = dnPred)
            for (i in seq(along = ncomp))
                pred[,,i] <- newX %*% B[-1,,i] + rep(B[1,,i], each = nobs)
            return(pred)
        } else {
            ## Predict with a model containing the components `comps'
            B <- rowSums(coef(object, comps = comps), dims = 2)
            B0 <- object$Ymeans - object$Xmeans %*% B
            pred <- newX %*% B + rep(B0, each = nobs)
            if (missing(newdata) && !is.null(object$na.action))
                pred <- napredict(object$na.action, pred)
            return(pred)
        }
    } else {
        ## Return predicted scores (for scores, `cumulative' has no meaning)
        ## When predicting scores, we allow ncomp as an alias for comps:
        if (missing(comps) || is.null(comps)) comps <- ncomp
        if (missing(newdata)) {
            TT <- object$scores[,comps]
            if (!is.null(object$na.action))
                TT <- napredict(object$na.action, TT)
        } else {
            if (is.null(object$projection))
                stop("`object' has no `projection' component.  Maybe it was fitted with `stripped = TRUE'.")
            TT <- (newX - rep(object$Xmeans, each = nobs)) %*%
                object$projection[,comps]
        }
        return(TT)
    }
}

### cppls.fit.R: The Canonical Powered PLS algorithm
###
### Implements the CPPLS algorithm as described in
### Indahl, U.G., Liland, K.H., Næs, T. (2009).
### Canonical partial least squares - a unified PLS approach to classification and regression problems,
### Journal of Chemometrics 23, pp. 495-504



#' @title CPPLS (Indahl et al.)
#'
#' @description Fits a PLS model using the CPPLS algorithm.
#'
#' @details This function should not be called directly, but through the generic
#' functions \code{cppls} or \code{mvr} with the argument
#' \code{method="cppls"}. Canonical Powered PLS (CPPLS) is a generalisation of
#' PLS incorporating discrete and continuous responses (also simultaneously),
#' additional responses, individual weighting of observations and power
#' methodology for sharpening focus on groups of variables. Depending on the
#' input to \code{cppls} it can produce the following special cases: \itemize{
#' \item PLS: uni-response continuous \code{Y} \item PPLS: uni-response
#' continuous \code{Y}, \code{(lower || upper) != 0.5} \item PLS-DA (using
#' correlation maximisation - B/W): dummy-coded descrete response \code{Y}
#' \item PPLS-DA: dummy-coded descrete response \code{Y}, \code{(lower ||
#' upper) != 0.5} \item CPLS: multi-response \code{Y} (continuous, discrete or
#' combination) \item CPPLS: multi-response \code{Y} (continuous, discrete or
#' combination), \code{(lower || upper) != 0.5} } The name "canonical" comes
#' from canonical correlation analysis which is used when calculating vectors
#' of loading weights, while "powered" refers to a reparameterisation of the
#' vectors of loading weights which can be optimised over a given interval.
#'
#' @param X a matrix of observations.  \code{NA}s and \code{Inf}s are not
#' allowed.
#' @param Y a vector or matrix of responses.  \code{NA}s and \code{Inf}s are
#' not allowed.
#' @param ncomp the number of components to be used in the modelling.
#' @param Y.add a vector or matrix of additional responses containing relevant
#' information about the observations.
#' @param center logical, determines if the \eqn{X} and \eqn{Y} matrices are
#' mean centered or not. Default is to perform mean centering.
#' @param stripped logical.  If \code{TRUE} the calculations are stripped as
#' much as possible for speed; this is meant for use with cross-validation or
#' simulations when only the coefficients are needed.  Defaults to
#' \code{FALSE}.
#' @param lower a vector of lower limits for power optimisation. Defaults to
#' \code{0.5}.
#' @param upper a vector of upper limits for power optimisation. Defaults to
#' \code{0.5}.
#' @param trunc.pow logical. If \code{TRUE} an experimental alternative power
#' algorithm is used. (Optional)
#' @param weights a vector of individual weights for the observations.
#' (Optional)
#' @param \dots other arguments.  Currently ignored.
#' @return A list containing the following components is returned:
#' \item{coefficients}{an array of regression coefficients for 1, \ldots{},
#' \code{ncomp} components.  The dimensions of \code{coefficients} are
#' \code{c(nvar, npred, ncomp)} with \code{nvar} the number of \code{X}
#' variables and \code{npred} the number of variables to be predicted in
#' \code{Y}.} \item{scores}{a matrix of scores.} \item{loadings}{a matrix of
#' loadings.} \item{loading.weights}{a matrix of loading weights.}
#' \item{Yscores}{a matrix of Y-scores.} \item{Yloadings}{a matrix of
#' Y-loadings.} \item{projection}{the projection matrix used to convert X to
#' scores.} \item{Xmeans}{a vector of means of the X variables.}
#' \item{Ymeans}{a vector of means of the Y variables.} \item{fitted.values}{an
#' array of fitted values.  The dimensions of \code{fitted.values} are
#' \code{c(nobj, npred, ncomp)} with \code{nobj} the number samples and
#' \code{npred} the number of Y variables.} \item{residuals}{an array of
#' regression residuals.  It has the same dimensions as \code{fitted.values}.}
#' \item{Xvar}{a vector with the amount of X-variance explained by each
#' component.} \item{Xtotvar}{total variance in \code{X}.}
#' \item{gammas}{gamma-values obtained in power optimisation.}
#' \item{canonical.correlations}{Canonical correlation values from the
#' calculations of loading weights.} \item{A}{matrix containing vectors of
#' weights \code{a} from canonical correlation (\code{cor(Za,Yb)}).}
#' \item{smallNorms}{vector of indices of explanatory variables of length close
#' to or equal to 0.}
#'
#' If \code{stripped} is \code{TRUE}, only the components \code{coefficients},
#' \code{Xmeans}, \code{Ymeans} and \code{gammas} are returned.
#' @author Kristian Hovde Liland
#' @seealso \code{\link{mvr}} \code{\link{plsr}} \code{\link{pcr}}
#' \code{\link{widekernelpls.fit}} \code{\link{simpls.fit}}
#' \code{\link{oscorespls.fit}}
#' @references Indahl, U. (2005) A twist to partial least squares regression.
#' \emph{Journal of Chemometrics}, \bold{19}, 32--44.
#'
#' Liland, K.H and Indahl, U.G (2009) Powered partial least squares
#' discriminant analysis, \emph{Journal of Chemometrics}, \bold{23}, 7--18.
#'
#' Indahl, U.G., Liland, K.H. and Næs, T. (2009) Canonical partial least
#' squares - a unified PLS approach to classification and regression problems.
#' \emph{Journal of Chemometrics}, \bold{23}, 495--504.
#' @keywords regression classification multivariate
#' @examples
#'
#' data(mayonnaise)
#' # Create dummy response
#' mayonnaise$dummy <-
#'     I(model.matrix(~y-1, data.frame(y = factor(mayonnaise$oil.type))))
#'
#' # Predict CPLS scores for test data
#' may.cpls <- cppls(dummy ~ NIR, 10, data = mayonnaise, subset = train)
#' may.test <- predict(may.cpls, newdata = mayonnaise[!mayonnaise$train,], type = "score")
#'
#' # Predict CPLS scores for test data (experimental used design as additional Y information)
#' may.cpls.yadd <- cppls(dummy ~ NIR, 10, data = mayonnaise, subset = train, Y.add=design)
#' may.test.yadd <- predict(may.cpls.yadd, newdata = mayonnaise[!mayonnaise$train,], type = "score")
#'
#' # Classification by linear discriminant analysis (LDA)
#' library(MASS)
#' error <- matrix(ncol = 10, nrow = 2)
#' dimnames(error) <- list(Model = c('CPLS', 'CPLS (Y.add)'), ncomp = 1:10)
#' for (i in 1:10) {
#'     fitdata1 <- data.frame(oil.type = mayonnaise$oil.type[mayonnaise$train],
#'                            NIR.score = I(may.cpls$scores[,1:i,drop=FALSE]))
#'     testdata1 <- data.frame(oil.type = mayonnaise$oil.type[!mayonnaise$train],
#'                             NIR.score = I(may.test[,1:i,drop=FALSE]))
#'     error[1,i] <-
#'         (42 - sum(predict(lda(oil.type ~ NIR.score, data = fitdata1),
#'                   newdata = testdata1)$class == testdata1$oil.type)) / 42
#'     fitdata2 <- data.frame(oil.type = mayonnaise$oil.type[mayonnaise$train],
#'                            NIR.score = I(may.cpls.yadd$scores[,1:i,drop=FALSE]))
#'     testdata2 <- data.frame(oil.type = mayonnaise$oil.type[!mayonnaise$train],
#'                             NIR.score = I(may.test.yadd[,1:i,drop=FALSE]))
#'     error[2,i] <-
#'         (42 - sum(predict(lda(oil.type ~ NIR.score, data = fitdata2),
#'                   newdata = testdata2)$class == testdata2$oil.type)) / 42
#' }
#' round(error,2)
#'
#' @export
cppls.fit <- function(X, Y, ncomp, Y.add = NULL, center = TRUE,
                      stripped = FALSE, lower = 0.5, upper = 0.5,
                      trunc.pow = FALSE, weights = NULL, ...)
{
    ## X       - the data matrix
    ## Y       - the primary response matrix
    ## Y.add   - the additional response matrix (optional)
    ## ncomp   - number of components
    ## lower   - lower bounds for power algorithm (default=0.5)
    ## upper   - upper bounds for power algorithm (default=0.5)
    ## weights - prior weighting of observations (optional)

    Y.orig <- Yprim <- as.matrix(Y)
    Y <- cbind(Yprim, Y.add)

    if (!stripped) {
        ## Save dimnames:
        dnX <- dimnames(X)
        dnY <- dimnames(Yprim)
    }
    dimnames(X) <- dimnames(Y) <- dimnames(Yprim) <- NULL

    nobj  <- dim(X)[1]
    npred <- dim(X)[2]
    nresp <- dim(Yprim)[2]

    ## Center variables:
    if (center) {
        if (is.null(weights)) {
            Xmeans <- colMeans(X)
            X <- X - rep(Xmeans, each = nobj)
        } else {
            Xmeans <- c(crossprod(weights,X) / sum(weights))
            X <- X - rep(Xmeans, each = nobj)
        }
        Ymeans <- colMeans(Yprim)
        Yprim <- Yprim - rep(Ymeans, each = nobj)
        Y <- Y - rep(colMeans(Y), each = nobj)
    } else  {
        ## Set means to zero. Will ensure that predictions do not take the
        ## mean into account.
        Xmeans <- rep_len(0, npred)
        Ymeans <- rep_len(0, nresp)
    }

    X.orig <- X

    ## Declaration of variables
    W   <- matrix(0, npred, ncomp)    # W-loadings
    TT  <- matrix(0, nobj,  ncomp)    # T-scores
    P   <- matrix(0, npred, ncomp)    # P-loadings
    Q   <- matrix(0, nresp, ncomp)    # Q-loadings
    A   <- matrix(0, dim(Y)[2], ncomp)# Column weights for W0 (from CCA)
    cc  <- numeric(ncomp)
    pot <- rep(0.5, ncomp)           # Powers used to construct the w-s in R
    B   <- array(0, c(npred, nresp, ncomp))
    smallNorm <- numeric()
    if (!stripped) {
        U <- TT                     # U-scores
        tsqs <- rep.int(1, ncomp)   # t't
        fitted <- array(0, c(nobj, nresp, ncomp))
    }

    for (a in 1:ncomp) {
        if (length(lower) == 1 && lower == 0.5 && length(upper) == 1 && upper == 0.5 ) {
            Rlist <- Rcal(X, Y, Yprim, weights) # Default CPLS algorithm
        } else {
            Rlist <- RcalP(X, Y, Yprim, weights, lower, upper, trunc.pow) # Alternate CPPLS algorithm
            pot[a] <- Rlist$pot
        }
        cc[a] <- Rlist$cc
        w.a <- Rlist$w
        ifelse(!is.null(Rlist$a), aa <- Rlist$a, aa <- NA)
        A[,a] <- aa

        ## Make new vectors orthogonal to old ones?
        ## w.a <- w.a - W[,1:(a-1)]%*%crossprod(W[,1:(a-1)], w.a)
        w.a[abs(w.a) < pls.options()$w.tol] <- 0   # Removes insignificant values
        w.a <- w.a / norm(w.a)                     # Normalization
        t.a <- X %*% w.a                           # Score vectors
        tsq <- crossprod(t.a)[1]
        p.a <- crossprod(X,t.a) / tsq
        q.a <- crossprod(Yprim,t.a) / tsq
        X   <- X - tcrossprod(t.a,p.a)             # Deflation

        ## Check and compensate for small norms
        mm <- apply(abs(X), 2, sum)
        r <- which(mm < pls.options()$X.tol)
        if (length(r) > 0) {
            for (i in 1:length(r)) {
                if (sum(smallNorm == r[i]) == 0) {
                    ## Add new short small to list
                    smallNorm[length(smallNorm) + 1] <- r[i]
                }
            }
        }
        X[,smallNorm] <- 0 # Remove collumns having small norms

        W[,a]  <- w.a
        TT[,a] <- t.a
        P[,a]  <- p.a
        Q[,a]  <- q.a
        B[,,a] <- W[,1:a, drop=FALSE] %*%
            tcrossprod(
                solve(crossprod(P[,1:a, drop=FALSE], W[,1:a, drop=FALSE])),
                Q[,1:a, drop=FALSE]
            )

        if (!stripped) {
            tsqs[a] <- tsq
            ## Extra step to calculate Y scores:
            U[,a] <- Yprim %*% q.a / crossprod(q.a)[1] # Ok for nresp == 1 ??
            ## make u orth to previous X scores:
            if (a > 1) U[,a] <- U[,a] - TT %*% (crossprod(TT, U[,a]) / tsqs)
            fitted[,,a] <- X.orig %*% B[,,a]
        }
    }
    if (stripped) {
        ## Return as quickly as possible
        list(coefficients = B, Xmeans = Xmeans, Ymeans = Ymeans, gammas = pot)
    } else {
        fitted <- fitted + rep(Ymeans, each = nobj) # Add mean
        residuals <- - fitted + c(Y.orig)

        ## Add dimnames:
        objnames <- dnX[[1]]
        if (is.null(objnames)) objnames <- dnY[[1]]
        prednames <- dnX[[2]]
        respnames <- dnY[[2]]
        compnames <- paste("Comp", 1:ncomp)
        nCompnames <- paste(1:ncomp, "comps")
        dimnames(TT) <- list(objnames, compnames)
        dimnames(W) <- dimnames(P) <-
            list(prednames, compnames)
        dimnames(Q) <- list(respnames, compnames)
        dimnames(B) <- list(prednames, respnames, nCompnames)
        dimnames(fitted) <- dimnames(residuals) <-
            list(objnames, respnames, nCompnames)
        colnames(A) <- compnames
        class(TT) <- "scores"
        class(P) <- class(W) <- class(Q) <- "loadings"

        list(coefficients = B,
             scores = TT, loadings = P,
             loading.weights = W,
             Yscores = U, Yloadings = Q,
             projection = W %*% solve(crossprod(P,W)),
             Xmeans = Xmeans, Ymeans = Ymeans,
             fitted.values = fitted, residuals = residuals,
             Xvar = colSums(P * P) * tsqs,
             Xtotvar = sum(X.orig * X.orig),
             gammas = pot,
             canonical.correlations = cc,
             smallNorm = smallNorm,
             A = A, trunc.pow = trunc.pow)
    }
}


#######################
## Rcal function (CPLS)
Rcal <- function(X, Y, Yprim, weights) {
    W0 <- crossprod(X,Y)
    Ar <- cancorr(X %*% W0, Yprim, weights, FALSE) # Computes canonical correlations between columns in XW and Y with rows weighted according to 'weights'
    w  <- W0 %*% Ar$A[,1, drop=FALSE]  # Optimal loadings
    ifelse(exists('Ar'), a <- Ar$A[,1], a <- NA)
    list(w = w, cc = Ar$r^2, a = a)
}


#########################
## RcalP function (CPPLS)
RcalP <- function(X, Y, Yprim, weights, lower, upper, trunc.pow) {
    CS <- CorrXY(X, Y, weights)     # Matrix of corr(Xj,Yg) and vector of std(Xj)
    sng <- sign(CS$C)               # Signs of C {-1,0,1}
    C <- abs(CS$C)                  # Correlation without signs
    mS <- max(CS$S); S <- CS$S / mS # Divide by largest value
    mC <- max(C); C <- C / mC       #  -------- || --------

    ## Computation of the best vector of loadings
    lw <- lw_bestpar(X, S, C, sng, Yprim, weights, lower, upper, trunc.pow)
}


################
## lw_bestpar function
lw_bestpar <- function(X, S, C, sng, Yprim, weights, lower, upper,
                       trunc.pow)
{
    if (!is.null(weights))
        weights <- sqrt(weights) # Prepare weights for cca
    ## Compute for S and each columns of C the distance from the median scaled to [0,1]
    if (trunc.pow) {
        medC <- t(abs(t(sng * C) - apply(sng * C, 2, median)))
        medC <- t(t(medC) / apply(medC, 2, max))
        medS <- abs(S - median(S))
        medS <- medS / max(medS)
    } else {
        medS <- medC <- NULL
    }

    #########################
    # Optimization function #
    #########################
    f <- function(p, X, S, C, sng, Yprim, weights, trunc.pow, medS, medC) {
        if(p == 0) {        # Variable selection from standard deviation
            S[S < max(S)] <- 0
            W0 <- S
        } else if(p == 1) { # Variable selection from correlation
            C[C < max(C)] <- 0
            W0 <- rowSums(C)
        } else {            # Standard deviation and correlation with powers
            if (trunc.pow) {
                ps <- (1 - p) / p
                if (ps < 1) {
                    S <- S^ps
                } else {
                    S[medS < (1 - 2 * p)] <- 0
                }
                pc <- p / (1 - p)
                if (pc < 1) {
                    W0 <- (sng * (C^pc)) * S
                } else {
                    C[medC < (2 * p - 1)] <- 0
                    W0 <- (sng * C) * S
                }
            } else {
                S <- S^((1 - p) / p)
                W0 <- (sng * (C^(p / (1 - p)))) * S
            }
        }
        Z <- X %*% W0  # Transform X into W0
        -(cancorr(Z, Yprim, weights))^2
    }

    #####################################
    # Logic for optimization segment(s) #
    #####################################
    nOpt <- length(lower)
    pot  <- numeric(3 * nOpt)
    ca   <- numeric(3 * nOpt)

    for (i in 1:nOpt) {
        ca[ 1 + (i - 1) * 3] <-
            f(lower[i], X, S, C, sng, Yprim, weights, trunc.pow, medS, medC)
        pot[1 + (i - 1) * 3] <- lower[i]
        if (lower[i] != upper[i]) {
            Pc <- optimize(f = f, interval = c(lower[i], upper[i]),
                           tol = 10^-4, maximum = FALSE,
                           X = X, S = S, C = C, sng = sng, Yprim = Yprim,
                           weights = weights, trunc.pow = trunc.pow, medS, medC)
            pot[2 + (i - 1) * 3] <- Pc[[1]]
            ca[ 2 + (i - 1) * 3] <- Pc[[2]]
        }
        ca[ 3 + (i - 1) * 3] <-
            f(upper[i], X, S, C, sng, Yprim, weights, trunc.pow, medS, medC)
        pot[3 + (i - 1) * 3] <- upper[i]
    }


    ########################################################
    # Computation of final w-vectors based on optimization #
    ########################################################
    cc <- max(-ca)                      # Determine which is more succesful
    cmin <- which.max(-ca)              # Determine which is more succesful
    if (pot[cmin] == 0) {        # Variable selection from standard deviation
        S[S < max(S)] <- 0
        w <- S
    } else if (pot[cmin] == 1) { # Variable selection from correlation
        C[C < max(C)] <- 0
        w <- rowSums(C)
    } else {                     # Standard deviation and correlation with powers
        p <- pot[cmin]           # Power from optimization
        if (trunc.pow) {         # New power algorithm
            ps <- (1 - p) / p
            if (ps < 1) {
                S <- S^ps
            } else {
                S[medS < (1 - 2 * p)] <- 0
            }
            pc <- p / (1 - p)
            if (pc < 1) {
                W0 <- (sng * (C^pc)) * S
            } else {
                C[medC < (2 * p-1)] <- 0
                W0 <- (sng * C) * S
            }
        } else {
            S <- S^((1 - p) / p)
            W0 <- (sng * (C^(p / (1 - p)))) * S
        }

        Z <- X %*% W0                   # Transform X into W
        Ar <- cancorr(Z, Yprim, weights, FALSE) # Computes canonical correlations between columns in XW and Y with rows weighted according to 'weights'
        w <- W0 %*% Ar$A[,1, drop=FALSE]  # Optimal loadings
    }
    pot <- pot[cmin]
    ifelse(exists('Ar'), a <- Ar$A[,1], a <- NA)
    list(w = w, pot = pot, cc = cc, a = a)
}


################
## CorrXY function
CorrXY <- function(X, Y, weights) {
    ##  Computation of correlations between the columns of X and Y
    n  <- dim(X)[1]
    if (is.null(weights)) {
        cx <- colMeans(X)
        cy <- colMeans(Y)
        X <- X - rep(cx, each = n)
        Y <- Y - rep(cy, each = n)
    } else {
        cx <- crossprod(weights,X) / sum(weights)
        cy <- crossprod(weights,Y) / sum(weights)
        X  <- X - rep(cx, each = n)
        Y  <- Y - rep(cy, each = n)
        X  <- X * weights
        Y  <- Y * weights
    }

    sdX <- sqrt(apply(X^2, 2, mean))
    inds <- which(sdX == 0, arr.ind = FALSE)
    sdX[inds] <- 1

    ccxy <- crossprod(X, Y) / (n * tcrossprod(sdX, sqrt(apply(Y^2, 2, mean))))
    sdX[ inds ] <- 0
    ccxy[inds,] <- 0
    list(C = ccxy, S = sdX)
}


################
## function norm
norm <- function(vec) {
    sqrt(crossprod(vec)[1])
}


################
## Stripped version of canonical correlation (cancor)
cancorr <- function (x, y, weights, opt = TRUE) {
    nr  <- nrow(x)
    ncx <- ncol(x)
    ncy <- ncol(y)
    if (!is.null(weights)) {
        x <- x * c(weights)
        y <- y * c(weights)
    }
    qx <- qr(x, LAPACK = TRUE)
    qy <- qr(y, LAPACK = TRUE)
    qxR <- qr.R(qx)
    ## Compute rank like MATLAB does
    dx <- sum(abs(diag(qxR)) >
              .Machine$double.eps * 2^floor(log2(abs(qxR[1]))) * max(nr, ncx))
    if (!dx)
        stop("'x' has rank 0")
    qyR <- qr.R(qy)
    ## Compute rank like MATLAB does
    dy <- sum(abs(diag(qyR)) >
              .Machine$double.eps * 2^floor(log2(abs(qyR[1]))) * max(nr, ncy))
    if (!dy)
        stop("'y' has rank 0")
    dxy <- min(dx, dy)
    if (opt) {
        z <- svd(qr.qty(qx, qr.qy(qy, diag(1, nr, dy)))[1:dx,, drop = FALSE],
                 nu = 0, nv = 0)
        ret <- max(min(z$d[1], 1), 0)
    } else {
        z <- svd(qr.qty(qx, qr.qy(qy, diag(1, nr, dy)))[1:dx,, drop = FALSE],
                 nu = dxy, nv = 0)
        A <- backsolve((qx$qr)[1:dx,1:dx, drop = FALSE], z$u) * sqrt(nr - 1)
        if ((ncx - nrow(A)) > 0) {
            A <- rbind(A, matrix(0, ncx - nrow(A), dxy))
        }
        A[qx$pivot,] <- A
        ret <- list(A = A, r = max(min(z$d[1], 1), 0))
    }
    ret
}

### cvsegments.R: A utility function to generate segments for k-fold
### cross-validation.



#' @title Generate segments for cross-validation
#'
#' @description The function generates a list of segments for cross-validation.  It can
#' generate random, consecutive and interleaved segments, and supports keeping
#' replicates in the same segment.
#'
#' @details If \code{length.seg} is specified, it is used to calculate the number of
#' segments to generate.  Otherwise \code{k} must be specified.  If
#' \eqn{k*length.seg \ne N}{k*length.seg <> N}, the \eqn{k*length.seg - N} last
#' segments will contain only \eqn{length.seg - 1} indices.
#'
#' If \code{type} is \code{"random"}, the indices are allocated to segments in
#' random order.  If it is \code{"consecutive"}, the first segment will contain
#' the first \eqn{length.seg} indices, and so on.  If \code{type} is
#' \code{"interleaved"}, the first segment will contain the indices \eqn{1,
#' length.seg+1, 2*lenght.seg+1, \ldots, (k-1)*length.seg+1}, and so on.
#'
#' If \eqn{nrep > }, it is assumed that each \code{nrep} consecutive rows are
#' replicates (repeated measurements) of the same object, and care is taken
#' that replicates are never put in different segments.
#'
#' Warning: If \code{k} does not divide \code{N}, a specified \code{length.seg}
#' does not divide \code{N}, or \code{nrep} does not divide \code{length.seg},
#' the number of segments and/or the segment length will be adjusted as needed.
#' Warnings are printed for some of these cases, and one should always inspect
#' the resulting segments to make sure they are as expected.
#'
#' Stratification of samples is enabled by the \code{stratify} argument.  This
#' is useful if there are sub-groups in the data set that should have a
#' proportional representation in the cross-validation segments or if the
#' response is categorical (classifiation). If \code{stratify} is combined with
#' \code{nrep}, \code{stratify} corresponds to the sets of replicates (see
#' example).
#'
#' @param N Integer.  The number of rows in the data set.
#' @param k Integer.  The number of segments to return.
#' @param length.seg Integer.  The length of the segments.  If given, it
#' overrides \code{k}.
#' @param nrep Integer.  The number of (consecutive) rows that are replicates
#' of the same object.  Replicates will always be kept in the same segment.
#' @param type One of \code{"random"}, \code{"consecutive"} and
#' \code{"interleaved"}.  The type of segments to generate.  Default is
#' \code{"random"}.
#' @param stratify Either a \code{list} of indices or an integer \code{vector}
#' indicating which stratum each sample (or set of replicates) belongs to (see
#' Details).
#' @return A list of vectors.  Each vector contains the indices for one
#' segment.  The attribute \code{"incomplete"} contains the number of
#' incomplete segments, and the attribute \code{"type"} contains the type of
#' segments.
#' @seealso Converting a factor to segments: \code{\link{fac2seg}}.
#' @author Bjørn-Helge Mevik, Ron Wehrens and Kristian Hovde Liland
#' @keywords models
#' @examples
#'
#' ## Segments for 10-fold randomised cross-validation:
#' cvsegments(100, 10)
#'
#' ## Segments with four objects, taken consecutive:
#' (segs <- cvsegments(60, length.seg = 4, type = "cons"))
#' data(gasoline)
#' plsr(octane ~ NIR, data=gasoline, ncomp=5, validation="CV", segments=segs)
#'
#' ## Incomplete segments
#' segs <- cvsegments(50, length.seg = 3)
#' attr(segs, "incomplete")
#'
#' ## Leave-one-out cross-validation:
#' cvsegments(100, 100)
#' ## Leave-one-out with variable/unknown data set size n:
#' n <- 50
#' cvsegments(n, length.seg = 1)
#'
#' ## Data set with replicates
#' cvsegments(100, 25, nrep = 2)
#' ## Note that rows 1 and 2 are in the same segment, rows 3 and 4 in the
#' ## same segment, and so on.
#'
#' ## Stratification
#' cvsegments(10, 3, type = "consecutive", stratify = c(rep(1,7), rep(2,3)))
#' ## Note that the last three samples are spread across the segments
#' ## according to the stratification vector.
#' cvsegments(20, 3, type = "consecutive", nrep = 2, stratify = c(rep(1,7), rep(2,3)))
#' ## Note the length of stratify matching number of replicate sets, not samples.
#'
#' ## Converting a factor to segments
#' fac <- factor(c("a", "b", "a", "b", "c", "c"))
#' fac2seg(fac)
#' ## Starting from a numeric vector
#' num <- c(1, 2, 1, 2, 3, 3)
#' fac2seg(factor(num))
#' @export
cvsegments <- function(N, k, length.seg = ceiling(N / k), nrep = 1,
                            type = c("random", "consecutive", "interleaved"), stratify = NULL)
{
    ## length.seg overrides k:
    if (!missing(length.seg)) k <- ceiling(N / length.seg)

    ## Check arguments:
    if (k > N) stop("More segments than observations requested")
    if (N %% nrep != 0)
        stop("The number of replicates does not divide ",
             "the number of observations")
    if (length.seg %% nrep != 0)
        warning("Segment length is not a multiple of the number of ",
                "replicates.\n  A best effort segment size will be used.")
    if (!missing(length.seg) && N %% length.seg != 0)
        warning("Required segment length does not divide the number of ",
                "observations.\n  A best effort segment size will be used.")

    ## The idea is to generate a k times length.seg matrix with indices, and
    ## use each column as a segment.  If k*length.seg > N, the last element of
    ## the N - k*length.seg last rows will be NA.  Any NAs are stripped when
    ## the matrix is converted to a list of vectors.

    ## If nrep > 1, N and length.seg is first divided by nrep, and the matrix
    ## of indices is created as above.  The matrix is then expanded by
    ## replacing each element i with a column vector nrep * (i - 1) + 1:nrep,
    ## before using the columns as segments.

    ## Reduce N and length.seg if needed
    if (nrep > 1) {
        N <- N / nrep
        length.seg <- ceiling(N / k)
    }

    incomplete <- k * length.seg - N    # Number of incomplete segments
    complete   <- k - incomplete        # Number of complete segments

    ## Create matrix of indices
    type <- match.arg(type)
    if (is.null(stratify)) { # No stratification
        switch(type,
               random = {
                   inds <- matrix(c(sample(1:N), rep(NA, incomplete)),
                                  nrow = length.seg, byrow = TRUE)
               },
               consecutive = {
                   if (complete < k) {
                       inds <- cbind(matrix(1:(length.seg*complete),
                                            nrow = length.seg),
                                     rbind(matrix((length.seg*complete+1):N,
                                                  nrow = length.seg-1), NA))
                   } else {
                       inds <- matrix(1:N, nrow = length.seg)
                   }
               },
               interleaved = {
                   inds <- matrix(c(1:N, rep(NA, incomplete)),
                                  nrow = length.seg, byrow = TRUE)
               }
        )
    } else {

        ## Prepare stratification
        if (!is.list(stratify)) {
            stratify <- lapply(1:max(stratify), function(i)which(stratify==i))
        }
        stratVec <- unlist(lapply(1:length(stratify), function(i)rep(i,length(stratify[[i]]))))
        indVec   <- unlist(stratify)
        segs     <- integer(N)
        blinds   <- rep(1:k, length.out=N)
        switch(type,
               random = {
                   stratRand <- sample(length(stratify))
                   stratify  <- stratify[stratRand]
                   indVec    <- unlist(stratify)
                   stratVec  <- unlist(lapply(1:length(stratify), function(i)rep(i,length(stratify[[i]]))))
                   for(i in 1:length(stratify)){
                       segs[stratify[[i]]] <- blinds[indVec[stratVec==i]]
                   }
                   inds <- lapply(1:k,function(i)indVec[segs==i])
                   inds <- lapply(inds, function(i)i[sample(length(i))])
                   if(incomplete>0)
                       inds[-(1:complete)] <- lapply(inds[-(1:complete)], function(i){i[length.seg] <- NA;i})
                   inds <- do.call(cbind, inds)
                   inds <- inds[,sample(k)]
               },
               consecutive = {
                   for(i in 1:length(stratify)){
                       blinds[stratVec==i] <- sort(blinds[stratVec==i])
                   }
                   for(i in 1:length(stratify)){
                       segs[stratify[[i]]] <- blinds[stratVec==i]
                   }
                   inds <- lapply(1:k,function(i)which(segs==i))
                   if(incomplete>0)
                       inds[-(1:complete)] <- lapply(inds[-(1:complete)], function(i){i[length.seg] <- NA;i})
                   inds <- do.call(cbind, inds)
               },
               interleaved = {
                   for(i in 1:length(stratify)){
                       segs[stratify[[i]]] <- blinds[indVec[stratVec==i]]
                   }
                   inds <- lapply(1:k,function(i)indVec[segs==i])
                   if(incomplete>0)
                       inds[-(1:complete)] <- lapply(inds[-(1:complete)], function(i){i[length.seg] <- NA;i})
                   inds <- do.call(cbind, inds)
               }
        )
    }

    ## Expand matrix if needed
    if (nrep > 1) {
        inds <- outer(1:nrep, nrep * (inds - 1), '+')
        dims <- dim(inds)
        dim(inds) <- c(dims[1] * dims[2], dims[3])
    }

    ## Convert to list of segments defined by the columns, and add attributes
    res <- lapply(as.data.frame(inds), function(x) c(na.omit(x)))
    attr(res, "incomplete") <- incomplete
    attr(res, "type") <- if (length.seg == 1) "leave-one-out" else type
    res
}

### mvr_wrappers.R: plsr, pls and pcr wrappers for mvr

#' @rdname mvr
#' @export
plsr <- function(..., method = pls.options()$plsralg) {
    cl <- match.call()
    cl$method <- match.arg(method, c("kernelpls", "widekernelpls", "simpls",
                                     "oscorespls", "nipalspls", "model.frame"))
    cl[[1]] <- quote(pls::mvr)
    res <- eval(cl, parent.frame())
    ## Fix call component
    if (cl$method != "model.frame") res$call[[1]] <- quote(pls::plsr)
    if (missing(method)) res$call$method <- NULL
    res
}

#' @rdname mvr
#' @export
pcr <- function(..., method = pls.options()$pcralg) {
    cl <- match.call()
    cl$method <- match.arg(method, c("svdpc", "model.frame"))
    cl[[1]] <- quote(pls::mvr)
    res <- eval(cl, parent.frame())
    ## Fix call component
    if (cl$method != "model.frame") res$call[[1]] <- quote(pls::pcr)
    if (missing(method)) res$call$method <- NULL
    res
}

#' @rdname mvr
#' @export
cppls <- function(..., Y.add, weights, method = pls.options()$cpplsalg) {
  cl <- match.call()
  cl$method <- match.arg(method, c("cppls", "model.frame"))
  cl[[1]] <- quote(pls::mvr)
  res <- eval(cl, parent.frame())
  ## Fix call component
  if (cl$method != "model.frame") res$call[[1]] <- quote(pls::cppls)
  if (missing(method)) res$call$method <- NULL
  res
}

#' @rdname mvr
#' @export
nipals <- function(..., weights, method = "nipalspls") {
  cl <- match.call()
  cl$method <- match.arg(method, c("nipalspls", "model.frame"))
  cl[[1]] <- quote(pls::mvr)
  res <- eval(cl, parent.frame())
  ## Fix call component
  if (cl$method != "model.frame") res$call[[1]] <- quote(pls::nipals)
  if (missing(method)) res$call$method <- NULL
  res
}

#' @rdname mvr
#' @export
nipalspcr <- function(..., method = "nipalspc") {
  cl <- match.call()
  cl$method <- match.arg(method, c("nipalspc", "model.frame"))
  cl[[1]] <- quote(pls::mvr)
  res <- eval(cl, parent.frame())
  ## Fix call component
  if (cl$method != "model.frame") res$call[[1]] <- quote(pls::pcr)
  if (missing(method)) res$call$method <- NULL
  res
}

### runit.mvr_wrappers.R: test functions for checking the mvr wrappers
### By Bjørn-Helge Mevik
### Started 2013-07-13

## test.wrapper_results: Check that the wrappers give the same results as mvr:
test.wrapper_results <- function() {
    # plsr
    mvrmod <- mvr(octane ~ NIR, ncomp=10, data = gasoline, x = TRUE, y = TRUE, method = "kernelpls")
    wrapmod <- plsr(octane ~ NIR, ncomp=10, data = gasoline, x = TRUE, y = TRUE)
    checkEquals(mvrmod[-c(14,18)], wrapmod[-c(14,18)],
                "plsr() doesn't give the same as mvr()")

    # pcr
    mvrmod <- mvr(octane ~ NIR, ncomp=10, data = gasoline, x = TRUE, y = TRUE, method = "svdpc")
    wrapmod <- pcr(octane ~ NIR, ncomp=10, data = gasoline, x = TRUE, y = TRUE)
    checkEquals(mvrmod[-c(12,16)], wrapmod[-c(12,16)],
                "pcr() doesn't give the same as mvr()")

    # cppls
    mvrmod <- mvr(oil.type ~ NIR, Y.add = design, ncomp=10, data = mayonnaise, x = TRUE, y = TRUE, method = "cppls")
    wrapmod <- cppls(oil.type ~ NIR, Y.add = design, ncomp=10, data = mayonnaise, x = TRUE, y = TRUE)
    checkEquals(mvrmod[-c(19,23)], wrapmod[-c(19,23)],
                "cppls() doesn't give the same as mvr()")
}

## test.wrapper_eval: Check that we don't pick up variables from the wrappers
test.wrapper_eval <- function() {
    ## ('cl' is a variable that exists in the wrapper.)
    if (exists("cl")) rm(cl)

    ## plsr:
    res <- try(plsr(y ~ X, data = cl), silent = TRUE)
    checkTrue(class(res) == "try-error" && grepl("object 'cl' not found", res),
              "plsr() picked up local variable 'cl'")

    ## pcr:
    res <- try(pcr(y ~ X, data = cl), silent = TRUE)
    checkTrue(class(res) == "try-error" && grepl("object 'cl' not found", res),
              "pcr() picked up local variable 'cl'")

    ## cppls:
    res <- try(cppls(y ~ X, data = cl), silent = TRUE)
    checkTrue(class(res) == "try-error" && grepl("object 'cl' not found", res),
              "cppls() picked up local variable 'cl'")
}

### runit.fit.R: test fitting models
### By Bjørn-Helge Mevik
### Started 2013-07-12

## test.mvr: test calling pls::mvr directly
test.mvr <- function() {
    mvrmod <- pls::mvr(sensory ~ chemical, data = oliveoil)
    checkEquals(class(mvrmod), "mvr")
}

## test.plsr: test calling pls::plsr directly
test.plsr <- function() {
    plsrmod <- pls::plsr(sensory ~ chemical, data = oliveoil)
    checkEquals(class(plsrmod), "mvr")
}

## test.pcr: test calling pls::pcr directly
test.pcr <- function() {
    pcrmod <- pls::pcr(sensory ~ chemical, data = oliveoil)
    checkEquals(class(pcrmod), "mvr")
}

context('Tests of the basic PLS functionality')

# Temporary measure to pass the tests while production R and devel R have
# different algorithms for generating random numbers.  When 3.6.0 is out,
                                        # consider removing this and update the saved results:
#suppressWarnings(RNGversion("3.5.0"))

# Set seed to make random choices reproducible
set.seed(100)
data(yarn)

## Default methods:
yarn.pcr <- pcr(density ~ NIR, 6, data = yarn, validation = "CV")
yarn.pcr$fit.time = 0   # This is the time it took to fit the model, this is not constant. Setting it to zero makes it constant and checkable in this test.
yarn.pls <- plsr(density ~ NIR, 6, data = yarn, validation = "CV")
yarn.pls$fit.time = 0
yarn.pls.widekernel <- plsr(density ~ NIR, 6, data = yarn, validation = "CV", method = 'widekernelpls')
yarn.pls.widekernel$fit.time = 0
cppls_weights = runif(nrow(yarn))
yarn.cppls <- cppls(density ~ NIR, 6, data = yarn, validation = "CV", weights = cppls_weights)
yarn.cppls$fit.time = 0
## These tests might fail with alternative BLAS implementations:
## test_that('Basic model fitting methods works', {
##   expect_equal_to_reference(yarn.pcr, 'yarn.pcr.rds', info = 'Principal Component Regression (pcr), method = `svdpc`')
##   expect_equal_to_reference(yarn.pls, 'yarn.pls.rds', info = 'Partial Least Squars (pls), method = `kernelpls`')
##   expect_equal_to_reference(yarn.pls.widekernel, 'yarn.pls.widekernel.rds', info = 'Partial Least Squars (pls), method = `widekernelpls`')
##   expect_equal_to_reference(yarn.cppls, 'yarn.cppls.rds', info = 'Canocial Powered Partial Least Squares works (cppls), method = `cppls`')
## })

## Alternative methods:
yarn.oscorespls <- mvr(density ~ NIR, 6, data = yarn, validation = "CV",
                       method = "oscorespls")
yarn.oscorespls$fit.time = 0
yarn.simpls <- mvr(density ~ NIR, 6, data = yarn, validation = "CV",
                   method = "simpls")
yarn.simpls$fit.time = 0
## These tests might fail with alternative BLAS implementations:
## test_that('Alternative model fitting methods works', {
##   expect_equal_to_reference(yarn.oscorespls, 'yarn.oscorespls.rds', info = 'Classical orthogonal scores algorithm (oscorespls)')
##   expect_equal_to_reference(yarn.simpls, 'yarn.simpls.rds', info = 'SIMPLS algorithm')
## })

## Match predictions
context('Test if predictions of the different methods line up')
pred.pcr = predict(yarn.pcr)
pred.pls = predict(yarn.pls)
pred.pls.widekernel = predict(yarn.pls.widekernel)
pred.cppls = predict(yarn.cppls)
pred.oscorepls = predict(yarn.oscorespls)
pred.simpls = predict(yarn.simpls)

test_that('The predictions for PLS are the same and for PCR/CPPLS are diffrent than PLS', {
  # expect_snapshot(pred.pls)
  # expect_snapshot(pred.pls.widekernel, 'pred.pls.widekernel.rds', info = 'PLS widekernel predictions should not change')
  # expect_snapshot(pred.pcr, 'pred.pcr.rds', info = 'PCR predictions should not change')
  # expect_snapshot(pred.cppls, 'pred.cppls.rds', info = 'PCR predictions should not change')
  expect_equal_to_reference(pred.pls, 'pred.pls.rds', info = 'PLS predictions should not change')
  expect_equal_to_reference(pred.pls.widekernel, 'pred.pls.widekernel.rds', info = 'PLS widekernel predictions should not change')
  expect_equal_to_reference(pred.pcr, 'pred.pcr.rds', info = 'PCR predictions should not change')
  expect_equal_to_reference(pred.cppls, 'pred.cppls.rds', info = 'PCR predictions should not change')
  expect_equal(pred.pls, pred.oscorepls, info = 'PLS prediction should be the same for normal PLS and oscorepls')
  expect_equal(pred.pls, pred.pls.widekernel, info = 'PLS prediction should be the same for normal PLS and widekernel PLS')
  expect_equal(pred.pls, pred.simpls, info = 'PLS prediction should be the same for normal PLS and simpls')
  expect_false(isTRUE(all.equal(pred.pls, pred.pcr)), info = 'PCR should not provide same predictions as PLS')
  expect_false(isTRUE(all.equal(pred.pls, pred.cppls)), info = 'CPPLS should not provide same predictions as PLS')
})

## Look at results with mean centering disabled
context('Test if disabling mean centering works correctly')
yarn_nocenter.pcr <- pcr(density ~ NIR, 6, data = yarn, validation = "CV", center = FALSE)
yarn_nocenter.pls <- plsr(density ~ NIR, 6, data = yarn, validation = "CV", center = FALSE)
yarn_nocenter.pls.widekernel <- plsr(density ~ NIR, 6, data = yarn, validation = "CV", method = 'widekernel', center = FALSE)
yarn_nocenter.cppls <- cppls(density ~ NIR, 6, data = yarn, validation = "CV", weights = cppls_weights, center = FALSE)

## Alternative methods:
yarn_nocenter.oscorespls <- mvr(density ~ NIR, 6, data = yarn, validation = "CV",
                       method = "oscorespls", center = FALSE)
yarn_nocenter.simpls <- mvr(density ~ NIR, 6, data = yarn, validation = "CV",
                   method = "simpls", center = FALSE)

# Make predictions
pred_nocenter.pcr = predict(yarn_nocenter.pcr)
pred_nocenter.pls = predict(yarn_nocenter.pls)
pred_nocenter.pls.widekernel = predict(yarn_nocenter.pls.widekernel)
pred_nocenter.cppls = predict(yarn_nocenter.cppls)
pred_nocenter.oscorepls = predict(yarn_nocenter.oscorespls)
pred_nocenter.simpls = predict(yarn_nocenter.simpls)

test_that('Predictions with mean centering disabled are internally consistent', {
  expect_equal_to_reference(pred_nocenter.pls, 'pred_nocenter.pls.rds', info = 'PLS predictions should not change')
  expect_equal_to_reference(pred_nocenter.pls.widekernel, 'pred_nocenter.pls.widekernel.rds', info = 'PLS predictions should not change')
  expect_equal_to_reference(pred_nocenter.oscorepls, 'pred_nocenter.oscorepls.rds', info = 'PLS predictions should not change')
  expect_equal_to_reference(pred_nocenter.pcr, 'pred_nocenter.pcr.rds', info = 'PCR predictions should not change')
  expect_equal_to_reference(pred_nocenter.cppls, 'pred_nocenter.cppls.rds', info = 'PCR predictions should not change')
  expect_equal(pred_nocenter.pls, pred_nocenter.oscorepls, info = 'PLS prediction should be the same for normal PLS and oscorepls')
  expect_equal(pred_nocenter.pls, pred_nocenter.simpls, info = 'PLS prediction should be the same for normal PLS and simpls')
  expect_equal(pred_nocenter.pls, pred_nocenter.pls.widekernel, info = 'PLS prediction should be the same for normal PLS and PLS widekernel')
  expect_false(isTRUE(all.equal(pred_nocenter.pls, pred.pcr)), info = 'PCR should not provide same predictions as PLS')
  expect_false(isTRUE(all.equal(pred_nocenter.pls, pred.cppls)), info = 'CPPLS should not provide same predictions as PLS')
})

test_that('Predictions with mean centering disabled differ from normal predictions', {
  expect_false(isTRUE(all.equal(pred.pls, pred_nocenter.pls)), info = 'PLS with no mean center should differ')
  expect_false(isTRUE(all.equal(pred.pls.widekernel, pred_nocenter.pls.widekernel)), info = 'PLS widekernel with no mean center should differ')
  expect_false(isTRUE(all.equal(pred.simpls, pred_nocenter.simpls)), info = 'PLS simpls with no mean center should differ')
  expect_false(isTRUE(all.equal(pred.oscorepls, pred_nocenter.oscorepls)), info = 'PLS oscorepls with no mean center should differ')
  expect_false(isTRUE(all.equal(pred.cppls, pred_nocenter.cppls)), info = 'CPPLS with no mean center should differ')
  expect_false(isTRUE(all.equal(pred.pcr, pred_nocenter.pcr)), info = 'PCR with no mean center should differ')
})

context('Compare non-centered results to Unscrambler results')
# Results generated by The Unscrambler, another software package that can perform PLS.
unscrambler_results = structure(list(centering = c(99.54402, 80.03986, 80.06403, 60.68136,
                                                   60.42256, 60.82306, 40.03272, 40.23078, 41.24754, 40.77765, 20.25846,
                                                   20.27943, 20.41399, 20.6133, 20.28128, 0.1703148, -0.2173195,
                                                   0.05234528, 0.1929855, 0.2232819, 0.128685, 51.10463, 50.3434,
                                                   32.29767, 34.55276, 30.04396, 20.54807, 19.86818),
                                     no_centering = c(99.50454, 80.08273, 80.03199, 60.71494, 60.49206, 60.85455, 39.96323, 40.26112,
                                                   41.14723, 40.78748, 20.30801, 20.32145, 20.46079, 20.67536, 20.26472,
                                                   0.1926269, -0.2748941, 0.03286389, 0.1880585, 0.263258, 0.1293178,
                                                   51.07263, 50.36798, 32.26351, 34.51244, 30.06797, 20.57023, 19.77034)),
                                .Names = c("centering", "no_centering"), class = "data.frame", row.names = c(NA, -28L))

test_that('Predictions for both centering and non-centering match between the package and The Unscrambler', {
  expect_equal(as.numeric(pred.pls[,,6]), unscrambler_results$centering, info = 'centered results in R match that of The Unscrambler', tolerance = 1e-04)
  expect_equal(as.numeric(pred_nocenter.pls[,,6]), unscrambler_results$no_centering, info = 'non-centered results in R match that of The Unscrambler', tolerance = 1e-04)
})

### mvrVal.R: Functions for calculating validation statistics, such
### as MSEP, RMSEP and R2, for mvr objects.

## Calculate the validation statistics needed for (R)MSEP and R^2.
## Note that it accepts any values for `estimate', but only calculates
## statistics for "train", "test" and "CV".

#' @name mvrVal
#' @title MSEP, RMSEP and R2 of PLSR and PCR models
#'
#' @description Functions to estimate the mean squared error of prediction (MSEP), root mean
#' squared error of prediction (RMSEP) and \eqn{R^2} (A.K.A. coefficient of
#' multiple determination) for fitted PCR and PLSR models.  Test-set,
#' cross-validation and calibration-set estimates are implemented.
#'
#' @details \code{RMSEP} simply calls \code{MSEP} and takes the square root of the
#' estimates.  It therefore accepts the same arguments as \code{MSEP}.
#'
#' Several estimators can be used.  \code{"train"} is the training or
#' calibration data estimate, also called (R)MSEC.  For \code{R2}, this is the
#' unadjusted \eqn{R^2}.  It is overoptimistic and should not be used for
#' assessing models.  \code{"CV"} is the cross-validation estimate, and
#' \code{"adjCV"} (for \code{RMSEP} and \code{MSEP}) is the bias-corrected
#' cross-validation estimate.  They can only be calculated if the model has
#' been cross-validated.  Finally, \code{"test"} is the test set estimate,
#' using \code{newdata} as test set.
#'
#' Which estimators to use is decided as follows (see below for
#' \code{mvrValstats}).  If \code{estimate} is not specified, the test set
#' estimate is returned if \code{newdata} is specified, otherwise the CV and
#' adjusted CV (for \code{RMSEP} and \code{MSEP}) estimates if the model has
#' been cross-validated, otherwise the training data estimate.  If
#' \code{estimate} is \code{"all"}, all possible estimates are calculated.
#' Otherwise, the specified estimates are calculated.
#'
#' Several model sizes can also be specified.  If \code{comps} is missing (or
#' is \code{NULL}), \code{length(ncomp)} models are used, with \code{ncomp[1]}
#' components, \ldots{}, \code{ncomp[length(ncomp)]} components.  Otherwise, a
#' single model with the components \code{comps[1]}, \ldots{},
#' \code{comps[length(comps)]} is used.  If \code{intercept} is \code{TRUE}, a
#' model with zero components is also used (in addition to the above).
#'
#' The \eqn{R^2} values returned by \code{"R2"} are calculated as \eqn{1 -
#' SSE/SST}, where \eqn{SST} is the (corrected) total sum of squares of the
#' response, and \eqn{SSE} is the sum of squared errors for either the fitted
#' values (i.e., the residual sum of squares), test set predictions or
#' cross-validated predictions (i.e., the \eqn{PRESS}).  For \code{estimate =
#' "train"}, this is equivalent to the squared correlation between the fitted
#' values and the response.  For \code{estimate = "train"}, the estimate is
#' often called the prediction \eqn{R^2}.
#'
#' \code{mvrValstats} is a utility function that calculates the statistics
#' needed by \code{MSEP} and \code{R2}.  It is not intended to be used
#' interactively.  It accepts the same arguments as \code{MSEP} and \code{R2}.
#' However, the \code{estimate} argument must be specified explicitly: no
#' partial matching and no automatic choice is made.  The function simply
#' calculates the types of estimates it knows, and leaves the other untouched.
#'
#' @aliases MSEP MSEP.mvr RMSEP RMSEP.mvr R2 R2.mvr mvrValstats
#' @param object an \code{mvr} object
#' @param estimate a character vector.  Which estimators to use.  Should be a
#' subset of \code{c("all", "train", "CV", "adjCV", "test")}.  \code{"adjCV"}
#' is only available for (R)MSEP.  See below for how the estimators are chosen.
#' @param newdata a data frame with test set data.
#' @param ncomp,comps a vector of positive integers.  The components or number
#' of components to use.  See below.
#' @param intercept logical.  Whether estimates for a model with zero
#' components should be returned as well.
#' @param se logical.  Whether estimated standard errors of the estimates
#' should be calculated.  Not implemented yet.
#' @param \dots further arguments sent to underlying functions or (for
#' \code{RMSEP}) to \code{MSEP}
#' @section Value: \code{mvrValstats} returns a list with components \describe{
#' \item{SSE}{three-dimensional array of SSE values.  The first dimension is
#' the different estimators, the second is the response variables and the third
#' is the models.} \item{SST}{matrix of SST values.  The first dimension is the
#' different estimators and the second is the response variables.}
#' \item{nobj}{a numeric vector giving the number of objects used for each
#' estimator.} \item{comps}{the components specified, with \code{0} prepended
#' if \code{intercept} is \code{TRUE}.} \item{cumulative}{\code{TRUE} if
#' \code{comps} was \code{NULL} or not specified.} }
#'
#' The other functions return an object of class \code{"mvrVal"}, with
#' components \describe{ \item{val}{three-dimensional array of estimates.  The
#' first dimension is the different estimators, the second is the response
#' variables and the third is the models.} \item{type}{\code{"MSEP"},
#' \code{"RMSEP"} or \code{"R2"}.} \item{comps}{the components specified, with
#' \code{0} prepended if \code{intercept} is \code{TRUE}.}
#' \item{cumulative}{\code{TRUE} if \code{comps} was \code{NULL} or not
#' specified.} \item{call}{the function call} }
#' @author Ron Wehrens and Bjørn-Helge Mevik
#' @seealso \code{\link{mvr}}, \code{\link{crossval}}, \code{\link{mvrCv}},
#' \code{\link{validationplot}}, \code{\link{plot.mvrVal}}
#' @references Mevik, B.-H., Cederkvist, H. R. (2004) Mean Squared Error of
#' Prediction (MSEP) Estimates for Principal Component Regression (PCR) and
#' Partial Least Squares Regression (PLSR).  \emph{Journal of Chemometrics},
#' \bold{18}(9), 422--429.
#' @keywords regression multivariate
#' @examples
#'
#' data(oliveoil)
#' mod <- plsr(sensory ~ chemical, ncomp = 4, data = oliveoil, validation = "LOO")
#' RMSEP(mod)
#' \dontrun{plot(R2(mod))}
#'
#' @export
mvrValstats <- function(object, estimate,
                        newdata, ncomp = 1:object$ncomp, comps,
                        intercept = cumulative, se = FALSE, ...)
{
    ## Makes the code slightly simpler:
    cumulative <- missing(comps) || is.null(comps)

    if (any(estimate == "CV")) {
        ## Check that cross-validation is possible:
        if (!cumulative)
            stop("Cross-validation is not supported when `comps' is specified")
        if (is.null(object$validation))
            stop("`object' has no `validation' component")
    }

    ## The calculated stuff:
    nestimates <- length(estimate)
    nresp <- dim(fitted(object))[2]
    respnames <- dimnames(fitted(object))[[2]]
    SSE <- array(dim = c(nestimates, nresp,
                         if(cumulative) 1 + length(ncomp) else 2),
                 dimnames = list(estimate = estimate,
                 response = respnames,
                 model = if (cumulative) {
                     c("(Intercept)", paste(ncomp, "comps"))
                 } else {
                     c("(Intercept)", paste("(Intercept), Comp",
                                            paste(comps, collapse = ", ")))
                 }
                 ))
    SST <- array(dim = c(nestimates, nresp),
                 dimnames = list(estimate = estimate, response = respnames))
    nobj <- numeric(nestimates)
    names(nobj) <- estimate

    ## Calculate the statistics:
    for (i in seq(along = estimate)) {
        switch(estimate[i],
               train = {
                   resp <- as.matrix(model.response(model.frame(object)))
                   nobj[i] <- nrow(resp)
                   if (inherits(object$na.action, "exclude")) {
                       resp <- napredict(object$na.action, resp) # inserts NAs
                   }
                   res <- if (cumulative)
                       residuals(object, ...)[,,ncomp, drop=FALSE]
                   else
                       resp - predict(object, comps = comps, ...)

                   SST[i,] <- apply(resp, 2, var, na.rm = TRUE) *
                       (nobj[i] - 1)
                   SSE[i,,] <- cbind(SST[i,], colSums(res^2, na.rm = TRUE))
               },
               test = {
                   if (missing(newdata)) stop("Missing `newdata'.")
                   ## Remove any observations with NAs:
                   newdata <- model.frame(formula(object), data = newdata)
                   resp <- as.matrix(model.response(newdata))
                   pred <- if (cumulative)
                       predict(object, ncomp = ncomp, newdata = newdata,...)
                   else
                       predict(object, comps = comps, newdata = newdata,...)
                   nobj[i] <- nrow(newdata)
                   SST[i,] <- apply(resp, 2, var) * (nobj[i] - 1)
                   SSE[i,,] <- cbind(colSums(sweep(resp, 2, object$Ymeans)^2),
                                    colSums((pred - c(resp))^2))
               },
               CV = {
                   resp <- as.matrix(model.response(model.frame(object)))
                   nobj[i] <- nrow(resp)
                   SST[i,] <- apply(resp, 2, var) * (nobj[i] - 1)
                   SSE[i,,] <-
                       cbind(object$validation$PRESS0,
                             object$validation$PRESS[,ncomp, drop=FALSE])
               }
               )
    }

    if (cumulative) comps <- ncomp
    ## Either remove the intercept or add a "zeroth" component:
    if (isTRUE(intercept))
        comps <- c(0, comps)
    else
        SSE <- SSE[,,-1, drop=FALSE]

    return(list(SSE = SSE, SST = SST, nobj = nobj, comps = comps,
                cumulative = cumulative))
}


## R2: Return R^2
#' @rdname mvrVal
#' @export
R2 <- function(object, ...) UseMethod("R2")
#' @rdname mvrVal
#' @export
R2.mvr <- function(object, estimate, newdata, ncomp = 1:object$ncomp, comps,
                   intercept = cumulative, se = FALSE, ...)
{
    ## Makes the code slightly simpler:  FIXME: maybe remove
    cumulative <- missing(comps) || is.null(comps)

    ## Figure out which estimate(s) to calculate:
    allEstimates <- c("all", "train", "CV", "test")
    if (missing(estimate)) {
        ## Select the `best' available estimate
        if (!missing(newdata)) {
            estimate = "test"
        } else {
            if (!is.null(object$validation)) {
                estimate = "CV"
            } else {
                estimate = "train"
            }
        }
    } else {
        estimate <- allEstimates[pmatch(estimate, allEstimates)]
        if (any(is.na(estimate)))
            stop("`estimate' should be a subset of ",
                 paste(allEstimates, collapse = ", "))
        if (any(estimate == "all")) {
            estimate <- allEstimates[-1] # Try all estimates (except "all")
            if (missing(newdata))
                estimate <- setdiff(estimate, "test")
            if (is.null(object$validation) || !cumulative)
                estimate <- setdiff(estimate, "CV")
        }
    }

    ## Get the needed validation statistics:
    cl <- match.call(expand.dots = FALSE)
    cl$estimate <- estimate             # update estimate argument
    cl[[1]] <- quote(pls::mvrValstats)
    valstats <- eval(cl, parent.frame())

    ## Calculate the R^2s:
    R2 <- 1 - valstats$SSE / c(valstats$SST)

    return(structure(list(val = R2, type = "R2", comps = valstats$comps,
                          cumulative = valstats$cumulative, call = match.call()),
                     class = "mvrVal"))
}


## MSEP: Return MSEP
#' @rdname mvrVal
#' @export
MSEP <- function(object, ...) UseMethod("MSEP")
#' @rdname mvrVal
#' @export
MSEP.mvr <- function(object, estimate, newdata, ncomp = 1:object$ncomp, comps,
                       intercept = cumulative, se = FALSE, ...)
{
    ## Makes the code slightly simpler:
    cumulative <- missing(comps) || is.null(comps)

    ## Figure out which estimate(s) to calculate:
    allEstimates <- c("all", "train", "CV", "adjCV", "test")
    if (missing(estimate)) {
        ## Select the `best' available estimate
        if (!missing(newdata)) {
            estimate = "test"
        } else {
            if (!is.null(object$validation)) {
                estimate = c("CV", "adjCV")
            } else {
                estimate = "train"
            }
        }
    } else {
        estimate <- allEstimates[pmatch(estimate, allEstimates)]
        if (any(is.na(estimate)))
            stop("`estimate' should be a subset of ",
                 paste(allEstimates, collapse = ", "))
        if (any(estimate == "all")) {
            estimate <- allEstimates[-1] # Try all estimates (except "all")
            if (missing(newdata))
                estimate <- setdiff(estimate, "test")
            if (is.null(object$validation) || !cumulative)
                estimate <- setdiff(estimate, c("CV", "adjCV"))
        }
    }

    ## adjCV needs the statistics for CV and train, so we optionally
    ## have to add them:
    if (adjCV <- any(estimate == "adjCV")) {
        ## Note: this removes any duplicate elements
        calcestimates <- union(estimate, c("train", "CV"))
    } else {
        calcestimates <- estimate
    }
    ## Get the needed validation statistics:
    cl <- match.call(expand.dots = FALSE)
    cl$estimate <- calcestimates        # update estimate argument
    cl[[1]] <- quote(pls::mvrValstats)
    valstats <- eval(cl, parent.frame())

    ## Calculate the MSEPs:
    MSEP <- valstats$SSE / valstats$nobj
    if (adjCV) {
        ## Calculate the adjusted CV
        MSEP["adjCV",,] <- MSEP["CV",,]
        if (isTRUE(intercept)) {
            MSEP["adjCV",,-1] <- MSEP["adjCV",,-1] + MSEP["train",,-1] -
                object$validation$adj[,ncomp]
        } else {
            MSEP["adjCV",,] <- MSEP["adjCV",,] + MSEP["train",,] -
                object$validation$adj[,ncomp]
        }
        ## Remove any specially added estimates (this also adds back any
        ## duplicate elements):
        MSEP <- MSEP[estimate,,, drop=FALSE]
    }

    return(structure(list(val = MSEP, type = "MSEP", comps = valstats$comps,
                          cumulative = valstats$cumulative, call = match.call()),
                     class = "mvrVal"))
}

# RMSEP: A wrapper around MSEP to calculate RMSEPs
#' @rdname mvrVal
#' @export
RMSEP <- function(object, ...) UseMethod("RMSEP")
#' @rdname mvrVal
#' @export
RMSEP.mvr <- function(object, ...) {
    cl <- match.call()
    cl[[1]] <- quote(pls::MSEP)
    z <- eval(cl, parent.frame())
    z$val <- sqrt(z$val)
    z$type <- "RMSEP"
    z$call[[1]] <- quote(pls::RMSEP)
    z
}

### runit.mvrVal.R: test functions for R2(), MSEP() and RMSEP()
### By Bjørn-Helge Mevik
### Started 2007-06-20

###
### Check for capturing argument errors:
###
test.argErrors <- function() {
    mod <- mvr(density ~ NIR, ncomp = 6, data = yarn, validation = "CV")
    checkException(R2(mod, comps = 2:5), silent = TRUE)
    checkException(R2(mod, estimate = "adjCV"), silent = TRUE)
    checkException(MSEP(mod, comps = 1:4), silent = TRUE)
}

###
### Checking dimensions and dimnames of output
###
test.dimR2 <- function() {
    ##
    ## Uni-response models:
    ##
    n <- nrow(yarn)
    ncomp <- 7
    comps <- c(1,3,5)
    ## No CV
    mod <- mvr(density ~ NIR, ncomp = ncomp, data = yarn, method = "svdpc")
    res <- R2(mod)
    checkEquals(dim(res$val), c(1,1,ncomp + 1))
    checkIdentical(dimnames(res$val)[[1]], "train")
    checkEquals(res$comps, 0:ncomp)
    checkIdentical(res$cumulative, TRUE)
    res <- R2(mod, newdata = yarn)
    checkEquals(dim(res$val), c(1,1,ncomp + 1))
    checkIdentical(dimnames(res$val)[[1]], "test")
    checkEquals(res$comps, 0:ncomp)
    checkIdentical(res$cumulative, TRUE)
    res <- R2(mod, ncomp = 2:5)
    checkEquals(dim(res$val), c(1,1,4 + 1))
    checkIdentical(dimnames(res$val)[[1]], "train")
    checkEquals(res$comps, c(0,2:5))
    checkIdentical(res$cumulative, TRUE)
    res <- R2(mod, newdata = yarn, estimate = "all")
    checkEquals(dim(res$val), c(2,1,ncomp + 1))
    checkIdentical(dimnames(res$val)[[1]], c("train", "test"))
    checkEquals(res$comps, 0:ncomp)
    checkIdentical(res$cumulative, TRUE)
    res <- R2(mod, newdata = yarn, intercept = FALSE, estimate = c("test", "train"))
    checkEquals(dim(res$val), c(2,1,ncomp))
    checkIdentical(dimnames(res$val)[[1]], c("test", "train"))
    checkEquals(res$comps, 1:ncomp)
    checkIdentical(res$cumulative, TRUE)
    res <- R2(mod, intercept = TRUE, comps = comps)
    checkEquals(dim(res$val), c(1,1,2), "uni, intercept, comps, 1")
    checkIdentical(dimnames(res$val)[[1]], "train")
    checkEquals(res$comps, c(0, comps), "uni, intercept, comps, 3")
    checkIdentical(res$cumulative, FALSE)
    res <- R2(mod, intercept = FALSE, comps = comps)
    checkEquals(dim(res$val), c(1,1,1), "uni, no intercept, comps, 1")
    checkIdentical(dimnames(res$val)[[1]], "train")
    checkEquals(res$comps, comps)
    checkIdentical(res$cumulative, FALSE)
    ## With CV
    mod <- mvr(density ~ NIR, ncomp = ncomp, data = yarn, validation = "CV")
    res <- R2(mod)
    checkEquals(dim(res$val), c(1,1,ncomp + 1), "uni, CV, 1")
    checkIdentical(dimnames(res$val)[[1]], "CV")
    checkEquals(res$comps, 0:ncomp)
    checkIdentical(res$cumulative, TRUE)
    res <- R2(mod, newdata = yarn)
    checkEquals(dim(res$val), c(1,1,ncomp + 1))
    checkIdentical(dimnames(res$val)[[1]], "test")
    checkEquals(res$comps, 0:ncomp)
    checkIdentical(res$cumulative, TRUE)
    res <- R2(mod, ncomp = 2:5)
    checkEquals(dim(res$val), c(1,1,4 + 1))
    checkIdentical(dimnames(res$val)[[1]], "CV")
    checkEquals(res$comps, c(0,2:5))
    checkIdentical(res$cumulative, TRUE)
    res <- R2(mod, intercept = FALSE)
    checkEquals(dim(res$val), c(1,1,ncomp))
    checkIdentical(dimnames(res$val)[[1]], "CV")
    checkEquals(res$comps, 1:ncomp)
    checkIdentical(res$cumulative, TRUE)
    res <- R2(mod, newdata = yarn, estimate = "all")
    checkEquals(dim(res$val), c(3,1,ncomp + 1))
    checkIdentical(dimnames(res$val)[[1]], c("train", "CV", "test"), "uni, CV, newdata, all, 2")
    checkEquals(res$comps, 0:ncomp)
    checkIdentical(res$cumulative, TRUE)
    res <- R2(mod, newdata = yarn, intercept = FALSE, estimate = "CV")
    checkEquals(dim(res$val), c(1,1,ncomp))
    checkIdentical(dimnames(res$val)[[1]], c("CV"))
    checkEquals(res$comps, 1:ncomp)
    checkIdentical(res$cumulative, TRUE)
    ##
    ## Multi-response models:
    ##
    n <- nrow(oliveoil)
    nresp <- ncol(oliveoil$chemical)
    ncomp <- 4
    comps <- c(2,4)
    ## No CV
    mod <- mvr(chemical ~ sensory, ncomp = ncomp, data = oliveoil)
    res <- R2(mod)
    checkEquals(dim(res$val), c(1,nresp,ncomp + 1))
    checkIdentical(dimnames(res$val)[[1]], "train")
    checkEquals(res$comps, 0:ncomp)
    checkIdentical(res$cumulative, TRUE)
    res <- R2(mod, newdata = oliveoil)
    checkEquals(dim(res$val), c(1,nresp,ncomp + 1))
    checkIdentical(dimnames(res$val)[[1]], "test")
    checkEquals(res$comps, 0:ncomp)
    checkIdentical(res$cumulative, TRUE)
    res <- R2(mod, ncomp = 2:4)
    checkEquals(dim(res$val), c(1,nresp,3 + 1), "multi, ncomp, 1")
    checkIdentical(dimnames(res$val)[[1]], "train", "multi, ncomp,2")
    checkEquals(res$comps, c(0,2:4), "multi, ncomp,3")
    checkIdentical(res$cumulative, TRUE, "multi, ncomp,4")
    res <- R2(mod, newdata = oliveoil, estimate = "all")
    checkEquals(dim(res$val), c(2,nresp,ncomp + 1))
    checkIdentical(dimnames(res$val)[[1]], c("train", "test"))
    checkEquals(res$comps, 0:ncomp)
    checkIdentical(res$cumulative, TRUE)
    res <- R2(mod, newdata = oliveoil, intercept = FALSE, estimate = c("test", "train"))
    checkEquals(dim(res$val), c(2,nresp,ncomp))
    checkIdentical(dimnames(res$val)[[1]], c("test", "train"))
    checkEquals(res$comps, 1:ncomp)
    checkIdentical(res$cumulative, TRUE)
    res <- R2(mod, intercept = TRUE, comps = comps)
    checkEquals(dim(res$val), c(1,nresp,2), "multi, intercept, comps, 1")
    checkIdentical(dimnames(res$val)[[1]], "train")
    checkEquals(res$comps, c(0, comps), "multi, intercept, comps, 3")
    checkIdentical(res$cumulative, FALSE)
    res <- R2(mod, intercept = FALSE, comps = comps)
    checkEquals(dim(res$val), c(1,nresp,1), "multi, no intercept, comps, 1")
    checkIdentical(dimnames(res$val)[[1]], "train")
    checkEquals(res$comps, comps)
    checkIdentical(res$cumulative, FALSE)
    ## With CV
    mod <- mvr(chemical ~ sensory, ncomp = ncomp, data = oliveoil, validation = "CV", method = "svdpc")
    res <- R2(mod)
    checkEquals(dim(res$val), c(1,nresp,ncomp + 1), "multi, CV, 1")
    checkIdentical(dimnames(res$val)[[1]], "CV")
    checkEquals(res$comps, 0:ncomp)
    checkIdentical(res$cumulative, TRUE)
    res <- R2(mod, newdata = oliveoil)
    checkEquals(dim(res$val), c(1,nresp,ncomp + 1))
    checkIdentical(dimnames(res$val)[[1]], "test")
    checkEquals(res$comps, 0:ncomp)
    checkIdentical(res$cumulative, TRUE)
    res <- R2(mod, ncomp = 2:4)
    checkEquals(dim(res$val), c(1,nresp,3 + 1), "multi, CV, ncomp, 1")
    checkIdentical(dimnames(res$val)[[1]], "CV", "multi, CV, ncomp, 2")
    checkEquals(res$comps, c(0,2:4), "multi, CV, ncomp, 3")
    checkIdentical(res$cumulative, TRUE, "multi, CV, ncomp, 4")
    res <- R2(mod, intercept = FALSE)
    checkEquals(dim(res$val), c(1,nresp,ncomp))
    checkIdentical(dimnames(res$val)[[1]], "CV")
    checkEquals(res$comps, 1:ncomp)
    checkIdentical(res$cumulative, TRUE)
    res <- R2(mod, newdata = oliveoil, estimate = "all")
    checkEquals(dim(res$val), c(3,nresp,ncomp + 1))
    checkIdentical(dimnames(res$val)[[1]], c("train", "CV", "test"), "multi, CV, newdata, all, 2")
    checkEquals(res$comps, 0:ncomp)
    checkIdentical(res$cumulative, TRUE)
    res <- R2(mod, newdata = oliveoil, intercept = FALSE, estimate = "CV")
    checkEquals(dim(res$val), c(1,nresp,ncomp))
    checkIdentical(dimnames(res$val)[[1]], c("CV"))
    checkEquals(res$comps, 1:ncomp)
    checkIdentical(res$cumulative, TRUE)
}

test.dimMSEP <- function() {
    ##
    ## Uni-response models:
    ##
    n <- nrow(yarn)
    ncomp <- 7
    comps <- c(1,3,5)
    ## No CV
    mod <- mvr(density ~ NIR, ncomp = ncomp, data = yarn, method = "svdpc")
    res <- MSEP(mod)
    checkEquals(dim(res$val), c(1,1,ncomp + 1), "uni, 1")
    checkIdentical(dimnames(res$val)[[1]], "train", "uni, 2")
    checkEquals(res$comps, 0:ncomp, "uni, 3")
    checkIdentical(res$cumulative, TRUE, "uni, 4")
    res <- MSEP(mod, newdata = yarn)
    checkEquals(dim(res$val), c(1,1,ncomp + 1))
    checkIdentical(dimnames(res$val)[[1]], "test")
    checkEquals(res$comps, 0:ncomp)
    checkIdentical(res$cumulative, TRUE)
    res <- MSEP(mod, ncomp = 2:5)
    checkEquals(dim(res$val), c(1,1,4 + 1))
    checkIdentical(dimnames(res$val)[[1]], "train")
    checkEquals(res$comps, c(0,2:5))
    checkIdentical(res$cumulative, TRUE)
    res <- MSEP(mod, newdata = yarn, estimate = "all")
    checkEquals(dim(res$val), c(2,1,ncomp + 1))
    checkIdentical(dimnames(res$val)[[1]], c("train", "test"))
    checkEquals(res$comps, 0:ncomp)
    checkIdentical(res$cumulative, TRUE)
    res <- MSEP(mod, newdata = yarn, intercept = FALSE, estimate = c("test", "train"))
    checkEquals(dim(res$val), c(2,1,ncomp))
    checkIdentical(dimnames(res$val)[[1]], c("test", "train"))
    checkEquals(res$comps, 1:ncomp)
    checkIdentical(res$cumulative, TRUE)
    res <- MSEP(mod, intercept = TRUE, comps = comps)
    checkEquals(dim(res$val), c(1,1,2), "uni, intercept, comps, 1")
    checkIdentical(dimnames(res$val)[[1]], "train")
    checkEquals(res$comps, c(0, comps), "uni, intercept, comps, 3")
    checkIdentical(res$cumulative, FALSE)
    res <- MSEP(mod, intercept = FALSE, comps = comps)
    checkEquals(dim(res$val), c(1,1,1), "uni, no intercept, comps, 1")
    checkIdentical(dimnames(res$val)[[1]], "train")
    checkEquals(res$comps, comps)
    checkIdentical(res$cumulative, FALSE)
    ## With CV
    mod <- mvr(density ~ NIR, ncomp = ncomp, data = yarn, validation = "CV")
    res <- MSEP(mod)
    checkEquals(dim(res$val), c(2,1,ncomp + 1), "uni, CV, 1")
    checkIdentical(dimnames(res$val)[[1]], c("CV", "adjCV"))
    checkEquals(res$comps, 0:ncomp)
    checkIdentical(res$cumulative, TRUE)
    res <- MSEP(mod, newdata = yarn)
    checkEquals(dim(res$val), c(1,1,ncomp + 1))
    checkIdentical(dimnames(res$val)[[1]], "test")
    checkEquals(res$comps, 0:ncomp)
    checkIdentical(res$cumulative, TRUE)
    res <- MSEP(mod, ncomp = 2:5)
    checkEquals(dim(res$val), c(2,1,4 + 1))
    checkIdentical(dimnames(res$val)[[1]], c("CV", "adjCV"))
    checkEquals(res$comps, c(0,2:5))
    checkIdentical(res$cumulative, TRUE)
    res <- MSEP(mod, intercept = FALSE)
    checkEquals(dim(res$val), c(2,1,ncomp))
    checkIdentical(dimnames(res$val)[[1]], c("CV", "adjCV"))
    checkEquals(res$comps, 1:ncomp)
    checkIdentical(res$cumulative, TRUE)
    res <- MSEP(mod, newdata = yarn, estimate = "all")
    checkEquals(dim(res$val), c(4,1,ncomp + 1))
    checkIdentical(dimnames(res$val)[[1]], c("train", "CV", "adjCV", "test"), "uni, CV, newdata, all, 2")
    checkEquals(res$comps, 0:ncomp)
    checkIdentical(res$cumulative, TRUE)
    res <- MSEP(mod, newdata = yarn, intercept = FALSE, estimate = "CV")
    checkEquals(dim(res$val), c(1,1,ncomp))
    checkIdentical(dimnames(res$val)[[1]], c("CV"))
    checkEquals(res$comps, 1:ncomp)
    checkIdentical(res$cumulative, TRUE)
    res <- MSEP(mod, newdata = yarn, intercept = FALSE, estimate = "adjCV")
    checkEquals(dim(res$val), c(1,1,ncomp))
    checkIdentical(dimnames(res$val)[[1]], c("adjCV"))
    checkEquals(res$comps, 1:ncomp)
    checkIdentical(res$cumulative, TRUE)
    res <- MSEP(mod, newdata = yarn, intercept = FALSE, estimate = c("adjCV", "CV"))
    checkEquals(dim(res$val), c(2,1,ncomp))
    checkIdentical(dimnames(res$val)[[1]], c("adjCV", "CV"), "uni, newdata, no intercept, adjCV, CV")
    checkEquals(res$comps, 1:ncomp)
    checkIdentical(res$cumulative, TRUE)
    ##
    ## Multi-response models:
    ##
    n <- nrow(oliveoil)
    nresp <- ncol(oliveoil$chemical)
    ncomp <- 5
    comps <- c(1,3,4)
    ## No CV
    mod <- mvr(chemical ~ sensory, ncomp = ncomp, data = oliveoil, method = "svdpc")
    res <- MSEP(mod)
    checkEquals(dim(res$val), c(1,nresp,ncomp + 1))
    checkIdentical(dimnames(res$val)[[1]], "train")
    checkEquals(res$comps, 0:ncomp)
    checkIdentical(res$cumulative, TRUE)
    res <- MSEP(mod, newdata = oliveoil)
    checkEquals(dim(res$val), c(1,nresp,ncomp + 1))
    checkIdentical(dimnames(res$val)[[1]], "test")
    checkEquals(res$comps, 0:ncomp)
    checkIdentical(res$cumulative, TRUE)
    res <- MSEP(mod, ncomp = 2:5)
    checkEquals(dim(res$val), c(1,nresp,4 + 1))
    checkIdentical(dimnames(res$val)[[1]], "train")
    checkEquals(res$comps, c(0,2:5))
    checkIdentical(res$cumulative, TRUE)
    res <- MSEP(mod, newdata = oliveoil, estimate = "all")
    checkEquals(dim(res$val), c(2,nresp,ncomp + 1))
    checkIdentical(dimnames(res$val)[[1]], c("train", "test"))
    checkEquals(res$comps, 0:ncomp)
    checkIdentical(res$cumulative, TRUE)
    res <- MSEP(mod, newdata = oliveoil, intercept = FALSE, estimate = c("test", "train"))
    checkEquals(dim(res$val), c(2,nresp,ncomp))
    checkIdentical(dimnames(res$val)[[1]], c("test", "train"))
    checkEquals(res$comps, 1:ncomp)
    checkIdentical(res$cumulative, TRUE)
    res <- MSEP(mod, intercept = TRUE, comps = comps)
    checkEquals(dim(res$val), c(1,nresp,2), "uni, intercept, comps, 1")
    checkIdentical(dimnames(res$val)[[1]], "train")
    checkEquals(res$comps, c(0, comps), "uni, intercept, comps, 3")
    checkIdentical(res$cumulative, FALSE)
    res <- MSEP(mod, intercept = FALSE, comps = comps)
    checkEquals(dim(res$val), c(1,nresp,1), "uni, no intercept, comps, 1")
    checkIdentical(dimnames(res$val)[[1]], "train")
    checkEquals(res$comps, comps)
    checkIdentical(res$cumulative, FALSE)
    ## With CV
    mod <- mvr(chemical ~ sensory, ncomp = ncomp, data = oliveoil, validation = "CV")
    res <- MSEP(mod)
    checkEquals(dim(res$val), c(2,nresp,ncomp + 1), "uni, CV, 1")
    checkIdentical(dimnames(res$val)[[1]], c("CV", "adjCV"))
    checkEquals(res$comps, 0:ncomp)
    checkIdentical(res$cumulative, TRUE)
    res <- MSEP(mod, newdata = oliveoil)
    checkEquals(dim(res$val), c(1,nresp,ncomp + 1))
    checkIdentical(dimnames(res$val)[[1]], "test")
    checkEquals(res$comps, 0:ncomp)
    checkIdentical(res$cumulative, TRUE)
    res <- MSEP(mod, ncomp = 2:5)
    checkEquals(dim(res$val), c(2,nresp,4 + 1))
    checkIdentical(dimnames(res$val)[[1]], c("CV", "adjCV"))
    checkEquals(res$comps, c(0,2:5))
    checkIdentical(res$cumulative, TRUE)
    res <- MSEP(mod, intercept = FALSE)
    checkEquals(dim(res$val), c(2,nresp,ncomp))
    checkIdentical(dimnames(res$val)[[1]], c("CV", "adjCV"))
    checkEquals(res$comps, 1:ncomp)
    checkIdentical(res$cumulative, TRUE)
    res <- MSEP(mod, newdata = oliveoil, estimate = "all")
    checkEquals(dim(res$val), c(4,nresp,ncomp + 1))
    checkIdentical(dimnames(res$val)[[1]], c("train", "CV", "adjCV", "test"), "uni, CV, newdata, all, 2")
    checkEquals(res$comps, 0:ncomp)
    checkIdentical(res$cumulative, TRUE)
    res <- MSEP(mod, newdata = oliveoil, intercept = FALSE, estimate = "CV")
    checkEquals(dim(res$val), c(1,nresp,ncomp))
    checkIdentical(dimnames(res$val)[[1]], c("CV"))
    checkEquals(res$comps, 1:ncomp)
    checkIdentical(res$cumulative, TRUE)
    res <- MSEP(mod, newdata = oliveoil, intercept = FALSE, estimate = "adjCV")
    checkEquals(dim(res$val), c(1,nresp,ncomp))
    checkIdentical(dimnames(res$val)[[1]], c("adjCV"))
    checkEquals(res$comps, 1:ncomp)
    checkIdentical(res$cumulative, TRUE)
    res <- MSEP(mod, newdata = oliveoil, intercept = FALSE, estimate = c("adjCV", "CV"))
    checkEquals(dim(res$val), c(2,nresp,ncomp))
    checkIdentical(dimnames(res$val)[[1]], c("adjCV", "CV"), "uni, newdata, no intercept, adjCV, CV")
    checkEquals(res$comps, 1:ncomp)
    checkIdentical(res$cumulative, TRUE)
}


###
### Check some numerical values
###
test.estimates <- function() {
    mod <- mvr(chemical ~ sensory, data = oliveoil)
    res <- R2(mod, newdata = oliveoil, estimate = "all")
    checkEquals(res$val["train",,], res$val["test",,], "R2, train = test")
    checkEquals(res$val["train","Acidity","2 comps"], cor(fitted(mod)[,"Acidity","2 comps"], oliveoil$chemical[,"Acidity"])^2, "R2, train R^2 = cor^2")
    res <- MSEP(mod, newdata = oliveoil, estimate = "all")
    checkEquals(res$val["train",,], res$val["test",,], "MSEP, train = test")
    checkEqualsNumeric(res$val["train",,-1], colMeans((fitted(mod)- c(oliveoil$chemical))^2), "MSEP, train directly")
    mod <- mvr(chemical ~ sensory, data = oliveoil, validation = "CV")
    res <- MSEP(mod, estimate = "CV")
    checkEqualsNumeric(res$val["CV",,-1],
                colMeans((mod$validation$pred - c(oliveoil$chemical))^2))
    ## FIXME: Temporary check:
    checkEquals(res$val["CV",,1],
                apply(oliveoil$chemical, 2, var) * nrow(oliveoil) / (nrow(oliveoil)-1))
}


###
### Test handling of missing values
###

## Handling of missing data in train and/or test data
test.MSEPmissing <- function() {
    ## Two missing in X, one in Y
    olmiss <- oliveoil
    olmiss$chemical[2,3] <- NA
    olmiss$chemical[3,4] <- NA
    olmiss$sensory[4,2] <- NA
    ## Common segments for comparison:
    ## Parametres:
    ## - mvr: na.action: omit (default) / exclude
    ## - MSEP: ncomp (default) / comps

    ## na.action = na.omit (default)
    mod <- mvr(sensory ~ chemical, data = olmiss, validation = "LOO")
    res1 <- MSEP(mod, estimate = "all", newdata = olmiss)
    checkTrue(all(is.finite(res1$val)), "na.omit, all is finite 1")
    checkEquals(res1$val["train",,], res1$val["test",,],
                "na.omit, train == test")
    checkEqualsNumeric(res1$val["train",,-1],
                       colMeans((fitted(mod) - c(olmiss$sensory[-c(2:4),]))^2),
                       "na.omit, train == fitted")
    checkEqualsNumeric(res1$val["CV",,-1],
                       mod$validation$PRESS / (nrow(olmiss) - 3),
                       "na.omit, CV == PRESS/n")
    ## FIXME: Check adjCV

    res2 <- MSEP(mod, estimate = c("train", "test"), newdata = olmiss,
                 comps = 1:5)
    checkTrue(all(is.finite(res2$val)), "na.omit, all is finite 2")
    checkEquals(drop(res2$val), res1$val[c("train", "test"),,6],
                "na.omit, comps == ncomp")

    ## na.action = na.exclude
    mod <- mvr(sensory ~ chemical, data = olmiss, validation = "LOO",
                na.action = na.exclude)
    res3 <- MSEP(mod, estimate = "all", newdata = olmiss)
## Superfluous:
##     checkTrue(all(is.finite(res3$val)), "na.exclude, all is finite 1")
##     checkEquals(res3$val["train",,], res3$val["test",,],
##                 "na.exclude, train == test")
##     checkEqualsNumeric(res3$val["train",,-1],
##                        colMeans((fitted(mod)[-c(2:4),,] -
##                                  c(olmiss$sensory[-c(2:4),]))^2),
##                        "na.exclude, train == fitted")
##     checkEqualsNumeric(res3$val["CV",,-1],
##                        mod$validation$PRESS / (nrow(olmiss) - 3),
##                        "na.exclude, CV == PRESS/n")
##     ## FIXME: Check adjCV

    res4 <- MSEP(mod, estimate = c("train", "test"), newdata = olmiss,
                 comps = 1:5)
## Superfluous:
##     checkTrue(all(is.finite(res4$val)), "na.exclude, all is finite 2")
##     checkEquals(drop(res4$val), res3$val[c("train", "test"),,6],
##                 "na.exclude, comps == ncomp")

    ## na.omit vs. na.exclude
    checkIdentical(res1, res3, "res1 == res3")
    checkIdentical(res2, res4, "res2 == res4")
}

---
title: "Introduction to the pls Package"
author:
  - Bjørn-Helge Mevik
  - Ron Wehrens
  - Kristian Hovde Liland
output: rmarkdown::html_vignette
bibliography: pls-manual.bib
vignette: >
  %\VignetteIndexEntry{Introduction to the pls Package}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, echo=FALSE, results='hide'}
library(pls)
knitr::opts_chunk$set(fig.align = "center")
# Corresponds to pdf.options(pointsize=10) and options(digits = 4) in original
options(digits = 4)
```

# Abstract

The **pls** package implements Principal Component Regression (PCR) and Partial Least Squares Regression (PLSR) in **R**, and is freely available from the CRAN website, licensed under the Gnu General Public License (GPL). 

The user interface is modelled after the traditional formula interface, as exemplified by `lm`. This was done so that people used to **R** would not have to learn yet another interface, and also because we believe the formula interface is a good way of working interactively with models. It thus has methods for generic functions like `predict`, `update` and `coef`. It also has more specialised functions like `scores`, `loadings` and `RMSEP`, and a flexible cross-validation system. Visual inspection and assessment is important in chemometrics, and the **pls** package has a number of plot functions for plotting scores, loadings, predictions, coefficients and RMSEP estimates. 

The package implements PCR and several algorithms for PLSR. The design is modular, so that it should be easy to use the underlying algorithms in other functions. It is our hope that the package will serve well both for interactive data analysis and as a building block for other functions or packages using PLSR or PCR. 

We will here describe the package and how it is used for data analysis, as well as how it can be used as a part of other packages. Also included is a section about formulas and data frames, for people not used to the **R** modelling idioms.

# 1 Introduction

This vignette is meant as an introduction to the **pls** package. It is based on the paper 'The pls Package: Principal Component and Partial Least Squares Regression in R', published in *Journal of Statistical Software* [@MevWeh:plsJSS], but has been extended to reflect further developments in the package. The PLSR methodology is shortly described in Section 2. Section 3 presents an example session, to get an overview of the package. In Section 4 we describe formulas and data frames (as they are used in **pls**). Users familiar with formulas and data frames in **R** can skip this section on first reading. Fitting of models is described in Section 5, and cross-validatory choice of components is discussed in Section 6. Next, inspecting and plotting models is described (Section 7), followed by a section on predicting future observations (Section 8). Finally, Section 9 covers more advanced topics such as parallel computing, setting options, using the underlying functions directly, and implementation details.

# 2 Theory

Multivariate regression methods like Principal Component Regression (PCR) and Partial Least Squares Regression (PLSR) enjoy large popularity in a wide range of fields, including the natural sciences. The main reason is that they have been designed to confront the situation that there are many, possibly correlated, predictor variables, and relatively few samples—a situation that is common, especially in chemistry where developments in spectroscopy since the seventies have revolutionised chemical analysis. In fact, the origin of PLSR lies in chemistry (see, e.g., [@Wold2001; @Martens2001]). The field of *near-infrared* (NIR) spectroscopy, with its highly overlapping lines and difficult to interpret overtones, would not have existed but for a method to obtain quantitative information from the spectra. Also other fields have benefited greatly from multivariate regression methods like PLSR and PCR. In medicinal chemistry, for example, one likes to derive molecular properties from the molecular structure. Most of these Quantitative Structure-Activity Relations (QSAR, and also Quantitative Structure-Property Relations, QSPR), and in particular, Comparative Molecular Field Analysis (ComFA) [@Cramer1988], use PLSR. Other applications range from statistical process control [@Kresta1991] to tumour classification [@Nguyen2002] to spatial analysis in brain images [@McIntosh1996] to marketing [@Fornell1982].

In the usual multiple linear regression (MLR) context, the least-squares solution for

\begin{equation}
\mathbf{Y} = \mathbf{X}\mathbf{B} + \Epsilon
\end{equation}

is given by

\begin{equation}
\mathbf{B} = (\mathbf{X}^T \mathbf{X})^{-1} \mathbf{X}^T \mathbf{Y}
\label{eq:lsq}
\end{equation}

The problem often is that $\mathbf{X}^T \mathbf{X}$ is singular, either because the number of variables (columns) in $\mathbf{X}$ exceeds the number of objects (rows), or because of collinearities. Both PCR and PLSR circumvent this by decomposing $\mathbf{X}$ into orthogonal scores $\mathbf{T}$ and loadings $\mathbf{P}$ 

\begin{equation}
\mathbf{X} = \mathbf{T} \mathbf{P}^T
\end{equation}

and regressing $\mathbf{Y}$ not on $\mathbf{X}$ itself but on the first $a$ columns of the scores $\mathbf{T}$. In PCR, the scores are given by the left singular vectors of $\mathbf{X}$, multiplied with the corresponding singular values, and the loadings are the right singular vectors of $\mathbf{X}$. This, however, only takes into account information about $\mathbf{X}$, and therefore may be suboptimal for prediction purposes. PLSR aims to incorporate information on both $\mathbf{X}$ and $\mathbf{Y}$ in the definition of the scores and loadings. In fact, for one specific version of PLSR, called SIMPLS [@Jong1993a], it can be shown that the scores and loadings are chosen in such a way to describe as much as possible of the *covariance* between $\mathbf{X}$ and $\mathbf{Y}$, where PCR concentrates on the *variance* of $\mathbf{X}$. Other PLSR algorithms give identical results to SIMPLS in the case of one $\mathbf{Y}$ variable, but deviate slightly for the multivariate $\mathbf{Y}$ case ; the differences are not likely to be important in practice.

## 2.1 Algorithms

In PCR, we approximate the $\mathbf{X}$ matrix by the first $a$ Principal Components (PCs), usually obtained from the singular value decomposition (SVD):

\begin{equation}
\mathbf{X} = \tilde{\mathbf{X}}_{(a)} + \Epsilon_X
    = (\mathbf{U}_{(a)} \mathbf{D}_{(a)} ) \mathbf{V}^T_{(a)} + \Epsilon_X
    = \mathbf{T}_{(a)} \mathbf{P}_{(a)}^T + \Epsilon_X
\end{equation}

Next, we regress $\mathbf{Y}$ on the scores, which leads to regression coefficients

\begin{equation}
\mathbf{B} = \mathbf{P} (\mathbf{T}^T \mathbf{T})^{-1} \mathbf{T}^T \mathbf{Y}
    = \mathbf{V} \mathbf{D}^{-1} \mathbf{U}^T \mathbf{Y}
\end{equation}

where the subscripts $a$ have been dropped for clarity. For PLSR, the components, called Latent Variables (LVs) in this context, are obtained iteratively. One starts with the SVD of the crossproduct matrix $\mathbf{S} = \mathbf{X}^T \mathbf{Y}$, thereby including information on variation in both $\mathbf{X}$ and $\mathbf{Y}$, and on the correlation between them. The first left and right singular vectors, $w$ and $q$, are used as weight vectors for $\mathbf{X}$ and $\mathbf{Y}$, respectively, to obtain scores $t$ and $u$:

\begin{equation}
t = \mathbf{X} w = \mathbf{E} w
\end{equation}
  
\begin{equation}
u = \mathbf{Y} q = \mathbf{F} q
\end{equation}

where $\mathbf{E}$ and $\mathbf{F}$ are initialised as $\mathbf{X}$ and $\mathbf{Y}$, respectively. The X scores $t$ are often normalised:

\begin{equation}
t =  t / \sqrt{t^Tt}
\end{equation}

The Y scores $u$ are not actually necessary in the regression but are often saved for interpretation purposes. Next, X and Y loadings are obtained by regressing against the *same* vector $t$:

\begin{equation}
\label{eq:plspt}
p = \mathbf{E}^T t
\end{equation}
\begin{equation}
\label{eq:plsqt}
q = \mathbf{F}^T t
\end{equation}

Finally, the data matrices are 'deflated': the information related to this latent variable, in the form of the outer products $t p^T$ and $t q^T$, is subtracted from the (current) data matrices $\mathbf{E}$ and $\mathbf{F}$.

\begin{equation}
\mathbf{E}_{n+1} = \mathbf{E}_n - t p^T
\end{equation}
\begin{equation}
\mathbf{F}_{n+1} = \mathbf{F}_n - t q^T
\end{equation}

The estimation of the next component then can start from the SVD of the crossproduct matrix $\mathbf{E}_{n+1}^T\mathbf{F}_{n+1}$. After every iteration, vectors $w$, $t$, $p$ and $q$ are saved as columns in matrices $\mathbf{W}$, $\mathbf{T}$, $\mathbf{P}$ and $\mathbf{Q}$, respectively. One complication is that columns of matrix $\mathbf{W}$ can not be compared directly: they are derived from successively deflated matrices $\mathbf{E}$ and $\mathbf{F}$. It has been shown that an alternative way to represent the weights, in such a way that all columns relate to the original $\mathbf{X}$ matrix, is given by 

\begin{equation}
\mathbf{R} = \mathbf{W} (\mathbf{P}^T \mathbf{W})^{-1}
\end{equation}

Now, we are in the same position as in the PCR case: instead of regressing $\mathbf{Y}$ on $\mathbf{X}$, we use scores $\mathbf{T}$ to calculate the regression coefficients, and later convert these back to the realm of the original variables by pre-multiplying with matrix $\mathbf{R}$ (since $\mathbf{T} = \mathbf{X} \mathbf{R}$):

\begin{equation}
\mathbf{B} = \mathbf{R} (\mathbf{T}^T \mathbf{T})^{-1} \mathbf{T}^T \mathbf{Y}
    = \mathbf{R} \mathbf{T}^T \mathbf{Y}
    = \mathbf{R} \mathbf{Q}^T
\end{equation}

Again, here, only the first $a$ components are used. How many components are optimal has to be determined, usually by cross-validation.

Many alternative formulations can be found in literature. It has been shown, for instance, that only one of $\mathbf{X}$ and $\mathbf{Y}$ needs to be deflated ; alternatively, one can directly deflate the crossproduct matrix $\mathbf{S}$ (as is done in SIMPLS, for example). Moreover, there are many equivalent ways of scaling. In the example above, the scores $t$ have been normalised, but one can also choose to introduce normalisation at another point in the algorithm. Unfortunately, this can make it difficult to directly compare the scores and loadings of different PLSR implementations.

## 2.2 On the use of PLSR and PCR

In theory, PLSR should have an advantage over PCR. One could imagine a situation where a minor component in $\mathbf{X}$ is highly correlated with $\mathbf{Y}$ ; not selecting enough components would then lead to very bad predictions. In PLSR, such a component would be automatically present in the first LV. In practice, however, there is hardly any difference between the use of PLSR and PCR ; in most situations, the methods achieve similar prediction accuracies, although PLSR usually needs fewer latent variables than PCR. Put the other way around: with the same number of latent variables, PLSR will cover more of the variation in $\mathbf{Y}$ and PCR will cover more of $\mathbf{X}$. In turn, both behave very similar to ridge regression [@Frank1993].

It can also be shown that both PCR and PLSR behave as shrinkage methods [@TESL], although in some cases PLSR seems to increase the variance of individual regression coefficients, one possible explanation of why PLSR is not always better than PCR.

# 3 Example session

In this section we will walk through an example session, to get an overview of the package. To be able to use the package, one first has to load it:

```{r load-lib}
library(pls)
```

This prints a message telling that the package has been attached, and that the package implements a function `loadings` that masks a function of the same name in package **stats**. (The output of the commands have in some cases been suppressed to save space.)

Four example data sets are included in **pls**:

* **`yarn`**: A data set with 28 near-infrared spectra (`NIR`) of PET yarns, measured at 268 wavelengths, as predictors, and density as response (`density`) [@SwiWeiWijBuy_StrConRobMulCalMod]. The data set also includes a logical variable `train` which can be used to split the data into a training data set of size 21 and test data set of size 7. See `?yarn` for details.
* **`oliveoil`**: A data set with 5 quality measurements (`chemical`) and 6 panel sensory panel variables (`sensory`) made on 16 olive oil samples [@Mas_etal_HandChemQualB]. See `?oliveoil` for details.
* **`gasoline`**: A data set consisting of octane number (`octane`) and NIR spectra (`NIR`) of 60 gasoline samples [@Kal:2DatNIR]. Each NIR spectrum consists of 401 diffuse reflectance measurements from 900 to 1700 nm. See `?gasoline` for details.
* **`mayonnaise`**: A data set consisting of oil type (`oil.type`), experimental design (`design`), train-test split (`train`) and NIR spectra (`NIR`) of 162 mayonnaise samples [@indahl1999multivariate]. Each NIR spectrum consists of 351 measurements from 1100 to 2500 nm. See `?mayonnaise`for details.

These will be used in the examples that follow. To use the data sets, they must first be loaded:

```{r load-data}
data(yarn)
data(oliveoil)
data(gasoline)
data(mayonnaise)
```

For the rest of the paper, it will be assumed that the package and the data sets have been loaded as above. Also, all examples are run with `options(digits = 4)`.

```{r fig-NIR, echo=FALSE, fig.height=1.5, fig.cap="Gasoline NIR spectra"}
par(mar = c(2, 4, 0, 1) + 0.1)
matplot(t(gasoline$NIR), type = "l", lty = 1, ylab = "log(1/R)", xaxt = "n")
ind <- pretty(seq(from = 900, to = 1700, by = 2))
ind <- ind[ind >= 900 & ind <= 1700]
ind <- (ind - 898) / 2
axis(1, ind, colnames(gasoline$NIR)[ind])
```

In this section, we will do a PLSR on the `gasoline` data to illustrate the use of **pls**. The spectra are shown in Figure 1. We first divide the data set into train and test data sets:

```{r split-data}
gasTrain <- gasoline[1:50,]
gasTest <- gasoline[51:60,]
```

A typical way of fitting a PLSR model is

```{r fit-gas1}
gas1 <- plsr(octane ~ NIR, ncomp = 10, data = gasTrain, validation = "LOO")
```

This fits a model with 10 components, and includes *leave-one-out* (LOO) cross-validated predictions [@LacMic:EstERRDA]. We can get an overview of the fit and validation results with the `summary` method:

```{r summary-gas1}
summary(gas1)
```

The validation results here are *Root Mean Squared Error of Prediction* (RMSEP). There are two cross-validation estimates: `CV` is the ordinary CV estimate, and `adjCV` is a bias-corrected CV estimate [@MevCed:MSEPest]. (For a LOO CV, there is virtually no difference) .

It is often simpler to judge the RMSEPs by plotting them:

```{r plot-RMSEP-code, eval=FALSE}
plot(RMSEP(gas1), legendpos = "topright")
```

```{r fig-RMSEP, echo=FALSE, fig.height=2.5, fig.cap="Cross-validated RMSEP curves for the gasoline data"}
par(mar = c(4, 4, 2.5, 1) + 0.1)
plot(RMSEP(gas1), legendpos = "topright")
```

This plots the estimated RMSEPs as functions of the number of components (Figure 2). The `legendpos` argument adds a legend at the indicated position. Two components seem to be enough. This gives an RMSEP of `r format(drop(RMSEP(gas1, "CV", ncomp = 2, intercept = FALSE)$val), digits = 3)`.

As mentioned in the introduction, the main practical difference between PCR and PLSR is that PCR often needs more components than PLSR to achieve the same prediction error. On this data set, PCR would need three components to achieve the same RMSEP. Once the number of components has been chosen, one can inspect different aspects of the fit by plotting predictions, scores, loadings, etc. The default plot is a prediction plot:

```{r plot-pred-code, eval=FALSE}
plot(gas1, ncomp = 2, asp = 1, line = TRUE)
```

```{r fig-cvpreds, echo=FALSE, fig.width=3.5, fig.height=3.7, fig.cap="Cross-validated predictions for the gasoline data"}
par(mar = c(4, 4, 2.5, 1) + 0.1)
plot(gas1, ncomp = 2, asp = 1, line = TRUE)
```

This shows the cross-validated predictions with two components versus measured values (Figure 3). We have chosen an aspect ratio of 1, and to draw a target line. The points follow the target line quite nicely, and there is no indication of a curvature or other anomalies. Other plots can be selected with the argument `plottype`:

```{r plot-scores-code, eval=FALSE}
plot(gas1, plottype = "scores", comps = 1:3)
```

```{r fig-scores, echo=FALSE, fig.cap="Score plot for the gasoline data"}
plot(gas1, plottype = "scores", comps = 1:3)
```

This gives a pairwise plot of the score values for the three first components (Figure 4). Score plots are often used to look for patterns, groups or outliers in the data. (For instance, plotting the two first components for a model built on the `yarn` dataset clearly indicates the experimental design of that data.) In this example, there is no clear indication of grouping or outliers. The numbers in parentheses after the component labels are the relative amount of X variance explained by each component. The explained variances can be extracted explicitly with 

```{r explvar}
explvar(gas1)
```

The loading plot (Figure 5) is much used for interpretation purposes, for instance to look for known spectral peaks or profiles:

```{r fig-loadings, fig.cap="Loading plot for the gasoline data", fig.height=2.5}
par(mar = c(4, 4, 0.3, 1) + 0.1)
plot(gas1, "loadings", comps = 1:2, legendpos = "topleft",
     labels = "numbers", xlab = "nm")
abline(h = 0)
```

The `labels = "numbers"` argument makes the plot function try to interpret the variable names as numbers, and use them as $x$ axis labels. Score and loading plots also have their own plotting functions, `scoreplot` and `loadingplot`, which have more options for plotting scores and loadings, respectively. For instance, `scoreplot`and its companion extractor `scores` has the same argument as `RMSEP/MSEP/R2` called `estimate`, which can be used to plot cross-validated scores or test data scores, e.g.,

```{r fig-scores-cv, fig.cap="Calibrated and cross-validated score plots for the gasoline data", fig.height=2.5, fig.width=5}
par.old <- par(mfrow=c(1,2))
scoreplot(gas1)
scoreplot(gas1, estimate="CV")
par(par.old)
```

A fitted model is often used to predict the response values of new observations. The following predicts the responses for the ten observations in `gasTest`, using two components:

```{r predict}
predict(gas1, ncomp = 2, newdata = gasTest)
```

Because we know the true response values for these samples, we can calculate the test set RMSEP:

```{r rmsep-test}
RMSEP(gas1, newdata = gasTest)
```

For two components, we get `r format(drop(RMSEP(gas1, ncomp = 2, intercept = FALSE, newdata = gasTest)$val), digits = 3)`, which is quite close to the cross-validated estimate above (`r format(drop(RMSEP(gas1, "CV", ncomp = 2, intercept = FALSE)$val), digits = 3)`).

# 4 Formulas and data frames

The **pls** package has a formula interface that works like the formula interface in **R**'s standard `lm` functions, in most ways. This section gives a short description of formulas and data frames as they apply to **pls**. More information on formulas can be found in the `lm` help file, in Chapter 11 of 'An Introduction to R', and in Chapter 2 of 'The White Book' [@R:Chambers+Hastie:1992]. These are good reads for anyone wanting to understand how **R** works with formulas, and the user is strongly advised to read them.

## 4.1 Formulas

A *formula* consists of a *left hand side* (lhs), a tilde (`~`), and a *right hand side* (rhs). The lhs consists of a single term, representing the response(s). The rhs consists of one or more terms separated by `+`, representing the regressor(s). For instance, in the formula `a ~ b + c + d`, `a` is the response, and `b`, `c`, and `d` are the regressors. The intercept is handled automatically, and need not be specified in the formula. Each term represents a matrix, a numeric vector or a factor (a factor should not be used as the response). If the response term is a matrix, a multi-response model is fit. In **pls**, the right hand side quite often consists of a single term, representing a matrix regressor: `y ~ X`. It is also possible to specify transformations of the variables. For instance, `log(y) ~ msc(Z)` specifies a regression of the logarithm of `y` onto `Z` after `Z` has been transformed by *Multiplicative Scatter (or Signal) Correction* (MSC) [@Geladi1985], a pre-treatment that is very common in infrared spectroscopy. If the transformations contain symbols that are interpreted in the formula handling, e.g., `+`, `*` or `^`, the terms should be protected with the `I()` function, like this: `y ~ x1 + I(x2 + x3)`. This specifies *two* regressors: `x1`, and the sum of `x2` and `x3`.

## 4.2 Data frames

The fit functions first look for the specified variables in a supplied data frame, and it is advisable to collect all variables there. This makes it easier to know what data has been used for fitting, to keep different variants of the data around, and to predict new data. To create a data frame, one can use the `data.frame` function: if `v1`, `v2` and `v3` are factors or numeric vectors, `mydata <- data.frame(y = v1, a = v2, b = v3)` will result in a data frame with variables named `y`, `a` and `b`. PLSR and PCR are often used with a matrix as the single predictor term (especially when one is working with spectroscopic data). Also, multi-response models require a matrix as the response term. If `Z` is a matrix, it has to be protected by the 'protect function' `I()` in calls to `data.frame`: `mydata <- data.frame(..., Z = I(Z))`. Otherwise, it will be split into separate variables for each column, and there will be no variable called `Z` in the data frame, so we cannot use `Z` in the formula. One can also add the matrix to an existing data frame:

```
> mydata <- data.frame(...)
> mydata$Z <- Z
```

This will also prevent `Z` from being split into separate variables. Finally, one can use `cbind` to combine vectors and matrices into matrices on the fly in the formula. This is most useful for the response, e.g., `cbind(y1, y2) ~ X`. Variables in a data frame can be accessed with the `$` operator, e.g., `mydata$y`. However, the **pls** functions access the variables automatically, so the user should never use `$` in formulas.

# 5 Fitting models

The main functions for fitting models are `pcr` and `plsr`. (They are simply wrappers for the function `mvr`, selecting the appropriate fit algorithm) . We will use `plsr` in the examples in this section, but everything could have been done with `pcr` (or `mvr`). In its simplest form, the function call for fitting models is `plsr(formula, ncomp, data)` (where `plsr` can be substituted with `pcr` or `mvr`). The argument `formula` is a formula as described above, `ncomp` is the number of components one wishes to fit, and `data` is the data frame containing the variables to use in the model. The function returns a fitted model (an object of class `"mvr"`) which can be inspected (Section 7) or used for predicting new observations (Section 8). For instance:

```{r dens1}
dens1 <- plsr(density ~ NIR, ncomp = 5, data = yarn)
```

If the response term of the formula is a matrix, a multi-response model is fit, e.g.,:

```{r olive1}
dim(oliveoil$sensory)
plsr(sensory ~ chemical, data = oliveoil)
```

(As we see, the `print` method simply tells us what type of model this is, and how the fit function was called.)

The argument `ncomp` is optional. If it is missing, the maximal possible number of components are used. Also `data` is optional, and if it is missing, the variables specified in the formula is searched for in the global environment (the user's workspace). Usually, it is preferable to keep the variables in data frames, but it can sometimes be convenient to have them in the global environment. If the variables reside in a data frame, e.g. `yarn`, *do not* be tempted to use formulas like `yarn$density ~ yarn$NIR`!  Use `density ~ NIR` and specify the data frame with `data = yarn` as above. There are facilities for working interactively with models. To use only part of the samples in a data set, for instance the first 20, one can use arguments `subset = 1:20` or `data = yarn[1:20,]`. Also, if one wants to try different alternatives of the model, one can use the function `update`. For instance

```{r update}
trainind <- which(yarn$train == TRUE)
dens2 <- update(dens1, subset = trainind)
```

will refit the model `dens1` using only the observations which are marked as `TRUE` in `yarn$train`, and 

```{r update2}
dens3 <- update(dens1, ncomp = 10)
```

will change the number of components to 10. Other arguments, such as `formula`, can also be changed with `update`. This can save a bit of typing when working interactively with models (but it doesn't save computing time; the model is refitted each time). In general, the reader is referred to 'The White Book' [@R:Chambers+Hastie:1992] or 'An Introduction to R' for more information about fitting and working with models in **R**.

Missing data can sometimes be a problem. The PLSR and PCR algorithms currently implemented in **pls** do not handle missing values intrinsically, so observations with missing values must be removed. This can be done with the `na.action` argument. With `na.action = na.omit` (the default), any observation with missing values will be removed from the model completely. With `na.action = na.exclude`, they will be removed from the fitting process, but included as `NA`s in the residuals and fitted values. If you want an explicit error when there are missing values in the data, use `na.action = na.fail`. The default `na.action` can be set with `options()`, e.g., `options(na.action = quote(na.fail))`.

Standardisation and other pre-treatments of predictor variables are often called for. In **pls**, the predictor variables are always centered, as a part of the fit algorithm. Scaling can be requested with the `scale` argument. If `scale` is `TRUE`, each variable is standardised by dividing it by its standard deviation, and if `scale` is a numeric vector, each variable is divided by the corresponding number. For instance, this will fit a model with standardised chemical measurements:

```{r olive2}
olive1 <- plsr(sensory ~ chemical, scale = TRUE, data = oliveoil)
```

As mentioned earlier, MSC [@Geladi1985] is implemented in **pls** as a function `msc` that can be used in formulas:

```{r gas2}
gas2 <- plsr(octane ~ msc(NIR), ncomp = 10, data = gasTrain)
```

This scatter corrects `NIR` prior to the fitting, and arranges for new spectra to be automatically scatter corrected (using the same reference spectrum as when fitting) in `predict`:

```{r predict-gas2, eval=FALSE}
predict(gas2, ncomp = 3, newdata = gasTest)
```

There are other arguments that can be given in the fit call: `validation` is for selecting validation, and `...` is for sending arguments to the underlying functions, notably the cross-validation function `mvrCv`. For the other arguments, see `?mvr`.

# 6 Choosing the number of components with cross-validation

Cross-validation, commonly used to determine the optimal number of components to take into account, is controlled by the `validation` argument in the modelling functions (`mvr`, `plsr` and `pcr`). The default value is `"none"`. Supplying a value of `"CV"` or `"LOO"` will cause the modelling procedure to call `mvrCv` to perform cross-validation; `"LOO"` provides leave-one-out cross-validation, whereas `"CV"` divides the data into segments. Default is to use ten segments, randomly selected, but also segments of consecutive objects or interleaved segments (sometimes also referred to as 'Venetian blinds') are possible through the use of the argument `segment.type`. One can also specify the segments explicitly with the argument `segments`; see `?mvrCv` for details. When validation is performed in this way, the model will contain an element comprising information on the out-of-bag predictions (in the form of predicted values, as well as MSEP and R2 values). As a reference, the MSEP error using no components at all is calculated as well. The validation results can be visualised using the `plottype = "validation"` argument of the standard plotting function. An example is shown in Figure 2 for the gasoline data; typically, one would select a number of components after which the cross-validation error does not show a significant decrease. The decision on how many components to retain will to some extent always be subjective. However, especially when building large numbers of models (e.g., in simulation studies), it can be crucial to have a consistent strategy on how to choose the "optimal" number of components. Two such strategies have been implemented in function `selectNcomp`. The first is based on the so-called one-sigma heuristic [@TESL2013] and consists of choosing the model with fewest components that is still less than one standard error away from the overall best model. The second strategy employs a permutation approach, and basically tests whether adding a new component is beneficial at all [@Voet1994]. It is implemented backwards, again taking the global minimum in the crossvalidation curve as a starting point, and assessing models with fewer and fewer components: as long as no significant deterioration in performance is found (by default on the $\alpha = 0.01$ level), the algorithm continues to remove components. Applying the function is quite straightforward:

```{r selectNcomp-code, eval=FALSE}
ncomp.onesigma <- selectNcomp(gas2, method = "onesigma", plot = TRUE,
                              ylim = c(.18, .6))
ncomp.permut <- selectNcomp(gas2, method = "randomization", plot = TRUE,
                            ylim = c(.18, .6))
```

This leads to the plots in Figure 6 -- note that graphical arguments can be supplied to customize the plots. In both cases, the global minimum of the crossvalidation curve is indicated with gray dotted lines, and the suggestion for the optimal number of components with a vertical blue dashed line. The left plot shows the width of the one-sigma intervals on which the suggestion is based ; the right plot indicates which models have been assessed by the permutation approach through the large blue circles. The two criteria do not always agree (as in this case) but usually are quite close.

```{r fig-NComp, echo=FALSE, fig.height=4.5, fig.width=10, fig.cap="The two strategies for suggesting optimal model dimensions: the left plot shows the one-sigma strategy, the right plot the permutation strategy."}
par(mfrow = c(1,2))
ncomp.onesigma <- selectNcomp(gas1, "onesigma", plot = TRUE,
                              ylim = c(.18, .6))
ncomp.permut <- selectNcomp(gas1, "randomization", plot = TRUE,
                            ylim = c(.18, .6))
```

When a pre-treatment that depends on the composition of the training set is applied, the cross-validation procedure as described above is not optimal, in the sense that the cross-validation errors are biased downward. As long as the only purpose is to select the optimal number of components, this bias may not be very important, but it is not too difficult to avoid it. The modelling functions have an argument `scale` that can be used for auto-scaling per segment. However, more elaborate methods such as MSC need explicit handling per segment. For this, the function `crossval` is available. It takes an `mvr` object and performs the cross-validation as it should be done: applying the pre-treatment for each segment. The results can be shown in a plot (which looks very similar to Figure 2) or summarised in numbers.

```{r crossval}
gas2.cv <- crossval(gas2, segments = 10)
plot(MSEP(gas2.cv), legendpos="topright")
summary(gas2.cv, what = "validation")
```

Applying MSC in this case leads to nearly identical cross-validation estimates of prediction error. When the scaling does not depend on the division of the data into segments (e.g., log-scaling), functions `crossval` and `mvrCv` give the same results ; however, `crossval` is much slower.

Cross-validation can be computationally demanding (especially when using the function `crossval`). Therefore, both `mvrCv` and `crossval` can perform the calculations in parallel on a multi-core machine or on several machines. How to do this is described in Section 9.2.

# 7 Inspecting fitted models

A closer look at the fitted model may reveal interesting agreements or disagreements with what is known about the relations between X and Y. Several functions are implemented in **pls** for plotting, extracting and summarising model components.

## 7.1 Plotting

One can access all plotting functions through the `"plottype"` argument of the `plot` method for `mvr` objects. This is simply a wrapper function calling the actual plot functions; the latter are available to the user as well. The default plot is a prediction plot (`predplot`), showing predicted versus measured values. Test set predictions are used if a test set is supplied with the `newdata` argument. Otherwise, if the model was built using cross-validation, the cross-validated predictions are used, otherwise the predictions for the training set. This can be overridden with the `which` argument. An example of this type of plot can be seen in Figure 3. An optional argument can be used to indicate how many components should be included in the prediction. To assess how many components are optimal, a validation plot (`validationplot`) can be used such as the one shown in Figure 2; this shows a measure of prediction performance (either RMSEP, MSEP, or $R^2$) against the number of components. Usually, one takes the first local minimum rather than the absolute minimum in the curve, to avoid over-fitting. The regression coefficients can be visualised using `plottype = "coef"` in the `plot` method, or directly through function `coefplot`. This allows simultaneous plotting of the regression vectors for several different numbers of components at once. The regression vectors for the `gasoline` data set using MSC are shown in Figure 7 using the command 

```{r plot-coef-code, eval=FALSE}
plot(gas1, plottype = "coef", ncomp=1:3, legendpos = "bottomleft",
     labels = "numbers", xlab = "nm")
```

```{r fig-gascoefs, echo=FALSE, fig.height=3, fig.cap="Regression coefficients for the gasoline data"}
par(mar = c(4, 4, 2.5, 1) + 0.1)
plot(gas1, plottype = "coef", ncomp=1:3, legendpos = "bottomleft",
     labels = "numbers", xlab = "nm")
```

Note that the coefficients for two components and three components are similar. This is because the third component contributes little to the predictions. The RMSEPs (see Figure 2) and predictions (see Section 8) for two and three components are quite similar. Scores and loadings can be plotted using functions `scoreplot` (an example is shown in Figure 4) and `loadingplot` (in Figure 5), respectively. One can indicate the number of components with the `comps` argument ; if more than two components are given, plotting the scores will give a pairs plot, otherwise a scatter plot. For `loadingplot`, the default is to use line plots.

Finally, a 'correlation loadings' plot (function `corrplot`, or `plottype = "correlation"` in `plot`) shows the correlations between each variable and the selected components (see Figure 8). These plots are scatter plots of two sets of scores with concentric circles of radii given by `radii`. Each point corresponds to an X variable. The squared distance between the point and the origin equals the fraction of the variance of the variable explained by the components in the panel. The default values for `radii` correspond to 50% and 100% explained variance, respectively.

```{r fig-corrplot, echo=FALSE, fig.width=3.5, fig.height=3.4, fig.cap="Correlation loadings plot for the gasoline data"}
par(mar = c(4, 4, 0, 1) + 0.1)
plot(gas1, plottype = "correlation")
```

The plot functions accept most of the ordinary plot parameters, such as `col` and `pch`. If the model has several responses or one selects more than one model size, e.g. `ncomp = 4:6`, in some plot functions (notably prediction plots (see below), validation plots and coefficient plots) the plot window will be divided and one plot will be shown for each combination of response and model size. The number of rows and columns are chosen automatically, but can be specified explicitly with arguments `nRows` and `nCols`. If there are more plots than fit the plot window, one will be asked to press return to see the rest of the plots.

## 7.2 Extraction

Regression coefficients can be extracted using the generic function `coef`; the function takes several arguments, indicating the number of components to take into account, and whether the intercept is needed (default is `FALSE`). Scores and loadings can be extracted using functions `scores` and `loadings` for X, and `Yscores` and `Yloadings` for Y. These also return the percentage of variance explained as attributes. In PLSR, weights can be extracted using the function `loading.weights`. When applied to a PCR model, the function returns `NULL`. Note that commands like `plot(scores(gas1))` are perfectly correct, and lead to exactly the same plots as using `scoreplot`. The `vcov` function can be used to get the covariance matrix of `pcr` models, e.g., for uncertainty plotting (see `?vcov.mvr`).

## 7.3 Summaries

The `print` method for an object of class `"mvr"` shows the regression type used, perhaps indicating the form of validation employed, and shows the function call. The `summary` method gives more information: it also shows the amount of variance explained by the model (for all choices of $a$, the number of latent variables). The `summary` method has an additional argument (`what`) to be able to focus on the training phase or validation phase, respectively. Default is to print both types of information.

# 8 Predicting new observations

Fitted models are often used to predict future observations, and **pls** implements a `predict` method for PLSR and PCR models. The most common way of calling this function is `predict(mymod, ncomp = myncomp, newdata = mynewdata)`, where `mymod` is a fitted model, `myncomp` specifies the model size(s) to use, and `mynewdata` is a data frame with new X observations. The data frame can also contain response measurements for the new observations, which can be used to compare the predicted values to the measured ones, or to estimate the overall prediction ability of the model. If `newdata` is missing, `predict` uses the data used to fit the model, i.e., it returns fitted values. If the argument `ncomp` is missing, `predict` returns predictions for models with 1 component, 2 components, $\ldots$, $A$ components, where $A$ is the number of components used when fitting the model. Otherwise, the model size(s) listed in `ncomp` are used. For instance, to get predictions from the model built in Section 3, with two and three components, one would use 

```{r predict2}
predict(gas1, ncomp = 2:3, newdata = gasTest[1:5,])
```

(We predict only the five first test observations, to save space.) The predictions with two and three components are quite similar. This could be expected, given that the regression vectors (Figure 7) as well as the estimated RMSEPs for the two model sizes were similar. One can also specify explicitly which components to use when predicting. This is done by specifying the components in the argument `comps`. (If both `ncomp` and `comps` are specified, `comps` takes precedence over `ncomp`.) For instance, to get predictions from a model with only component 2, one can use 

```{r predict3}
predict(gas1, comps = 2, newdata = gasTest[1:5,])
```

The results are different from the predictions with two components (i.e., components one and two) above. (The intercept is always included in the predictions. It can be removed by subtracting `mymod$Ymeans` from the predicted values.) 

The `predict` method returns a three-dimensional array, in which the entry $(i,j,k)$ is the predicted value for observation $i$, response $j$ and model size $k$. Note that singleton dimensions are not dropped, so predicting five observations for a uni-response model with `ncomp = 3` gives an $5 \times 1 \times 1$ array, not a vector of length five. This is to make it easier to distinguish between predictions from models with one response and predictions with one model size. (When using the `comps` argument, the last dimension is dropped, because the predictions are always from a single model.) One can drop the singleton dimensions explicitly by using `drop(predict(...))`:

```{r predict4}
drop(predict(gas1, ncomp = 2:3, newdata = gasTest[1:5,]))
```

Missing values in `newdata` are propagated to `NA`s in the predicted response, by default. This can be changed with the `na.action` argument. See `?na.omit` for details. The `newdata` does not have to be a data frame. Recognising the fact that the right hand side of PLSR and PCR formulas very often are a single matrix term, the `predict` method allows one to use a matrix as `newdata`, so instead of 
```
newdataframe <- data.frame(X = newmatrix)
predict(..., newdata = newdataframe)
```
one can simply say
```
predict(..., newdata = newmatrix)
```
However, there are a couple of caveats: First, this *only* works in `predict`. Other functions that take a `newdata` argument (such as `RMSEP`) must have a data frame (because they also need the response values). Second, when `newdata` is a data frame, `predict` is able to perform more tests on the supplied data, such as the dimensions and types of variables. Third, with the exception of scaling (specified with the `scale` argument when fitting the model), any transformations or coding of factors and interactions have to be performed manually if `newdata` is a matrix. It is often interesting to predict scores from new observations, instead of response values. This can be done by specifying the argument `type = "scores"` in `predict`. One will then get a matrix with the scores corresponding to the components specified in `comps` (`ncomp` is accepted as a synonym for `comps` when predicting scores). Predictions can be plotted with the function `predplot`. This function is generic, and can also be used for plotting predictions from other types of models, such as `lm`. Typically, `predplot` is called like this:

```{r predplot-code, eval=FALSE}
predplot(gas1, ncomp = 2, newdata = gasTest, asp = 1, line = TRUE)
```

```{r fig-testPreds, echo=FALSE, fig.width=3.5, fig.height=3.7, fig.cap="Test set predictions"}
par(mar = c(4, 4, 2.5, 1))
predplot(gas1, ncomp = 2, newdata = gasTest, asp = 1, line = TRUE)
```

This plots predicted (with 2 components) versus measured response values. (Note that `newdata` must be a data frame with both X and Y variables.) 

# 9 Further topics

This section presents a couple of slightly technical topics for more advanced use of the package.

## 9.1 Selecting fit algorithms

There are several PLSR algorithms, and the **pls** package currently implements three of them: the kernel algorithm for tall matrices (many observations, few variables) [@DayMacGre:ImprPlsAlg], the classic orthogonal scores algorithm (A.K.A. NIPALS algorithm) [@MarNaes:MultCal] and the SIMPLS algorithm [@Jong1993a]. The kernel and orthogonal scores algorithms produce the same results (the kernel algorithm being the fastest of them for most problems). SIMPLS produces the same fit for single-response models, but slightly different results for multi-response models. It is also usually faster than the NIPALS algorithm.

The factory default is to use the kernel algorithm. One can specify a different algorithm with the `method` argument; i.e., `method = "oscorespls"`. If one's personal taste of algorithms does not coincide with the defaults in **pls**, it can be quite tedious (and error prone) having to write e.g. `method = "oscorespls"` every time (even though it can be shortened to e.g. `me = "o"` due to partial matching). Therefore, the defaults can be changed, with the function `pls.options`. Called without arguments, it returns the current settings as a named list:

```{r pls.options}
pls.options()
```

The options specify the default fit algorithm of `mvr`, `plsr`, and `pcr`. To return only a specific option, one can use `pls.options("plsralg")`. To change the default PLSR algorithm for the rest of the session, one can use, e.g.:

```{r pls.options-set}
pls.options(plsralg = "oscorespls")
```

Note that changes to the options only last until **R** exits. (Earlier versions of **pls** stored the changes in the global environment so they could be saved and restored, but current CRAN policies do not allow this.)

### 9.1.1 Additional responses and focused loadings

One of the alternative PLS algorithm is called Canonical (Powered) Partial Least Squares and is available as `cppls` when fitting. It uses canonical correlation for the loading calculations. For single response regression models, this coincides with the standard algorihms, while fore multiresponse and dummy-coded responses (or their combination), this criterion is more aggressive, typically leading to more parsimonious models and sometimes better prediction performance. The use of canonical correlation also opens for additional responses used when fitting, but not when predicting. These are enabled throught the extra argument `Y.add`. Weighting of samples is also possible using the `weights`argument. Finally, focused loading weights can be enabled or forced throught the `lower`and `upper`arguments (see the example in `?cppls` and [@indahl2009canonical])

### 9.1.2 Missing data handling

When data are missing completely at random (MCAR), algorithms `nipals`and `nipalspc` can be used in place of the standard algorithms. These omit the missing values without dropping observations or variables, calculating loading weights, scores and loadings through an iterative process called NIPALS.

## 9.2 Parallel cross-validation

Cross-validation is a computationally demanding procedure. A new model has to be fitted for each segment. The underlying fit functions have been optimised, and the implementation of cross-validation that is used when specifying the `validation` argument to `mvr` tries to avoid any unneeded calculations (and house-keeping things like the formula handling, which can be surprisingly expensive). Even so, cross-validation can take a long time, for models with large matrices, many components or many segments. By default, the cross-validation calculations in **pls** is performed serially, on one CPU (core). (In the following, we will use 'CPU' to denote both CPUs and cores.) 

Since version 2.14.0, **R** has shipped with a package **parallel** for running calculations in parallel, on multi-CPU machines or on several machines. The **pls** package can use the facilities of **parallel** to run the cross-validations in parallel. The **parallel** package has several ways of running calculations in parallel, and not all of them are available on all systems. Therefore, the support in **pls** is quite general, so one can select the ways that work well on the given system. To specify how to run calculations in parallel, one sets the option `parallel` in `pls.options`. After setting the option, one simply runs cross-validatons as before, and the calculations will be performed in parallel. This works both when using the `crossval` function and the `validation` argument to `mvr`. The parallel specification has effect until it is changed.

The default value for `parallel` is `NULL`, which specifies that the calculations are done serially, using one CPU. Specifying the value 1 has the same effect.

Specifying an integer $> 1$ makes the calculations use the function `mclapply` with the given number as the number of CPUs to use. Note: `mclapply` depends on 'forking' which does not exist on MS Windows, so `mclapply` cannot be used there. Example:

```r
pls.options(parallel = 4) # Use mclapply with 4 CPUs
gas1.cv <- plsr(octane ~ NIR, ncomp = 10, data = gasTrain, validation = "LOO")
```

The `parallel` option can also be specified as a cluster object created by the `makeCluster` function from the package **parallel**. Any following cross-validation will then be performed with the function `parLapply` on that cluster. Any valid cluster specification can be used. The user should stop the cluster with `stopCluster(pls.options()$parallel)` when it is no longer needed.

```r
library(parallel) # Needed for the makeCluster call
pls.options(parallel = makeCluster(4, type = "PSOCK")) # PSOCK cluster, 4 CPUs
gas1.cv <- plsr(octane ~ NIR, ncomp = 10, data = gasTrain, validation = "LOO")
## later:
stopCluster(pls.options()$parallel)
```

Several types of clusters are available: FORK uses forking, so starting the cluster is very quick, however it is not available on MS Windows. PSOCK starts **R** processes with the `Rscript` command, which is slower, but is supported on MS Windows. It can also start worker processes on different machines (see `?makeCluster` for how). MPI uses MPI to start and communicate with processes. This is the most flexible, but is often slower to start up than the other types. It also dependens on the packages **snow** and **Rmpi** to be installed and working. It is especially useful when running batch jobs on a computing cluster, because MPI can interact with the queue system on the cluster to find out which machines to use when the job starts. Here is an example of running a batch job on a cluster using MPI:

R script (myscript.R):
```r
library(parallel) # for the makeCluster call
pls.options(parallel = makeCluster(16, type = "MPI") # MPI cluster, 16 CPUs
gas1.cv <- plsr(octane ~ NIR, ncomp = 10, data = gasTrain, validation = "LOO")
## later:
save.image(file = "results.RData")
stopCluster(pls.options()$parallel)
mpi.exit() # stop Rmpi
```

To run the job:
```bash
mpirun -np 1 R --slave --file=myscript.R
```
The details of how to run `mpirun` varies between the different MPI implementations and how they interact with the queue system used (if any). The above should work for OpenMPI or Intel MPI running under the Slurm queue system. In other situations, one might have to specify which machines to use with, e.g., the `-host` or `-machinefile` switch.

## 9.3 Package design

The **pls** package is designed such that an interface function `mvr` handles the formula and data, and calls an underlying fit function (and possibly a cross-validation function) to do the real work. There are several reasons for this design: it makes it easier to implement new algorithms, one can easily skip the time-consuming formula and data handling in computing-intensive applications (simulations, etc.), and it makes it easier to use the **pls** package as a building block in other packages. The plotting facilities are implemented similarly: the `plot` method simply calls the correct plot function based on the `plottype` argument. Here, however, the separate plot functions are meant to be callable interactively, because some people like to use the generic `plot` function, while others like to use separate functions for each plot type. There are also `plot` methods for some of the components of fitted models that can be extracted with extract functions, like score and loading matrices. Thus there are several ways to get some plots, e.g.:

```r
plot(mymod, plottype = "scores", ...)
scoreplot(mymod, ...)
plot(scores(mymod), ...)
```

One example of a package that uses **pls** is **lspls**, available on CRAN. In that package LS is combined with PLS in a regression procedure. It calls the fit functions of **pls** directly, and also uses the plot functions to construct score and loading plots. There is also the **plsgenomics** package, which includes a modified version of (an earlier version of) the SIMPLS fit function `simpls.fit`.

## 9.4 Calling fit functions directly

The underlying fit functions are called `kernelpls.fit`, `oscorespls.fit`, and `simpls.fit` for the PLSR methods, and `svdpc.fit` for the PCR method. They all take arguments `X`, `Y`, `ncomp`, and `stripped`. Arguments `X`, `Y`, and `ncomp` specify $\mathbf{X}$ and $\mathbf{Y}$ (as matrices, not data frames), and the number of components to fit, respectively. The argument `stripped` defaults to `FALSE`. When it is `TRUE`, the calculations are stripped down to the bare minimum required for returning the $\mathbf{X}$ means, $\mathbf{Y}$ means, and the regression coefficients. This is used to speed up cross-validation procedures.

The fit functions can be called directly, for instance when one wants to avoid the overhead of formula and data handling in repeated fits. As an example, this is how a simple leave-one-out cross-validation for a uni-response-model could be implemented, using the SIMPLS:

```{r manual-cv}
X <- gasTrain$NIR
Y <- gasTrain$octane
ncomp <- 5
cvPreds <- matrix(nrow = nrow(X), ncol = ncomp)
for (i in 1:nrow(X)) {
    fit <- simpls.fit(X[-i,], Y[-i], ncomp = ncomp, stripped = TRUE)
    cvPreds[i,] <- (X[i,] - fit$Xmeans) %*% drop(fit$coefficients) +
        fit$Ymeans
}
```

The RMSEP of the cross-validated predictions are

```{r manual-cv-rmsep}
sqrt(colMeans((cvPreds - Y)^2))
```

which can be seen to be the same as the (unadjusted) CV results for the `gas1` model in Section 3.

## 9.5 Formula handling in more detail

The handling of formulas and variables in the model fitting is very similar to what happens in the function `lm`: The variables specified in the formula are looked up in the data frame given in the `data` argument of the fit function (`plsr`, `pcr` or `mvr`), or in the calling environment if not found in the data frame. Factors are coded into one or more of columns, depending on the number of levels, and on the contrasts option. All (possibly coded) variables are then collected in a numerical model matrix. This matrix is then handed to the underlying fit or cross-validation functions. A similar handling is used in the `predict` method. The intercept is treated specially in **pls**. After the model matrix has been constructed, the intercept column is removed. This ensures that any factors are coded as if the intercept was present. The underlying fit functions then center the rest of the variables as a part of the fitting process. (This is intrinsic to the PLSR and PCR algorithms.) The intercept is handled separately. A consequence of this is that explicitly specifying formulas without the intercept (e.g., `y ~ a + b - 1`) will only result in the coding of any factors to change ; the intercept will still be fitted.

# References

::: {#refs}

:::

=head1 NAME

PLS - Perl Language Server

=head1 DESCRIPTION

The Perl Language Server implements a subset of the Language Server Protocol
for the Perl language. Features currently implemented are:

=over

=item * Go to definition (for packages, subroutines, and variables)

=item * Listing all symbols in a document

=item * Hovering to show documentation

=item * Signature help (showing parameters for a function as you type)

=item * Formatting

=item * Range Formatting

=item * Auto-completion

=item * Syntax checking

=item * Linting (using perlcritic)

=item * Sorting imports

=back

=head1 OPTIONS

This application does not take any command line options.
The following settings may be configured using your text editor:

=over

=item pls.cmd - path to pls

Make sure that C<pls.cmd> is set to the path to the C<pls> script on your system.
If you rely on your C<$PATH>, ensure that your editor is configured with the correct
path, which may not be the same one that your terminal uses.

=item pls.args - args to pass to the pls command

Add any additional arguments needed to execute C<pls> to the C<pls.args> setting.
For example, if you run C<pls> in a docker container, C<pls.cmd> would be C<docker>, and
C<pls.args> would be C<< ["run", "--rm", "-i", "<image name>", "pls"] >>.

=item pls.inc - a list of paths to add to C<@INC>

You can use the C<$ROOT_PATH> mnemonic to stand in for your project's root directory,
for example C<$ROOT_PATH/lib>. If you are using multiple workspace folders and use
C<$ROOT_PATH>, the path will be multiplied by the number of workspace folders,
and will be replaced that many times. This is useful if you use SVN and check out
each branch to a different directory.

=item pls.cwd - the working directory to use for pls

If you use C<$ROOT_PATH>, it will be replaced by your workspace's first
or only folder.

=item pls.perltidy.perltidyrc - the location of your C<.perltidyrc> file.

Defaults to C<~/.perltidyrc> if not configured.

=item pls.perlcritic.enabled - whether to enable linting using L<perlcritic>.

=item pls.perlcritic.perlcriticrc - the location of your C<.perlcriticrc> file.

Defaults to C<~/.perlcriticrc> if not configured.

=item pls.syntax.enabled - whether to enable syntax checking.

=item pls.syntax.perl - path to an alternate C<perl> to use for syntax checking.

Defaults to the C<perl> used to run PLS.

=item pls.syntax.args - additional arguments to pass when syntax checking.

This is useful if there is a BEGIN block in your code that changes
behavior depending on the contents of @ARGV.

=back

You may configure a .plsignore file in your project's root directory, with
a list of Perl glob patterns which you do not want pls to index.

By default, PLS will index all files ending with `.pl`, `.pm`,
or have `perl` in the shebang line that are not `.t` files.

=head1 CAVEATS

pls is known to be compatible with Visual Studio Code, Neovim, and BBEdit.

pls will perform much better if you have an XS JSON module installed.
If you install L<Cpanel::JSON::XS> or L<JSON::XS>, it will use one of those
before falling back to L<JSON::PP>, similar to L<JSON::MaybeXS>.

=head1 NOTES

Refer to this README for instructions on configuring your specific editor:
L<https://marketplace.visualstudio.com/items?itemName=FractalBoy.pls>

=head1 COPYRIGHT

Copyright 2022 Marc Reisner

=head1 LICENSE

This library is free software; you may redistribute it and/or
modify it under the same terms as Perl itself.

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/stdize.R
\name{stdize}
\alias{stdize}
\alias{predict.stdized}
\alias{makepredictcall.stdized}
\title{Standardization of Data Matrices}
\usage{
stdize(x, center = TRUE, scale = TRUE)

\method{predict}{stdized}(object, newdata, ...)

\method{makepredictcall}{stdized}(var, call)
}
\arguments{
\item{x, newdata}{numeric matrices.  The data to standardize.}

\item{center}{logical value or numeric vector of length equal to the number
of coloumns of \code{x}.}

\item{scale}{logical value or numeric vector of length equal to the number
of coloumns of \code{x}.}

\item{object}{an object inheriting from class \code{"stdized"}, normally the
result of a call to \code{stdize}.}

\item{\dots}{other arguments.  Currently ignored.}

\item{var}{A variable.}

\item{call}{The term in the formula, as a call.}
}
\value{
Both \code{stdize} and \code{predict.stdized} return a scaled and/or
centered matrix, with attributes \code{"stdized:center"} and/or
\code{"stdized:scale"} the vector used for centering and/or scaling.  The
matrix is given class \code{c("stdized", "matrix")}.
}
\description{
Performs standardization (centering and scaling) of a data matrix.
}
\details{
\code{makepredictcall.stdized} is an internal utility function; it is not
meant for interactive use.  See \code{\link{makepredictcall}} for details.

If \code{center} is \code{TRUE}, \code{x} is centered by subtracting the
coloumn mean from each coloumn.  If \code{center} is a numeric vector, it is
used in place of the coloumn means.

If \code{scale} is \code{TRUE}, \code{x} is scaled by dividing each coloumn
by its sample standard deviation.  If \code{scale} is a numeric vector, it
is used in place of the standard deviations.
}
\note{
\code{stdize} is very similar to \code{\link[base]{scale}}.  The
difference is that when \code{scale = TRUE}, \code{stdize} divides the
coloumns by their standard deviation, while \code{scale} uses the
root-mean-square of the coloumns.  If \code{center} is \code{TRUE}, this is
equivalent, but in general it is not.
}
\examples{

data(yarn)
## Direct standardization:
Ztrain <- stdize(yarn$NIR[yarn$train,])
Ztest <- predict(Ztrain, yarn$NIR[!yarn$train,])

## Used in formula:
mod <- plsr(density ~ stdize(NIR), ncomp = 6, data = yarn[yarn$train,])
pred <- predict(mod, newdata = yarn[!yarn$train,]) # Automatically standardized

}
\seealso{
\code{\link{mvr}}, \code{\link{pcr}}, \code{\link{plsr}},
\code{\link{msc}}, \code{\link[base]{scale}}
}
\author{
Bjørn-Helge Mevik and Ron Wehrens
}
\keyword{multivariate}
\keyword{regression}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plots.R
\name{biplot.mvr}
\alias{biplot.mvr}
\title{Biplots of PLSR and PCR Models.}
\usage{
\method{biplot}{mvr}(
  x,
  comps = 1:2,
  which = c("x", "y", "scores", "loadings"),
  var.axes = FALSE,
  xlabs,
  ylabs,
  main,
  ...
)
}
\arguments{
\item{x}{an \code{mvr} object.}

\item{comps}{integer vector of length two.  The components to plot.}

\item{which}{character.  Which matrices to plot.  One of \code{"x"} (X
scores and loadings), \code{"y"} (Y scores and loadings), \code{"scores"} (X
and Y scores) and \code{"loadings"} (X and Y loadings).}

\item{var.axes}{logical.  If \code{TRUE}, the second set of points have
arrows representing them.}

\item{xlabs}{either a character vector of labels for the first set of
points, or \code{FALSE} for no labels.  If missing, the row names of the
first matrix is used as labels.}

\item{ylabs}{either a character vector of labels for the second set of
points, or \code{FALSE} for no labels.  If missing, the row names of the
second matrix is used as labels.}

\item{main}{character.  Title of plot.  If missing, a title is constructed
by \code{biplot.mvr}.}

\item{\dots}{Further arguments passed on to \code{biplot.default}.}
}
\description{
Biplot method for \code{mvr} objects.
}
\details{
\code{biplot.mvr} can also be called through the \code{mvr} plot method by
specifying \code{plottype = "biplot"}.
}
\examples{

data(oliveoil)
mod <- plsr(sensory ~ chemical, data = oliveoil)
\dontrun{
## These are equivalent
biplot(mod)
plot(mod, plottype = "biplot")

## The four combinations of x and y points:
par(mfrow = c(2,2))
biplot(mod, which = "x") # Default
biplot(mod, which = "y")
biplot(mod, which = "scores")
biplot(mod, which = "loadings")
}

}
\seealso{
\code{\link{mvr}}, \code{\link{plot.mvr}},
\code{\link{biplot.default}}
}
\author{
Ron Wehrens and Bjørn-Helge Mevik
}
\keyword{hplot}
\keyword{multivariate}
\keyword{regression}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mvr.R, R/mvr_wrappers.R
\name{mvr}
\alias{mvr}
\alias{pcr}
\alias{plsr}
\alias{cppls}
\alias{nipals}
\alias{nipalspcr}
\title{Partial Least Squares and Principal Component Regression}
\usage{
mvr(
  formula,
  ncomp,
  Y.add,
  data,
  subset,
  na.action,
  method = pls.options()$mvralg,
  scale = FALSE,
  center = TRUE,
  validation = c("none", "CV", "LOO"),
  model = TRUE,
  x = FALSE,
  y = FALSE,
  ...
)

plsr(..., method = pls.options()$plsralg)

pcr(..., method = pls.options()$pcralg)

cppls(..., Y.add, weights, method = pls.options()$cpplsalg)

nipals(..., weights, method = "nipalspls")

nipalspcr(..., method = "nipalspc")
}
\arguments{
\item{formula}{a model formula.  Most of the \code{\link{lm}} formula
constructs are supported.  See below.}

\item{ncomp}{the number of components to include in the model (see below).}

\item{Y.add}{a vector or matrix of additional responses containing relevant
information about the observations.  Only used for \code{cppls}.}

\item{data}{an optional data frame with the data to fit the model from.}

\item{subset}{an optional vector specifying a subset of observations to be
used in the fitting process.}

\item{na.action}{a function which indicates what should happen when the data
contain missing values.  The default is set by the \code{na.action} setting
of \code{\link{options}}, and is \code{\link{na.fail}} if that is unset.
The \sQuote{factory-fresh} default is \code{\link{na.omit}}.  Another
possible value is \code{NULL}, no action.  Value \code{\link{na.exclude}}
can be useful.  See \code{\link{na.omit}} for other alternatives.}

\item{method}{the multivariate regression method to be used.  If
\code{"model.frame"}, the model frame is returned.}

\item{scale}{numeric vector, or logical.  If numeric vector, \eqn{X} is
scaled by dividing each variable with the corresponding element of
\code{scale}.  If \code{scale} is \code{TRUE}, \eqn{X} is scaled by dividing
each variable by its sample standard deviation.  If cross-validation is
selected, scaling by the standard deviation is done for every segment.}

\item{center}{logical, determines if the \eqn{X} and \eqn{Y} matrices are
mean centered or not. Default is to perform mean centering.}

\item{validation}{character.  What kind of (internal) validation to use.
See below.}

\item{model}{a logical.  If \code{TRUE}, the model frame is returned.}

\item{x}{a logical.  If \code{TRUE}, the model matrix is returned.}

\item{y}{a logical.  If \code{TRUE}, the response is returned.}

\item{\dots}{additional optional arguments, passed to the underlying fit
functions, and \code{\link{mvrCv}}.

Currently, the fit functions \code{\link{oscorespls.fit}} and
\code{\link{widekernelpls.fit}} implement these extra arguments: \describe{
\item{tol:}{numeric.  Tolerance used for determining convergence.}
\item{maxit:}{positive integer.  The maximal number of iterations used.} }
and \code{\link{cppls.fit}} implements: \describe{ \item{lower:}{a vector of
lower limits for power optimisation.} \item{upper:}{a vector of upper limits
for power optimisation.} \item{trunc.pow:}{logical. Whether to use an
experimental alternative power algorithm.} } \code{\link{mvrCv}} implements
several arguments; the following are probably the most useful of them:
\describe{ \item{segments:}{the number of segments to use, or a list with
segments.} \item{segment.type:}{the type of segments to use.}
\item{length.seg:}{Positive integer.  The length of the segments to use.}
\item{jackknife:}{logical.  Whether to perform jackknifing of regression
coefficients.} }

See the functions' documentation for details.}

\item{weights}{a vector of individual weights for the observations.  Only
used for \code{cppls}.  (Optional)}
}
\value{
If \code{method = "model.frame"}, the model frame is returned.
Otherwise, an object of class \code{mvr} is returned.  The object contains
all components returned by the underlying fit function.  In addition, it
contains the following components: \item{validation}{if validation was
requested, the results of the cross-validation.  See \code{\link{mvrCv}} for
details.} \item{fit.time}{the elapsed time for the fit.  This is used by
\code{\link{crossval}} to decide whether to turn on tracing.}
\item{na.action}{if observations with missing values were removed,
\code{na.action} contains a vector with their indices.  The class of this
vector is used by functions like \code{fitted} to decide how to treat the
observations.} \item{ncomp}{the number of components of the model.}
\item{method}{the method used to fit the model.  See the argument
\code{method} for possible values.} \item{center}{use of centering in the model}
\item{scale}{if scaling was requested
(with \code{scale}), the scaling used.} \item{call}{the function call.}
\item{terms}{the model terms.} \item{model}{if \code{model = TRUE}, the
model frame.} \item{x}{if \code{x = TRUE}, the model matrix.} \item{y}{if
\code{y = TRUE}, the model response.}
}
\description{
Functions to perform partial least squares regression (PLSR), canonical
powered partial least squares (CPPLS) or principal component regression
(PCR), with a formula interface.  Cross-validation can be used.  Prediction,
model extraction, plot, print and summary methods exist.
}
\details{
The functions fit PLSR, CPPLS or PCR models with 1, \eqn{\ldots},
\code{ncomp} number of components.  Multi-response models are fully
supported.

The type of model to fit is specified with the \code{method} argument. Four
PLSR algorithms are available: the kernel algorithm (\code{"kernelpls"}),
the wide kernel algorithm (\code{"widekernelpls"}), SIMPLS (\code{"simpls"})
and the classical orthogonal scores algorithm (\code{"oscorespls"}). One
CPPLS algorithm is available (\code{"cppls"}) providing several extensions
to PLS. One PCR algorithm is available: using the singular value
decomposition (\code{"svdpc"}).  If \code{method} is \code{"model.frame"},
the model frame is returned.  The functions \code{pcr}, \code{plsr} and
\code{cppls} are wrappers for \code{mvr}, with different values for
\code{method}.

The \code{formula} argument should be a symbolic formula of the form
\code{response ~ terms}, where \code{response} is the name of the response
vector or matrix (for multi-response models) and \code{terms} is the name of
one or more predictor matrices, usually separated by \code{+}, e.g.,
\code{water ~ FTIR} or \code{y ~ X + Z}.  See \code{\link{lm}} for a
detailed description.  The named variables should exist in the supplied
\code{data} data frame or in the global environment.  Note: Do not use
\code{mvr(mydata$y ~ mydata$X, \ldots{})}, instead use \code{mvr(y ~ X, data
= mydata, \ldots{})}.  Otherwise, \code{\link{predict.mvr}} will not work
properly.  The chapter \samp{Statistical models in R} of the manual \samp{An
Introduction to R} distributed with is a good reference on formulas in .

The number of components to fit is specified with the argument \code{ncomp}.
It this is not supplied, the maximal number of components is used (taking
account of any cross-validation).

All implemented algorithms mean-center both predictor and response matrices.
This can be turned off by specifying \code{center = FALSE}.  See Seasholtz
and Kowalski for a discussion about centering in PLS regression.

If \code{validation = "CV"}, cross-validation is performed.  The number and
type of cross-validation segments are specified with the arguments
\code{segments} and \code{segment.type}.  See \code{\link{mvrCv}} for
details.  If \code{validation = "LOO"}, leave-one-out cross-validation is
performed.  It is an error to specify the segments when \code{validation =
"LOO"} is specified.

By default, the cross-validation will be performed serially.  However, it
can be done in parallel using functionality in the \code{\link{parallel}}
package by setting the option \code{parallel} in \code{\link{pls.options}}.
See \code{\link{pls.options}} for the differnt ways to specify the
parallelism.  See also Examples below.

Note that the cross-validation is optimised for speed, and some generality
has been sacrificed.  Especially, the model matrix is calculated only once
for the complete cross-validation, so models like \code{y ~ msc(X)} will not
be properly cross-validated.  However, scaling requested by \code{scale =
TRUE} is properly cross-validated.  For proper cross-validation of models
where the model matrix must be updated/regenerated for each segment, use the
separate function \code{\link{crossval}}.
}
\examples{

data(yarn)
## Default methods:
yarn.pcr <- pcr(density ~ NIR, 6, data = yarn, validation = "CV")
yarn.pls <- plsr(density ~ NIR, 6, data = yarn, validation = "CV")
yarn.cppls <- cppls(density ~ NIR, 6, data = yarn, validation = "CV")

## Alternative methods:
yarn.oscorespls <- mvr(density ~ NIR, 6, data = yarn, validation = "CV",
                      method = "oscorespls")
yarn.simpls <- mvr(density ~ NIR, 6, data = yarn, validation = "CV",
                  method = "simpls")
# See ?simpls.fit for example of numerical instability in SIMPLS

\dontrun{
## Parallelised cross-validation, using transient cluster:
pls.options(parallel = 4) # use mclapply
pls.options(parallel = quote(makeCluster(4, type = "PSOCK"))) # use parLapply
## A new cluster is created and stopped for each cross-validation:
yarn.pls <- plsr(density ~ NIR, 6, data = yarn, validation = "CV")
yarn.pcr <- pcr(density ~ NIR, 6, data = yarn, validation = "CV")

## Parallelised cross-validation, using persistent cluster:
library(parallel)
## This creates the cluster:
pls.options(parallel = makeCluster(4, type = "PSOCK"))
## The cluster can be used several times:
yarn.pls <- plsr(density ~ NIR, 6, data = yarn, validation = "CV")
yarn.pcr <- pcr(density ~ NIR, 6, data = yarn, validation = "CV")
## The cluster should be stopped manually afterwards:
stopCluster(pls.options()$parallel)

## Parallelised cross-validation, using persistent MPI cluster:
## This requires the packages snow and Rmpi to be installed
library(parallel)
## This creates the cluster:
pls.options(parallel = makeCluster(4, type = "MPI"))
## The cluster can be used several times:
yarn.pls <- plsr(density ~ NIR, 6, data = yarn, validation = "CV")
yarn.pcr <- pcr(density ~ NIR, 6, data = yarn, validation = "CV")
## The cluster should be stopped manually afterwards:
stopCluster(pls.options()$parallel)
## It is good practice to call mpi.exit() or mpi.quit() afterwards:
mpi.exit()
}

## Multi-response models:
data(oliveoil)
sens.pcr <- pcr(sensory ~ chemical, ncomp = 4, scale = TRUE, data = oliveoil)
sens.pls <- plsr(sensory ~ chemical, ncomp = 4, scale = TRUE, data = oliveoil)

## Classification
# A classification example utilizing additional response information
# (Y.add) is found in the cppls.fit manual ('See also' above).

}
\references{
Martens, H., Næs, T. (1989) \emph{Multivariate calibration.}
Chichester: Wiley.

Seasholtz, M. B. and Kowalski, B. R. (1992) The effect of mean centering on
prediction in multivariate calibration.  \emph{Journal of Chemometrics},
\bold{6}(2), 103--111.
}
\seealso{
\code{\link{kernelpls.fit}}, \code{\link{widekernelpls.fit}},
\code{\link{simpls.fit}}, \code{\link{oscorespls.fit}}, \code{\link{nipals.fit}},
\code{\link{cppls.fit}}, \code{\link{svdpc.fit}}, \code{\link{mvrCv}},
\code{\link{crossval}}, \code{\link[stats]{loadings}}, \code{\link{scores}},
\code{\link{loading.weights}}, \code{\link{coef.mvr}},
\code{\link{predict.mvr}}, \code{\link{R2}}, \code{\link{MSEP}},
\code{\link{RMSEP}}, \code{\link{plot.mvr}}
}
\author{
Ron Wehrens and Bjørn-Helge Mevik
}
\keyword{multivariate}
\keyword{regression}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/extract.R
\name{coef.mvr}
\alias{coef.mvr}
\alias{fitted.mvr}
\alias{residuals.mvr}
\alias{model.frame.mvr}
\alias{model.matrix.mvr}
\alias{prednames}
\alias{respnames}
\alias{compnames}
\alias{explvar}
\title{Extract Information From a Fitted PLSR or PCR Model}
\usage{
\method{coef}{mvr}(object, ncomp = object$ncomp, comps, intercept = FALSE, ...)

\method{fitted}{mvr}(object, ...)

\method{residuals}{mvr}(object, ...)

\method{model.frame}{mvr}(formula, ...)

\method{model.matrix}{mvr}(object, ...)

respnames(object)

prednames(object, intercept = FALSE)

compnames(object, comps, explvar = FALSE, ...)

explvar(object)
}
\arguments{
\item{object, formula}{an \code{mvr} object.  The fitted model.}

\item{ncomp, comps}{vector of positive integers.  The components to include
in the coefficients or to extract the names of.  See below.}

\item{intercept}{logical.  Whether coefficients for the intercept should be
included.  Ignored if \code{comps} is specified.  Defaults to \code{FALSE}.}

\item{\dots}{other arguments sent to underlying functions.  Currently only
used for \code{model.frame.mvr} and \code{model.matrix.mvr}.}

\item{explvar}{logical.  Whether the explained \eqn{X} variance should be
appended to the component names.}
}
\value{
\code{coef.mvr} returns an array of regression coefficients.

\code{fitted.mvr} returns an array with fitted values.

\code{residuals.mvr} returns an array with residuals.

\code{model.frame.mvr} returns a data frame.

\code{model.matrix.mvr} returns the \eqn{X} matrix.

\code{prednames}, \code{respnames} and \code{compnames} return a character
vector with the corresponding names.

\code{explvar} returns a numeric vector with the explained variances, or
\code{NULL} if not available.
}
\description{
Functions to extract information from \code{mvr} objects: Regression
coefficients, fitted values, residuals, the model frame, the model matrix,
names of the variables and components, and the \eqn{X} variance explained by
the components.
}
\details{
These functions are mostly used inside other functions.  (Functions
\code{coef.mvr}, \code{fitted.mvr} and \code{residuals.mvr} are usually
called through their generic functions \code{\link{coef}},
\code{\link{fitted}} and \code{\link{residuals}}, respectively.)

\code{coef.mvr} is used to extract the regression coefficients of a model,
i.e. the \eqn{B} in \eqn{y = XB} (for the \eqn{Q} in \eqn{y = TQ} where
\eqn{T} is the scores, see \code{\link{Yloadings}}).  An array of dimension
\code{c(nxvar, nyvar, length(ncomp))} or \code{c(nxvar, nyvar,
length(comps))} is returned.

If \code{comps} is missing (or is \code{NULL}), \code{coef()[,,ncomp[i]]}
are the coefficients for models with \code{ncomp[i]} components, for \eqn{i
= 1, \ldots, length(ncomp)}.  Also, if \code{intercept = TRUE}, the first
dimension is \eqn{nxvar + 1}, with the intercept coefficients as the first
row.

If \code{comps} is given, however, \code{coef()[,,comps[i]]} are the
coefficients for a model with only the component \code{comps[i]}, i.e. the
contribution of the component \code{comps[i]} on the regression
coefficients.

\code{fitted.mvr} and \code{residuals.mvr} return the fitted values and
residuals, respectively.  If the model was fitted with \code{na.action =
na.exclude} (or after setting the default \code{na.action} to
\code{"na.exclude"} with \code{\link{options}}), the fitted values (or
residuals) corresponding to excluded observations are returned as \code{NA};
otherwise, they are omitted.

\code{model.frame.mvr} returns the model frame; i.e. a data frame with all
variables neccessary to generate the model matrix.  See
\code{\link[stats]{model.frame}} for details.

\code{model.matrix.mvr} returns the (possibly coded) matrix used as \eqn{X}
in the fitting.  See \code{\link[stats]{model.matrix}} for details.

\code{prednames}, \code{respnames} and \code{compnames} extract the names of
the \eqn{X} variables, responses and components, respectively.  With
\code{intercept = TRUE} in \code{prednames}, the name of the intercept
variable (i.e. \code{"(Intercept)"}) is returned as well.  \code{compnames}
can also extract component names from score and loading matrices.  If
\code{explvar = TRUE} in \code{compnames}, the explained variance for each
component (if available) is appended to the component names.  For optimal
formatting of the explained variances when not all components are to be
used, one should specify the desired components with the argument
\code{comps}.

\code{explvar} extracts the amount of \eqn{X} variance (in per cent)
explained by each component in the model.  It can also handle score and
loading matrices returned by \code{\link{scores}} and
\code{\link{loadings}}.
}
\examples{

data(yarn)
mod <- pcr(density ~ NIR, data = yarn[yarn$train,], ncomp = 5)
B <- coef(mod, ncomp = 3, intercept = TRUE)
## A manual predict method:
stopifnot(drop(B[1,,] + yarn$NIR[!yarn$train,] \%*\% B[-1,,]) ==
          drop(predict(mod, ncomp = 3, newdata = yarn[!yarn$train,])))

## Note the difference in formatting:
mod2 <- pcr(density ~ NIR, data = yarn[yarn$train,])
compnames(mod2, explvar = TRUE)[1:3]
compnames(mod2, comps = 1:3, explvar = TRUE)

}
\seealso{
\code{\link{mvr}}, \code{\link{coef}}, \code{\link{fitted}},
\code{\link{residuals}}, \code{\link{model.frame}},
\code{\link{model.matrix}}, \code{\link{na.omit}}
}
\author{
Ron Wehrens and Bjørn-Helge Mevik
}
\keyword{multivariate}
\keyword{regression}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{gasoline}
\alias{gasoline}
\title{Octane numbers and NIR spectra of gasoline}
\format{
A data frame with 60 observations on the following 2 variables.
\describe{ \item{octane}{a numeric vector.  The octane number.}
\item{NIR}{a matrix with 401 columns.  The NIR spectrum.} }
}
\source{
Kalivas, John H. (1997) Two Data Sets of Near Infrared Spectra
\emph{Chemometrics and Intelligent Laboratory Systems}, \bold{37}, 255--259.
}
\description{
A data set with NIR spectra and octane numbers of 60 gasoline samples.  The
NIR spectra were measured using diffuse reflectance as log(1/R) from 900 nm
to 1700 nm in 2 nm intervals, giving 401 wavelengths.  Many thanks to John
H. Kalivas.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/jackknife.R
\name{var.jack}
\alias{var.jack}
\title{Jackknife Variance Estimates of Regression Coefficients}
\usage{
var.jack(object, ncomp = object$ncomp, covariance = FALSE, use.mean = TRUE)
}
\arguments{
\item{object}{an \code{mvr} object.  A cross-validated model fitted with
\code{jackknife = TRUE}.}

\item{ncomp}{the number of components to use for estimating the
(co)variances}

\item{covariance}{logical.  If \code{TRUE}, covariances are calculated;
otherwise only variances.  The default is \code{FALSE}.}

\item{use.mean}{logical.  If \code{TRUE} (default), the mean coefficients
are used when estimating the (co)variances; otherwise the coefficients from
a model fitted to the entire data set.  See Details.}
}
\value{
If \code{covariance} is \code{FALSE}, an \eqn{p\times q \times c}
array of variance estimates, where \eqn{p} is the number of predictors,
\eqn{q} is the number of responses, and \eqn{c} is the number of components.

If \code{covariance} id \code{TRUE}, an \eqn{pq\times pq \times c} array of
variance-covariance estimates.
}
\description{
Calculates jackknife variance or covariance estimates of regression
coefficients.

The original (Tukey) jackknife variance estimator is defined as \eqn{(g-1)/g
\sum_{i=1}^g(\tilde\beta_{-i} - \bar\beta)^2}, where \eqn{g} is the number
of segments, \eqn{\tilde\beta_{-i}} is the estimated coefficient when
segment \eqn{i} is left out (called the jackknife replicates), and
\eqn{\bar\beta} is the mean of the \eqn{\tilde\beta_{-i}}.  The most common
case is delete-one jackknife, with \eqn{g = n}, the number of observations.

This is the definition \code{var.jack} uses by default.

However, Martens and Martens (2000) defined the estimator as \eqn{(g-1)/g
\sum_{i=1}^g(\tilde\beta_{-i} - \hat\beta)^2}, where \eqn{\hat\beta} is the
coefficient estimate using the entire data set.  I.e., they use the original
fitted coefficients instead of the mean of the jackknife replicates.  Most
(all?) other jackknife implementations for PLSR use this estimator.
\code{var.jack} can be made to use this definition with \code{use.mean =
FALSE}.  In practice, the difference should be small if the number of
observations is sufficiently large.  Note, however, that all theoretical
results about the jackknife refer to the `proper' definition.  (Also note
that this option might disappear in a future version.)
}
\section{Warning}{
 Note that the Tukey jackknife variance estimator is not
unbiased for the variance of regression coefficients (Hinkley 1977).  The
bias depends on the \eqn{X} matrix.  For ordinary least squares regression
(OLSR), the bias can be calculated, and depends on the number of
observations \eqn{n} and the number of parameters \eqn{k} in the mode.  For
the common case of an orthogonal design matrix with \eqn{\pm 1}{?1} levels,
the delete-one jackknife estimate equals \eqn{(n-1)/(n-k)} times the
classical variance estimate for the regression coefficients in OLSR.
Similar expressions hold for delete-d estimates.  Modifications have been
proposed to reduce or eliminate the bias for the OLSR case, however, they
depend on the number of parameters used in the model.  See e.g. Hinkley
(1977) or Wu (1986).

Thus, the results of \code{var.jack} should be used with caution.
}

\examples{

data(oliveoil)
mod <- pcr(sensory ~ chemical, data = oliveoil, validation = "LOO",
           jackknife = TRUE)
var.jack(mod, ncomp = 2)

}
\references{
Tukey J.W. (1958) Bias and Confidence in Not-quite Large
Samples. (Abstract of Preliminary Report).  \emph{Annals of Mathematical
Statistics}, \bold{29}(2), 614.

Martens H. and Martens M. (2000) Modified Jack-knife Estimation of Parameter
Uncertainty in Bilinear Modelling by Partial Least Squares Regression
(PLSR).  \emph{Food Quality and Preference}, \bold{11}, 5--16.

Hinkley D.V. (1977), Jackknifing in Unbalanced Situations.
\emph{Technometrics}, \bold{19}(3), 285--292.

Wu C.F.J. (1986) Jackknife, Bootstrap and Other Resampling Methods in
Regression Analysis.  \emph{Te Annals of Statistics}, \bold{14}(4),
1261--1295.
}
\seealso{
\code{\link{mvrCv}}, \code{\link{jack.test}}
}
\author{
Bjørn-Helge Mevik
}
\keyword{univar}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{mayonnaise}
\alias{mayonnaise}
\title{NIR measurements and oil types of mayonnaise}
\format{
A data frame with 162 observations on the following 4 variables.
\describe{ \item{NIR}{a matrix with 351 columns}
\item{oil.type}{a numeric vector} \item{design}{a matrix
with 5 columns} \item{train}{a logical vector} }
}
\source{
Indahl U, Sahni NS, Kirkhus B, Næs T.  Multivariate strategies for
classification based on NIR-spectra-with application to mayonnaise.
Chemometr. Intell. Lab. Sys. 1999; 49: 19-31.
}
\description{
Raw NIR measurements (351 wavelengths, 1100-2500 nm in steps of 4 nm) taken
on 54 samples of mayonnaise based on six different oil types (soybean,
sunflower, canola, olive, corn, and grapeseed). The resulting 54 samples
were measured in triplicates, resulting in 54 x 3 = 162 different spectra
(120/42 training/test).
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plots.R
\name{validationplot}
\alias{validationplot}
\alias{plot.mvrVal}
\title{Validation Plots}
\usage{
validationplot(
  object,
  val.type = c("RMSEP", "MSEP", "R2"),
  estimate,
  newdata,
  ncomp,
  comps,
  intercept,
  ...
)

\method{plot}{mvrVal}(
  x,
  nCols,
  nRows,
  type = "l",
  lty = 1:nEst,
  lwd = par("lwd"),
  pch = 1:nEst,
  cex = 1,
  col = 1:nEst,
  legendpos,
  xlab = "number of components",
  ylab = x$type,
  main,
  ask = nRows * nCols < nResp && dev.interactive(),
  ...
)
}
\arguments{
\item{object}{an \code{mvr} object.}

\item{val.type}{character.  What type of validation statistic to plot.}

\item{estimate}{character.  Which estimates of the statistic to calculate.
See \code{\link{RMSEP}}.}

\item{newdata}{data frame.  Optional new data used to calculate statistic.}

\item{ncomp, comps}{integer vector.  The model sizes to compute the statistic
for.  See \code{\link{RMSEP}}.}

\item{intercept}{logical.  Whether estimates for a model with zero
components should be calculated as well.}

\item{\dots}{Further arguments sent to underlying plot functions.}

\item{x}{an \code{mvrVal} object.  Usually the result of a
\code{\link{RMSEP}}, \code{\link{MSEP}} or \code{\link{R2}} call.}

\item{nCols, nRows}{integers.  The number of coloumns and rows the plots will
be laid out in.  If not specified, \code{plot.mvrVal} tries to be
intelligent.}

\item{type}{character.  What type of plots to create.  Defaults to
\code{"l"} (lines).  Alternative types include \code{"p"} (points) and
\code{"b"} (both).  See \code{\link{plot}} for a complete list of types.}

\item{lty}{vector of line types (recycled as neccessary).  Line types can be
specified as integers or character strings (see \code{\link{par}} for the
details).}

\item{lwd}{vector of positive numbers (recycled as neccessary), giving the
width of the lines.}

\item{pch}{plot character.  A character string or a vector of single
characters or integers (recycled as neccessary).  See \code{\link{points}}
for all alternatives.}

\item{cex}{numeric vector of character expansion sizes (recycled as
neccessary) for the plotted symbols.}

\item{col}{character or integer vector of colors for plotted lines and
symbols (recycled as neccessary).  See \code{\link{par}} for the details.}

\item{legendpos}{Legend position.  Optional.  If present, a legend is drawn
at the given position.  The position can be specified symbolically (e.g.,
\code{legendpos = "topright"}).  This requires >= 2.1.0.  Alternatively, the
position can be specified explicitly (\code{legendpos = t(c(x,y))}) or
interactively (\code{legendpos = \link{locator}()}).  This only works well
for plots of single-response models.}

\item{xlab, ylab}{titles for \eqn{x} and \eqn{y} axes.  Typically character
strings, but can be expressions (e.g., \code{expression(R^2)} or lists.  See
\code{\link{title}} for details.}

\item{main}{optional main title for the plot.  See Details.}

\item{ask}{logical.  Whether to ask the user before each page of a plot.}
}
\description{
Functions to plot validation statistics, such as RMSEP or \eqn{R^2}, as a
function of the number of components.
}
\details{
\code{validationplot} calls the proper validation function (currently
\code{\link{MSEP}}, \code{\link{RMSEP}} or \code{\link{R2}}) and plots the
results with \code{plot.mvrVal}.  \code{validationplot} can be called
through the \code{mvr} plot method, by specifying \code{plottype =
"validation"}.

\code{plot.mvrVal} creates one plot for each response variable in the model,
laid out in a rectangle.  It uses \code{\link{matplot}} for performing the
actual plotting.  If \code{legendpos} is given, a legend is drawn at the
given position.

The argument \code{main} can be used to specify the main title of the plot.
It is handled in a non-standard way.  If there is only on (sub) plot,
\code{main} will be used as the main title of the plot.  If there is
\emph{more} than one (sub) plot, however, the presence of \code{main} will
produce a corresponding \sQuote{global} title on the page.  Any graphical
parametres, e.g., \code{cex.main}, supplied to \code{coefplot} will only
affect the \sQuote{ordinary} plot titles, not the \sQuote{global} one.  Its
appearance can be changed by setting the parameters with \code{\link{par}},
which will affect \emph{both} titles.  (To have different settings for the
two titles, one can override the \code{par} settings with arguments to the
plot function.)
}
\note{
\code{\link{legend}} has many options.  If you want greater control
over the appearance of the legend, omit the \code{legendpos} argument and
call \code{legend} manually.
}
\examples{

data(oliveoil)
mod <- plsr(sensory ~ chemical, data = oliveoil, validation = "LOO")
\dontrun{
## These three are equivalent:
validationplot(mod, estimate = "all")
plot(mod, "validation", estimate = "all")
plot(RMSEP(mod, estimate = "all"))
## Plot R2:
plot(mod, "validation", val.type = "R2")
## Plot R2, with a legend:
plot(mod, "validation", val.type = "MSEP", legendpos = "top") # R >= 2.1.0
}

}
\seealso{
\code{\link{mvr}}, \code{\link{plot.mvr}}, \code{\link{RMSEP}},
\code{\link{MSEP}}, \code{\link{R2}}, \code{\link{matplot}},
\code{\link{legend}}
}
\author{
Ron Wehrens and Bjørn-Helge Mevik
}
\keyword{hplot}
\keyword{multivariate}
\keyword{regression}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plots.R
\name{plot.mvr}
\alias{plot.mvr}
\title{Plot Method for MVR objects}
\usage{
\method{plot}{mvr}(
  x,
  plottype = c("prediction", "validation", "coefficients", "scores", "loadings",
    "biplot", "correlation"),
  ...
)
}
\arguments{
\item{x}{an object of class \code{mvr}.  The fitted model to plot.}

\item{plottype}{character.  What kind of plot to plot.}

\item{\dots}{further arguments, sent to the underlying plot functions.}
}
\value{
\code{plot.mvr} returns whatever the underlying plot function
returns.
}
\description{
\code{plot.mvr} plots predictions, coefficients, scores, loadings, biplots,
correlation loadings or validation plots (RMSEP curves, etc.).
}
\details{
The function is simply a wrapper for the underlying plot functions used to
make the selected plots.  See \code{\link{predplot.mvr}},
\code{\link{validationplot}}, \code{\link{coefplot}},
\code{\link{scoreplot}}, \code{\link{loadingplot}}, \code{\link{biplot.mvr}}
or \code{\link{corrplot}} for details.  Note that all arguments except
\code{x} and \code{plottype} must be named.
}
\examples{

data(yarn)
nir.pcr <- pcr(density ~ NIR, ncomp = 9, data = yarn, validation = "CV")
\dontrun{
plot(nir.pcr, ncomp = 5) # Plot of cross-validated predictions
plot(nir.pcr, "scores") # Score plot
plot(nir.pcr, "loadings", comps = 1:3) # The three first loadings
plot(nir.pcr, "coef", ncomp = 5) # Coefficients
plot(nir.pcr, "val") # RMSEP curves
plot(nir.pcr, "val", val.type = "MSEP", estimate = "CV") # CV MSEP
}

}
\seealso{
\code{\link{mvr}}, \code{\link{predplot.mvr}},
\code{\link{validationplot}}, \code{\link{coefplot}},
\code{\link{scoreplot}}, \code{\link{loadingplot}},
\code{\link{biplot.mvr}}, \code{\link{corrplot}}
}
\author{
Ron Wehrens and Bjørn-Helge Mevik
}
\keyword{hplot}
\keyword{multivariate}
\keyword{regression}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plots.R
\name{predplot}
\alias{predplot}
\alias{predplot.default}
\alias{predplot.mvr}
\alias{predplotXy}
\title{Prediction Plots}
\usage{
predplot(object, ...)

\method{predplot}{default}(object, ...)

\method{predplot}{mvr}(
  object,
  ncomp = object$ncomp,
  which,
  newdata,
  nCols,
  nRows,
  xlab = "measured",
  ylab = "predicted",
  main,
  ask = nRows * nCols < nPlots && dev.interactive(),
  ...,
  font.main,
  cex.main
)

predplotXy(
  x,
  y,
  line = FALSE,
  labels,
  type = "p",
  main = "Prediction plot",
  xlab = "measured response",
  ylab = "predicted response",
  line.col = par("col"),
  line.lty = NULL,
  line.lwd = NULL,
  ...
)
}
\arguments{
\item{object}{a fitted model.}

\item{\dots}{further arguments sent to underlying plot functions.}

\item{ncomp}{integer vector.  The model sizes (numbers of components) to use
for prediction.}

\item{which}{character vector.  Which types of predictions to plot.  Should
be a subset of \code{c("train", "validation", "test")}.  If not specified,
\code{plot.mvr} selects test set predictions if \code{newdata} is supplied,
otherwise cross-validated predictions if the model has been cross-validated,
otherwise fitted values from the calibration data.}

\item{newdata}{data frame.  New data to predict.}

\item{nCols, nRows}{integer.  The number of coloumns and rows the plots will
be laid out in.  If not specified, \code{plot.mvr} tries to be intelligent.}

\item{xlab, ylab}{titles for \eqn{x} and \eqn{y} axes.  Typically character
strings, but can be expressions or lists.  See \code{\link{title}} for
details.}

\item{main}{optional main title for the plot.  See Details.}

\item{ask}{logical.  Whether to ask the user before each page of a plot.}

\item{font.main}{font to use for main titles.  See \code{\link{par}} for
details.  Also see Details below.}

\item{cex.main}{numeric.  The magnification to be used for main titles
relative to the current size.  Also see Details below.}

\item{x}{numeric vector.  The observed response values.}

\item{y}{numeric vector.  The predicted response values.}

\item{line}{logical.  Whether a target line should be drawn.}

\item{labels}{optional.  Alternative plot labels to use.  Either a vector of
labels, or \code{"names"} or \code{"numbers"} to use the row names or row
numbers of the data as labels.}

\item{type}{character.  What type of plot to make.  Defaults to \code{"p"}
(points).  See \code{\link{plot}} for a complete list of types.  The
argument is ignored if \code{labels} is specified.}

\item{line.col, line.lty, line.lwd}{character or numeric.  The \code{col},
\code{lty} and \code{lwd} parametres for the target line.  See
\code{\link{par}} for details.}
}
\value{
The functions invisibly return a matrix with the (last) plotted
data.
}
\description{
Functions to plot predicted values against measured values for a fitted
model.
}
\details{
\code{predplot} is a generic function for plotting predicted versus measured
response values, with default and \code{mvr} methods currently implemented.
The default method is very simple, and doesn't handle multiple responses or
new data.

The \code{mvr} method, handles multiple responses, model sizes and types of
predictions by making one plot for each combination.  It can also be called
through the plot method for \code{mvr}, by specifying \code{plottype =
"prediction"} (the default).

The argument \code{main} can be used to specify the main title of the plot.
It is handled in a non-standard way.  If there is only on (sub) plot,
\code{main} will be used as the main title of the plot.  If there is
\emph{more} than one (sub) plot, however, the presence of \code{main} will
produce a corresponding \sQuote{global} title on the page.  Any graphical
parametres, e.g., \code{cex.main}, supplied to \code{coefplot} will only
affect the \sQuote{ordinary} plot titles, not the \sQuote{global} one.  Its
appearance can be changed by setting the parameters with \code{\link{par}},
which will affect \emph{both} titles (with the exception of \code{font.main}
and \code{cex.main}, which will only affect the \sQuote{global} title when
there is more than one plot).  (To have different settings for the two
titles, one can override the \code{par} settings with arguments to
\code{predplot}.)

\code{predplotXy} is an internal function and is not meant for interactive
use.  It is called by the \code{predplot} methods, and its arguments, e.g,
\code{line}, can be given in the \code{predplot} call.
}
\note{
The \code{font.main} and \code{cex.main} must be (completely) named.
This is to avoid that any argument \code{cex} or \code{font} matches them.

Tip: If the labels specified with \code{labels} are too long, they get
clipped at the border of the plot region.  This can be avoided by supplying
the graphical parameter \code{xpd = TRUE} in the plot call.
}
\examples{

data(yarn)
mod <- plsr(density ~ NIR, ncomp = 10, data = yarn[yarn$train,], validation = "CV")
\dontrun{
predplot(mod, ncomp = 1:6)
plot(mod, ncomp = 1:6) # Equivalent to the previous
## Both cross-validated and test set predictions:
predplot(mod, ncomp = 4:6, which = c("validation", "test"),
         newdata = yarn[!yarn$train,])
}

data(oliveoil)
mod.sens <- plsr(sensory ~ chemical, ncomp = 4, data = oliveoil)
\dontrun{plot(mod.sens, ncomp = 2:4) # Several responses gives several plots}

}
\seealso{
\code{\link{mvr}}, \code{\link{plot.mvr}}
}
\author{
Ron Wehrens and Bjørn-Helge Mevik
}
\keyword{hplot}
\keyword{multivariate}
\keyword{regression}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pls.options.R
\name{pls.options}
\alias{pls.options}
\title{Set or return options for the pls package}
\usage{
pls.options(...)
}
\arguments{
\item{\dots}{a single list, a single character vector, or any number of
named arguments (\var{name = value}).}
}
\value{
A list with the (possibly changed) options.  If any named argument
(or list element) was provided, the list is returned invisibly.
}
\description{
A function to set options for the \pkg{pls} package, or to return the
current options.
}
\details{
If called with no arguments, or with an empty list as the single argument,
\code{pls.options} returns the current options.

If called with a character vector as the single argument, a list with the
arguments named in the vector are returned.

If called with a non-empty list as the single argument, the list elements
should be named, and are treated as named arguments to the function.

Otherwise, \code{pls.options} should be called with one or more named
arguments \var{name = value}.  For each argument, the option named
\var{name} will be given the value \var{value}.

The recognised options are: \describe{ \item{mvralg}{The fit method to use
in \code{\link{mvr}} and \code{\link{mvrCv}}.  The value should be one of
the allowed methods.  Defaults to \code{"kernelpls"}.  Can be overridden
with the argument \code{method} in \code{mvr} and \code{mvrCv}.}
\item{pcralg}{The fit method to use in \code{\link{pcr}}.  The value should
be one of the allowed methods.  Defaults to \code{"svdpc"}.  Can be
overridden with the argument \code{method} in \code{pcr}.}
\item{plsralg}{The fit method to use in \code{\link{plsr}}.  The value
should be one of the allowed methods.  Defaults to \code{"kernelpls"}.  Can
be overridden with the argument \code{method} in \code{plsr}.}
\item{cpplsalg}{The fit method to use in \code{\link{cppls}}.  The value
should be one of the allowed methods.  Defaults to \code{"cppls"}.  Can be
overridden with the argument \code{method} in \code{cppls}.}
\item{parallel}{Specification of how the cross-validation (CV) in
\code{\link{mvr}} should be performed.  If the specification is \code{NULL}
(default) or \code{1}, the CV is done serially, otherwise it is done in
parallel using functionality from the \code{\link{parallel}} package.

If it is an integer greater than 1, the CV is done in parallel with the
specified number of processes, using \code{\link{mclapply}}.

If it is a cluster object created by \code{\link{makeCluster}}, the CV is
done in parallel on that cluster, using \code{\link{parLapply}}.  The user
should stop the cluster herself when it is no longer needed, using
\code{\link{stopCluster}}.

Finally, if the specification is an unevaluated call to
\code{\link{makeCluster}}, the call is evaluated, and the CV is done in
parallel on the resulting cluster, using \code{\link{parLapply}}.  In this
case, the cluster will be stopped (with \code{\link{stopCluster}}) after the
CV.  Thus, in the final case, the cluster is created and destroyed for each
CV, just like when using \code{\link{mclapply}}.} \item{w.tol}{The tolerance
used for removing values close to 0 in the vectors of loading weights in
\code{\link{cppls}}.  Defaults to .Machine$double.eps.} \item{X.tol}{The
tolerance used for removing predictor variables with L1 norms close to 0 in
\code{\link{cppls}}.  Defaults to 10^-12.} }
}
\note{
The function is a slight modification of the function
\code{\link[sm]{sm.options}} from the package \pkg{sm}.
}
\examples{

## Return current options:
pls.options()
pls.options("plsralg")
pls.options(c("plsralg", "pcralg"))

## Set options:
pls.options(plsralg = "simpls", mvralg = "simpls")
pls.options(list(plsralg = "simpls", mvralg = "simpls")) # Equivalent
pls.options()

## Restore `factory settings':
pls.options(list(mvralg = "kernelpls", plsralg = "kernelpls", cpplsalg = "cppls",
                 pcralg = "svdpc", parallel = NULL,
                 w.tol = .Machine$double.eps, X.tol = 10^-12))
pls.options()

}
\author{
Bjørn-Helge Mevik and Ron Wehrens
}
\keyword{multivariate}
\keyword{regression}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nipals.fit.R
\name{nipals.fit}
\alias{nipals.fit}
\title{NIPALS PLS with missing values}
\usage{
nipals.fit(
  X,
  Y,
  ncomp,
  center = TRUE,
  stripped = FALSE,
  maxiter = 500,
  tol = 1e-06,
  ...
)
}
\arguments{
\item{X}{numeric matrix (or coercible) of predictors. Missing values are
allowed and handled internally.}

\item{Y}{numeric matrix (or coercible) of responses. Missing values are also
handled internally.}

\item{ncomp}{number of PLS components to extract.}

\item{center}{logical whether to center \code{X} and \code{Y} before
fitting. Means ignore missing entries.}

\item{stripped}{logical. If \code{TRUE} only the coefficients and the mean
vectors are returned.}

\item{maxiter}{maximum number of inner iterations to force convergence on
each component.}

\item{tol}{tolerance used to stop the inner loop when the direction vector
changes very little.}

\item{...}{currently ignored.}
}
\value{
A list with the same components as \code{nipals.fit}, but the
computations never fail in the presence of missing entries.
}
\description{
A NIPALS implementation that tolerates \code{NA}s in both
\code{X} and \code{Y} by ignoring them when updating scores and loadings.
This is useful when the design matrix is incomplete but the number of
components is relatively low.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plots.R
\name{coefplot}
\alias{coefplot}
\title{Plot Regression Coefficients of PLSR and PCR models}
\usage{
coefplot(
  object,
  ncomp = object$ncomp,
  comps,
  intercept = FALSE,
  separate = FALSE,
  se.whiskers = FALSE,
  nCols,
  nRows,
  labels,
  type = "l",
  lty,
  lwd = NULL,
  pch,
  cex = NULL,
  col,
  legendpos,
  xlab = "variable",
  ylab = "regression coefficient",
  main,
  pretty.xlabels = TRUE,
  xlim,
  ylim,
  ask = nRows * nCols < nPlots && dev.interactive(),
  ...
)
}
\arguments{
\item{object}{an \code{mvr} object.  The fitted model.}

\item{ncomp, comps}{vector of positive integers.  The components to plot.
See \code{\link{coef.mvr}} for details.}

\item{intercept}{logical.  Whether coefficients for the intercept should be
plotted.  Ignored if \code{comps} is specified.  Defaults to \code{FALSE}.
See \code{\link{coef.mvr}} for details.}

\item{separate}{logical.  If \code{TRUE}, coefficients for different model
sizes are blotted in separate plots.}

\item{se.whiskers}{logical.  If \code{TRUE}, whiskers at plus/minus 1
estimated standard error are added to the plot.  This is only available if
the model was cross-validated with \code{jackknife = TRUE}.  Also, in the
current implementation, \code{intercept} must be \code{FALSE}, and
\code{separate} must be \code{TRUE} if \code{length(ncomp) > 1}.}

\item{nCols, nRows}{integer.  The number of coloumns and rows the plots will
be laid out in.  If not specified, \code{coefplot} tries to be intelligent.}

\item{labels}{optional.  Alternative \eqn{x} axis labels.  See Details.}

\item{type}{character.  What type of plot to make.  Defaults to \code{"l"}
(lines).  Alternative types include \code{"p"} (points) and \code{"b"}
(both).  See \code{\link{plot}} for a complete list of types.}

\item{lty}{vector of line types (recycled as neccessary).  Line types can be
specified as integers or character strings (see \code{\link{par}} for the
details).}

\item{lwd}{vector of positive numbers (recycled as neccessary), giving the
width of the lines.}

\item{pch}{plot character.  A character string or a vector of single
characters or integers (recycled as neccessary).  See \code{\link{points}}
for all alternatives.}

\item{cex}{numeric vector of character expansion sizes (recycled as
neccessary) for the plotted symbols.}

\item{col}{character or integer vector of colors for plotted lines and
symbols (recycled as neccessary).  See \code{\link{par}} for the details.}

\item{legendpos}{Legend position.  Optional.  Ignored if \code{separate} is
\code{TRUE}.  If present, a legend is drawn at the given position.  The
position can be specified symbolically (e.g., \code{legendpos =
"topright"}).  This requires >= 2.1.0.  Alternatively, the position can be
specified explicitly (\code{legendpos = t(c(x,y))}) or interactively
(\code{legendpos = \link{locator}()}).  This only works well for plots of
single-response models.}

\item{xlab, ylab}{titles for \eqn{x} and \eqn{y} axes.  Typically character
strings, but can be expressions (e.g., \code{expression(R^2)} or lists.  See
\code{\link{title}} for details.}

\item{main}{optional main title for the plot.  See Details.}

\item{pretty.xlabels}{logical.  If \code{TRUE}, \code{coefplot} tries to
plot the \eqn{x} labels more nicely.  See Details.}

\item{xlim, ylim}{optional vector of length two, with the \eqn{x} or \eqn{y}
limits of the plot.}

\item{ask}{logical.  Whether to ask the user before each page of a plot.}

\item{\dots}{Further arguments sent to the underlying plot functions.}
}
\description{
Function to plot the regression coefficients of an \code{mvr} object.
}
\details{
\code{coefplot} handles multiple responses by making one plot for each
response.  If \code{separate} is \code{TRUE}, separate plots are made for
each combination of model size and response.  The plots are laid out in a
rectangular fashion.

If \code{legendpos} is given, a legend is drawn at the given position
(unless \code{separate} is \code{TRUE}).

The argument \code{labels} can be a vector of labels or one of
\code{"names"} and \code{"numbers"}.  The labels are used as \eqn{x} axis
labels.  If \code{labels} is \code{"names"} or \code{"numbers"}, the
variable names are used as labels, the difference being that with
\code{"numbers"}, the variable names are converted to numbers, if possible.
Variable names of the forms \samp{"number"} or \samp{"number text"} (where
the space is optional), are handled.

The argument \code{main} can be used to specify the main title of the plot.
It is handled in a non-standard way.  If there is only on (sub) plot,
\code{main} will be used as the main title of the plot.  If there is
\emph{more} than one (sub) plot, however, the presence of \code{main} will
produce a corresponding \sQuote{global} title on the page.  Any graphical
parametres, e.g., \code{cex.main}, supplied to \code{coefplot} will only
affect the \sQuote{ordinary} plot titles, not the \sQuote{global} one.  Its
appearance can be changed by setting the parameters with \code{\link{par}},
which will affect \emph{both} titles.  (To have different settings for the
two titles, one can override the \code{par} settings with arguments to
\code{coefplot}.)

The argument \code{pretty.xlabels} is only used when \code{labels} is
specified.  If \code{TRUE} (default), the code tries to use a
\sQuote{pretty} selection of labels.  If \code{labels} is \code{"numbers"},
it also uses the numerical values of the labels for horisontal spacing.  If
one has excluded parts of the spectral region, one might therefore want to
use \code{pretty.xlabels = FALSE}.

When \code{separate} is \code{TRUE}, the arguments \code{lty}, \code{col},
and \code{pch} default to their \code{par()} setting.  Otherwise, the
default for all of them is \code{1:nLines}, where \code{nLines} is the
number of model sizes specified, i.e., the length of \code{ncomp} or
\code{comps}.

The function can also be called through the \code{mvr} plot method by
specifying \code{plottype = "coefficients"}.
}
\note{
\code{\link{legend}} has many options.  If you want greater control
over the appearance of the legend, omit the \code{legendpos} argument and
call \code{legend} manually.

The handling of \code{labels} and \code{pretty.xlabels} is experimental.
}
\examples{

data(yarn)
mod.nir <- plsr(density ~ NIR, ncomp = 8, data = yarn)
\dontrun{
coefplot(mod.nir, ncomp = 1:6)
plot(mod.nir, plottype = "coefficients", ncomp = 1:6) # Equivalent to the previous
## Plot with legend:
coefplot(mod.nir, ncom = 1:6, legendpos = "bottomright")
}

data(oliveoil)
mod.sens <- plsr(sensory ~ chemical, ncomp = 4, data = oliveoil)
\dontrun{coefplot(mod.sens, ncomp = 2:4, separate = TRUE)}

}
\seealso{
\code{\link{mvr}}, \code{\link{plot.mvr}}, \code{\link{coef.mvr}},
\code{\link{plot}}, \code{\link{legend}}
}
\author{
Ron Wehrens and Bjørn-Helge Mevik
}
\keyword{hplot}
\keyword{multivariate}
\keyword{regression}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/widekernelpls.fit.R
\name{widekernelpls.fit}
\alias{widekernelpls.fit}
\title{Wide Kernel PLS (Rännar et al.)}
\usage{
widekernelpls.fit(
  X,
  Y,
  ncomp,
  center = TRUE,
  stripped = FALSE,
  tol = .Machine$double.eps^0.5,
  maxit = 100,
  ...
)
}
\arguments{
\item{X}{a matrix of observations.  \code{NA}s and \code{Inf}s are not
allowed.}

\item{Y}{a vector or matrix of responses.  \code{NA}s and \code{Inf}s are
not allowed.}

\item{ncomp}{the number of components to be used in the modelling.}

\item{center}{logical, determines if the \eqn{X} and \eqn{Y} matrices are
mean centered or not. Default is to perform mean centering.}

\item{stripped}{logical.  If \code{TRUE} the calculations are stripped as
much as possible for speed; this is meant for use with cross-validation or
simulations when only the coefficients are needed.  Defaults to
\code{FALSE}.}

\item{tol}{numeric.  The tolerance used for determining convergence in the
algorithm.}

\item{maxit}{positive integer.  The maximal number of iterations used in the
internal Eigenvector calculation.}

\item{\dots}{other arguments.  Currently ignored.}
}
\value{
A list containing the following components is returned:
\item{coefficients}{an array of regression coefficients for 1, \ldots{},
\code{ncomp} components.  The dimensions of \code{coefficients} are
\code{c(nvar, npred, ncomp)} with \code{nvar} the number of \code{X}
variables and \code{npred} the number of variables to be predicted in
\code{Y}.} \item{scores}{a matrix of scores.} \item{loadings}{a matrix of
loadings.} \item{loading.weights}{a matrix of loading weights.}
\item{Yscores}{a matrix of Y-scores.} \item{Yloadings}{a matrix of
Y-loadings.} \item{projection}{the projection matrix used to convert X to
scores.} \item{Xmeans}{a vector of means of the X variables.}
\item{Ymeans}{a vector of means of the Y variables.} \item{fitted.values}{an
array of fitted values.  The dimensions of \code{fitted.values} are
\code{c(nobj, npred, ncomp)} with \code{nobj} the number samples and
\code{npred} the number of Y variables.} \item{residuals}{an array of
regression residuals.  It has the same dimensions as \code{fitted.values}.}
\item{Xvar}{a vector with the amount of X-variance explained by each
component.} \item{Xtotvar}{Total variance in \code{X}.}

If \code{stripped} is \code{TRUE}, only the components \code{coefficients},
\code{Xmeans} and \code{Ymeans} are returned.
}
\description{
Fits a PLSR model with the wide kernel algorithm.
}
\details{
This function should not be called directly, but through the generic
functions \code{plsr} or \code{mvr} with the argument
\code{method="widekernelpls"}.  The wide kernel PLS algorithm is efficient
when the number of variables is (much) larger than the number of
observations.  For very wide \code{X}, for instance 12x18000, it can be
twice as fast as \code{\link{kernelpls.fit}} and \code{\link{simpls.fit}}.
For other matrices, however, it can be much slower.  The results are equal
to the results of the NIPALS algorithm.
}
\note{
The current implementation has not undergone extensive testing yet,
and should perhaps be regarded as experimental.  Specifically, the internal
Eigenvector calculation does not always converge in extreme cases where the
Eigenvalue is close to zero.  However, when it does converge, it always
converges to the same results as \code{\link{kernelpls.fit}}, up to
numerical inacurracies.

The algorithm also has a bit of overhead, so when the number of observations
is moderately high, \code{\link{kernelpls.fit}} can be faster even if the
number of predictors is much higher.  The relative speed of the algorithms
can also depend greatly on which BLAS and/or LAPACK library is linked
against.
}
\references{
Rännar, S., Lindgren, F., Geladi, P. and Wold, S. (1994) A PLS
Kernel Algorithm for Data Sets with Many Variables and Fewer Objects.  Part
1: Theory and Algorithm.  \emph{Journal of Chemometrics}, \bold{8},
111--125.
}
\seealso{
\code{\link{mvr}} \code{\link{plsr}} \code{\link{cppls}}
\code{\link{pcr}} \code{\link{kernelpls.fit}} \code{\link{simpls.fit}}
\code{\link{oscorespls.fit}}
}
\author{
Bjørn-Helge Mevik
}
\keyword{multivariate}
\keyword{regression}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cvsegments.R
\name{cvsegments}
\alias{cvsegments}
\title{Generate segments for cross-validation}
\usage{
cvsegments(
  N,
  k,
  length.seg = ceiling(N/k),
  nrep = 1,
  type = c("random", "consecutive", "interleaved"),
  stratify = NULL
)
}
\arguments{
\item{N}{Integer.  The number of rows in the data set.}

\item{k}{Integer.  The number of segments to return.}

\item{length.seg}{Integer.  The length of the segments.  If given, it
overrides \code{k}.}

\item{nrep}{Integer.  The number of (consecutive) rows that are replicates
of the same object.  Replicates will always be kept in the same segment.}

\item{type}{One of \code{"random"}, \code{"consecutive"} and
\code{"interleaved"}.  The type of segments to generate.  Default is
\code{"random"}.}

\item{stratify}{Either a \code{list} of indices or an integer \code{vector}
indicating which stratum each sample (or set of replicates) belongs to (see
Details).}
}
\value{
A list of vectors.  Each vector contains the indices for one
segment.  The attribute \code{"incomplete"} contains the number of
incomplete segments, and the attribute \code{"type"} contains the type of
segments.
}
\description{
The function generates a list of segments for cross-validation.  It can
generate random, consecutive and interleaved segments, and supports keeping
replicates in the same segment.
}
\details{
If \code{length.seg} is specified, it is used to calculate the number of
segments to generate.  Otherwise \code{k} must be specified.  If
\eqn{k*length.seg \ne N}{k*length.seg <> N}, the \eqn{k*length.seg - N} last
segments will contain only \eqn{length.seg - 1} indices.

If \code{type} is \code{"random"}, the indices are allocated to segments in
random order.  If it is \code{"consecutive"}, the first segment will contain
the first \eqn{length.seg} indices, and so on.  If \code{type} is
\code{"interleaved"}, the first segment will contain the indices \eqn{1,
length.seg+1, 2*lenght.seg+1, \ldots, (k-1)*length.seg+1}, and so on.

If \eqn{nrep > }, it is assumed that each \code{nrep} consecutive rows are
replicates (repeated measurements) of the same object, and care is taken
that replicates are never put in different segments.

Warning: If \code{k} does not divide \code{N}, a specified \code{length.seg}
does not divide \code{N}, or \code{nrep} does not divide \code{length.seg},
the number of segments and/or the segment length will be adjusted as needed.
Warnings are printed for some of these cases, and one should always inspect
the resulting segments to make sure they are as expected.

Stratification of samples is enabled by the \code{stratify} argument.  This
is useful if there are sub-groups in the data set that should have a
proportional representation in the cross-validation segments or if the
response is categorical (classifiation). If \code{stratify} is combined with
\code{nrep}, \code{stratify} corresponds to the sets of replicates (see
example).
}
\examples{

## Segments for 10-fold randomised cross-validation:
cvsegments(100, 10)

## Segments with four objects, taken consecutive:
(segs <- cvsegments(60, length.seg = 4, type = "cons"))
data(gasoline)
plsr(octane ~ NIR, data=gasoline, ncomp=5, validation="CV", segments=segs)

## Incomplete segments
segs <- cvsegments(50, length.seg = 3)
attr(segs, "incomplete")

## Leave-one-out cross-validation:
cvsegments(100, 100)
## Leave-one-out with variable/unknown data set size n:
n <- 50
cvsegments(n, length.seg = 1)

## Data set with replicates
cvsegments(100, 25, nrep = 2)
## Note that rows 1 and 2 are in the same segment, rows 3 and 4 in the
## same segment, and so on.

## Stratification
cvsegments(10, 3, type = "consecutive", stratify = c(rep(1,7), rep(2,3)))
## Note that the last three samples are spread across the segments
## according to the stratification vector.
cvsegments(20, 3, type = "consecutive", nrep = 2, stratify = c(rep(1,7), rep(2,3)))
## Note the length of stratify matching number of replicate sets, not samples.

## Converting a factor to segments
fac <- factor(c("a", "b", "a", "b", "c", "c"))
fac2seg(fac)
## Starting from a numeric vector
num <- c(1, 2, 1, 2, 3, 3)
fac2seg(factor(num))
}
\seealso{
Converting a factor to segments: \code{\link{fac2seg}}.
}
\author{
Bjørn-Helge Mevik, Ron Wehrens and Kristian Hovde Liland
}
\keyword{models}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/extract.R
\name{scores}
\alias{scores}
\alias{loadings}
\alias{scores.default}
\alias{loadings.default}
\alias{loading.weights}
\alias{Yscores}
\alias{Yloadings}
\title{Extract Scores and Loadings from PLSR and PCR Models}
\usage{
loadings(object, ...)

\method{loadings}{default}(object, ...)

scores(object, ...)

\method{scores}{default}(object, estimate, newdata, ...)

Yscores(object)

loading.weights(object)

Yloadings(object)
}
\arguments{
\item{object}{a fitted model to extract from.}

\item{\dots}{extra arguments, currently not used.}

\item{estimate}{optional character vector ("train", "CV", "test") used
by \code{scores} to select the desired estimate.}

\item{newdata}{optional data frame passed to \code{scores}
when \code{estimate = "test"}.}
}
\value{
A matrix with scores or loadings.
}
\description{
These functions extract score and loading matrices from fitted \code{mvr}
models.
}
\details{
All functions extract the indicated matrix from the fitted model, and will
work with any object having a suitably named component.

The default \code{scores} and \code{loadings} methods also handle
\code{prcomp} objects (their scores and loadings components are called
\code{x} and \code{rotation}, resp.), and add an attribute \code{"explvar"}
with the variance explained by each component, if this is available.  (See
\code{\link{explvar}} for details.)
}
\note{
There is a \code{loadings} function in package \pkg{stats}.  It simply
returns any element named \code{"loadings"}.  See
\code{\link[stats]{loadings}} for details.  The function can be accessed as
\code{stats::loadings(...)}.
}
\examples{

data(yarn)
plsmod <- plsr(density ~ NIR, 6, data = yarn)
scores(plsmod)
loadings(plsmod)[,1:4]

}
\seealso{
\code{\link{mvr}}, \code{\link{coef.mvr}}
}
\author{
Ron Wehrens and Bjørn-Helge Mevik
}
\keyword{multivariate}
\keyword{regression}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mvrVal.R
\name{mvrVal}
\alias{mvrVal}
\alias{mvrValstats}
\alias{MSEP}
\alias{MSEP.mvr}
\alias{RMSEP}
\alias{RMSEP.mvr}
\alias{R2}
\alias{R2.mvr}
\title{MSEP, RMSEP and R2 of PLSR and PCR models}
\usage{
mvrValstats(
  object,
  estimate,
  newdata,
  ncomp = 1:object$ncomp,
  comps,
  intercept = cumulative,
  se = FALSE,
  ...
)

R2(object, ...)

\method{R2}{mvr}(
  object,
  estimate,
  newdata,
  ncomp = 1:object$ncomp,
  comps,
  intercept = cumulative,
  se = FALSE,
  ...
)

MSEP(object, ...)

\method{MSEP}{mvr}(
  object,
  estimate,
  newdata,
  ncomp = 1:object$ncomp,
  comps,
  intercept = cumulative,
  se = FALSE,
  ...
)

RMSEP(object, ...)

\method{RMSEP}{mvr}(object, ...)
}
\arguments{
\item{object}{an \code{mvr} object}

\item{estimate}{a character vector.  Which estimators to use.  Should be a
subset of \code{c("all", "train", "CV", "adjCV", "test")}.  \code{"adjCV"}
is only available for (R)MSEP.  See below for how the estimators are chosen.}

\item{newdata}{a data frame with test set data.}

\item{ncomp, comps}{a vector of positive integers.  The components or number
of components to use.  See below.}

\item{intercept}{logical.  Whether estimates for a model with zero
components should be returned as well.}

\item{se}{logical.  Whether estimated standard errors of the estimates
should be calculated.  Not implemented yet.}

\item{\dots}{further arguments sent to underlying functions or (for
\code{RMSEP}) to \code{MSEP}}
}
\description{
Functions to estimate the mean squared error of prediction (MSEP), root mean
squared error of prediction (RMSEP) and \eqn{R^2} (A.K.A. coefficient of
multiple determination) for fitted PCR and PLSR models.  Test-set,
cross-validation and calibration-set estimates are implemented.
}
\details{
\code{RMSEP} simply calls \code{MSEP} and takes the square root of the
estimates.  It therefore accepts the same arguments as \code{MSEP}.

Several estimators can be used.  \code{"train"} is the training or
calibration data estimate, also called (R)MSEC.  For \code{R2}, this is the
unadjusted \eqn{R^2}.  It is overoptimistic and should not be used for
assessing models.  \code{"CV"} is the cross-validation estimate, and
\code{"adjCV"} (for \code{RMSEP} and \code{MSEP}) is the bias-corrected
cross-validation estimate.  They can only be calculated if the model has
been cross-validated.  Finally, \code{"test"} is the test set estimate,
using \code{newdata} as test set.

Which estimators to use is decided as follows (see below for
\code{mvrValstats}).  If \code{estimate} is not specified, the test set
estimate is returned if \code{newdata} is specified, otherwise the CV and
adjusted CV (for \code{RMSEP} and \code{MSEP}) estimates if the model has
been cross-validated, otherwise the training data estimate.  If
\code{estimate} is \code{"all"}, all possible estimates are calculated.
Otherwise, the specified estimates are calculated.

Several model sizes can also be specified.  If \code{comps} is missing (or
is \code{NULL}), \code{length(ncomp)} models are used, with \code{ncomp[1]}
components, \ldots{}, \code{ncomp[length(ncomp)]} components.  Otherwise, a
single model with the components \code{comps[1]}, \ldots{},
\code{comps[length(comps)]} is used.  If \code{intercept} is \code{TRUE}, a
model with zero components is also used (in addition to the above).

The \eqn{R^2} values returned by \code{"R2"} are calculated as \eqn{1 -
SSE/SST}, where \eqn{SST} is the (corrected) total sum of squares of the
response, and \eqn{SSE} is the sum of squared errors for either the fitted
values (i.e., the residual sum of squares), test set predictions or
cross-validated predictions (i.e., the \eqn{PRESS}).  For \code{estimate =
"train"}, this is equivalent to the squared correlation between the fitted
values and the response.  For \code{estimate = "train"}, the estimate is
often called the prediction \eqn{R^2}.

\code{mvrValstats} is a utility function that calculates the statistics
needed by \code{MSEP} and \code{R2}.  It is not intended to be used
interactively.  It accepts the same arguments as \code{MSEP} and \code{R2}.
However, the \code{estimate} argument must be specified explicitly: no
partial matching and no automatic choice is made.  The function simply
calculates the types of estimates it knows, and leaves the other untouched.
}
\section{Value}{
 \code{mvrValstats} returns a list with components \describe{
\item{SSE}{three-dimensional array of SSE values.  The first dimension is
the different estimators, the second is the response variables and the third
is the models.} \item{SST}{matrix of SST values.  The first dimension is the
different estimators and the second is the response variables.}
\item{nobj}{a numeric vector giving the number of objects used for each
estimator.} \item{comps}{the components specified, with \code{0} prepended
if \code{intercept} is \code{TRUE}.} \item{cumulative}{\code{TRUE} if
\code{comps} was \code{NULL} or not specified.} }

The other functions return an object of class \code{"mvrVal"}, with
components \describe{ \item{val}{three-dimensional array of estimates.  The
first dimension is the different estimators, the second is the response
variables and the third is the models.} \item{type}{\code{"MSEP"},
\code{"RMSEP"} or \code{"R2"}.} \item{comps}{the components specified, with
\code{0} prepended if \code{intercept} is \code{TRUE}.}
\item{cumulative}{\code{TRUE} if \code{comps} was \code{NULL} or not
specified.} \item{call}{the function call} }
}

\examples{

data(oliveoil)
mod <- plsr(sensory ~ chemical, ncomp = 4, data = oliveoil, validation = "LOO")
RMSEP(mod)
\dontrun{plot(R2(mod))}

}
\references{
Mevik, B.-H., Cederkvist, H. R. (2004) Mean Squared Error of
Prediction (MSEP) Estimates for Principal Component Regression (PCR) and
Partial Least Squares Regression (PLSR).  \emph{Journal of Chemometrics},
\bold{18}(9), 422--429.
}
\seealso{
\code{\link{mvr}}, \code{\link{crossval}}, \code{\link{mvrCv}},
\code{\link{validationplot}}, \code{\link{plot.mvrVal}}
}
\author{
Ron Wehrens and Bjørn-Helge Mevik
}
\keyword{multivariate}
\keyword{regression}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summaries.R
\name{print.mvr}
\alias{print.mvr}
\alias{summary.mvr}
\alias{print.mvrVal}
\alias{as.data.frame.mvrVal}
\title{Summary and Print Methods for PLSR and PCR objects}
\usage{
\method{print}{mvr}(x, ...)

\method{summary}{mvr}(
  object,
  what = c("all", "validation", "training"),
  digits = 4,
  print.gap = 2,
  ...
)

\method{print}{mvrVal}(x, digits = 4, print.gap = 2, ...)

\method{as.data.frame}{mvrVal}(x, row.names = NULL, optional = FALSE, shortAlgs = TRUE, ...)
}
\arguments{
\item{x, object}{an \code{mvr} object}

\item{\dots}{Other arguments sent to underlying methods.}

\item{what}{one of \code{"all"}, \code{"validation"} or \code{"training"}}

\item{digits}{integer.  Minimum number of significant digits in the output.
Default is 4.}

\item{print.gap}{Integer.  Gap between coloumns of the printed tables.}

\item{row.names}{NULL or a character vector giving the row names for the data frame. Missing values are not allowed.}

\item{optional}{Not used, only included to match signature of \code{as.data.frame}.}

\item{shortAlgs}{Logical.  Shorten algorithm names (default = TRUE).}
}
\value{
\code{print.mvr} and \code{print.mvrVal} return the object
invisibly.
}
\description{
Summary and print methods for \code{mvr} and \code{mvrVal} objects.
}
\details{
If \code{what} is \code{"training"}, the explained variances are given; if
it is \code{"validation"}, the cross-validated RMSEPs (if available) are
given; if it is \code{"all"}, both are given.
}
\examples{

data(yarn)
nir.mvr <- mvr(density ~ NIR, ncomp = 8, validation = "LOO", data = yarn)
nir.mvr
summary(nir.mvr)
RMSEP(nir.mvr)
# Extract MVR validation statistics as data.frame:
as.data.frame(RMSEP(nir.mvr, estimate = "CV"))
as.data.frame(R2(nir.mvr))

}
\seealso{
\code{\link{mvr}}, \code{\link{pcr}}, \code{\link{plsr}},
\code{\link{RMSEP}}, \code{\link{MSEP}}
}
\author{
Ron Wehrens and Bjørn-Helge Mevik
}
\keyword{multivariate}
\keyword{regression}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/crossval.R
\name{crossval}
\alias{crossval}
\title{Cross-validation of PLSR and PCR models}
\usage{
crossval(
  object,
  segments = 10,
  segment.type = c("random", "consecutive", "interleaved"),
  length.seg,
  jackknife = FALSE,
  trace = 15,
  ...
)
}
\arguments{
\item{object}{an \code{mvr} object; the regression to cross-validate.}

\item{segments}{the number of segments to use, or a list with segments (see
below).}

\item{segment.type}{the type of segments to use.  Ignored if \code{segments}
is a list.}

\item{length.seg}{Positive integer.  The length of the segments to use.  If
specified, it overrides \code{segments} unless \code{segments} is a list.}

\item{jackknife}{logical.  Whether jackknifing of regression coefficients
should be performed.}

\item{trace}{if \code{TRUE}, tracing is turned on.  If numeric, it denotes a
time limit (in seconds).  If the estimated total time of the
cross-validation exceeds this limit, tracing is turned on.}

\item{\dots}{additional arguments, sent to the underlying fit function.}
}
\value{
The supplied \code{object} is returned, with an additional component
\code{validation}, which is a list with components \item{method}{euqals
\code{"CV"} for cross-validation.} \item{pred}{an array with the
cross-validated predictions.} \item{coefficients}{(only if \code{jackknife}
is \code{TRUE}) an array with the jackknifed regression coefficients.  The
dimensions correspond to the predictors, responses, number of components,
and segments, respectively.} \item{PRESS0}{a vector of PRESS values (one for
each response variable) for a model with zero components, i.e., only the
intercept.} \item{PRESS}{a matrix of PRESS values for models with 1,
\ldots{}, \code{ncomp} components.  Each row corresponds to one response
variable.} \item{adj}{a matrix of adjustment values for calculating bias
corrected MSEP.  \code{MSEP} uses this.} \item{segments}{the list of
segments used in the cross-validation.} \item{ncomp}{the number of
components.} \item{gammas}{if method \code{cppls} is used, gamma values for
the powers of each CV segment are returned.}
}
\description{
A \dQuote{stand alone} cross-validation function for \code{mvr} objects.
}
\details{
This function performs cross-validation on a model fit by \code{mvr}.  It
can handle models such as \code{plsr(y ~ msc(X), \dots{})} or other models
where the predictor variables need to be recalculated for each segment.
When recalculation is not needed, the result of
\code{crossval(mvr(\dots{}))} is identical to \code{mvr(\dots{}, validation
= "CV")}, but slower.

Note that to use \code{crossval}, the data \emph{must} be specified with a
\code{data} argument when fitting \code{object}.

If \code{segments} is a list, the arguments \code{segment.type} and
\code{length.seg} are ignored.  The elements of the list should be integer
vectors specifying the indices of the segments.  See
\code{\link{cvsegments}} for details.

Otherwise, segments of type \code{segment.type} are generated.  How many
segments to generate is selected by specifying the number of segments in
\code{segments}, or giving the segment length in \code{length.seg}.  If both
are specified, \code{segments} is ignored.

If \code{jackknife} is \code{TRUE}, jackknifed regression coefficients are
returned, which can be used for for variance estimation
(\code{\link{var.jack}}) or hypothesis testing (\code{\link{jack.test}}).

When tracing is turned on, the segment number is printed for each segment.

By default, the cross-validation will be performed serially.  However, it
can be done in parallel using functionality in the \code{\link{parallel}}
package by setting the option \code{parallel} in \code{\link{pls.options}}.
See \code{\link{pls.options}} for the different ways to specify the
parallelism.  See also Examples below.
}
\note{
The \code{PRESS0} is always cross-validated using leave-one-out
cross-validation.  This usually makes little difference in practice, but
should be fixed for correctness.

The current implementation of the jackknife stores all jackknife-replicates
of the regression coefficients, which can be very costly for large matrices.
This might change in a future version.
}
\examples{

data(yarn)
yarn.pcr <- pcr(density ~ msc(NIR), 6, data = yarn)
yarn.cv <- crossval(yarn.pcr, segments = 10)
\dontrun{plot(MSEP(yarn.cv))}

\dontrun{
## Parallelised cross-validation, using transient cluster:
pls.options(parallel = 4) # use mclapply (not available on Windows)
pls.options(parallel = quote(parallel::makeCluster(4, type = "PSOCK"))) # parLapply
## A new cluster is created and stopped for each cross-validation:
yarn.cv <- crossval(yarn.pcr)
yarn.loocv <- crossval(yarn.pcr, length.seg = 1)

## Parallelised cross-validation, using persistent cluster:
library(parallel)
## This creates the cluster:
pls.options(parallel = makeCluster(4, type = "FORK")) # not available on Windows
pls.options(parallel = makeCluster(4, type = "PSOCK"))
## The cluster can be used several times:
yarn.cv <- crossval(yarn.pcr)
yarn.loocv <- crossval(yarn.pcr, length.seg = 1)
## The cluster should be stopped manually afterwards:
stopCluster(pls.options()$parallel)

## Parallelised cross-validation, using persistent MPI cluster:
## This requires the packages snow and Rmpi to be installed
library(parallel)
## This creates the cluster:
pls.options(parallel = makeCluster(4, type = "MPI"))
## The cluster can be used several times:
yarn.cv <- crossval(yarn.pcr)
yarn.loocv <- crossval(yarn.pcr, length.seg = 1)
## The cluster should be stopped manually afterwards:
stopCluster(pls.options()$parallel)
## It is good practice to call mpi.exit() or mpi.quit() afterwards:
mpi.exit()
}

}
\references{
Mevik, B.-H., Cederkvist, H. R. (2004) Mean Squared Error of
Prediction (MSEP) Estimates for Principal Component Regression (PCR) and
Partial Least Squares Regression (PLSR).  \emph{Journal of Chemometrics},
\bold{18}(9), 422--429.
}
\seealso{
\code{\link{mvr}} \code{\link{mvrCv}} \code{\link{cvsegments}}
\code{\link{MSEP}} \code{\link{var.jack}} \code{\link{jack.test}}
}
\author{
Ron Wehrens and Bjørn-Helge Mevik
}
\keyword{multivariate}
\keyword{regression}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/svdpc.fit.R
\name{svdpc.fit}
\alias{svdpc.fit}
\title{Principal Component Regression}
\usage{
svdpc.fit(X, Y, ncomp, center = TRUE, stripped = FALSE, ...)
}
\arguments{
\item{X}{a matrix of observations.  \code{NA}s and \code{Inf}s are not
allowed.}

\item{Y}{a vector or matrix of responses.  \code{NA}s and \code{Inf}s are
not allowed.}

\item{ncomp}{the number of components to be used in the modelling.}

\item{center}{logical, determines if the \eqn{X} and \eqn{Y} matrices are
mean centered or not. Default is to perform mean centering.}

\item{stripped}{logical.  If \code{TRUE} the calculations are stripped as
much as possible for speed; this is meant for use with cross-validation or
simulations when only the coefficients are needed.  Defaults to
\code{FALSE}.}

\item{\dots}{other arguments.  Currently ignored.}
}
\value{
A list containing the following components is returned:
\item{coefficients}{an array of regression coefficients for 1, \ldots{},
\code{ncomp} components.  The dimensions of \code{coefficients} are
\code{c(nvar, npred, ncomp)} with \code{nvar} the number of \code{X}
variables and \code{npred} the number of variables to be predicted in
\code{Y}.} \item{scores}{a matrix of scores.} \item{loadings}{a matrix of
loadings.} \item{Yloadings}{a matrix of Y-loadings.} \item{projection}{the
projection matrix used to convert X to scores.} \item{Xmeans}{a vector of
means of the X variables.} \item{Ymeans}{a vector of means of the Y
variables.} \item{fitted.values}{an array of fitted values.  The dimensions
of \code{fitted.values} are \code{c(nobj, npred, ncomp)} with \code{nobj}
the number samples and \code{npred} the number of Y variables.}
\item{residuals}{an array of regression residuals.  It has the same
dimensions as \code{fitted.values}.} \item{Xvar}{a vector with the amount of
X-variance explained by each component.} \item{Xtotvar}{Total variance in
\code{X}.}

If \code{stripped} is \code{TRUE}, only the components \code{coefficients},
\code{Xmeans} and \code{Ymeans} are returned.
}
\description{
Fits a PCR model using the singular value decomposition.
}
\details{
This function should not be called directly, but through the generic
functions \code{pcr} or \code{mvr} with the argument \code{method="svdpc"}.
The singular value decomposition is used to calculate the principal
components.
}
\references{
Martens, H., Næs, T. (1989) \emph{Multivariate calibration.}
Chichester: Wiley.
}
\seealso{
\code{\link{mvr}} \code{\link{plsr}} \code{\link{pcr}}
\code{\link{cppls}}
}
\author{
Ron Wehrens and Bjørn-Helge Mevik
}
\keyword{multivariate}
\keyword{regression}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plots.R
\name{scoreplot}
\alias{scoreplot}
\alias{scoreplot.default}
\alias{plot.scores}
\alias{loadingplot}
\alias{loadingplot.default}
\alias{plot.loadings}
\alias{corrplot}
\title{Plots of Scores, Loadings and Correlation Loadings}
\usage{
scoreplot(object, ...)

\method{scoreplot}{default}(
  object,
  comps = 1:2,
  labels,
  identify = FALSE,
  type = "p",
  xlab,
  ylab,
  estimate,
  newdata,
  ...
)

\method{plot}{scores}(x, ...)

loadingplot(object, ...)

\method{loadingplot}{default}(
  object,
  comps = 1:2,
  scatter = FALSE,
  labels,
  identify = FALSE,
  type,
  lty,
  lwd = NULL,
  pch,
  cex = NULL,
  col,
  legendpos,
  xlab,
  ylab,
  pretty.xlabels = TRUE,
  xlim,
  ...
)

\method{plot}{loadings}(x, ...)

corrplot(
  object,
  comps = 1:2,
  labels,
  plotx = TRUE,
  ploty = FALSE,
  radii = c(sqrt(1/2), 1),
  identify = FALSE,
  type = "p",
  xlab,
  ylab,
  col,
  ...
)
}
\arguments{
\item{object}{an object.  The fitted model.}

\item{\dots}{further arguments sent to the underlying plot function(s).}

\item{comps}{integer vector.  The components to plot.}

\item{labels}{optional.  Alternative plot labels or \eqn{x} axis labels.
See Details.}

\item{identify}{logical.  Whether to use \code{identify} to interactively
identify points.  See below.}

\item{type}{character.  What type of plot to make.  Defaults to \code{"p"}
(points) for scatter plots and \code{"l"} (lines) for line plots.  See
\code{\link{plot}} for a complete list of types (not all types are
possible/meaningful for all plots).}

\item{xlab, ylab}{titles for \eqn{x} and \eqn{y} axes.  Typically character
strings, but can be expressions or lists.  See \code{\link{title}} for
details.}

\item{estimate}{optional character vector passed to \code{scores()} so
\code{scoreplot()} can request the training, CV or test estimate when
plotting.}

\item{newdata}{optional data frame supplied to \code{scores()} when
\code{estimate = "test"}.}

\item{x}{a \code{scores} or \code{loadings} object.  The scores or loadings
to plot.}

\item{scatter}{logical.  Whether the loadings should be plotted as a scatter
instead of as lines.}

\item{lty}{vector of line types (recycled as neccessary).  Line types can be
specified as integers or character strings (see \code{\link{par}} for the
details).}

\item{lwd}{vector of positive numbers (recycled as neccessary), giving the
width of the lines.}

\item{pch}{plot character.  A character string or a vector of single
characters or integers (recycled as neccessary).  See \code{\link{points}}
for all alternatives.}

\item{cex}{numeric vector of character expansion sizes (recycled as
neccessary) for the plotted symbols.}

\item{col}{character or integer vector of colors for plotted lines and
symbols (recycled as neccessary).  See \code{\link{par}} for the details.}

\item{legendpos}{Legend position.  Optional.  Ignored if \code{scatter} is
\code{TRUE}.  If present, a legend is drawn at the given position.  The
position can be specified symbolically (e.g., \code{legendpos =
"topright"}).  This requires >= 2.1.0.  Alternatively, the position can be
specified explicitly (\code{legendpos = t(c(x,y))}) or interactively
(\code{legendpos = \link{locator}()}).}

\item{pretty.xlabels}{logical.  If \code{TRUE}, \code{loadingplot} tries to
plot the \eqn{x} labels more nicely.  See Details.}

\item{xlim}{optional vector of length two, with the \eqn{x} limits of the
plot.}

\item{plotx}{locical.  Whether to plot the \eqn{X} correlation loadings.
Defaults to \code{TRUE}.}

\item{ploty}{locical.  Whether to plot the \eqn{Y} correlation loadings.
Defaults to \code{FALSE}.}

\item{radii}{numeric vector, giving the radii of the circles drawn in
\code{corrplot}.  The default radii represent 50\% and 100\% explained
variance of the \eqn{X} variables by the chosen components.}
}
\value{
The functions return whatever the underlying plot function (or
\code{identify}) returns.
}
\description{
Functions to make scatter plots of scores or correlation loadings, and
scatter or line plots of loadings.
}
\details{
\code{plot.scores} is simply a wrapper calling \code{scoreplot}, passing all
arguments.  Similarly for \code{plot.loadings}.

\code{scoreplot} is generic, currently with a default method that works for
matrices and any object for which \code{\link{scores}} returns a matrix.
The default \code{scoreplot} method makes one or more scatter plots of the
scores, depending on how many components are selected.  If one or two
components are selected, and \code{identify} is \code{TRUE}, the function
\code{\link{identify}} is used to interactively identify points.

Also \code{loadingplot} is generic, with a default method that works for
matrices and any object where \code{\link{loadings}} returns a matrix.  If
\code{scatter} is \code{TRUE}, the default method works exactly like the
default \code{scoreplot} method.  Otherwise, it makes a lineplot of the
selected loading vectors, and if \code{identify} is \code{TRUE}, uses
\code{\link{identify}} to interactively identify points.  Also, if
\code{legendpos} is given, a legend is drawn at the position indicated.

\code{corrplot} works exactly like the default \code{scoreplot} method,
except that at least two components must be selected.  The
\dQuote{correlation loadings}, i.e. the correlations between each variable
and the selected components (see References), are plotted as pairwise
scatter plots, with concentric circles of radii given by \code{radii}.  Each
point corresponds to a variable.  The squared distance between the point and
origin equals the fraction of the variance of the variable explained by the
components in the panel.  The default \code{radii} corresponds to 50\% and
100\% explained variance.  By default, only the correlation loadings of the
\eqn{X} variables are plotted, but if \code{ploty} is \code{TRUE}, also the
\eqn{Y} correlation loadings are plotted.

\code{scoreplot}, \code{loadingplot} and \code{corrplot} can also be called
through the plot method for \code{mvr} objects, by specifying
\code{plottype} as \code{"scores"}, \code{"loadings"} or
\code{"correlation"}, respectively.  See \code{\link{plot.mvr}}.

The argument \code{labels} can be a vector of labels or one of
\code{"names"} and \code{"numbers"}.

If a scatter plot is produced (i.e., \code{scoreplot}, \code{corrplot}, or
\code{loadingplot} with \code{scatter = TRUE}), the labels are used instead
of plot symbols for the points plotted.  If \code{labels} is \code{"names"}
or \code{"numbers"}, the row names or row numbers of the matrix (scores,
loadings or correlation loadings) are used.

If a line plot is produced (i.e., \code{loadingplot}), the labels are used
as \eqn{x} axis labels.  If \code{labels} is \code{"names"} or
\code{"numbers"}, the variable names are used as labels, the difference
being that with \code{"numbers"}, the variable names are converted to
numbers, if possible.  Variable names of the forms \samp{"number"} or
\samp{"number text"} (where the space is optional), are handled.

The argument \code{pretty.xlabels} is only used when \code{labels} is
specified for a line plot.  If \code{TRUE} (default), the code tries to use
a \sQuote{pretty} selection of labels.  If \code{labels} is
\code{"numbers"}, it also uses the numerical values of the labels for
horisontal spacing.  If one has excluded parts of the spectral region, one
might therefore want to use \code{pretty.xlabels = FALSE}.
}
\note{
\code{\link{legend}} has many options.  If you want greater control
over the appearance of the legend, omit the \code{legendpos} argument and
call \code{legend} manually.

Graphical parametres (such as \code{pch} and \code{cex}) can also be used
with \code{scoreplot} and \code{corrplot}.  They are not listed in the
argument list simply because they are not handled specifically in the
function (unlike in \code{loadingplot}), but passed directly to the
underlying plot functions by \code{\dots{}}.

Tip: If the labels specified with \code{labels} are too long, they get
clipped at the border of the plot region.  This can be avoided by supplying
the graphical parameter \code{xpd = TRUE} in the plot call.

The handling of \code{labels} and \code{pretty.xlabels} in \code{coefplot}
is experimental.
}
\examples{

data(yarn)
mod <- plsr(density ~ NIR, ncomp = 10, data = yarn)
## These three are equivalent:
\dontrun{
scoreplot(mod, comps = 1:5)
plot(scores(mod), comps = 1:5)
plot(mod, plottype = "scores", comps = 1:5)

loadingplot(mod, comps = 1:5)
loadingplot(mod, comps = 1:5, legendpos = "topright") # With legend
loadingplot(mod, comps = 1:5, scatter = TRUE) # Plot as scatterplots

corrplot(mod, comps = 1:2)
corrplot(mod, comps = 1:3)
}

# Use of labels in plots and x scales
data(gasoline)
colnames(gasoline$NIR) <- paste(seq(900, 1700, 2), "nm")
gas <- plsr(octane ~ NIR, ncomp = 10, data = gasoline)
loadingplot(gas, labels="numbers")
loadingplot(gas, labels="names")
loadingplot(gas, labels="names", scatter=TRUE)

}
\references{
Martens, H., Martens, M. (2000) Modified Jack-knife Estimation
of Parameter Uncertainty in Bilinear Modelling by Partial Least Squares
Regression (PLSR).  \emph{Food Quality and Preference}, \bold{11}(1--2),
5--16.
}
\seealso{
\code{\link{mvr}}, \code{\link{plot.mvr}}, \code{\link{scores}},
\code{\link{loadings}}, \code{\link{identify}}, \code{\link{legend}}
}
\author{
Ron Wehrens and Bjørn-Helge Mevik
}
\keyword{hplot}
\keyword{multivariate}
\keyword{regression}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/crossval.R
\name{mvrCv}
\alias{mvrCv}
\title{Cross-validation}
\usage{
mvrCv(
  X,
  Y,
  ncomp,
  Y.add = NULL,
  weights = NULL,
  method = pls.options()$mvralg,
  scale = FALSE,
  segments = 10,
  segment.type = c("random", "consecutive", "interleaved"),
  length.seg,
  jackknife = FALSE,
  trace = FALSE,
  ...
)
}
\arguments{
\item{X}{a matrix of observations.  \code{NA}s and \code{Inf}s are not
allowed.}

\item{Y}{a vector or matrix of responses.  \code{NA}s and \code{Inf}s are
not allowed.}

\item{ncomp}{the number of components to be used in the modelling.}

\item{Y.add}{a vector or matrix of additional responses containing relevant
information about the observations.  Only used for \code{cppls}.}

\item{weights}{a vector of individual weights for the observations.  Only
used for \code{cppls}.  (Optional)}

\item{method}{the multivariate regression method to be used.}

\item{scale}{logical.  If \code{TRUE}, the learning \eqn{X} data for each
segment is scaled by dividing each variable by its sample standard
deviation.  The prediction data is scaled by the same amount.}

\item{segments}{the number of segments to use, or a list with segments (see
below).}

\item{segment.type}{the type of segments to use.  Ignored if \code{segments}
is a list.}

\item{length.seg}{Positive integer.  The length of the segments to use.  If
specified, it overrides \code{segments} unless \code{segments} is a list.}

\item{jackknife}{logical.  Whether jackknifing of regression coefficients
should be performed.}

\item{trace}{logical; if \code{TRUE}, the segment number is printed for each
segment.}

\item{\dots}{additional arguments, sent to the underlying fit function.}
}
\value{
A list with the following components: \item{method}{equals
\code{"CV"} for cross-validation.} \item{pred}{an array with the
cross-validated predictions.} \item{coefficients}{(only if \code{jackknife}
is \code{TRUE}) an array with the jackknifed regression coefficients.  The
dimensions correspond to the predictors, responses, number of components,
and segments, respectively.} \item{PRESS0}{a vector of PRESS values (one for
each response variable) for a model with zero components, i.e., only the
intercept.} \item{PRESS}{a matrix of PRESS values for models with 1,
\ldots{}, \code{ncomp} components.  Each row corresponds to one response
variable.} \item{adj}{a matrix of adjustment values for calculating bias
corrected MSEP.  \code{MSEP} uses this.} \item{segments}{the list of
segments used in the cross-validation.} \item{ncomp}{the actual number of
components used.} \item{gamma}{if method \code{cppls} is used, gamma values
for the powers of each CV segment are returned.}
}
\description{
Performs the cross-validation calculations for \code{mvr}.

This function is not meant to be called directly, but through the generic
functions \code{pcr}, \code{plsr}, \code{cppls} or \code{mvr} with the
argument \code{validation} set to \code{"CV"} or \code{"LOO"}.  All
arguments to \code{mvrCv} can be specified in the generic function call.

If \code{segments} is a list, the arguments \code{segment.type} and
\code{length.seg} are ignored.  The elements of the list should be integer
vectors specifying the indices of the segments.  See
\code{\link{cvsegments}} for details.

Otherwise, segments of type \code{segment.type} are generated.  How many
segments to generate is selected by specifying the number of segments in
\code{segments}, or giving the segment length in \code{length.seg}.  If both
are specified, \code{segments} is ignored.

If \code{jackknife} is \code{TRUE}, jackknifed regression coefficients are
returned, which can be used for for variance estimation
(\code{\link{var.jack}}) or hypothesis testing (\code{\link{jack.test}}).

\code{X} and \code{Y} do not need to be centered.

Note that this function cannot be used in situations where \eqn{X} needs to
be recalculated for each segment (except for scaling by the standard
deviation), for instance with \code{msc} or other preprocessing.  For such
models, use the more general (but slower) function \code{\link{crossval}}.

Also note that if needed, the function will silently(!) reduce \code{ncomp}
to the maximal number of components that can be cross-validated, which is
\eqn{n - l - 1}, where \eqn{n} is the number of observations and \eqn{l} is
the length of the longest segment.  The (possibly reduced) number of
components is returned as the component \code{ncomp}.

By default, the cross-validation will be performed serially.  However, it
can be done in parallel using functionality in the \code{\link{parallel}}
package by setting the option \code{parallel} in \code{\link{pls.options}}.
See \code{\link{pls.options}} for the different ways to specify the
parallelism.
}
\note{
The \code{PRESS0} is always cross-validated using leave-one-out
cross-validation.  This usually makes little difference in practice, but
should be fixed for correctness.

The current implementation of the jackknife stores all jackknife-replicates
of the regression coefficients, which can be very costly for large matrices.
This might change in a future version.
}
\examples{

data(yarn)
yarn.pcr <- pcr(density ~ NIR, 6, data = yarn, validation = "CV", segments = 10)
\dontrun{plot(MSEP(yarn.pcr))}

}
\references{
Mevik, B.-H., Cederkvist, H. R. (2004) Mean Squared Error of
Prediction (MSEP) Estimates for Principal Component Regression (PCR) and
Partial Least Squares Regression (PLSR).  \emph{Journal of Chemometrics},
\bold{18}(9), 422--429.
}
\seealso{
\code{\link{mvr}} \code{\link{crossval}} \code{\link{cvsegments}}
\code{\link{MSEP}} \code{\link{var.jack}} \code{\link{jack.test}}
}
\author{
Ron Wehrens and Bjørn-Helge Mevik
}
\keyword{multivariate}
\keyword{regression}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/extract.R
\name{naExcludeMvr}
\alias{naExcludeMvr}
\title{Adjust for Missing Values}
\usage{
naExcludeMvr(omit, x, ...)
}
\arguments{
\item{omit}{an object produced by an \code{na.action} function, typically
the \code{"na.action"} attribute of the result of \code{na.omit} or
\code{na.exclude}.}

\item{x}{a three-dimensional array to be adjusted based upon the missing
value information in \code{omit}.}

\item{\dots}{further arguments.  Currently not used.}
}
\value{
\code{x}, padded with \code{NA}s along the first dimension
(\sQuote{rows}).
}
\description{
Use missing value information to adjust residuals and predictions.  This is
the \sQuote{mvr equivalent} of the \code{naresid.exclude} and
\code{napredict.exclude} functions.
}
\details{
This is a utility function used to allow \code{predict.mvr} and
\code{residuals.mvr} to compensate for the removal of \code{NA}s in the
fitting process.

It is called only when the \code{na.action} is \code{na.exclude}, and pads
\code{x} with \code{NA}s in the correct positions to have the same number of
rows as the original data frame.
}
\seealso{
\code{\link{predict.mvr}}, \code{\link{residuals.mvr}},
\code{\link{napredict}}, \code{\link{naresid}}
}
\author{
Bjørn-Helge Mevik and Ron Wehrens
}
\keyword{internal}
\keyword{multivariate}
\keyword{regression}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/kernelpls.fit.R
\name{kernelpls.fit}
\alias{kernelpls.fit}
\title{Kernel PLS (Dayal and MacGregor)}
\usage{
kernelpls.fit(X, Y, ncomp, center = TRUE, stripped = FALSE, ...)
}
\arguments{
\item{X}{a matrix of observations.  \code{NA}s and \code{Inf}s are not
allowed.}

\item{Y}{a vector or matrix of responses.  \code{NA}s and \code{Inf}s are
not allowed.}

\item{ncomp}{the number of components to be used in the modelling.}

\item{center}{logical, determines if the \eqn{X} and \eqn{Y} matrices are
mean centered or not. Default is to perform mean centering.}

\item{stripped}{logical.  If \code{TRUE} the calculations are stripped as
much as possible for speed; this is meant for use with cross-validation or
simulations when only the coefficients are needed.  Defaults to
\code{FALSE}.}

\item{\dots}{other arguments.  Currently ignored.}
}
\value{
A list containing the following components is returned:
\item{coefficients}{an array of regression coefficients for 1, \ldots{},
\code{ncomp} components.  The dimensions of \code{coefficients} are
\code{c(nvar, npred, ncomp)} with \code{nvar} the number of \code{X}
variables and \code{npred} the number of variables to be predicted in
\code{Y}.} \item{scores}{a matrix of scores.} \item{loadings}{a matrix of
loadings.} \item{loading.weights}{a matrix of loading weights.}
\item{Yscores}{a matrix of Y-scores.} \item{Yloadings}{a matrix of
Y-loadings.} \item{projection}{the projection matrix used to convert X to
scores.} \item{Xmeans}{a vector of means of the X variables.}
\item{Ymeans}{a vector of means of the Y variables.} \item{fitted.values}{an
array of fitted values.  The dimensions of \code{fitted.values} are
\code{c(nobj, npred, ncomp)} with \code{nobj} the number samples and
\code{npred} the number of Y variables.} \item{residuals}{an array of
regression residuals.  It has the same dimensions as \code{fitted.values}.}
\item{Xvar}{a vector with the amount of X-variance explained by each
component.} \item{Xtotvar}{Total variance in \code{X}.}

If \code{stripped} is \code{TRUE}, only the components \code{coefficients},
\code{Xmeans} and \code{Ymeans} are returned.
}
\description{
Fits a PLSR model with the kernel algorithm.
}
\details{
This function should not be called directly, but through the generic
functions \code{plsr} or \code{mvr} with the argument
\code{method="kernelpls"} (default).  Kernel PLS is particularly efficient
when the number of objects is (much) larger than the number of variables.
The results are equal to the NIPALS algorithm.  Several different forms of
kernel PLS have been described in literature, e.g.  by De Jong and Ter
Braak, and two algorithms by Dayal and MacGregor.  This function implements
the fastest of the latter, not calculating the crossproduct matrix of X.  In
the Dyal & MacGregor paper, this is \dQuote{algorithm 1}.
}
\references{
de Jong, S. and ter Braak, C. J. F. (1994) Comments on the PLS
kernel algorithm.  \emph{Journal of Chemometrics}, \bold{8}, 169--174.

Dayal, B. S. and MacGregor, J. F. (1997) Improved PLS algorithms.
\emph{Journal of Chemometrics}, \bold{11}, 73--85.
}
\seealso{
\code{\link{mvr}} \code{\link{plsr}} \code{\link{cppls}}
\code{\link{pcr}} \code{\link{widekernelpls.fit}} \code{\link{simpls.fit}}
\code{\link{oscorespls.fit}}
}
\author{
Ron Wehrens and Bjørn-Helge Mevik
}
\keyword{multivariate}
\keyword{regression}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/jackknife.R
\name{jack.test}
\alias{jack.test}
\alias{print.jacktest}
\title{Jackknife approximate t tests of regression coefficients}
\usage{
jack.test(object, ncomp = object$ncomp, use.mean = TRUE)

\method{print}{jacktest}(x, P.values = TRUE, ...)
}
\arguments{
\item{object}{an \code{mvr} object.  A cross-validated model fitted with
\code{jackknife = TRUE}.}

\item{ncomp}{the number of components to use for estimating the variances}

\item{use.mean}{logical.  If \code{TRUE} (default), the mean coefficients
are used when estimating the (co)variances; otherwise the coefficients from
a model fitted to the entire data set.  See \code{\link{var.jack}} for
details.}

\item{x}{an \code{jacktest} object, the result of \code{jack.test}.}

\item{P.values}{logical.  Whether to print \eqn{p} values (default).}

\item{\dots}{Further arguments sent to the underlying print function
\code{\link{printCoefmat}}.}
}
\value{
\code{jack.test} returns an object of class \code{"jacktest"}, with
components \item{coefficients }{The estimated regression coefficients}
\item{sd}{The square root of the jackknife variance estimates}
\item{tvalues}{The \eqn{t} statistics} \item{df}{The `degrees of freedom'
used for calculating \eqn{p} values} \item{pvalues}{The calculated \eqn{p}
values}

\code{print.jacktest} returns the \code{"jacktest"} object (invisibly).
}
\description{
Performes approximate t tests of regression coefficients based on jackknife
variance estimates.
}
\details{
\code{jack.test} uses the variance estimates from \code{var.jack} to perform
\eqn{t} tests of the regression coefficients.  The resulting object has a
print method, \code{print.jacktest}, which uses \code{\link{printCoefmat}}
for the actual printing.
}
\section{Warning}{
 The jackknife variance estimates are known to be biased
(see \code{\link{var.jack}}).  Also, the distribution of the regression
coefficient estimates and the jackknife variance estimates are unknown (at
least in PLSR/PCR).  Consequently, the distribution (and in particular, the
degrees of freedom) of the resulting \eqn{t} statistics is unknown.  The
present code simply assumes a \eqn{t} distribution with \eqn{m - 1} degrees
of freedom, where \eqn{m} is the number of cross-validation segments.

Therefore, the resulting \eqn{p} values should not be used uncritically, and
should perhaps be regarded as mere indicator of (non-)significance.

Finally, also keep in mind that as the number of predictor variables
increase, the problem of multiple tests increases correspondingly.
}

\examples{

data(oliveoil)
mod <- pcr(sensory ~ chemical, data = oliveoil, validation = "LOO", jackknife = TRUE)
jack.test(mod, ncomp = 2)

}
\references{
Martens H. and Martens M. (2000) Modified Jack-knife Estimation
of Parameter Uncertainty in Bilinear Modelling by Partial Least Squares
Regression (PLSR).  \emph{Food Quality and Preference}, \bold{11}, 5--16.
}
\seealso{
\code{\link{var.jack}}, \code{\link{mvrCv}}
}
\author{
Bjørn-Helge Mevik
}
\keyword{htest}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{oliveoil}
\alias{oliveoil}
\title{Sensory and physico-chemical data of olive oils}
\format{
A data frame with 16 observations on the following 2 variables.
\describe{ \item{sensory}{a matrix with 6 columns.  Scores for
attributes \sQuote{yellow}, \sQuote{green}, \sQuote{brown}, \sQuote{glossy},
\sQuote{transp}, and \sQuote{syrup}.} \item{chemical}{a matrix with
5 columns.  Measurements of acidity, peroxide, K232, K270, and DK.} }
}
\source{
Massart, D. L., Vandeginste, B. G. M., Buydens, L. M. C., de Jong,
S., Lewi, P. J., Smeyers-Verbeke, J. (1998) \emph{Handbook of Chemometrics
and Qualimetrics: Part B}.  Elsevier. Tables 35.1 and 35.4.
}
\description{
A data set with scores on 6 attributes from a sensory panel and measurements
of 5 physico-chemical quality parameters on 16 olive oil samples.  The first
five oils are Greek, the next five are Italian and the last six are Spanish.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vcov.R
\name{vcov.mvr}
\alias{vcov.mvr}
\title{Calculate Variance-Covariance Matrix for a Fitted Model Object}
\usage{
\method{vcov}{mvr}(object, ncomp, ...)
}
\arguments{
\item{object}{a fitted PCR object of class \code{mvr}.}

\item{ncomp}{number of principal components to estimate \code{vcov} for.}

\item{...}{additional arguments (not used).}
}
\value{
A matrix of estimated covariances between regression coefficients.
}
\description{
Returns the variance-covariance matrix of the coefficients
of a Principal Component Regression.
}
\examples{
data(yarn)
yarn.pcr <- pcr(density ~ NIR, 6, data = yarn)
vc <- vcov(yarn.pcr, 3)

# Standard error of coefficients
se <- sqrt(diag(vc))
beta <- coef(yarn.pcr, ncomp = 3)

# Plot regression coefficients with two standard errors shading.
plot(beta, type = 'l',
     panel.first = polygon(x = c(1:268, 268:1),
                           y = c(beta+2*se, rev(beta-2*se)),
                           col = 'lightblue',
                           border = NA))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cppls.fit.R
\name{cppls.fit}
\alias{cppls.fit}
\title{CPPLS (Indahl et al.)}
\usage{
cppls.fit(
  X,
  Y,
  ncomp,
  Y.add = NULL,
  center = TRUE,
  stripped = FALSE,
  lower = 0.5,
  upper = 0.5,
  trunc.pow = FALSE,
  weights = NULL,
  ...
)
}
\arguments{
\item{X}{a matrix of observations.  \code{NA}s and \code{Inf}s are not
allowed.}

\item{Y}{a vector or matrix of responses.  \code{NA}s and \code{Inf}s are
not allowed.}

\item{ncomp}{the number of components to be used in the modelling.}

\item{Y.add}{a vector or matrix of additional responses containing relevant
information about the observations.}

\item{center}{logical, determines if the \eqn{X} and \eqn{Y} matrices are
mean centered or not. Default is to perform mean centering.}

\item{stripped}{logical.  If \code{TRUE} the calculations are stripped as
much as possible for speed; this is meant for use with cross-validation or
simulations when only the coefficients are needed.  Defaults to
\code{FALSE}.}

\item{lower}{a vector of lower limits for power optimisation. Defaults to
\code{0.5}.}

\item{upper}{a vector of upper limits for power optimisation. Defaults to
\code{0.5}.}

\item{trunc.pow}{logical. If \code{TRUE} an experimental alternative power
algorithm is used. (Optional)}

\item{weights}{a vector of individual weights for the observations.
(Optional)}

\item{\dots}{other arguments.  Currently ignored.}
}
\value{
A list containing the following components is returned:
\item{coefficients}{an array of regression coefficients for 1, \ldots{},
\code{ncomp} components.  The dimensions of \code{coefficients} are
\code{c(nvar, npred, ncomp)} with \code{nvar} the number of \code{X}
variables and \code{npred} the number of variables to be predicted in
\code{Y}.} \item{scores}{a matrix of scores.} \item{loadings}{a matrix of
loadings.} \item{loading.weights}{a matrix of loading weights.}
\item{Yscores}{a matrix of Y-scores.} \item{Yloadings}{a matrix of
Y-loadings.} \item{projection}{the projection matrix used to convert X to
scores.} \item{Xmeans}{a vector of means of the X variables.}
\item{Ymeans}{a vector of means of the Y variables.} \item{fitted.values}{an
array of fitted values.  The dimensions of \code{fitted.values} are
\code{c(nobj, npred, ncomp)} with \code{nobj} the number samples and
\code{npred} the number of Y variables.} \item{residuals}{an array of
regression residuals.  It has the same dimensions as \code{fitted.values}.}
\item{Xvar}{a vector with the amount of X-variance explained by each
component.} \item{Xtotvar}{total variance in \code{X}.}
\item{gammas}{gamma-values obtained in power optimisation.}
\item{canonical.correlations}{Canonical correlation values from the
calculations of loading weights.} \item{A}{matrix containing vectors of
weights \code{a} from canonical correlation (\code{cor(Za,Yb)}).}
\item{smallNorms}{vector of indices of explanatory variables of length close
to or equal to 0.}

If \code{stripped} is \code{TRUE}, only the components \code{coefficients},
\code{Xmeans}, \code{Ymeans} and \code{gammas} are returned.
}
\description{
Fits a PLS model using the CPPLS algorithm.
}
\details{
This function should not be called directly, but through the generic
functions \code{cppls} or \code{mvr} with the argument
\code{method="cppls"}. Canonical Powered PLS (CPPLS) is a generalisation of
PLS incorporating discrete and continuous responses (also simultaneously),
additional responses, individual weighting of observations and power
methodology for sharpening focus on groups of variables. Depending on the
input to \code{cppls} it can produce the following special cases: \itemize{
\item PLS: uni-response continuous \code{Y} \item PPLS: uni-response
continuous \code{Y}, \code{(lower || upper) != 0.5} \item PLS-DA (using
correlation maximisation - B/W): dummy-coded descrete response \code{Y}
\item PPLS-DA: dummy-coded descrete response \code{Y}, \code{(lower ||
upper) != 0.5} \item CPLS: multi-response \code{Y} (continuous, discrete or
combination) \item CPPLS: multi-response \code{Y} (continuous, discrete or
combination), \code{(lower || upper) != 0.5} } The name "canonical" comes
from canonical correlation analysis which is used when calculating vectors
of loading weights, while "powered" refers to a reparameterisation of the
vectors of loading weights which can be optimised over a given interval.
}
\examples{

data(mayonnaise)
# Create dummy response
mayonnaise$dummy <-
    I(model.matrix(~y-1, data.frame(y = factor(mayonnaise$oil.type))))

# Predict CPLS scores for test data
may.cpls <- cppls(dummy ~ NIR, 10, data = mayonnaise, subset = train)
may.test <- predict(may.cpls, newdata = mayonnaise[!mayonnaise$train,], type = "score")

# Predict CPLS scores for test data (experimental used design as additional Y information)
may.cpls.yadd <- cppls(dummy ~ NIR, 10, data = mayonnaise, subset = train, Y.add=design)
may.test.yadd <- predict(may.cpls.yadd, newdata = mayonnaise[!mayonnaise$train,], type = "score")

# Classification by linear discriminant analysis (LDA)
library(MASS)
error <- matrix(ncol = 10, nrow = 2)
dimnames(error) <- list(Model = c('CPLS', 'CPLS (Y.add)'), ncomp = 1:10)
for (i in 1:10) {
    fitdata1 <- data.frame(oil.type = mayonnaise$oil.type[mayonnaise$train],
                           NIR.score = I(may.cpls$scores[,1:i,drop=FALSE]))
    testdata1 <- data.frame(oil.type = mayonnaise$oil.type[!mayonnaise$train],
                            NIR.score = I(may.test[,1:i,drop=FALSE]))
    error[1,i] <-
        (42 - sum(predict(lda(oil.type ~ NIR.score, data = fitdata1),
                  newdata = testdata1)$class == testdata1$oil.type)) / 42
    fitdata2 <- data.frame(oil.type = mayonnaise$oil.type[mayonnaise$train],
                           NIR.score = I(may.cpls.yadd$scores[,1:i,drop=FALSE]))
    testdata2 <- data.frame(oil.type = mayonnaise$oil.type[!mayonnaise$train],
                            NIR.score = I(may.test.yadd[,1:i,drop=FALSE]))
    error[2,i] <-
        (42 - sum(predict(lda(oil.type ~ NIR.score, data = fitdata2),
                  newdata = testdata2)$class == testdata2$oil.type)) / 42
}
round(error,2)

}
\references{
Indahl, U. (2005) A twist to partial least squares regression.
\emph{Journal of Chemometrics}, \bold{19}, 32--44.

Liland, K.H and Indahl, U.G (2009) Powered partial least squares
discriminant analysis, \emph{Journal of Chemometrics}, \bold{23}, 7--18.

Indahl, U.G., Liland, K.H. and Næs, T. (2009) Canonical partial least
squares - a unified PLS approach to classification and regression problems.
\emph{Journal of Chemometrics}, \bold{23}, 495--504.
}
\seealso{
\code{\link{mvr}} \code{\link{plsr}} \code{\link{pcr}}
\code{\link{widekernelpls.fit}} \code{\link{simpls.fit}}
\code{\link{oscorespls.fit}}
}
\author{
Kristian Hovde Liland
}
\keyword{classification}
\keyword{multivariate}
\keyword{regression}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/msc.R
\name{msc}
\alias{msc}
\alias{predict.msc}
\alias{makepredictcall.msc}
\title{Multiplicative Scatter Correction}
\usage{
msc(X, reference = NULL)

\method{predict}{msc}(object, newdata, ...)

\method{makepredictcall}{msc}(var, call)
}
\arguments{
\item{X, newdata}{numeric matrices.  The data to scatter correct.}

\item{reference}{numeric vector.  Spectre to use as reference.  If
\code{NULL}, the column means of \code{X} are used.}

\item{object}{an object inheriting from class \code{"msc"}, normally the
result of a call to \code{msc} with a single matrix argument.}

\item{\dots}{other arguments.  Currently ignored.}

\item{var}{A variable.}

\item{call}{The term in the formula, as a call.}
}
\value{
Both \code{msc} and \code{predict.msc} return a multiplicative
scatter corrected matrix, with attribute \code{"reference"} the vector used
as reference spectre. The matrix is given class \code{c("msc", "matrix")}.
For \code{predict.msc}, the \code{"reference"} attribute of \code{object} is
used as reference spectre.
}
\description{
Performs multiplicative scatter/signal correction on a data matrix.
}
\details{
\code{makepredictcall.msc} is an internal utility function; it is not meant
for interactive use.  See \code{\link{makepredictcall}} for details.
}
\examples{

data(yarn)
## Direct correction:
Ztrain <- msc(yarn$NIR[yarn$train,])
Ztest <- predict(Ztrain, yarn$NIR[!yarn$train,])

## Used in formula:
mod <- plsr(density ~ msc(NIR), ncomp = 6, data = yarn[yarn$train,])
pred <- predict(mod, newdata = yarn[!yarn$train,]) # Automatically scatter corrected

}
\references{
Martens, H., Næs, T. (1989) \emph{Multivariate calibration.}
Chichester: Wiley.
}
\seealso{
\code{\link{mvr}}, \code{\link{pcr}}, \code{\link{plsr}},
\code{\link{stdize}}
}
\author{
Bjørn-Helge Mevik and Ron Wehrens
}
\keyword{multivariate}
\keyword{regression}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{yarn}
\alias{yarn}
\title{NIR spectra and density measurements of PET yarns}
\format{
A data frame with components \describe{ \item{NIR}{Numeric matrix of
NIR measurements} \item{density}{Numeric vector of densities}
\item{train}{Logical vector with \code{TRUE} for the training samples and
\code{FALSE} for the test samples} }
}
\source{
Swierenga H., de Weijer A. P., van Wijk R. J., Buydens L. M. C.
(1999) Strategy for constructing robust multivariate calibration models
\emph{Chemometrics and Intelligent Laboratoryy Systems}, \bold{49}(1),
1--17.
}
\description{
A training set consisting of 21 NIR spectra of PET yarns, measured at 268
wavelengths, and 21 corresponding densities.  A test set of 7 samples is
also provided.  Many thanks to Erik Swierenga.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/predict.mvr.R
\name{predict.mvr}
\alias{predict.mvr}
\title{Predict Method for PLSR and PCR}
\usage{
\method{predict}{mvr}(
  object,
  newdata,
  ncomp = 1:object$ncomp,
  comps,
  type = c("response", "scores"),
  na.action = na.pass,
  ...
)
}
\arguments{
\item{object}{an \code{mvr} object.  The fitted model}

\item{newdata}{a data frame.  The new data.  If missing, the training data
is used.}

\item{ncomp, comps}{vector of positive integers.  The components to use in
the prediction.  See below.}

\item{type}{character.  Whether to predict scores or response values}

\item{na.action}{function determining what should be done with missing
values in \code{newdata}.  The default is to predict \code{NA}.  See
\code{\link{na.omit}} for alternatives.}

\item{\dots}{further arguments.  Currently not used}
}
\value{
When \code{type} is \code{"response"}, a three dimensional array of
predicted response values is returned.  The dimensions correspond to the
observations, the response variables and the model sizes, respectively.

When \code{type} is \code{"scores"}, a score matrix is returned.
}
\description{
Prediction for mvr (PCR, PLSR) models.  New responses or scores are
predicted using a fitted model and a new matrix of observations.
}
\details{
When \code{type} is \code{"response"} (default), predicted response values
are returned.  If \code{comps} is missing (or is \code{NULL}), predictions
for \code{length(ncomp)} models with \code{ncomp[1]} components,
\code{ncomp[2]} components, etc., are returned.  Otherwise, predictions for
a single model with the exact components in \code{comps} are returned.
(Note that in both cases, the intercept is always included in the
predictions.  It can be removed by subtracting the \code{Ymeans} component
of the fitted model.)

When \code{type} is \code{"scores"}, predicted score values are returned for
the components given in \code{comps}.  If \code{comps} is missing or
\code{NULL}, \code{ncomps} is used instead.

It is also possible to supply a matrix instead of a data frame as
\code{newdata}, which is then assumed to be the \eqn{X} data matrix.  Note
that the usual checks for the type of the data are then omitted.  Also note
that this is \emph{only} possible with \code{predict}; it will not work in
functions like \code{\link{predplot}}, \code{\link{RMSEP}} or
\code{\link{R2}}, because they also need the response variable of the new
data.
}
\note{
A warning message like \samp{'newdata' had 10 rows but variable(s)
found have 106 rows} means that not all variables were found in the
\code{newdata} data frame.  This (usually) happens if the formula contains
terms like \code{yarn$NIR}.  Do not use such terms; use the \code{data}
argument instead.  See \code{\link{mvr}} for details.
}
\examples{

data(yarn)
nir.mvr <- mvr(density ~ NIR, ncomp = 5, data = yarn[yarn$train,])

## Predicted responses for models with 1, 2, 3 and 4 components
pred.resp <- predict(nir.mvr, ncomp = 1:4, newdata = yarn[!yarn$train,])

## Predicted responses for a single model with components 1, 2, 3, 4
predict(nir.mvr, comps = 1:4, newdata = yarn[!yarn$train,])

## Predicted scores
predict(nir.mvr, comps = 1:3, type = "scores", newdata = yarn[!yarn$train,])

}
\seealso{
\code{\link{mvr}}, \code{\link{summary.mvr}},
\code{\link{coef.mvr}}, \code{\link{plot.mvr}}
}
\author{
Ron Wehrens and Bjørn-Helge Mevik
}
\keyword{multivariate}
\keyword{regression}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fac2seg.R
\name{fac2seg}
\alias{fac2seg}
\title{Factor to Segments}
\usage{
fac2seg(fac)
}
\arguments{
\item{fac}{A factor where each level represents a segment}
}
\value{
A list of vectors, each vector contains the indices of the elements
of the corresponding segment
}
\description{
Factor to Segments
}
\examples{
fac <- factor(c("a", "b", "a", "b", "c", "c"))
fac2seg(fac)
}
\seealso{
CV segments by various patterns: \code{\link{cvsegments}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/svdpc.fit.R
\name{nipalspc.fit}
\alias{nipalspc.fit}
\title{NIPALS PCR with missing values}
\usage{
nipalspc.fit(
  X,
  Y,
  ncomp,
  center = TRUE,
  stripped = FALSE,
  maxiter = 500,
  tol = 1e-06,
  ...
)
}
\arguments{
\item{X}{numeric matrix (or coercible) of predictors. Missing values are
allowed and handled internally.}

\item{Y}{numeric matrix (or coercible) of responses. Missing values are also
handled internally during the final regression step.}

\item{ncomp}{number of PCR components to extract.}

\item{center}{logical. If \code{TRUE} both \code{X} and \code{Y} are
centered column-wise (ignoring missing entries).}

\item{stripped}{logical. When \code{TRUE} only the coefficients and mean
vectors are returned for faster use in resampling.}

\item{maxiter}{maximum number of inner iterations per component.}

\item{tol}{convergence tolerance used when the direction vector stabilizes.}

\item{...}{currently ignored.}
}
\value{
A list mirroring the return value of \code{svdpc.fit} but computed
via the NA-robust NIPALS PCR updates.
}
\description{
A NIPALS-based PCR that tolerates missing entries in both
predictors and responses by only using observed cells when updating scores
and loadings.  It follows the same API as \code{svdpc.fit} so it can be used
whenever low-level PCR needs to handle incomplete data.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/extract.R
\name{delete.intercept}
\alias{delete.intercept}
\title{Delete intercept from model matrix}
\usage{
delete.intercept(mm)
}
\arguments{
\item{mm}{Model matrix.}
}
\value{
A model matrix without intercept column.
}
\description{
A utility function to delete any intercept column from a model matrix, and
adjust the \code{"assign"} attribute correspondingly.  It is used by formula
handling functions like \code{mvr} and \code{model.matrix.mvr}.
}
\seealso{
\code{\link{mvr}}, \code{\link{model.matrix.mvr}}
}
\author{
Bjørn-Helge Mevik and Ron Wehrens
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/oscorespls.fit.R
\name{oscorespls.fit}
\alias{oscorespls.fit}
\title{Orthogonal scores PLSR}
\usage{
oscorespls.fit(
  X,
  Y,
  ncomp,
  center = TRUE,
  stripped = FALSE,
  tol = .Machine$double.eps^0.5,
  maxit = 100,
  ...
)
}
\arguments{
\item{X}{a matrix of observations.  \code{NA}s and \code{Inf}s are not
allowed.}

\item{Y}{a vector or matrix of responses.  \code{NA}s and \code{Inf}s are
not allowed.}

\item{ncomp}{the number of components to be used in the modelling.}

\item{center}{logical, determines if the \eqn{X} and \eqn{Y} matrices are
mean centered or not. Default is to perform mean centering.}

\item{stripped}{logical.  If \code{TRUE} the calculations are stripped as
much as possible for speed; this is meant for use with cross-validation or
simulations when only the coefficients are needed.  Defaults to
\code{FALSE}.}

\item{tol}{numeric.  The tolerance used for determining convergence in
multi-response models.}

\item{maxit}{positive integer.  The maximal number of iterations used in the
internal Eigenvector calculation.}

\item{\dots}{other arguments.  Currently ignored.}
}
\value{
A list containing the following components is returned:
\item{coefficients}{an array of regression coefficients for 1, \ldots{},
\code{ncomp} components.  The dimensions of \code{coefficients} are
\code{c(nvar, npred, ncomp)} with \code{nvar} the number of \code{X}
variables and \code{npred} the number of variables to be predicted in
\code{Y}.} \item{scores}{a matrix of scores.} \item{loadings}{a matrix of
loadings.} \item{loading.weights}{a matrix of loading weights.}
\item{Yscores}{a matrix of Y-scores.} \item{Yloadings}{a matrix of
Y-loadings.} \item{projection}{the projection matrix used to convert X to
scores.} \item{Xmeans}{a vector of means of the X variables.}
\item{Ymeans}{a vector of means of the Y variables.} \item{fitted.values}{an
array of fitted values.  The dimensions of \code{fitted.values} are
\code{c(nobj, npred, ncomp)} with \code{nobj} the number samples and
\code{npred} the number of Y variables.} \item{residuals}{an array of
regression residuals.  It has the same dimensions as \code{fitted.values}.}
\item{Xvar}{a vector with the amount of X-variance explained by each
component.} \item{Xtotvar}{Total variance in \code{X}.}

If \code{stripped} is \code{TRUE}, only the components \code{coefficients},
\code{Xmeans} and \code{Ymeans} are returned.
}
\description{
Fits a PLSR model with the orthogonal scores algorithm (aka the NIPALS
algorithm).
}
\details{
This function should not be called directly, but through the generic
functions \code{plsr} or \code{mvr} with the argument
\code{method="oscorespls"}.  It implements the orthogonal scores algorithm,
as described in \cite{Martens and Næs (1989)}.  This is one of the two
\dQuote{classical} PLSR algorithms, the other being the orthogonal loadings
algorithm.
}
\references{
Martens, H., Næs, T. (1989) \emph{Multivariate calibration.}
Chichester: Wiley.
}
\seealso{
\code{\link{mvr}} \code{\link{plsr}} \code{\link{pcr}}
\code{\link{kernelpls.fit}} \code{\link{widekernelpls.fit}}
\code{\link{simpls.fit}}
}
\author{
Ron Wehrens and Bjørn-Helge Mevik
}
\keyword{multivariate}
\keyword{regression}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ncomp.R
\name{selectNcomp}
\alias{selectNcomp}
\title{Suggestions for the optimal number of components in PCR and PLSR models}
\usage{
selectNcomp(
  object,
  method = c("randomization", "onesigma"),
  nperm = 999,
  alpha = 0.01,
  ncomp = object$ncomp,
  plot = FALSE,
  ...
)
}
\arguments{
\item{object}{an \code{mvr} object.  The fitted model. It should contain a
\code{validation} element.}

\item{method}{character string, indicating the heuristic to use.}

\item{nperm}{number of permutations in the \code{"randomization"} approach -
not used in the \code{"onesigma"} approach.}

\item{alpha}{cutoff for p values in the \code{"randomization"} approach -
not used in the \code{"onesigma"} approach.}

\item{ncomp}{maximum number of components to consider when determining the
global minimum in the cross-validation curve.}

\item{plot}{whether or not to show a cross-validation plot. The plot for the
\code{"randomization"} approach shows models that do not differ
significantly from the global RMSEP minimum with open circles; the
\code{"onesigma"} approach shows the one-sigma bands around the RMSEP
values. In both cases, the selection is indicated with a blue dashed line.}

\item{\dots}{Further plotting arguments, e.g., to add a title to the plot,
or to limit the plotting range.}
}
\value{
A number indicating the suggested number of components in the model.
}
\description{
Choosing the best number of components in PCR and PLSR models is difficult
and usually done on the basis of visual inspection of the validation plots.
In cases where large numbers of models are built this choice needs to be
automated. This function implements two proposals, one based on
randomization (permutation) testing, and an approach based on the standard
error of the cross-validation residuals.
}
\details{
In both approaches the results of cross-validation are used, so the model
should have been calculated with some form of cross-validation. First, the
absolute minimum in the CV curve is determined (considering only the first
ncomp components), leading to the reference model. The randomization test
approach (Van der Voet, 1994) checks whether the squared prediction errors
of models with fewer components are significantly larger than in the
reference model. This leads for each model considered to a \eqn{p} value;
the smallest model not significantly worse than the reference model is
returned as the selected one.

The approach \code{"onesigma"} simply returns the first model where the
optimal CV is within one standard error of the absolute optimum (Hastie,
Tibshirani and Friedman, 2009). Note that here we simply use the standard
deviation of the cross-validation residuals, in line with the procedure used
to calculate the error measure itself. Some other packages implementing
similar procedures (such as \code{glmnet}) calculate an error measure for
each validation segment separately and use the average as the final
estimate. In such cases the standard error across segments is the relevant
measure of spread. For LOO, the two procedures are identical. In other forms
of validation, small differences will occur.
}
\examples{

data(yarn)
yarn.pls <- plsr(density ~ NIR, data = yarn, scale = TRUE,
                 ncomp = 20, validation = "LOO")
selectNcomp(yarn.pls, "onesigma", plot = TRUE, ylim = c(0, 3))
selectNcomp(yarn.pls, "randomization", plot = TRUE)
selectNcomp(yarn.pls, "randomization", plot = TRUE,
            ncomp = 10, ylim = c(0, 3))

}
\references{
Van der Voet, H. (1994) Comparing the predictive accuracy of
models using a simple randomization test. Chemom. Intell. Lab. Syst. 25 (2),
313-323

Hastie, T., Friedman, J. and Tibshirani, R. The Elements of Statistical
Learning: data mining, inference, and prediction, Springer (2013), 10th
printing with corrections, paragraph 7.10.
}
\seealso{
\code{\link{mvr}}, \code{\link{summary.mvr}}
}
\author{
Ron Wehrens, Hilko van der Voet and Gerie van der Heijden
}
\keyword{multivariate}
\keyword{regression}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simpls.fit.R
\name{simpls.fit}
\alias{simpls.fit}
\title{Sijmen de Jong's SIMPLS}
\usage{
simpls.fit(
  X,
  Y,
  ncomp,
  center = TRUE,
  orthScores = FALSE,
  stripped = FALSE,
  ...
)
}
\arguments{
\item{X}{a matrix of observations.  \code{NA}s and \code{Inf}s are not
allowed.}

\item{Y}{a vector or matrix of responses.  \code{NA}s and \code{Inf}s are
not allowed.}

\item{ncomp}{the number of components to be used in the modelling.}

\item{center}{logical, determines if the \eqn{X} and \eqn{Y} matrices are
mean centered or not. Default is to perform mean centering.}

\item{orthScores}{logical.  If \code{TRUE} the scores are orthogonalised for
increased numerical precision (default = FALSE, for speed).}

\item{stripped}{logical.  If \code{TRUE} the calculations are stripped as
much as possible for speed; this is meant for use with cross-validation or
simulations when only the coefficients are needed.  Defaults to
\code{FALSE}.}

\item{\dots}{other arguments.  Currently ignored.}
}
\value{
A list containing the following components is returned:
\item{coefficients}{an array of regression coefficients for 1, \ldots{},
\code{ncomp} components.  The dimensions of \code{coefficients} are
\code{c(nvar, npred, ncomp)} with \code{nvar} the number of \code{X}
variables and \code{npred} the number of variables to be predicted in
\code{Y}.} \item{scores}{a matrix of scores.} \item{loadings}{a matrix of
loadings.} \item{Yscores}{a matrix of Y-scores.} \item{Yloadings}{a matrix
of Y-loadings.} \item{projection}{the projection matrix used to convert X to
scores.} \item{Xmeans}{a vector of means of the X variables.}
\item{Ymeans}{a vector of means of the Y variables.} \item{fitted.values}{an
array of fitted values.  The dimensions of \code{fitted.values} are
\code{c(nobj, npred, ncomp)} with \code{nobj} the number samples and
\code{npred} the number of Y variables.} \item{residuals}{an array of
regression residuals.  It has the same dimensions as \code{fitted.values}.}
\item{Xvar}{a vector with the amount of X-variance explained by each
component.} \item{Xtotvar}{Total variance in \code{X}.}

If \code{stripped} is \code{TRUE}, only the components \code{coefficients},
\code{Xmeans} and \code{Ymeans} are returned.
}
\description{
Fits a PLSR model with the SIMPLS algorithm.
}
\details{
This function should not be called directly, but through the generic
functions \code{plsr} or \code{mvr} with the argument
\code{method="simpls"}.  SIMPLS is much faster than the NIPALS algorithm,
especially when the number of X variables increases, but gives slightly
different results in the case of multivariate Y.  SIMPLS truly maximises the
covariance criterion.  According to de Jong, the standard PLS2 algorithms
lie closer to ordinary least-squares regression where a precise fit is
sought; SIMPLS lies closer to PCR with stable predictions.
}
\examples{
## Simulation of SIMPLS stability
# The graphics produced, demonstrate the numeric instability of the original
# SIMPLS without score orthogonalization.
set.seed(42)
N <- 100
p <- 2000
ncomp <- 40
simData <- data.frame(X = I(matrix(rnorm(N*p),N)), y = rnorm(N))
pls    <- plsr(y ~ X, data=simData, ncomp=ncomp)
simps  <- plsr(y ~ X, data=simData, ncomp=ncomp, method="simpls")
simpsO <- plsr(y ~ X, data=simData, ncomp=ncomp, method="simpls", orthScores=TRUE)
normScores <- pls$scores
for(i in 1:ncomp)
  normScores[,i] <- normScores[,i]/sqrt(sum(normScores[,i]^2))

# Check number of equal digits
eqDig <- function(x,y){
  xy <- abs(x - y)
  xy[xy == 0] <- 10^-16
  -colMeans(log10(xy))
}
eqDig_PLS_oSIMPLS    <- eqDig(normScores, simpsO$scores)
eqDig_SIMPLS_PLS     <- eqDig(simps$scores, normScores)
eqDig_SIMPLS_oSIMPLS <- eqDig(simps$scores, simpsO$scores)
# Correlation between models
cor_PLS_oSIMPLS    <- diag(cor(pls$scores,simps$scores))
cor_SIMPLS_oSIMPLS <- diag(cor(pls$scores,simpsO$scores))
cor_SIMPLS_PLS     <- diag(cor(simps$scores,simpsO$scores))

par.old <- par(mfrow=c(2,1), mar=c(4,4,1,1), las=1)
matplot(2:ncomp,cbind(eqDig_PLS_oSIMPLS,eqDig_SIMPLS_PLS, eqDig_SIMPLS_oSIMPLS)[-1,],
        type="l", xlab="component", ylab="equal digits", ylim=c(0,17),
        panel.first=grid())
legend("bottomleft", legend=c("PLS, SIMPLS", "PLS, OrthSIMPLS", "SIMPLS, OrthSIMPLS"),
       col=1:3, lty=1:3)
matplot(1:ncomp,cbind(cor_PLS_oSIMPLS,cor_SIMPLS_oSIMPLS,cor_SIMPLS_PLS), type="l",
        ylab="correlation", xlab="component", panel.first=grid())
legend("bottomleft", legend=c("PLS, SIMPLS", "PLS, OrthSIMPLS", "SIMPLS, OrthSIMPLS"),
       col=1:3, lty=1:3)
par(par.old)
}
\references{
de Jong, S. (1993) SIMPLS: an alternative approach to partial
least squares regression.  \emph{Chemometrics and Intelligent Laboratory
Systems}, \bold{18}, 251--263.
}
\seealso{
\code{\link{mvr}} \code{\link{plsr}} \code{\link{pcr}}
\code{\link{kernelpls.fit}} \code{\link{widekernelpls.fit}}
\code{\link{oscorespls.fit}}
}
\author{
Ron Wehrens and Bjørn-Helge Mevik
}
\keyword{multivariate}
\keyword{regression}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pls-package.R
\docType{package}
\name{pls}
\alias{pls-package}
\alias{pls}
\title{Partial Least Squares and Principal Component Regression}
\description{
Multivariate regression methods Partial Least Squares Regression (PLSR), Principal Component
Regression (PCR) and Canonical Powered Partial Least Squares (CPPLS).
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/khliland/pls}
  \item \url{https://khliland.github.io/pls/}
  \item Report bugs at \url{https://github.com/khliland/pls/issues}
}

}
\author{
\strong{Maintainer}: Kristian Hovde Liland \email{kristian.liland@nmbu.no}

Authors:
\itemize{
  \item Bjørn-Helge Mevik \email{b-h@mevik.net}
  \item Ron Wehrens
}

Other contributors:
\itemize{
  \item Paul Hiemstra [contributor]
}

}
\keyword{internal}
